+{
   locale_version => 1.31,
# in cldr test/fi.xml why x{110}x < xdx though xd < x{110} ?
   entry => <<'ENTRY', # for DUCET v13.0.0
0111      ; [.1FEB.0021.0002][.0000.0039.0002] # LATIN SMALL LETTER D WITH STROKE
0110      ; [.1FEB.0021.0008][.0000.0039.0002] # LATIN CAPITAL LETTER D WITH STROKE
01E5      ; [.2051.0021.0002][.0000.0039.0002] # LATIN SMALL LETTER G WITH STROKE
01E4      ; [.2051.0021.0008][.0000.0039.0002] # LATIN CAPITAL LETTER G WITH STROKE
014B      ; [.2118.0021.0002][.0000.0039.0002] # LATIN SMALL LETTER ENG
014A      ; [.2118.0021.0008][.0000.0039.0002] # LATIN CAPITAL LETTER ENG
0167      ; [.21F7.0021.0002][.0000.0039.0002] # LATIN SMALL LETTER T WITH STROKE
0166      ; [.21F7.0021.0008][.0000.0039.0002] # LATIN CAPITAL LETTER T WITH STROKE
0077      ; [.2247.0021.0002] # LATIN SMALL LETTER W
0057      ; [.2247.0021.0008] # LATIN CAPITAL LETTER W
00FC      ; [.2270.0021.0002] # LATIN SMALL LETTER U WITH DIAERESIS
0075 0308 ; [.2270.0021.0002] # LATIN SMALL LETTER U WITH DIAERESIS
00DC      ; [.2270.0021.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
0055 0308 ; [.2270.0021.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
01DC      ; [.2270.0021.0002][.0000.0025.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
01DB      ; [.2270.0021.0008][.0000.0025.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE
01D8      ; [.2270.0021.0002][.0000.0024.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
01D7      ; [.2270.0021.0008][.0000.0024.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE
01D6      ; [.2270.0021.0002][.0000.0032.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
01D5      ; [.2270.0021.0008][.0000.0032.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON
01DA      ; [.2270.0021.0002][.0000.0028.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND CARON
01D9      ; [.2270.0021.0008][.0000.0028.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON
0292      ; [.2286.0021.0002][.0000.0039.0002] # LATIN SMALL LETTER EZH
01B7      ; [.2286.0021.0008][.0000.0039.0002] # LATIN CAPITAL LETTER EZH
01EF      ; [.2286.0021.0002][.0000.0039.0002][.0000.0028.0002] # LATIN SMALL LETTER EZH WITH CARON
01EE      ; [.2286.0021.0008][.0000.0039.0002][.0000.0028.0002] # LATIN CAPITAL LETTER EZH WITH CARON
00E5      ; [.22FB.0020.0002] # LATIN SMALL LETTER A WITH RING ABOVE
0061 030A ; [.22FB.0020.0002] # LATIN SMALL LETTER A WITH RING ABOVE
00C5      ; [.22FB.0020.0008] # LATIN CAPITAL LETTER A WITH RING ABOVE
0041 030A ; [.22FB.0020.0008] # LATIN CAPITAL LETTER A WITH RING ABOVE
212B      ; [.22FB.0020.0008] # ANGSTROM SIGN
01FB      ; [.22FB.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER A WITH RING ABOVE AND ACUTE
01FA      ; [.22FB.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER A WITH RING ABOVE AND ACUTE
00E4      ; [.22FC.0020.0002] # LATIN SMALL LETTER A WITH DIAERESIS
0061 0308 ; [.22FC.0020.0002] # LATIN SMALL LETTER A WITH DIAERESIS
00C4      ; [.22FC.0020.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
0041 0308 ; [.22FC.0020.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
01DF      ; [.22FC.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
01DE      ; [.22FC.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON
00E6      ; [.22FC.0021.0002] # LATIN SMALL LETTER AE
1DD4      ; [.22FC.0021.0002] # COMBINING LATIN SMALL LETTER AE
00C6      ; [.22FC.0021.0008] # LATIN CAPITAL LETTER AE
1D2D      ; [.22FC.0021.0014] # MODIFIER LETTER CAPITAL AE
01FD      ; [.22FC.0021.0002][.0000.0024.0002] # LATIN SMALL LETTER AE WITH ACUTE
01FC      ; [.22FC.0021.0008][.0000.0024.0002] # LATIN CAPITAL LETTER AE WITH ACUTE
01E3      ; [.22FC.0021.0002][.0000.0032.0002] # LATIN SMALL LETTER AE WITH MACRON
01E2      ; [.22FC.0021.0008][.0000.0032.0002] # LATIN CAPITAL LETTER AE WITH MACRON
00F6      ; [.22FD.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
006F 0308 ; [.22FD.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
00D6      ; [.22FD.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
004F 0308 ; [.22FD.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
022B      ; [.22FD.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
022A      ; [.22FD.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
00F8      ; [.22FD.0021.0002] # LATIN SMALL LETTER O WITH STROKE
006F 0338 ; [.22FD.0021.0002] # LATIN SMALL LETTER O WITH STROKE
00D8      ; [.22FD.0021.0008] # LATIN CAPITAL LETTER O WITH STROKE
004F 0338 ; [.22FD.0021.0008] # LATIN CAPITAL LETTER O WITH STROKE
01FF      ; [.22FD.0021.0002][.0000.0024.0002] # LATIN SMALL LETTER O WITH STROKE AND ACUTE
01FE      ; [.22FD.0021.0008][.0000.0024.0002] # LATIN CAPITAL LETTER O WITH STROKE AND ACUTE
ENTRY
};
