# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V34
13312
19904
19968
40960
64014
64016
64017
64018
64019
64021
64031
64032
64033
64034
64035
64037
64039
64042
131072
173792
173824
177978
177984
178206
178208
183970
183984
191457
191472
192094
196608
201547
201552
205744
END
