# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.



# The name this table is to be known by, with the format of the mappings in
# the main body of the table, and what all code points missing from this file
# map to.
$Unicode::UCD::SwashInfo{'ToBpt'}{'format'} = 's'; # string
$Unicode::UCD::SwashInfo{'ToBpt'}{'missing'} = 'n';

return <<'END';
28		O
29		C
5B		O
5D		C
7B		O
7D		C
F3A		O
F3B		C
F3C		O
F3D		C
169B		O
169C		C
2045		O
2046		C
207D		O
207E		C
208D		O
208E		C
2308		O
2309		C
230A		O
230B		C
2329		O
232A		C
2768		O
2769		C
276A		O
276B		C
276C		O
276D		C
276E		O
276F		C
2770		O
2771		C
2772		O
2773		C
2774		O
2775		C
27C5		O
27C6		C
27E6		O
27E7		C
27E8		O
27E9		C
27EA		O
27EB		C
27EC		O
27ED		C
27EE		O
27EF		C
2983		O
2984		C
2985		O
2986		C
2987		O
2988		C
2989		O
298A		C
298B		O
298C		C
298D		O
298E		C
298F		O
2990		C
2991		O
2992		C
2993		O
2994		C
2995		O
2996		C
2997		O
2998		C
29D8		O
29D9		C
29DA		O
29DB		C
29FC		O
29FD		C
2E22		O
2E23		C
2E24		O
2E25		C
2E26		O
2E27		C
2E28		O
2E29		C
2E55		O
2E56		C
2E57		O
2E58		C
2E59		O
2E5A		C
2E5B		O
2E5C		C
3008		O
3009		C
300A		O
300B		C
300C		O
300D		C
300E		O
300F		C
3010		O
3011		C
3014		O
3015		C
3016		O
3017		C
3018		O
3019		C
301A		O
301B		C
FE59		O
FE5A		C
FE5B		O
FE5C		C
FE5D		O
FE5E		C
FF08		O
FF09		C
FF3B		O
FF3D		C
FF5B		O
FF5D		C
FF5F		O
FF60		C
FF62		O
FF63		C
END
