/**************************************************************************
* WARNING: 'git_version.h' is automatically generated by make_patchnum.pl
*          DO NOT EDIT DIRECTLY - edit make_patchnum.pl instead
***************************************************************************/
#define PERL_PATCHNUM "v5.42.2"
#define PERL_GIT_UNPUSHED_COMMITS		\
    /*leave-this-comment*/
#define PERL_GIT_UNCOMMITTED_CHANGES
