# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.


return <<'END';
V62
94
95
96
97
168
169
175
176
180
181
184
185
706
710
722
736
741
748
749
750
751
768
885
886
900
902
2184
2185
8125
8126
8127
8130
8141
8144
8157
8160
8173
8176
8189
8191
12443
12445
42752
42775
42784
42786
42889
42891
43867
43868
43882
43884
64434
64451
65342
65343
65344
65345
65507
65508
127995
128000
END
