# !!!!!!!   DO NOT EDIT THIS FILE   !!!!!!!
# This file is machine-generated by lib/unicore/mktables from the Unicode
# database, Version 16.0.0.  Any changes made here will be lost!

# !!!!!!!   INTERNAL PERL USE ONLY   !!!!!!!
# This file is for internal use by core Perl only.  The format and even the
# name or existence of this file are subject to change without notice.  Don't
# use it directly.  Use Unicode::UCD to access the Unicode character data
# base.



# The name this table is to be known by, with the format of the mappings in
# the main body of the table, and what all code points missing from this file
# map to.
$Unicode::UCD::SwashInfo{'TokEHDesc'}{'format'} = 's'; # string
$Unicode::UCD::SwashInfo{'TokEHDesc'}{'missing'} = ''; # code point maps to the empty string

return <<'END';
13000		Man, seated, right knee raised, right arm raised, left arm in front of body.
13001		Man, seated, right knee raised, right arm raised with hand to mouth, left arm hanging beside the body.
13002		Man, seated on heel, right knee raised, right arm raised, left arm in front of body.
13003		Man, seated, right knee raised, back straight, both arms raised in front, handpalms outwards.
13005		Man, seated, right knee raised, back straight, both arms raised in front, handpalms outwards, under a corner of a wall.
13006		Man, seated, right knee raised, with a vase on its side, with liquid issuing from it (W54), orientated to the front, on his head, both arms raised towards the front, handpalms outwards.
13007		Man, seated, both knees down, with with a vase on its side, with liquid issuing from it (W54), orientated to the front, on his head, with the water ending on a block, both arms forward, extended downwards toward the block.
13009		Man, seated on heel, right leg extended with foot in front of left leg, right and left arm sloping downward to either side of the body.
1300A		Man, seated, right knee raised, right arm angled in front of the body, left arm raised, hand held verically, handpalm inwards (hnw gesture).
1300B		Man, seated, right knee raised, a vessel on his head, right arm raised in front, with the hand at the vessel, left arm hanging beside the body.
1300C		Man, seated, right knee raised, right arm extended forward, holding the shaft of an oar, with the blade at the level of the feet, left arm hanging beside the body.
1300D		Man, seated, right knee raised, right arm raised In front, holding a sceptre (S42), angled towards the head, left arm in front of the body, holding a stick, resting against the shoulder.
1300E		Man, seated on heel, right knee up, with a feather (H6) on his head, angling backwards, right arm forwards, forearm horizontal, holding a composite bow (T10), left arm in front of the body, holding a stick which angles backwards, leaning against the left shoulder.
1300F		Man (enemy), seated on heel, both knees down, both arms behind the back, bound together with rope.
13010		Man, curled up, right and left lower legs separated, back nearly horizontal, both arms forward, forearms horizontal, grasping a line of liquid (blood) which comes from the head.
13011		Man, seated on heel, right knee raised, with a band of cloth around the middle, both arms forward, holding an axe with the axeblade in the forehead of the man.
13012		Man, written horizontally, facing down, arms raised in front.
13013		Man, standing, bend forwards, arms hanging downwards, extended towards the front.
13014		A child, seated on nothing, forelegs separated, right arm raised with hand to mouth, left arm hanging beside the body.
13015		Child, seated on nothing, forelegs spread, right arm forward, extended towards the knee, handpalm down, left arm hanging beside the body.
13016		Child, wearing the red crown (S3), seated on nothing, forelegs spread, right arm raised in front with hand to mouth, left arm hanging beside the body.
13017		Man, standing, with a bend back, right arm forward, holding a staff/stick near the top, left arm hanging beside the body.
13018		Man, standing, with a slightly bent back, right arm in front holding a long staff or pole with a two-pronged end piece for holding down snakes (U116), left arm hanging beside the body.
13019		Man, standing, right arm in front, holding a long stick/staff, left arm hanging beside the body, holding a piece of cloth.
1301A		Man, standing on a base (statue), right arm forward, holding a stick/staff, which rests on the same base, left arm hanging beside the body holding a sceptre (S42), sceptre-head extending in front of the body.
1301B		The king, standing, with a long straight beard and short wig/coif, with a uraeus bound to the forehead, right arm forward, hand at the hight of the shoulder, holding a staff with a triangular piece (point down) at the hight of the waist, left arm hanging beside the body, holding a mace (T3) horizontally.
1301C		Man, standing, both arms towards the front, holding a branch, as if to strike with it.
1301D		Man, standing, back bend forward, right arm forward, holding a stick which slightly curves towards the body, left arm hanging beside the body.
1301E		Man, standing, right arm outstretched before him, with the left arm hanging beside the body.
1301F		Man, running, right arm raised in front, left arm in front of the body.
13020		Man, standing, with arms raised at either side of the body , handpalms inwards.
13021		Man, upside down, both arms raised at either side of the body, handpalms inwards.
13022		Man, standing, both arms raised in front, hands held vertically, handpalms outwards.
13023		Man, standing, head facing forwards, arms raised behind him, hand palms outwards.
13024		Man, dancing, left leg straight, right knee in front of the left leg, right foot behind the left leg, toes at the hight of the left foot, right arm in front of the body, left arm raised at the back, hand at the hight of the head.
13026		Man, standing, bend forwards, both arms forward, holding a stick with a bundle or a mat over his shoulders, bundle behind the back.
13027		Man, standing, back slightly bend forwards, both arms forward, holding a long stick (rammer), which ends at the bottom in a mortar.
13028		Man, standing, both arms raised in front, touching a vertically written wall with battlements near the top.
13029		Man, standing, back bend forwards, both arms forward, extending downward into a large vessel.
1302A		Man, standing inside a cross-section of a vessel that reaches to his shoulders, arms extended at either side of the body, holding the rim of the vessel.
1302B		Man, standing, arms at either side of the body, holding the necks of two emblematic animals with panther heads, connected by a line which runs under the feet of the man.
1302C		Man, standing on the back of two animals, which look either way, man arms extended at either side of the body, holding both heads of the animals.
1302D		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig.
1302E		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig, holding a holding a sceptre with a straight shaft, topped with the head of the Seth animal.
1302F		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing a coif with uraeus.
13030		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing a coif with uraeus, holding a flagellum (S45).
13031		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing a coif with uraeus, holding a flagellum (S45) and crook (S38) with the opening inwards.
13032		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown with uraeus (S1A).
13033		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown (S1), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13034		King, seated, both knees up, with covered legs and arms, without beard, wearing the white crown with uraeus (S1A), holding a flagellum (S45).
13035		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the red crown with (S3).
13036		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the red crown with (S3), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13037		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the red crown with (S3), holding a flagellum (S45).
13038		Man (shepherd), seated, both knees up, with covered legs and arms, with a short straight beard and short hair/wig, holding a stick which curves inwards, with a band of string or fabric, with the ties away from the body.
13039		Man, seated, both knees up, with covered legs and arms, with short hair/wig, holding an unknown object (mace/knife/stick?)
1303A		Man, seated, both knees up, with covered legs and arms, with a short bushy beard and long wig, holding a stick which angles slightly towards the body.
1303B		Man, with short straight beard, seated on a chair, no arms visible.
1303C		Man, with a short straight beard, seated on a chair, both arms extended forward, forearms horizontal, one above another, holding a flagellum (S45).
1303D		Man, seated, both knees down, with short straight beard, both arms forward, forearms horizontal, one above the other, holding a flagellum (S45).
1303E		A mummy with a long curved beard, standing upright.
1303F		A mummy with a long curved beard, lying horizontally.
13040		A mummy with a long curved beard, lying horizontally on top of a bed.
13041		Man, seated, right knee raised, right arm raised in front, left arm in front of body, holding a stick which which rests against the left shoulder.
13042		King, standing, with a long straight beard, wearing the red crown (S3), both arms forwards, hands at the hight of the belly/waist, holding a loaf of bread on a reed mat (R4) horizontally.
13043		Man, standing, back bend forward, both arms toward the front, holding a hoe with a rope connecting the two pieces (U6), with the forward piece at the level of the feet.
13044		Man, standing, right arm forward, forearm and hand at the level of the waist, left arm raised, forearm vertically, holding a stick nearly horizontally.
13045		Man, standing, right arm forward, forearm horizontal, handpalm upwards, left arm backwards, upper arm horizontal, forearm nearly vertical, handpalm inwards, with a dotted line running between the handpalms.
13046		Man, standing, head oriented backwards, with the arms hanging at either side of the body.
13047		Foreign man, with a bushy beard, standing, wearing a long dress, with  the arms hanging at either side of the body.
13049		Man, seated on heel, both knees down, back straight, arm extended in front, forearm horizontal, holding a wide cup (W10).
1304A		Stylistic early dynastic form of a man, standing, holding a stick.
1304B		Man, standing, wearing kilt, right arm extended in front, held horizontally, holding a sistrum (Y18), left arm hanging beside the body.
1304C		A dwarf, standing, arms hanging at either side of the body.
1304D		Man, standing, right arm forward, forearm horizontal at the hight of the waist, holding a knife (T30) at an forward angle, left arm hanging beside the body.
1304E		Man, seated, right knee raised, right arm raised, left arm hanging beside the body.
1304F		Man, seated, right knee raised, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
13050		Woman, seated, both knees up, with covered legs and arms, with long hair.
13051		Woman, pregnant, seated, both knees down, with a band of cloth around the head, right arm forward, angling down towards the knee, left arm hanging beside the body.
13052		Woman, pregnant, seated, both knees down, right arm forward, angling down towards the knee, left arm hanging beside the body, with the head and arms of a child coming from the legs.
13053		Woman, pregnant, seated, both knees down, right arm forward, angling down towards the knee, left arm hanging beside the body, with 3 vertical lines coming from the legs.
13054		Woman, seated, both knees down, nursing a child, seated on her lap, left arm forward, hand over the chest of the woman, right arm hanging beside the body, right arm forward, embracing/holding the child, left arm over the chest.
13055		Woman, seated, both knees down, with long hair and a band of cloth around the head, bound at the back, with two sticks forming a cross-like structure over the legs.
13056		Woman, seated on a block throne, with a child on her lap, facing her, both arms down, both arms forward, right arm behind the child, left arm in front of the child.
13057		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a diadem, holding a flower which curves forward, with the flower orientated downwards.
13058		Woman, seated, both knees down, no arms visible, with a flower on the knee, which curves towards the face, with the flower in front of the face.
13059		Woman, standing, with long hair, right arm extended forward, hand at the hight of the shoulder, holding a sistrum (Y18), vertically, left arm hanging beside the body.
1305A		God, seated, knees up, with covered legs and arms, with a long curved beard and long wig/hair,  with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head.
1305B		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a sun disk (N5) on its head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
1305C		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a sun disk (N5) on its head.
1305F		God, seated, knees up, with covered legs and arms, with the head of an ibis.
13060		God, seated, knees up, with covered legs and arms, with the head of a ram, with horizontal twisted horns.
13061		God, seated, knees up, with covered legs and arms, with the head of a ram with horizontal twisted horns, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
13062		God, seated, knees up, with covered legs and arms, with the head of a jackal.
13063		God, seated, knees up, with covered legs and arms, with the head of a Seth animal.
13064		An ithyphallic god, with long, curved beard, standing on a platform, wearing a double plume headdress, with a line coming from the back of the head, running down to the base, arm raised in back, with a flagellum (S45) written over the arm.
13065		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102).
13066		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head.
13067		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
13068		God, seated on heel, right knee raised, with long curved beard and coif, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
13069		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), with a line running from the crown down the back, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1306B		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), with a line running from the crown down the back, holding a khopesh sword (T16A) vertically.
1306D		God, seated, both knees up, with covered legs and arms, with a long, curved beard, wearing the double crown (S5), holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
1306E		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing a headdress consisting of two plumes with a sun disk (S63A/S70), holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
1306F		God, seated, both knees down, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers on rams horns (S77).
13070		God, in mummyform, standing upright on a platform, with a long straight beard, wearing a cap, both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically.
13071		God, in mummyform, standing upright on a platform, with a long straight beard, wearing a cap, both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, adorned with an ankh-sign (S34) and a djed-pillar (R11), inside a shrine supported by pillars resembling a stem of papyrus with a bud (M13).
13072		Bes figure, seen from the front, with tail between the legs and headdress, arms downwards, hands behind the back.
13073		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a crescent moon with the entire moon disk (N62) on its head.
13074		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head.
13075		Goddess, seated, knees up, with covered legs and arms, wearing the red crown (S3), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13076		The head of a human man, seen in profile, with a short straight beard.
13077		The face of a human man.
13078		A lock of hair with three strands.
13079		An eye.
1307A		An eye with two touches of eye paint above the eye.
1307B		An eye, with a line of eye-paint above the eye.
1307C		An eye with a painted lower lid.
1307D		An eye, written inside an oval.
1307E		An eye with a painted lower lid inside an oval.
1307F		An eye, with three lines coming from the bottom.
13080		An eye, with the markings of the head of a falcon.
13081		The left part of the white of an eye.
13082		The pupil of an eye.
13083		An eyebrow.
13084		The right part of the white of an eye.
13085		The right lower marking of an Udjat eye.
13086		The left lower marking of an Udjat eye.
13087		The left and right lower marking of an Udjat eye.
13088		The ear of a human.
13089		The nose, eye and cheek of a human.
1308A		The nose, eye and cheek of a human, written in a semi-cursive fashion.
1308B		A mouth.
1308C		A mouth with two vertical lines at the bottom of the sign.
1308D		A mouth with three vertical lines at the bottom of the sign
1308E		The upper lip of a human with teeth.
1308F		The upper and lower lip of a human, with teeth.
13090		A human mouth, seen from the side, with a wavy downwards sloping line coming from the lips.
13091	13092	A human breast.
13093		Two arms, raised upwards, with the forearms and hands vertical, handpalms inwards.
13094		Two arms, raised upwards, with the forearms and hands vertical, handpalms inwards, on top of a standard used for carrying religious symbols (R12).
13095		Two arms, raised, with the palms of the hands towards each other, written on it side, with a downwards curving line at the back.
13096		Two arms, lowered, with the elbows bent outwards, and the palms of the hands facing towards each other (D32), with a club used by washer-men for beating laundry (U36) written between the arms.
13097		Two arms, raised upwards, with the forearms and hands vertical, handpalms inwards, with a club used by washer-men for beating laundry (U36), written between the arms.
13098		Two arms, lowered, with the elbows bent outwards, and the palms of the hands facing towards each other.
13099		Two arms, holding an oar, in a rowing motion, below a cover.
1309A		Two arms, connected at the shoulders, one arm forwards, with a horizontal forearm, holding a shield as seen in profile, top curving inwards, other arm downwards, forearm vertical, holding a mace (T3) horizontally.
1309B		Two arms, connected at the shoulders, one arm forwards, with a horizontal forearm, holding a rectangular shield with inward curving sides, other arm downwards, forearm vertical, holding a mace (T3) horizontally.
1309C		Two arms, connected at the shoulders, spread sideways, forearms horizontal, with the palms of the hands facing upwards
1309D		A forearm, with the palm of the hand facing upwards.
1309E		A forearm with the hand holding a conical loaf of bread (X8).
1309F		A forearm with the hand holding a loaf of bread, without internal detail (X3).
130A0		A forearm with the hand holding a round vessel with an upstanding rim (W24).
130A1		A forearm with the hand holding a stick.
130A2		A forearm, with the palm of the hand downwards, with the upper arm bend forwards.
130A3		A forearm, with the palm of the hand downwards, upper arm vertical.
130A4		A forearm with the hand holding a flagellum (S45).
130A5		A forearm with the hand holding a sceptre (S42).
130A6		An arm, bend at the elbow, forearm nearly horizontal, holding a wand or brush.
130A7		A human hand.
130A8		A human hand with a forwards downwards line coming from the thumb.
130A9		A human hand with the thumb upwards and the palm curved upwards.
130AA		A human hand without a thumb.
130AB		A human hand, holding an oval.
130AC		A human hand, held as a fist, with the thumb on top.
130AD		A vertical human finger.
130B7		A horizontal human finger.
130B8		A phallus with a scrotum.
130B9		A phallus with a scrotum (D52), written over a folded piece of cloth (S29).
130BA		A phallus with a scrotum, with a line of liquid issuing from it.
130BB		Legs in a walking posture, feet orientated towards the reading direction.
130BC		Legs in a walking posture, feet orientated towards the reading direction, with a hook added to the front leg.
130BD		Legs in a walking posture, feet orientated away from the reading direction.
130BE		A leg with a bend knee.
130BF		A leg with a bend knee (D56), with a knife with a rounded blade and rounded handle (T30) written over it.
130C0		A human foot and lower leg.
130C1		A forearm, with the palm of the hand facing upwards (D36) written over a human foot and lower leg (D58).
130C2		A human foot and lower leg (D58), with a vase on its side, with liquid issuing from it (W54) on top of it.
130C3		A stylised row of three toes, with the ovals at the left side.
130C4		A stylised row of three toes, with the ovals at the right side.
130C5		A stylised set of two toes, resembling a cloth wound on a pole, an emblem of divinity (R8), on top of a base with a loop at the far end.
130C6		A human hand with the thumb downwards and the palm curved downwards.
130C7		A sidelock, usually worn by a child.
130C8		An arm, bend at the elbow, forearm nearly horizontal, hand held like a fist, with a stylus angled over the tip of the fist.
130D2		A bovid (bull), standing, with outwards curving horns.
130D3		A bovid (bull), standing, with its head lowered as if to attack.
130D4		A calf, standing.
130D5		A cult image of a bovid (cow), lying down, with a sun-disk (N5) with two feathers between the horns, with a flagellum (S45) on its back, and a necklace around the neck.
130D6		A bovid (cow), standing, head facing backwards over its body, with a calf below the cow, suckling.
130D7		A horse, about to rear, front legs nearly vertical.
130D8		A donkey/ass, standing.
130D9		A immature goat (kid).
130DA		A immature goat (kid), rearing.
130DB		A newborn bubalis antilope, lying down, legs folded beneath the body, long tail downwards.
130DC		A bovid (cow), lying down, legs folded beneath the body.
130DD		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard.
130DE		A ram (Ovis longipes palaeo-aegyptiacus), standing, without a beard.
130DF		A pig.
130E0		A cat, seated, tail curled over the body towards the back.
130E1		A hunting hound, standing, tail curling upwards.
130E2		A jackal, lying down, tail downwards.
130E3		A jackal, lying down, tail downwards, on top of a shrine.
130E4		A jackal, lying down, tail downwards, with a flagellum (S45) on its back, on top of a shrine.
130E5		A jackal, standing, tail down.
130E6		A jackal, standing, tail down, head looking backwards.
130E7		A jackal, standing, tail down (E17), on top of a standard used for the carrying of religious symbols, with an uraeus at the front of the standard.
130E8		A jackal, standing, tail down (E17), on top of a standard used for the carrying of religious symbols, with an uraeus and SdSd-pretuberance at the front of the standard, with a mace (T3) written horizontally over the vertical pole of the standard.
130E9		The animal of Seth, seated, tail up.
130EB		The animal of Seth, lying down, tail up.
130EC		A lion, standing, tail downwards.
130ED		A lion, lying down, tail curled over the body towards the back.
130EE		A panther or leopard, standing.
130EF		A hippopotamus (Hippopotamus amphibius).
130F0		An elephant.
130F1		A giraffe.
130F2		An oryx, standing.
130F3		A basket with four pieces of grain or fruit (M39) in front of an oryx, standing (E28), on top of a parcel of land with irrigation ditches (N24).
130F4		A gazelle, standing, with one horn visible.
130F5		A Nubian ibex (Capra nubiana).
130F6		A goat, standing, with a seal, attached to a neck string (S20) around its neck.
130F7		A hamadryas baboon (Papio hamadryas).
130F8		A monkey.
130F9		A desert hare, lying down.
130FB		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail folded under the rear.
130FC		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail folded under the rear (E35), in front of a sheath or receptacle with V shaped indentation at the top, bound at the top with a loop a the backside (V36G), on top of an alabaster basin with a diamond shaped inner marking (W3).
130FE		The head of a bovid (ox) with outwards curving horns.
130FF		The head of a bovid (cow), without horns, with an ear.
13100		The head of a bovid (bull), angled forward as if to attack.
13101		The head of a hippopotamus.
13102		The forepart of a lion, lying down.
13103		The head of a bubalis antilope.
13104		The forepart of a bubalis antilope.
13105		The head of a ram (Ovis longipes palaeo-aegyptiacus), with beard.
13106		The forepart of a ram (Ovis longipes palaeo-aegyptiacus), with beard.
13107		The head of a leopard.
13108		The head and neck of a long-necked horned animal, with a forked end to the neck with a horizontal line over the forked end.
13109		The head and neck of a long-necked horned animal.
1310A		The head and neck of a canine animal.
1310B		The horns of a bovid (ox).
1310D		A palm branch, stripped of leaves and notched with a round notch (M4), written within the horns of a bovid (F13).
1310E		A palm branch, stripped of leaves and notched with a round notch (M4), on top of a sun-disk (N5), written within the horns of a bovid (F13).
1310F		A horn of a bovid
13110		A vase on its side, with liquid issuing from it (W54), written over the horn of a bovid (F16).
13111		A tusk of an elephant.
13112		The lower jaw-bone of a bovid (ox).
13113		A tongue.
13114		The ear of a bovid (ox), without internal detail.
13115		The ear of a bovid (ox), without internal detail, with two stokes on the base.
13116		The hind-quarters of a seated lion or leopard.
13117		A foreleg of a bovid (ox), hoof orientated towards the reading direction.
13118		A foreleg of a bovid (ox), hoof orientated away from the reading direction.
13119		The leg and hoof of a bovid (ox).
1311A		The skin of a goat.
1311B		The skin of a cow, with the tail curved backwards.
1311C		The skin of a cow, with the tail hanging straight down.
1311D		The skin of a cow, with the tail hanging straight down, pierced by an arrow with rounded fletching (T11).
1311E		A water skin.
1311F		Three skins of foxes, tied together at the top.
13120		A simplified representation of three skins of foxes, tied together at the top.
13121		The udder and tail of an animal, seen from below.
13122		The tail of an animal.
13123		A heart without internal detail.
13124		A windpipe and heart, with a single horizontal stroke at the top.
13125		A windpipe and lungs.
13126		A spine with four ribs, with a downwards curving loop at the end of the spine.
13127		A spine with four ribs, written at an angle.
13128		A piece of spine with three vertebrae, angling forwards.
1312A		A spine, with the spinal cord hanging out at the backside.
1312B		A spine, with the spinal cord hanging out at either side.
1312C		A section of vertebrae, written vertically, with a small dot on top.
1312D		A rib.
1312E		Ribs of beef.
1312F		A leg-bone with meat.
13130		The uterus of a cow.
13131		An object resembling a long vertical line with two pointed lines on top.
13132		An intestine, with the opening to the bottom right.
13133		An intestine, with the opening to the top left.
13134		An intestine, with the opening to the bottom left.
13135		An intestine, with the opening to the top right.
13136		An intestine, with the openings top left and bottom right.
13137		An intestine, with the openings top right and bottom left.
13138		A folded piece of cloth (S29), written over an intestine, with the opening to the bottom right (F46).
13139		A piece of flesh, point towards the back, curving downwards.
1313A		Three pieces of flesh, point towards the back, curving downwards, arranged horizontally.
1313B		Three pieces of flesh, point towards the back, curving downwards, arranged vertically.
1313D		A pile of excrement.
1313E		The head of a ram (Ovis longipes palaeo-aegyptiacus), with beard, on top of a staff with a forked end.
1313F		An Egyptian vulture (Neophron percnopterus).
13140		Two Egyptian vultures (Neophron percnopterus), overlapping each other.
13141		An Egyptian vulture (Neophron percnopterus) (G1), written over a sickle (U1), with the handle towards the back.
13142		A long-legged buzzard (Buteo rufinus).
13143		A falcon.
13144		A falcon (G5) with a flagellum (S45) on its back.
13145		A falcon (G5) on top of a wickerwork basket (V30).
13146		A falcon (G5), on top of a standard used for carrying religious symbols (R12).
13147		A falcon (G5), on top of a moon sickle shaped boat.
13148		A falcon (G5), on top of a moon sickle shaped boat, which rests upon a pedestal (P15).
13149		A falcon (G5), on top of a collar of beads (S12).
1314A		A falcon (G5) with a sun-disk (N5) on its head.
1314B		A falcon (G5), on top of a moon-sickle shaped boat with an higher prow than stern, with a rudder, with the head of an antilope (F81) on top of the prow, facing inwards, with a downwards line with multiple horizontal lines written over it coming from the head, on top of a sledge (P60B).
1314C		An archaic image of a falcon/cult image of a falcon.
1314E		An archaic image of a falcon/cult image of a falcon, with a flagellum (S45) on its back.
1314F		An archaic image of a falcon/cult image of a falcon, wearing a headdress consisting F1E34of two plumes (S9).
13150		A griffon vulture (Gyps fulvus).
13151		A griffon vulture (Gyps fulvus), with a flagellum (S45) on its back.
13152		A griffon vulture (Gyps fulvus), on top of a wickerwork basket (V30), in front of a cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), upon a wickerwork basket (V30).
13153		An owl.
13154		Two owls, overlapping each other.
13156		A forearm, with the palm of the hand facing upwards (D36), written over an owl (G17).
13157		A mouth (D21), written over an owl (G17).
13158		A saharan helmeted guinea fowl (Numida m. meleagris), with a lappet.
13159		A hoopoe (Upupa epops).
1315A		A lapwing (Vanellus cristatus).
1315B		A lapwing (Vanellus cristatus), legs drawn towards the body, wings extended above the body.
1315C		A crested ibis (Ibis comata).
1315D		An African sacred ibis (Threskiornis aethiopicus) (G26A), on top of a standard used for carrying religious symbols (R12).
1315E		An African sacred ibis (Threskiornis aethiopicus).
1315F		A greater flamingo (Phoenicopterus roseus).
13160		A black ibis (Plegadis falcinellus).
13161		A saddle-billed stork (Ephippiorhynchus senegalensis), with lappet.
13162		Three saddle-billed stork (Ephippiorhynchus senegalensis), overlapping each other, the first bird having a lappet.
13163		A gray heron (Ardea cinerea).
13164		A gray heron (Ardea cinerea), legs drawn towards the body, seated on a pole, held upright through ropes (R91).
13165		A cattle egret (Bubulcus ibis).
13166		An ostrich (Struthio camelus), with both wings upwards.
13167		A great cormorant (Phalacrocorax carbo).
13168		A swallow (Hirundo rustica savignii).
1316A		A house sparrow (Passer domesticus niloticus).
1316C		A greater white-fronted goose (Anser albifrons).
1316D		A pintail duck (Anas acuta).
1316E		A greater white-fronted goose (Anser albifrons), flying, with a wing at either side of its body.
1316F		A greater white-fronted goose (Anser albifrons), alighting or flying up, with both wings behind the body.
13170		A widgeon (Mareca penelope).
13171		A quail chick (Coturnix coturnix).
13172		A half round loaf of bread (X1), written in front of a quail chick (G43).
13173		Two quail chicks (Coturnix coturnix), overlapping each other.
13174		A forearm, with the palm of the hand facing upwards (D36), written over a quail chick (G43).
13175		A forearm with the hand holding a conical loaf of bread (D37), written over a quail chick (G43).
13176		A quail chick (G43), written over a sickle (U1).
13177		A duckling, with open beak, outstretched wings on either side of the body, and the legs drawn towards the body.
13178		Three ducklings, with open beak, outstretched wings on either side of the body, and the legs drawn towards the body (G47), inside a nest.
1317A		Two plovers, overlapping each other.
1317B		A cattle egret (Bubulcus ibis), pecking and standing upon a fish with a long broad fin on its back, and two small fins on its belly.
1317C		A goose, standing, with its head downwards, as if it is picking up food
1317D		A bowl for incense with smoke rising from the bowl (R7), written in front of the legs of a human headed bird (G53A).
1317E		A trussed and plucked goose or duck, with the head backwards below its body.
1317F		The head of a pintail duck (Anas acuta).
13180		The head of a gray heron (Ardea cinerea).
13181		The head of a spoonbill (Platalea leucorodia).
13182		The head of a griffon vulture (Gyps fulvus), with a backwards curved neck.
13183		The fully extended wing of a bird.
13184		A feather.
13185		A feather with tow upwards angled ticks on the front.
13186		The claw of a bird, with two frontal digits visible.
13187		An egg.
13188		A lizard or gecko.
13189		A freshwater turtle.
1318A		A Nile crocodile (Crocodylus niloticus).
1318B		A Nile crocodile (Crocodylus niloticus) on top of a shrine.
1318C		A Nile crocodile (Crocodylus niloticus) with its tail curved forward, below the body.
1318D		A cult image of a crocodile.
1318E		A piece of crocodile skin with horizontal spines, or tail of a crocodile.
1318F		A frog or toad.
13190		A tadpole.
13191		A horned desert viper (Cerastes cerastes).
13192		A horned desert viper (Cerastes cerastes) (I9), coming out of an open rectangle, with the top line shorter than the bottom line.
13193		A cobra in repose (Naja haja).
13194		A cobra in repose (Naja haja) (I10), with a feather (H6) on its back, angling backwards.
13195		Two cobras in repose (Naja haja), overlapping each other.
13196		A half round loaf of bread (X1) above a a strip of land (17), written below a cobra in repose (Naja haja) (I10).
13197		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail.
13198		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), upon a wickerwork basket (V30).
13199		A snake with a coiled tail, consisting of two coils.
1319A		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail, consisting of two coils.
1319B		A Nile tilapia (Oreochromis niloticus).
1319C		A fish (Labeobarbus bynni)
1319D		A mullet (Mugil cephalus).
1319E		A freshwater elephant fish (Mormyrus kannume).
1319F		A fish with a long broad fin on its back, and two small fins on its belly.
131A0		The scale of a fish.
131A1		A fahaka puffer fish (Tetrodon fahaka).
131A2		An electric catfish (Malapterurus electricus).
131A3		A dung beetle, scarab (scarabaeus sacer).
131A4		A bee.
131A5		A sedge (M23) written above a half round loaf of bread (X1), in front of a bee (L2) written above a half round loaf of bread (X1).
131A7		A common locust.
131A8		A centipede.
131A9		A bivalve shell.
131AB		A water scorpion (nep cinerea).
131AC		A click beetle (Lanelater Notodonta), without legs.
131AD		A tree.
131AE		A branch, horizontally written (M3), written over a tree (M1).
131AF		A horned desert viper (Cerastes cerastes), written over a tree (M1).
131B0		A herb, with three flowers or leaves, curving upwards, with the middle flower/leaf higher than the other two.
131B1		A branch, horizontally written.
131B2		A branch, horizontally written (M3), over an owl (G17).
131B3		A palm branch, stripped of leaves and notched with a round notch.
131B4		A palm branch, stripped of leaves and notched with a round notch (M4), on top of a half round loaf of bread (X1).
131B5		A palm branch, stripped of leaves and notched with a round notch (M4), on top of a mouth (D21).
131B6		A palm branch, stripped of leaves and notched with a round notch (M4), on top of a stool made of reed matting (Q3).
131B7		A pool with lotus flowers, consisting of 3 flowers and two buds.
131B8		A lotus flower, with the flower facing forward, and the stem facing downward.
131B9		A lotus bud, with the bud facing forward, and the stem facing downward.
131BA		A lotus bud with a long stalk, folded under itself.
131BB		A lotus flower, facing forwards, with a long twisted stalk, which is folded under itself.
131BC		A leaf, stem and root of a lotus plant.
131C5		A stem of papyrus with a bud.
131C6		A cobra in repose (Naja haja, I10), written over a stem of papyrus with a bud (M13).
131C7		A clump of three papyrus flowers, with two buds bent down.
131C8		Three thin stems of papyrus with a bud, with two buds bend down, on top of a plan of a crossroads in a village (O49).
131C9		A clump of three papyrus flowers.
131CA		Three thin stems of papyrus with a bud, on top of a plan of a crossroads in a village (O49).
131CB		A flowering reed.
131CC		Two flowering reeds, arranged horizontally.
131CD		A flowering reed (M17), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
131CE		A flowering reed (M17) and a club used by washer-men for beating laundry as part of the cleaning process (U36), connected by a network consisting of four horizontal lines and three lines going from bottom corner to top corner.
131CF		A field of reeds with three flowering reeds and three stumps.
131D0		A field of reeds with three flowering reeds and a downwards loop at the far end.
131D1		A rush with two shoots.
131D2		Two rushes with two shoots (M22), arranged horizontally.
131D3		A sedge.
131D4		A sedge (M23), written on top of a mouth (D21).
131D5		A stem of papyrus with a flowering bud.
131D6		A desert plant, with four branches, with flowers on every branch (M26A), written on top of a mouth (D21).
131D7		A desert plant, with four branches, with flowers on every branch, on a horizontal base.
131D8		A forearm, with the palm of the hand facing upwards (D36), written over a desert plant, with four branches, with flowers on every branch, on a horizontal base. (M26).
131D9		A desert plant, with four branches, with flowers on every branch (M26A), written on top of a hobble for cattle without a crossbar (V20).
131DA		Three thin stems of papyrus with a flowering bud, on top of a plan of a crossroads in a village (O49).
131DB		A pod of a carob (Ceratonia siliqua) tree.
131DC		A root.
131DD		A rhizome of a lotus, resembling vertical lines coming from a vessel.
131DE		Either a tree growing in a pot, or a rhizome of a lotus, with a leaf coming from it.
131DF		A rhizome of a lotus, with two shoots at the bottom.
131E0		Three grains of corn, arranged horizontally.
131E1		Three grains of corn, arranged vertically.
131E2		Three grains of corn, arranged in a triangular form, with a single grain at the bottom.
131E3		An ear of corn.
131E4		A heap of corn.
131E5		A bundle of flax stems, bound together with string, loop and tie at the back, with the bolls visible on the top, squared at the bottom.
131E6		A bundle of flax stems, bound together with string near the top of the bundle.
131E7		A bundle of flax stems, bound together with string near the bottom of the bundle.
131E8		A basket with four pieces of grain or fruit.
131E9		A vertical bundle of reeds, tied together, with the ties at the front
131EA		A vertical bundle of reeds, tied together, with the ties resembling a triangle.
131EB		A log of wood.
131EC		A flower with four petals.
131ED		A vine with fruits on two forked supports, with the back of the vine descending down, into a basket or vessel.
131EE		A thorn.
131EF		The sky.
131F0		The sky with a sceptre with a forked end hanging from it.
131F1		The sky with a sceptre with a cone at the end hanging from it.
131F2		The sky, with lines of moisture coming from it.
131F3		The sun, within a halo.
131F4		The sun within a halo, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus).
131F5		The sun-disk upon a butchers block.
131F6		The sun, with thee beams of sunlight coming from it.
131F7		The moon, with one half obscured.
131F8		The moon, partly obscured to form a crescent at the top of the circle.
131F9		A crescent moon.
131FA		A crescent moon with a part of the moon disc.
131FB		The back half of a crescent moon (N63A), written over a star (N14).
131FC		A star.
131FD		A star, within a circle.
131FE		A strip of land, with three grains of sand written beneath it.
131FF		A strip of land.
13200		An oval of sand.
13201		A ripple of water (N35), written within an oval.
13203		Two ovals of sand, arranged vertically.
13204		A long tongue of land.
13205		A tongue of land.
13206		A tongue of land with a vertical line at the front.
13207		An irrigation canal.
13208		A parcel of land with irrigation ditches.
13209		The hill country over the edge of the cultivated areas.
1320B		A sand covered mountain over the edge of the cultivated areas.
1320C		The sun rising over a sand covered mountain over the edge of the cultivated areas.
1320D		A hill with a halo of sun rays behind it.
1320E		A sandy hill slope.
1320F		A hill with shrubs.
13210		A road with two shrubs at the top, and one at the bottom.
13211		A lump of clay.
13212		A grain of sand.
13213		Three grains of sand that are horizontally aligned to each other.
13214		An ingot of metal.
13215		A slightly less broad ingot of metal.
13216		A ripple of water.
13217		Three ripples of water, vertically aligned above one another.
13218		A canal.
13219		A rectangular pool.
1321A		A rectangular pool, with two diagonal strokes written inside.
1321B		A rectangular pool with sloping sides.
1321C		A rectangular pool, with vertical ripples of water written within it.
1321D		A rectangular pool (N37), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
1321E		A well with a ripple of water.
1321F		A well with a straight line of water.
13220		A wall, written vertically, with battlements on the corners, and one in the middle of the long sides (O36), in front of a mace with a pear-shaped head, written vertically (T3), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13221		The tigh of a bovid (ox), on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13222		A feather (H6), in front of a falcon (G5), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13223		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62) in front of a sedge written on top of a mouth (M24), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13224		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62), in front of a clump of three papyrus flowers, with two buds bent down (M15), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13225		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62) in front of a whip with a loop at the front (V23A) on top of a half round loaf of bread (X1), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13226		A sand covered mountain over the edge of the cultivated areas (N26), in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13227		An one-barbed harpoon, with handle, written horizontally (T21), on top of a crescent moon shape, connected by three lines, in front of a feather (H6), angled forwards on top of a standard with a round top, with an short vertical line beside the main pole (R14), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13228		An one-barbed harpoon, with handle, written horizontally (T21), on top of a crescent moon shape, connected by three lines, in front of a spear made into a standard, with a circle on either side of the speartip, with a loop over the standard (Jsesh/Gardiner R15), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13229		Man/god, standing, with a double plume/feather headdress, right arm forward, hand at the hight of the shoulder, holding a crook (S38) of the size of the man, left arm hanging beside the body, holding a flagellum (S45) horizontally, on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
1322A		A piece of crocodile skin with horizontal spines, or tail of a crocodile (I6), in front of a bovid (bull), standing, on top of a standard used for the carrying of religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1322B		A pustule or gland, or a packet of mummy bandages (Aa2), in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1322C		A newborn bubalis antilope, lying down, legs folded beneath the body, long tail downwards (E9), in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1322D		A double forked tool for the mending of nets, with a broad middle section (V26), written veritically, in front of a crook (S38), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1322E		A schematic representation of three sealed water pots in a rack, without horizontal line at the bottom, with the downwards lines descending to the bottom (W17E), in front of a spear made into a standard, with a circle on either side of the speartip, with a loop over the standard (Jsesh R15), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1322F		An African sacred ibis (Threskiornis aethiopicus) (G26A), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13230		A fish, with its tail orientated downwards (K4A), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13231		A tusk of an elephant (F18), above a human hand (D46), above a half round loaf of bread (X1) beside a plan of a crossroads in a village (O49), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13232		A windpipe and lungs (F36), in front of a cluster of signs consisting of a tusk of an elephant (F18), above a human hand (D46), above a half round loaf of bread (X1) beside a plan of a crossroads in a village (O49), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13236		A strip of land (N17), above a cluster of signs consisting of a bow, of an archaic type (Aa23), in front of a half round loaf of bread (X1), over a strip of land (N17), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13237		The pole of a balance, with the cross-bar resembling a feather (H6), written horizontally (U39D), in front of a falcon (G5), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13238		Two feathers (H6), orientated outwards, on top of a plan of a prehistoric building at the town of Nekhen, written as a circle / A circle with two vertical strokes, arragned horizontally, written within it (O48), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13239		A sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), with a fillet and a backwards angled feather (H6) on top of the staff (R19), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1323A		Two falcons (G5), arranged horizontally, on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1323B		A Nile crocodile (Crocodylus niloticus), with a feather (H6), angled backwards, on its head, on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1323C		An emblem with a human face with cow ears, with two upwards line which curl inwards (R129), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1323D		A wig, with a fillet, with a headdress consisting of two feathers, on top of a vertical pole, on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1323E		A feather (H6), on top of two bones of a fossil squid (belemnite) (R22), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1323F		A cobra in repose (Naja haja) (I10), with a feather (H6) on its back, angling backwards, on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13240		A stem of papyrus with a bud (M13), in front of a half round loaf of bread (X1), written below a cobra in repose (Naja haja) (I10), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13241		The animal of Seth, lying down, tail up (E21), on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13242		A wick of twisted flax, consisting of 3 loops (V28), in front of the red crown (S3), in front of three ripples of water, vertically aligned above one another (N35A), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13243		A sand covered mountain over the edge of the cultivated areas (N26), above a horned desert viper (Cerastes cerastes) (I9), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13244		A horned desert viper (Cerastes cerastes) (I9), written over a tree (M1), in front of a schematic representation of three sealed water pots in a rack, without horizontal line at the bottom, with the downwards lines descending to the bottom (W17E), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13245		A horned desert viper (Cerastes cerastes) (I9), written over a tree (M1), in front of hind-quarters of a seated lion or leopard (F22) above a half round loaf of bread (X1), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13246		A desert hare, lying down (E34), on top of a standard used for the carrying of religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13247		A basket with four pieces of grain or fruit (M39) in front of an oryx, standing (E28), on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13248		A jackal, lying down, tail downwards (E15), on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13249		A falcon, standing, with extended wings at either side of the body, at a 45° angle, on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1324B		A human foot and lower leg (D58), written between two sceptres with a straight shaft, a forked base, topped with the head of the Seth animal (S40), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1324C		A tree (M1), in front of a schematic representation of three sealed water pots in a rack, without horizontal line at the bottom, with the downwards lines descending to the bottom (W17E), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
1324D		A tree (M1), in front of hind-quarters of a seated lion or leopard (F22) above a half round loaf of bread (X1), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13250		A house or enclosure.
13251		A tie or strap, used with sandals (ankh-sign) (S34), written with its loop inside a house or enclosure (O1).
13252		A mace with a pear-shaped head, written vertically (T3), intruding with the top inside a a house or enclosure (O1).
13253		An oar, written vertically (P8), intruding with the top inside a a house or enclosure (O1), with a loaf of bread, without internal detail (X3) left of the oar, and a beer-jug (W22) right of the oar.
13254		A plan of a courtyard or reed shelter.
13255		A plan of a winding wall.
13257		A plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction.
1325E		A half round loaf of bread (X1), written inside a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6).
1325F		A wooden column, written horizontally (O29), written over a half round loaf of bread (X1), written inside a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6).
13260		A wickerwork basket (V30), on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6), with a half round loaf of bread (X1), written inside the enclosure.
13261		A falcon (G5), written inside a plan of a rectangular enclosure, with an internal rectangle in the top corner away from the reading direction.
13262		A tie or strap, used with sandals (ankh-sign) (S34), written inside a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6).
13265		A wall of the palace, with ornamental chevaux de frise on top of the wall, with internal decoration.
13266		A forearm, with the palm of the hand facing upwards (D36), written over a wall of the palace, with ornamental chevaux de frise on top of the wall, with internal decoration (O11).
13267		An enclosure wall with battlements on the front side and top, and on the vertial line of the opening.
13268		The corner of a wall, with battlements, with the opening in the bottom right corner.
13269		A wide cup (W10) above a half round loaf of bread (X1), written inside the enclosed part of an enclosure wall without battlements, with a block on each corner.
1326A		A plan and façade of a hall, with a cornice with Uraei on top, with half orientated forwards and half orientated backwards.
1326B		A plan of a hall, with a cornice with uraei.
1326C		A shrine, seen from the side, with a downwards sloping roof.
1326D		An archaic shrine, seen from the side, with a downwards sloping roof, with vertical poles in front of the shrine itself, connected to the shrine.
1326E		An archaic shrine, seen from the side, with a downwards sloping roof, with upwards curving lines coming from the roof, and short vertical lines in front of the shrine.
1326F		A tall shrine with a round roof, with two vertical lines enclosing the top, on a base, with internal decoration consisting of two vertical lines
13271		A façade of a shrine with a flat roof, with vertical sides, with a small doorway.
13272		An open booth supported by a pole.
13273		A double pavilion, with flower-topped capitals on the columns, with curved roofs, with two carrying chairs (Q2) facing away from one another.
13274		A pyramid with an enclosure wall.
13275		An Old Kingdom solar temple with an enclosure wall.
13276		An obelisk, with base.
13277		An obelisk of an Old Kingdom sun temple, with an circle (sun disk) on top of the obelisk.
13278		A stela on a base.
13279		A hall supported by two columns.
1327A		A column with a tenon at the top, represented as a vertical line.
1327B		A wooden column, written horizontally.
1327C		A wooden column, written vertically.
1327D		A forked pole for supporting a roof.
1327F		A tenoned door, written horizontally.
13280		A gateway.
13281		A façade of a palace or tomb, with the top resembling two floating horizontal lines.
13283		A door-bolt.
13284		A door-bolt (O34) written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13285		A wall, written vertically, with battlements on the corners, and one in the middle of the long sides.
1328A		A wall, with battlements on the corners, and one in the middle of the long sides, written at a forward angle.
1328B		The corner of a wall, with the opening in the bottom right corner.
1328C		A rectangular stone slab or brick.
1328D		A stairway.
1328E		A double stairway.
1328F		A fence, consisting of a horizontal base with four lines with bulbs at the end, angled backwards, with a curved line connecting the front of the base and the first line with a bulb, with a horizontal line with a loop at the front through the bulbs.
13290		A fence, consisting of a horizontal base, with five vertical lines with a bulb on top of it, with a horizontal line through the bulbs.
13291		The emblem erected outside the temple of Min, resembling the horns of a bovid (F13) on top of a stem of papyrus with a bud (M13), with a coil of rope (V1) between the horns.
13292		A domed building, in the form of a quarter of a circle, with a smaller quarter circle inside the corner away from the reading direction.
13293		A domed building, with a rectangle in the corner away from the reading direction.
13294		A plan of a prehistoric building at the town of Nekhen, written as an oval, without the angled lines touching the oval.
13295		A plan of a prehistoric building at the town of Nekhen, written as a circle / A circle with two vertical strokes, arragned horizontally, written within it.
13296		A plan of a crossroads in a village.
13297		A circular threshing floor covered with grain.
13299		A circle, with two lines at 45° coming from the front of the circle.
1329A		A heap of grain on a raised floor.
1329B		A boat/ship, resembling a crescent moon, on top of a rectangle representing water, with a carrying chair (Q2) inside the boat/ship, with an oar/rudder at the back.
1329C		An upside down boat/ship, resembling a crescent moon, on top of a rectangle representing water, with a carrying chair (Q2) inside the boat/ship, with an oar/rudder at the back.
1329E		A boat/ship with a vertical prow and a stern resembling the handle of a handle of a sickle (U1), on top of a rectangle resembling water, with an oval with rounded corners in the boat/ship, with an oar/rudder at the back.
132A0		A fishersman's boat, on top of a rectangle representing water, with a net inside the boat, without internal detail.
132A1		A rectangular sail, on a mast of a ship with two prongs, connected by vertical lines (P6), written behind the sail.
132A2		A mast of a ship with two prongs, connected by vertical lines.
132A3		A forearm, with the palm of the hand facing upwards (D36), written over a mast of a ship with two prongs, connected by vertical lines (P6).
132A4		An oar, written vertically.
132A5		A horned desert viper (Cerastes cerastes) (I9), written over an oar, written vertically (P8).
132A6		An oar used for the steering of a ship (rudder).
132A7		A mooring post.
132A8		A seat.
132A9		A carrying chair.
132AA		A stool made of reed matting.
132AB		A headrest.
132AC		A rectangular chest, with legs at the side.
132AD		A rectangular coffin with a rounded lid, and upstanding sides, with legs at the side.
132AE		A brazier with a flame rising from it, flame angling towards the front.
132AF		A loaf of bread (X3), a flat bottomed jar with handles (W23) and a round loaf of bread (X6), arranged horizontally on an one-legged table.
132B0		A one legged table with long upright loaves of bread, cut in halves.
132B1		Three circular items, arranged horizontally, on a one legged table with a bowl-shaped surface.
132B2		A round loaf of bread (X6), a tall water-jug (W14) and a loaf of bread (X3), arranged horizontally on a table with an inwards incline of the legs, with a horizontal beam connecting the legs( R3P).
132B3		A table with an inwards incline of the legs, with a horizontal beam connecting the legs.
132B5		A loaf of bread upon a reed mat.
132B6		A censer for fumigation, written horizontally, with a forwards, downwards line at the front.
132B7		A censer for fumigation, written horizontally, with a triangular shape at the front, with a backwards, downwards line at the back.
132B8		A bowl for incense with smoke rising from the bowl.
132B9		A cloth wound on a pole, an emblem of divinity.
132BA		A cloth wound on a pole, an emblem of divinity (R8), on top of a bag or a sack of linen, with a tie towards the front, and a (small) loop at the back (V33).
132BB		A cloth wound on a pole, an emblem of divinity (R8), written between a butchers block and a sandy hill slope (N29).
132BC		A cloth wound on a pole, an emblem of divinity (R8), written over a butchers block, with the pole at the bottom of the block.
132BD		A column imitating a bundle of stalks tied together.
132BE		A standard used for carrying religious symbols.
132BF		A falcon (G5) on a standard with a rounded top, with a feather (H6), angled forwards, attached to the standard in front of the falcon.
132C0		A feather (H6), angled forwards on top of a standard with a round top, with an short vertical line beside the main pole.
132C1		A spear made into a standard, with a circle on either side of the speartip, with a loop over the standard.
132C2		Two plumes (S9), mounted symmetrically on a papyrus stalk (M13), with a band of string or fabric (V12) written over the stem.
132C3		A standard resembling a lotus flower with two plumes on top.
132C4		A wig with a fillet, with a headdress consisting of two plumes (S9), on top of a pole.
132C5		A wig, with a fillet, with a headdress consisting of two feathers on top of the horns of a ram (S77), on top of a vertical pole, on top of a parcel of land with irrigation ditches (N24).
132C6		A sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), with a fillet and a backwards angled feather (H6) on top of the staff.
132C7		A flower with a long stem, enclosed by the horns of a bovid, upside down.
132C8		A flower with a long stem, enclosed by the horns of a bovid, upside down, with two plumes (S9) on top of the horns.
132C9		Two bones of a fossil squid (belemnite).
132CA		Two bones of a fossil squid (belemnite), with an extra triangular shape at each side of the circle.
132CB		Two bows, strung together as a package.
132CC		Two bows, strung together as a package, written vertically, with internal detail in the package.
132CD		A windpipe and lungs (F36), with a wide cup on each side of the lungs, from which two flowers come, one vertical, the other crossing the windpipe, to angle downwards on top of the vertical flower on the other side, with a stem of papyrus with a flowering bud between the two flowers.
132CF		An emblem with a human face with cow ears, with two upwards line which curl inwards.
132D0		A snake with a coiled tail, written vertically (I114), written inside a narrow shrine or stela on a base (O195A).
132D1		The white crown of Upper Egypt.
132D2		The white crown (S1), on top of a wickerwork basket (V30).
132D3		The white crown (S1), on top of a plan of a crossroads in a village (O49).
132D4		The red crown of Lower Egypt.
132D5		The red crown (S3), on top of a wickerwork basket (V30).
132D6		The union of the red crown of Lower Egypt and the white crown of Upper Egypt, with the white crown worn within the red crown. (i.e., the double crown).
132D7		The union of the red crown of Lower Egypt and the white crown of Upper Egypt, with the white crown worn within the red crown. (i.e., the double crown, S5), on top of a wickerwork basket (V30).
132D8		The red crown (S3), on top of a plan of a crossroads in a village (O49).
132D9		The blue crown.
132DA		The Atef crown with rams horns and a sun disk.
132DB		A headdress consisting of two plumes.
132DC		A band of cloth, used as a fillet.
132DD		A collar of beads with ornamental terminals.
132DE		A collar of beads.
132DF		A collar of beads (S12), written over a human foot and lower leg (D58).
132E0		A collar of beads (S12), written over a mace with a pear-shaped head, written vertically (T3).
132E1		A collar of beads (S12), written over a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40).
132E3		A pectoral of three glass or faïence beads.
132E4		A pectoral consisting of a circle with three strands of glass or faïence beads.
132E5		A pectoral consisting of a horizontal rectangle with a loop on top, with three vertical strands of glass or faience beads, resembling ripples.
132E6		A girdle worn by various gods, with four hanging pieces.
132E7		A bead necklace with a counterweight.
132E8		A seal, attached to a neck string, with the sting curving backwards and downwards, with a loop below the seal.
132E9		A seal, attached to a neck string.
132EA		A ring or bracelet.
132EB		A shoulder knot, with loops a the top ends and ties at the bottom ends.
132EC		Two pieces of cloth strung together / Two flagellums (S45) on top of/written through a cartouche in a round form (V9).
132ED		A girdle knot.
132EE		A garment with downwards curving ties at both sides.
132EF		A kilt or apron.
132F0		An oval bale of cloth.
132F2		A horizontal strip of cloth with two fringes.
132F3		A horizontal strip of cloth with fringes, combined with a folded piece of cloth (S29).
132F4		A folded piece of cloth.
132F5		A horned desert viper (Cerastes cerastes) (I9), written over a folded piece of cloth (S29).
132F6		A sickle (U1), written over a folded piece of cloth (S29).
132F7		A rectangular piece of cloth, with sloping fringes on its front short side, without internal decoration.
132F8		A sandal.
132F9		A tie or strap, used with sandals (ankh-sign).
132FA		A sun shade, made of a staff which holds feathers at the top, resembling a half circle, with internal detail.
132FC		A sun shade, made of a staff which holds feathers at the top, resembling a half circle, without internal detail.
132FD		A one handed fan, with a handle resembling a stem of papyrus with a bud (M13).
132FE		A crook.
132FF		A shepherd's crook.
13300		A sceptre with a straight shaft, a forked base, topped with the head of the Seth animal.
13301		A sceptre with a spiral shaft, a forked base, topped with the head of the Seth animal.
13302		A sceptre.
13303		A staff which widens towards the bottom.
13304		A vertical staff, with a flagellum (S45) attached to the back.
13305		A flagellum or flail, with the stick oriented forwards.
13306		A head-cloth.
13307		A mace with cup or dish shaped head, written horizontally.
13308		A mace with a pear-shaped head, written at a 45° angle forwards.
13309		A mace with a pear-shaped head, written vertically.
1330A		A mace with pear shaped head, written vertically (T3), on top of a sand covered mountain over the edge of the cultivated areas (N26).
1330B		A mace with pear shaped head, written vertically, with a strap around the handle, loop towards the front.
1330C		A cobra in repose (Naja haja) (I10), written over a mace with a pear-shaped head, written vertically (T3).
1330D		Two cobras in repose (Naja haja) (I10), arranged vertically, written over a mace with a pear-shaped head, written vertically (T3).
1330E		An axe with a round blade and a curved handle.
1330F		An axe with a hourglass-shaped blade, with a straight handle, written vertically.
13310		A dagger within a round tipped sheath.
13311		A dagger with a pointed tip.
13312		A bow, written horizontally, with a rectangular central section, with the tips of the bow forming a bowl-shaped loop.
13313		A bow, written horizontally, with a straight middle section, with the limbs forming a bowl-shaped loop.
13314		A composite bow, with handle bound to the bowstring when out of use.
13315		An arrow with rounded fletching.
13317		A bow-string, coiled up, with a loop at the front end, and the back end angled downwards, with a loop on it.
13318		Pieces of wood, lashed together with the top piece at an angle, with a loop and tie at the back.
13319		A throwing-stick, or a club used by foreign people.
1331A		A throwing-stick, or a club used by foreign people (T14), angled backwards.
1331B		A scimitar (khopesh), written horizontally.
1331C		A scimitar (khopesh), written vertically, blade curving forwards, with a handle resembling a stem of papyrus with a bud (M13).
1331D		A chariot.
1331E		A shepherds crook with a package lashed to it, with a tie above and below the package.
1331F		A harpoon-head with two horizontal strokes on top and an angled stroke below a curl as point.
13320		A harpoon-head with one horizontal stroke on top, and two angled strokes as point.
13321		An one-barbed harpoon, with handle, written horizontally.
13322		A two-barbed arrow-head, with the point facing downwards.
13323		A rounded two-barbed arrow-head, with the point facing downwards.
13324		A net used for fishing or fowling.
13325		A float, made of reed, with a tie towards the front at the top.
13326		A bird trap consisting of a horizontal beam and vertical beam, with two lines running from the horizontal beam to the vertical beam, crossing in the middle.
13327		A bird trap, with a horizontal and vertical pole, with one line running from the front of the horizontal beam to the middle of the vertical beam, and one line running from the middle of the horizontal beam to the top of the vertical beam.
13328		A butcher's block.
13329		A knife with a rounded blade and rounded handle (T30), written on top of a butcher's block (T28).
1332A		A knife with a rounded blade and rounded handle.
1332B		A knife sharpener with a rounded body and a loop at the back.
1332C		A knife sharpener with a rounded body and a loop at the back (T31), on top of legs in a walking posture, feet orientated towards the reading direction (D54)
1332D		A knife sharpener with a rounded body and a loop at the back (T31), written over a folded piece of cloth (S29).
1332E		A knife sharpener, with a thin, curved body, and a loop at the end.
13330		A vertical slaughtering knife, with cup-shaped handle.
13331		A vertical slaughtering knife, without distinct handle.
13332		A shield with a rounded top, with a loop on top, with a boss in the center of the shield.
13333		A sickle.
13334		A sickle with a more horizontal blade.
13335		An eye (D4), written above the blade of a sickle (U1).
13336		A platform with a sloping front (Aa11), written above the blade of a sickle (U1).
13337		A platform with a sloping front (Aa11), written above the blade of a sickle with a more horizontal blade (U2).
13338		A hoe with a rope connecting the two pieces, written at an angle.
1333B		A hoe with a rope connecting the two pieces, written horizontally.
1333C		A hoe without a rope connecting the two pieces.
1333D		A grain measure, with a forwards and downwards line of grain coming from the front.
1333E		A grain measure, with a forwards and downwards line of grain coming from the front, with three grains of corn, arranged horizontally (M33), above the grain measure.
1333F		A grain measure, with a forwards and downwards line of grain coming from the front, with a hook on top of the measure, resembling a crook (S38).
13340		A grain measure, with a forwards and downwards line of grain coming from the front, with a vertical human finger (D50) on top of the measure.
13341		A plough, with a cross-bar and two vertical lines on the end of the long beam.
13342		Two branches of wood, joined at one end.
13343		A sledge.
13344		A sledge, with the head of a canine at the front, with a rectangle on the sledge.
13345		A hoe or pick excavating a rectangular pool, with the handle angling downwards.
13346		A hoe or pick excavating a rectangular pool with sloping sides (N38), with the handle angling downwards.
13347		An adze.
13348		An adze without the downwards curve at the end of the handle.
13349		An adze, with an attached blade at the front (U19A), in a piece of wood.
1334A		A chisel with a conical handle and short blade.
1334B		A hair-pin or burin, with a horizontal line above and below the oval of the pin.
1334C		A hair-pin or burin, without the horizontal line on the top.
1334D		A drill, with a half-circle handle, with a backwards tick on top of the handle, with a forked drill-bit with a horizontal line on top of the forked drill-bit, with a short vertical stroke next to the drill, connected to the handle.
1334E		A stoneworker's drill weighted at the top with stones, lashed together with strings.
1334F		A drill with a truncated pyramid-shaped handle, with a round drill-bit with a horizontal line over it.
13350		A drill for drilling holes in beads, with an rectangular handle, and a round drill bit.
13351		A fire-drill in a piece of wood.
13352		A fire-drill on top of a piece of wood.
13353		A spiral, winding counter-clockwise away from its central point, ending at the right lower corner after about 1,5 turns (Z7), written behind a fire-drill in a piece of wood (U28).
13354		A pottery kiln.
13355		A bakers raker or two crossed castanet sticks.
13356		A pestle, inside a round-bottomed mortar.
13357		A pestle with a handle/tick at the front.
13358		A pestle.
13359		A spindle.
1335A		A horned desert viper (Cerastes cerastes) (I9), written over a spindle (U34).
1335B		A club used by washer-men for beating laundry.
1335C		A razor.
1335D		A balance, with a plummet resembling a heart (F34).
1335E		The pole of a balance, on a horizontal base, with a angled line on each side as support.
1335F		The semi-Hieratic form of a pole of a balance.
13360		The plummet of a balance.
13361		A staff or pole with a two-pronged end piece for holding down snakes.
13362		A coil of rope.
1336C		A horizontal line, intersected in the middle by a coil of rope (V1).
1336D		Hieroglyphic representation of the Hieratic form of a horizontal line, intersected in the middle by a coil of rope (V2).
1336E		A horizontal line, intersected in the middle by three coils of rope (V1).
1336F		A lasso, with a big loop at the front, with a tie at the front and a small loop at the back, with an upwards, downwards cuving line behind the loop.
13370		A looped rope, with two small loops at the bottom, and a small loop within a big loop above the small loops.
13371		A loop of cord with the ends upwards.
13372		A loop of cord with the ends downwards.
13373		A folded piece of cloth.
13374		A loop of cord with the ends downwards (V7), on top of a vertical line, on top of a horizontal base.
13375		A half-round loaf of bread (X1), written inside a loop of cord with the ends downwards (V7).
13376		A cartouche in a round form.
13377		An oval cartouche.
13378		The rear half of a cartouche, with the opening to the back.
1337C		A band of string or fabric.
1337D		A feather bend forward.
1337E		A wide band of string or fabric, with the shorter section lying on top of the larger section.
1337F		A rope used for the tethering of animals.
13380		A rope used for the tethering of animals, with an diacritic stroke added to the large loop.
13381		A rope used for the tethering of animals (V13), written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13382		A looped cord, used as a hobble for cattle, with two horizontal loops and eight vertical loops.
13383		A rolled up herdsman's shelter of papyrus with an rectangle at either side of the loop.
13384		A rolled up herdsman's shelter of papyrus.
13385		A hobble for cattle with a crossbar.
13386		A hobble for cattle without a crossbar.
13393		A cobra in repose (Naja haja) (I10), written over a hobble for cattle without a crossbar (V20).
13394		A whip.
13395		A whip, without internal detail.
13396		A whip with a loop at the front.
13397		A string wound on a stick.
13398		A string wound on a stick, with a tie sticking out at the front.
13399		A double forked tool for the mending of nets, with a broad middle section.
1339A		A double forked tool for the mending of nets, with a thin middle section.
1339B		A wick of twisted flax, consisting of three loops.
1339C		A forearm, with the palm of the hand facing upwards (D36), written over a wick of twisted flax, consisting of three loops (V28).
1339D		A swab made from a hank of fibre with two loops.
1339E		A swab made from a hank of fibre with two loops (V29), written over a wickerwork basket with a handle, orientated with the handle to the back (V31).
1339F		A wickerwork basket.
133A1		A wickerwork basket with a handle, orientated with the handle to the back.
133A2		A wickerwork basket with a handle, orientated with the handle to the front.
133A3		A wickerwork frail with ties.
133A4		A bag or a sack of linen, with a tie towards the front, and a (small) loop at the back.
133A5		An oval bag or sack with linen, without ties or loops, with three horizontal lines over the bag.
133A6		A bag or a sack of linen, with a tie towards the front, and a (small) loop at the back, with a horizontal line over the bag.
133A7		A bag or sack of linen, with a hollow bottom.
133A8		A sheath or receptacle with V shaped indentation at the top, bound at the top with a tie at the backside.
133A9		A fastening knot of a dress on the shoulder.
133AA		A basket-shaped bag with a tied of end at the front, resembling a loop / A uterus of a cow, tied of at the front.
133AB		A bandage.
133AC		A tie or straps with a loop at the top, and a downward hanging loop at either side of the strings.
133AF		A sealed ointment jar, with ties.
133B0		A sealed ointment jar, without ties.
133B1		An alabaster basin with a diamond shaped inner marking.
133B3		An open booth supported by a pole (O22), on top of an alabaster basin with a diamond shaped inner marking (W3).
133B4		A butcher's block (T28), on top of an alabaster basin with a diamond shaped inner marking (W3).
133B5		A flat bottomed cauldron with handles and an upstanding rim.
133B6		A granite vessel with small handles.
133B7		A low broad oval, with a triangular shape at both short ends of the oval.
133B8		A stone jug with a large handle connecting to the rim, and a small round handle.
133BA		A wide cup.
133BB		A wide cup with a spout.
133BC		A ring-stand for jars.
133BD		A ring-stand for jars, with a level bottom.
133BE		A red earthenware pot.
133BF		A tall water pot.
133C1		A tall water pot, with a forwards, downwards line of liquid coming from the top of the vessel.
133C2		A tall water pot with a spout at the back, with a forwards, downwards line of liquid coming from the top of the vessel, inside a ring-stand.
133C3		Three sealed water pots in a rack.
133C4		A schematic representation of three sealed water pots in a rack, without horizontal line at the bottom, with the downwards curving lines descending to the bottom.
133C5		Four sealed water pots in a rack.
133C7		A milk jar in a carrying sling.
133C8		A milk jar with two horizontal lines over the jar, with a leaf covering the milk.
133C9		A twin set of conical wine jars, bound together.
133CA		A beer-jug.
133CB		A flat bottomed jar with handles.
133CC		A round vessel with an upstanding rim.
133CE		A round vessel with an upstanding rim (W24), on top of long legs in a walking posture, feet orientated towards the reading direction.
133CF		A half round loaf of bread.
133D0		A loaf of bread with a horizontal line inside the bread.
133D1		A loaf of bread, without internal detail.
133D2		A roll of bread, with a diamond in the middle.
133D3		An roll of bread, with round imprints at the var sides and in the middle.
133D4		A roll of bread without internal detail.
133D5		An horizontal oval shape, with two upwards curving strokes added at the front short side.
133D6		A round loaf with a finger mark at the bottom.
133D8		A half loaf of bread.
133D9		A conical loaf of bread, with a triangular finger mark at the bottom.
133DA		A conical loaf of bread without internal detail.
133DB		A papyrus scroll, rolled up, tied and sealed, written horizontally, with ties.
133DC		A papyrus scroll, rolled up, tied and sealed, written vertically, with ties.
133DD		A papyrus scroll, rolled up, tied and sealed, written horizontally.
133DE		A scribe's kit, consisting of a palette, ink or paint pouch and a reed pen.
133DF		A scribe's kit, consisting of a palette, ink or paint pouch and reed pen, with the reed pen towards the front.
133E0		A game board (of Senet) with game pieces.
133E1		A game piece, pawn, or draughtsman.
133E2		A rounded harp.
133E3		A sistrum, with the top piece in the form of a shrine encosed by two curls.
133E4		A vertical stroke.
133E5		Three vertical strokes in a horizontal orientation.
133E8		Three vertical strokes in a triangular orientation with one stroke a the top.
133E9		Three vertical strokes in a triangular orientation with one stroke a the bottom.
133EA		Three vertical strokes in a vertical orientation.
133EB		Three horizontal strokes arranged vertically.
133ED		Two diagonal strokes arranged horizontally.
133EE		Two vertical strokes, arranged horizontally.
133EF		A doubly curved, diagonal stroke.
133F0		A diagonal stroke.
133F1		A slightly inclined stroke with two smaller strokes protruding at its right lower end, the upper one being shorter than the lower of them.
133F2		A spiral, winding counter-clockwise away from its central point, ending at the right lower corner after about 1,5 turns.
133F3		A short horizontal oval.
133F4		A cross.
133F5		Two crossed diagonal strokes with acute angles at the left and right side, and obtuse angles at the upper and lower side.
133F6		A short horizontal stroke overlapped by a longer vertical stroke, the crossing point is in the middle of both strokes.
133F7		Two short horizontal lines, arranged vertically, connected by a downwards, outwards sloping line.
133F8		A  geometrical circle
133F9		A curved diagonal stroke.
1340D		A human placenta (a circle with lines written within horizontally).
1340E		A pustule or gland, or a packet of mummy bandages.
1340F		A pustule or gland, with a forward, downwards line of fluid issuing forth.
13410		A spouted cup.
13411		A part of the steering gear of ships, consisiting of two beams.
13412		Two pieces of cloth strung together / a line of thread being woven into a weave.
13413		A vertical tool, with a upwards curved handle, with the handle towards the back.
13414		A vertical tool, with a upwards curved handle, with the handle towards the front.
13416		A single line of irrigation canals.
13417		Part of a carrying-chair.
13418		A graver.
13419		A platform with a sloping front.
1341A		A rectangular platform.
1341B		A representation of two ribs, top line curving downwards.
1341C		A representation of two ribs, lines curving outwards.
1341D		A representation of two ribs, written parallel.
1341E		A representation of two ribs, written parallel, short form.
1341F		A cover of a quiver.
13420		A cover of a quiver with a smaller rectangle inset.
13421		A parabolic shape with a dot at each side.
13422		A type of ornament, with two ties at the top.
13423		An object consisting of a horizontal rectangle with a triangular indentation at the bottom, with a vertical line on top of it, connected through a triangle.
13424		A forearm, with the palm of the hand facing upwards (D36), written over an object consisting of a horizontal rectangle with a triangular indentation at the bottom, with a vertical line on top of it, connected through a triangle (Aa21).
13425		A warp, stretched between two forked poles.
13426		A warp, stretched between two poles resembling cup-like shapes with upwards ticks on them.
13427		A crescent moon shape written over a thin triangle, point downwards.
13428		A pole with a hook at the front to which one binds an enemy.
13429		A milling tool, with a long vertical stick under a circle with triangular shapes at the sides and the top.
1342A		An instrument used by bricklayers, with a triangular extension with a horizontal top at the front.
1342B		An instrument used by bricklayers, with a round extension at the front.
1342C		An ornamental chevaux de frise on top of walls, with a circle dividing the top and base.
1342D		An ornamental chevaux de frise on top of walls, with a flat base.
1342E		A bow, of an archaic type, with the ends towards the back.
13460		Man, seated, right knee raised, right arm raised, left arm behind body.
13461		Man, seated, both knees raised, foot flat on the ground, right arm raised, left arm in front of the body.
13462		Man, seated, right knee raised, right foot in front of the left knee, left foot horizontal, right arm raised in front, left arm in front of the body.
13463		Man, seated on heel, both knees down, right arm raised, left arm in front of body.
13464		Man, squatting, both knees forwards, knees horizontal, right arm raised, left arm in front of body.
13465		Man, seated, both knees raised, foot flat on the ground, right arm raised, left arm hanging beside the body.
13466		Man, seated, right knee raised, right arm raised, righ hand horizontal, handpalm upwards, left arm in front of body.
13467		Man, seated, right knee raised, right arm raised, left arm in front of body, hand extending beyond the torso.
13468		Man, seated on heel, right leg extended with foot in front of left leg, right arm raised with hand held vertically, hand palm outwards, left arm hanging beside the body.
13469		Man, seated on heel, both knees down, right arm raised, hand held vertically, hand palm outwards, left arm hanging beside the body.
1346A		Man, seated, right knee raised, right arm raised, hand held vertically, handpalm inwards, left arm hanging beside the body.
1346B		Man, seated, right knee raised, right arm raised, hand held horizontally, hand palm upwards, left arm hanging beside the body.
1346C		Man, seated, both knees down, righ arm raised, hand held vertically, handpalm inwards, left arm held in front of the body, lower arm and hand horizontal, handpalm upwards.
1346D		Man, seated on heel, both knees down, one arm visible, raised in front, hand at the hight of the shoulder.
1346F		Man, seated, both knees down, right arm raised with hand to mouth, left arm hanging beside the body.
13470		Man seated, right knee raised, right arm raised with hand to mouth, left arm raised in front of the body.
13471		Man, seated, both knees raised, foot flat on the ground, right arm raised with hand to mouth, left arm not visible.
13472		Man, seated, right knee raised, rigth arm raised, hand vertical, handpalm towards the face, in front of the face, left arm hanging beside the body.
13473		Man, seated, right knee raised, right foot in front of the left knee, left foot horizontal, right arm raised with hand to mouth, left arm hanging beside the body.
13474		Man, seated, both knees down, right and left arm sloping downwards to either side of the body.
13476		Man, seated, right knee raised, back bent forward, both arm raised in front, handpalms outwards.
13477		Man, seated on heel, right knee raised, with arms raised in front of him, hand-palms facing outwards.
13478		Man, kneeling with both knees on the ground, with arms raised in front of him, hand-palms facing outwards.
13479		Man, kneeling with both knees on the ground, with arms raised in front of him, hand-palms facing inwards.
1347A		Man, kneeling with both knees on the ground, back bend forwards, arms extended forwards, foreams horizontal at the hight of the waist, handpalms downwards.
1347B		Man, seated, right knee raised, head facing backwards, both arms raised at the back, hands vertical, handpalms outwards.
1347C		Man (foreigner), seated on heel, right knee raised, head facing backwards, wearing a syrian clothing with long open sleeves, both arms raised towards the back, hands vertical, handpalms outwards.
1347D		Man, seated, both knees raised, with covered knees, arms covered by cloth.
1347E		Man, seated on heel, right leg extended with foot in front of left leg, right arm angled in front of the body, left arm raised, hand held vertically, handpalm inwards (hnw gesture).
1347F		Man, seated on heel, both knees downward, right arm angled in front of the body, left arm raised, hand held vertically, handpalm inwards (hnw gesture).
13480		Man, seated, both knees downward, right arm sloping downward, left arm raised, hand held vertically, handpalm inwards.
13481		Man, seated, right knee raised, right arm raised in front, left arm raised.
13482		Man, seated, right knee raised, right arm angled behind the back, left arm raised, hand held vertically, handpalm outwards.
13483		Man, seated, right knee raised, right arm raised in front, left arm not shown.
13484		man, seated, right knee raised, without visible arms.
13485		Man, seated, right knee raised, with a hair-lock, arm raised in front, hand held horizontal, handpalm upwards.
13486		Man, seated, right knee raised, with arms raised at either side of the body, hands on either side of the head.
13487		Man, seated, right knee raised, with arms raised at either side of the body, elbows bent, handpalms inwards.
13489		Man, standing, with the arms hanging at either side of the body.
1348A		Man, standing, bent slightly forwards, with a feather/cone on its head, angled backwards, with the arms hanging at either side of the body.
1348B		Two men standing, facing each other holding hands, with the other arm hanging beside the body.
1348C		Two men standing upon a base, facing each other holding hands, with the other arm hanging beside the body.
1348D		Two man standing, the first leading the second by the hand.
1348E		Two man standing, facing each other, holding each other by the arms (wrestling).
1348F		Naked man, standing, with spread legs, arms hanging beside the body, with a dotted line between the legs.
13490		Naked man, standing, with spread legs, arms hanging beside the body.
13491		Naked man, standing, fallus erect, producing a line of liquid, right arm hanging downwards, hand above the fallus, left arm hanging beside the body.
13492		Man, standing, with the arms folded over each other.
13493		Man, standing, right arm outstretched before him, with the left arm bend, hand on the waist.
13494		Man, standing, bend slightly forward, right arm stretched in front at waist hight, left arm hanging beside the body.
13496		Man, standing, right arm hanging beside the body, left arm raised, Hand on top of the head.
13498		Man, standing, head facing backwards, arms raised in front, hand palms outwards.
13499		Man, standing, head facing backwards, arms raised behind him, hand palms outwards.
1349A		Man, standing, right arm raised with hand held vertical, handpalm outwards, left arm hanging beside the body.
1349B		Man, standing, right arm raised with hand to mouth, left arm hanging beside the body.
1349C		Man, standing, right arm raised in front, handpalm outwards, left arm at a 45° angle downwards at the front.
1349D		Man, standing, both arms held in front, about waist hight.
1349E		Man, standing, both arms held downwards at the front.
134A0		Man, standing, with arms raised at either side of the body bent at a 90° angle, handpalms inwards.
134A1		Man, standing, bend forward, arms hanging beside the body.
134A2		Man, standing, wearing a lion/leopard-skin, left hand raised in front.
134A3		Man, standing, right arm hanging beside the body, left arm raised, hand on the top of the head.
134A4		Man, standing, with arms raised at either side of the body, hands on either side of the head.
134A5		Man, standing, bend forwards, both arms raised in front, handpalms outwards.
134A7		Man standing, bend slightly forwards, both arms extended forwards, right hand at the hight of the shoulder, handpalm downwards, left arm below the right arm, handpalm downwards, as if to place the hands on something to leap over it.
134A8		Man, standing, bend forwards, front arm hanging downwards towards the front, rear arm bend at a 90° angle, forearm and hand horizontal, rear arm hand extending beyond the front arm, at hip level.
134AA		Man, standing, bend forwards with back nearly horizontal, arms hanging downwards, extended towards the front.
134AB		Man, standing on top of a base, bend forwards with back nearly horizontal, arms hanging downwards, extended towards the front.
134AC		Man, standing, back slightly bend forwards, arms twisted together at the front on a downward angle, line angling forewards from the head (braid, hairlock).
134AD		Man, on an incline line, one leg straight at the bottom, the other bend and raised, right arm holding the top of the incline, left arm haning beside the body at the back.
134AF		Man, standing, back bend forwards, back horizontal, head slightly raised, arms hanging downwards, extended towards the front, fingers/hands at the same level as the feet.
134B0		Man, written at a 45° degree angle forward, one leg extended forwards, arms raised in front.
134B1		Man, written at an angle, right knee raised at slightly over 90° degree angle, arms extended forwards, with a knife with a triangular blade and straight handle (T30A) in the head, blade downwards.
134B2		Man, written horizontally, facing downwards, back slightly bent, arms raised in front, knife with the blade down, angled with the tip at the waist.
134B3		Man, prostracting, right leg extended, left leg bend and raised, both arms downwards, slightly bend, with the hands at the same level as the left knee and toes of the right leg.
134B4		Man, prostracting on a strip of land, right leg extended, left leg bend and raised, both arms downwards, slightly bend, with the hands at the same level as the left knee and toes of the right leg.
134B5		Man, written horizontally, facing downwards, both arms raised in front, written upon a base, with the hands and the toes touching the base.
134B6		Man, prostracting on a strip of land, with grains of sand written below it, right leg extended, left leg bend and raised, both arms downwards, slightly bend, with the hands at the same level as the left knee and toes of the right leg.
134B8		Man, prostracting, right leg extended, left leg bend and raised, left arm downwards, slightly bend, with the hands at the same level as the left knee and toes of the right leg, right arm raised, handpalm outwards.
134B9		Man, written horizontally, facing downwards, with knees and hips bend, with arms hanging loosly on either side beside the body (i.e., a corpse lying on the ground).
134BA		Man, written horizontally, facing upwards, with knees and hips bend, with the arms bend at either side of the body, elbows downwards, hands upwards, facing outwards at either side.
134BB		Man. naked, written horizontally, facing upwards, with the head looking downwards, with knees and hips bend, lower arm bend beside the body, elbow downwards, hand upwards, facing outwards, upper arm hanging loosely beside the body.
134BC		Man, written horizontally, facing upwards, arms hanging on either side of the body.
134BD		Man, written horizontally, facing downwards, one leg extended, one leg bend, top arm hanging beside the body, bottom arm bend, forearm written horizontally, handpalm downwards.
134BF		Man, written horizontally, facing downwards, knees and hips bend, arms raised at either side of the body, elbow bend downwards, hands upwards, facing inwards, with a round dotted line running between the hands (i.e., a man swimming).
134C0		Man, horizontal, in a swimming pose, right arm forwards, left arm beside the body, above a rectangle representing water.
134C1		Man, horizontal, in a swimming pose, right arm forwards, left arm beside the body, above a rectangle representing water, with a curved line of water over the man starting at the right hand.
134C3		A man, written at a 35° forwards, as if standing, with both arms raised in front, written inside a circle of wavy lines.
134C4		Man, written at a 45° angle forward, hips at a 90° angle, knees at an 90° angle legs slightly spread out, both arms forward, at a downwards angle, forearms separated, written inside a circle of wavy lines.
134C5		Man, written at a 45° angle forward, both arms raised at either side of the body, handpalms outwards, inside an oval with a dotted line.
134C6		Man, standing, right arm in front, holding a long stick/staff, left arm hanging beside the body.
134C7		Man, standing, right arm in front, holding a long stick/staff, left arm hanging beside the body, on top of a standard with the support in the center.
134C8		Man, standing, wearing a long sash-kilt, right arm in front, holding a long stick/staff, left arm hanging beside the body, holding a piece of cloth.
134C9		Man, standing, wearing a rammiside long garment, right arm in front, holding a long stick/staff, which widens at the top, left arm hanging beside the body, holding a piece of cloth.
134CA		Man, standing, wearing a long sash-kilt, right arm in front, holding a long stick/staff, left arm hanging beside the body.
134CB		Man, standing, wearing a fillet with uraeus on its head, right arm in front, holding a long stick/staff, left arm hanging beside the body holding an ankh (tie or strap used with sandals) sign.
134CD		Man, standing, with a bend back, right arm forward, holding a staff/stick near the top, with the staff at an backwards angle, left arm hanging beside the body.
134CE		Man, standing on top of the sky (N1), right arm forward, holding a stick/staff, left arm hanging beside the body.
134CF		Man, standing on a base (statue), right arm in front, holding a long stick/staff, which rests on the same base, left arm hanging beside the body.
134D0		Man, standing, both arms in front, holding a stick/staff.
134D1		Man, standing, with a backwards line on the head, right arm in forward, holding a stick/staff, left arm hanging beside the body.
134D2		Man, standing, wearing a long sash-kilt, with a backwards line on its head, right arm in front, holding a long stick/staff, left arm hanging beside the body, holding a flagellum (S45) horizontally.
134D3		Man, standing, wearing a long sash-kilt, with a cone-like shape on its head, right arm in front, holding a long stick/staff, left arm hanging beside the body.
134D4		Man, standing, wearing a long sash-kilt, with a feather (H6) on his head, right arm in front, holding a stick/staff, left arm hanging beside the body.
134D5		Man, standing, wearing a long sash-kilt, with two feathers on his head, right arm in front, holding a stick/staff, left arm hanging beside the body.
134D6		Man, standing, with a palm branch, stripped of leaves and a round notch on front, with the curve at the top backwards (M4), on the head, right arm in forward, holding a stick/staff, left arm hanging beside the body.
134D7		Man, standing, right arm forward, holding a stick/staff, left arm hanging beside the body holding a sceptre (S42), sceptre-head extending in front of the body.
134D8		Man, standing, right arm forward, right arm forward, holding a body-length wAs scepter (sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40)), left arm hanging beside the body, holding a stick.
134D9		Man standing, with a feather on his head (H8), right arm forward, holding a body-length wAs scepter (sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40)), left arm hanging beside the body.
134DA		Man, standing, wearing a lion/leopard skin, tail between the legs, with two backwards curving lines on the head, right arm extended forwards, forearm on waist hight, holding a staff with a triangular shape below the right hand (point downwards), left arm raised towards the front hand at shoulder hight, holding a bookroll, held vertically.
134DB		Man, seated, right knee raised, right arm raised, holding a conical shape at the hight of the shoulder, angled forward, left arm in front of body.
134DC		Man, seated, right knee raised, right arm raised, holding a conical shape at the hight of the shoulder, angled forward, left arm hanging beside the body.
134DD		Man, seated, right knee raised, right arm raised, holding a cup or vessel, about the size of the hand, at the hight of the shoulder, left arm in front of the body.
134DE		Man, seated, right knee raised, right arm raised, holding a basin or wickerwork basket at the hight of the shoulder, left arm hanging beside the body.
134DF		Man, seated, right knee raised, right arm raised, holding a cup or vessel, with the vessel at the mouth, left arm hanging beside the body.
134E0		Man, seated, right knee raised, both arms raised at the front, holding a cup or vessel, with the vessel at the mouth.
134E1		Man, seated on heel, right knee raised, with arms raised in front, hands horizontal, at the hight of the shoulder, handpalms upwards, holding a tray with  a pile of offerings, resembling a half circle.
134E2		The king, seated on heel, right knee raised, with long straight beard and ureaeus, with arms raised in front, hands horizontal at the hight of the shoulder, handpalms upwards, holding a tray with three circular items on top of it.
134E3		The king, seated on heel, right knee raised, with long straight beard and ureaeus, with arms raised in front, hands horizontal at the hight of the shoulder, handpalms upwards, holding a tray with five circular loafs of bread covered by vegetables.
134E4		The king, seated on heel, both knees down, upon a base, with uraeus on brow, long, straight beard and coif, arms extended to the front, carrying a loaf of bread upon a reed mat at the hight of the face.
134E5		Man, seated, right knee raised, a basket with multiple pieces of grain or fruit on his head, right arm raised in front, with the hand at the vessel, left arm hanging beside the body.
134E6		Man, seated, right knee raised, a vessel on his head, right arm raised in front, with the hand at the vessel, left arm in front of the body.
134E7		Man, seated, both knees raised, foot horizontal, a vessel on his head, right arm raised in front, with the hand at the vessel, left arm hanging beside the body.
134E8		Man, seated, both knees down, a vessel on his head, right arm raised in front, with the hand at the vessel, left arm hanging beside the body.
134E9		Man, seated, both knees raised, foot horizontal, a vessel on his head, right arm raised in front, with the hand at the vessel, left arm raised at the back, with the hand at the vessel.
134EA		Man, seated, right knee raised, horned viper on his head, right arm raised in front, holding the viper at the neck, left arm hanging beside the body.
134EB		Man, seated, both knees raised, foot flat on the ground, arms raised at either side of the body, supporting a horned desert viper (Cerastes cerastes) (I9).
134EC		Man, seated, right knee raised, a forearm, with the palm of the hand facing upwards (D36) on top of his head, right arm raised in front, hand horizontal, handpalm up, supporting the forearm, left arm hanging beside the body.
134ED		Man, seated, right knee raised, with a circle (sun-disk) on his head, arms raised at either side of the body, supporting the sign for the sky (N1) which has the points at the hands, with the sign on top of the circle.
134EE		Man, seated, right knee raised, arms raised at either side of the body, supporting the sign for the sky (N1).
134EF		Man, seated, both knees down, arms raised at either side of the body, supporting the sign for the sky (N1).
134F0		Man, seated, right knee raised, right arm raised in front, left arm behind the back, holding a stick.
134F1		Man, seated, right knee raised, right arm raised in front, holding a loop of cord with the ties at the top, at the hight of the shoulder, left arm hanging beside the body.
134F2		Man, seated, rigth knee raised, right arm raised in front, holding a round vessel with an upstanding rim (nw-vase, W24), at the hight of the shoulder, left arm hanging beside the body.
134F3		Man, seated, both knees down, foot flat, both arms forwards, hands horizontal, palm upwards, supporting an African sacred ibis (Threskiornis aethiopicus (G26A)).
134F4		Man, seated, right knee raised, right arm raised in front, right hand held horizontally, supporting a duck or goose (G38-G39), left arm hanging beside the body.
134F6		Man, seated, right knee raised, right arm in front, right hand horrizontally, supporting a heart (F34), left arm in front of the body.
134F7		Man, seated, right knee raised, right arm in front, right hand horizontally, supporting a face (D2), left arm hanging beside the body.
134F8		Man, seated, right knee raised, right arm in front, holding a sheaf of grain in the right hand, left arm hanging beside the body.
134F9		Man, seated, right knee raised, right arm raised in front, holding a papyrus scroll at a 45° degree towards the face, left arm in front of the body.
134FA		Man, seated, right knee raised, right arm raised in front, holding a sedge  (M23) at the top, left arm raised to the back, holding a sedge (M23) at the top.
134FB		Man, seated, right knee raised, both arms raised at the front, holding an oval cartouche on his head.
134FC		Man, seated on heel, both knees down, both arms extended in front, supporting a child (A17: knees at 90°, both lower legs visible, slightly apart, right arm raised with hand to mouth, left arm hanging beside the body).
134FD		Man, standing, a vessel (W10) on his head, right arm raised in front, with the hand at the vessel, left arm hanging beside the body.
134FE		Man, standing, a horizontal oval on his head, right arm raised in front, with the hand at the oval, left arm hanging beside the body.
134FF		Man, standing, a tall oval vessel with upstanding rim on his head, right arm raised in front, with the hand at the vessel, left arm hanging beside the body.
13500		Man, standing, a circle on his head, right arm raised in front, with the hand at the circle, left arm hanging beside the body.
13501		Man, standing, arms raised at either side of the body, supporting a vessel with a small base and a broad rim (W77), with three vertical oval loaves of bread on top of it.
13502		Man, standing, arms raised at either side of the body, supporting a vessel with a small base and a broad rim (W77), with three vertical oval loaves of bread on top of it, interspaced by two round loaves.
13504		Man, standing, arms raised at either side of the body, supporting the sign for the sky (N1).
13505		Man, standing, with a shrine, seen from the side (O18) on his head, both arms raised in front, supporting the shrine.
13507		Man, standing, with a child (A17: knees at 90°, both lower legs visible, slightly apart, right arm raised with hand to mouth, left arm hanging beside the body) on his head, arms raised at either side of the body, supporting the child.
13508		Man, standing, with a child (A17: knees at 90°, both lower legs visible, slightly apart, right arm raised with hand to mouth, left arm hanging beside the body) on his head, right arm raised, supporting the child, left arm hanging beside the body.
13509		Man, standing, both arms towards the front, holding an key for a tumbler lock consisting of a slightly bend vertical line with two shorter horizontal lines attached to the top akin to a flag, pointing outwards. , as if to strike with it.
1350A		Man, running, right arm raised in front, holding a tall water pot (W14) vertically, left arm over the body, holding a tall water pot (W14) which balances against the left shoulder.
1350B		Man, running, right arm raised in front, hand horizontal, handpalm upwards, holding a round vessel with an upstanding rim (W24), left arm over the body, holding a lotus flower at the stem, which rests against the shoulder, with the flower above the shoulder.
1350C		Man, running, right arm raised in front, hand horizontal, handpalm upwards, holding a round vessel with an upstanding rim (W24), left arm over the body.
1350D		Man, standing, both arms towards the front, holding a branch, as if to strike with it.
1350E		Man, standing, back bend forward, right arm forward, holding a branch, left arm hanging beside the body.
1350F		Man, standing, bend forwards, both arms forwards, holding a cup or vessel at the hight of the knees.
13510		Man, seated, right knee raised, with a vase on its side, with liquid issuing from it (W54), orientated to the front, on his head, both arms raised towards the front, handpalms inwards.
13511		Man, seated on heel, right leg extended with the foot in front of the left knee, with a vase on its side, with liquid issuing from it (W54), orientated to the front, on his head, both arms raised towards the front, handpalms outwards.
13512		Man, seated, both knees down, with a vase on its side, with liquid issuing from it (W54), orientated to the front, on his head, both arms raised towards the front, handpalms upwards.
13513		Man, seated, right knee raised, with a vase on its side, with liquid issuing from it (W54), orientated to the front, on his head, right arm extended forwards, hand in the line of water, handpalm forwards, left arm hanging besides the body.
13514		Man, seated, right knee raised, with a vase on its side, with liquid issuing from it (W54), orientated to the front, on his head, right arm extended forwards, hand in the line of water, handpalm forwards, left arm in front of the body.
13516		Man, standing, knees bend, bend forwards with back nearly horizontal, arms hanging downwards, extended towards the front, touching a block, with a vase with liquid issuing from it floating in front of its head, with the water ending on the block.
13517		Man, seated, right knee raised, both arms forward, holding a round vessel on its side, with liquid issuing from it, orientated to the front, at the hight of his head.
13518		Man, seated, right knee raised, both arms forward, holding a tall water pot (W14) on its side, with liquid issuing from it, orientated to the front, at the hight of his head.
13519		Man, seated, right knee raised, with a tall water pot (W14) on its side, with liquid issuing from it, orientated to the back, on his head, both arms forward, holding the base of the water pot.
1351A		Man, seated on heel, both knees down, arms raised in front, holding a round vessel on its side, with liquid issuing from it, orientated to the back, going over the head, at the hight of the head.
1351C		Man, seated, right knee raised, with a vessel on its head with an opening at the left and right side, with liquid issuing forth from either side, arms at either side of the body, hands touching the head.
1351D		Man, seated on heel, right leg extended with the foot in front of the left knee, both arms extended forwards, one arm at the hight of the shoulder, one at the height of the of the chest, handpalms horizontal, handpalms upwards, inside a double dotted line running from two vessels with a small base and a broad rim (W77), in front and behind the man, with the double line of dots curving over the man.
1351E		Man, seated, right knee raised, both arms in front, forearms horizontal, hands upward, with a rectangular vessel below the arms, and two lines of liquid running from the hands over the head to the back, ending around the waist.
1351F		Man, standing, both arms forward, holding a tall water pot with a spout (W14A), as if to pour.
13520		Man, standing, both arms forward, holding a tall water pot with a spout (W14A), with a forwards, downwards ripple of water coming from the spout.
13522		Man, standing, right arm forward, holding a tall water pot with a spout at a 45° angle, with water issuing from the spout, left arm hanging beside the body.
13523		Man, standing, both arms forward, holding a vessel at the hight of the waist, with a dotted line coming from the vessel.
13524		Man, standing, both arms forward, right forearm above left forearm, with a riple of water (N35) written nearly horizontally above the right forearm and hand.
13525		Man, standing, right arm forward, forearm horizontal, handpalm upwards, left arm backwards, upper arm horizontal, forearm nearly vertical, handpalm inwards, with a curved line running between the handpalms, continuing downwards to the level of the feet.
13526		Man, standing, right arm forward, forearm horizontal, handpalm upwards, left arm backwards, upper arm horizontal, forearm nearly vertical, handpalm inwards, with a dotted line running between the handpalms, continuing downwards to the level of the feet.
13527		Man, seated, both knees down, back straight, both arms raised in front, handpalms outwards, under a corner of a wall.
13528		Man, seated, both knees raised, feet flat, near its bottom both arms raised in front, hands horizontal, handpalms upwards, under a corner of a wall.
13529		Man, standing, both arms raised in front, handpalms outwards, under a corner of a wall.
1352A		Man, seated, right knee raised, back straight, both arms raised in front, handpalms outward, behind and below a clump of three papyrus flowers, which curve over the man.
1352B		Man, seated on heel, right knee raised, back slightly bent, both arms forward, holding a reed pipe ending in a tuyère at his mouth, behind a vessel from which a flame, which angles away from the man, comes.
1352C		Man, seated on heel, right knee raised, right hand forward, holding the top of a horizontal drill of a bow-drill, with the point of the drill in a N25-like base, left arm in front of the body, holding the bow of the bow-drill.
1352E		Man, kneeling, right leg extended, left leg bend, foot at an angle, toes down, both arms forwards, extending downwards towards a grind-stone.
1352F		Man, standing, back bend forwards, both arms downwards, holding a rectangular grain measure vessel.
13530		Man, standing, back bend forward, right arm forward, holding a bundle of grain stalks, left arm in front of the body, holding a sickle with the point at the base of the bundle of grain stalks.
13531		Man, standing, back bend forward, both arms toward the front, holding a hammer with a round head, above a spike or pointy pole.
13533		Man, standing, both arms forwards, holding a long stick vertically, levering a vertical, rectangular stone.
13534		Man, standing, both arms forwards, holding a long stick vertically, levering a vertical, rectangular stone, with the stick in front of the stone.
13535		Man, standing, both arms raised in front, touching a vertically written wall with battlements near the top, left leg raised, touching the wall near the bottom.
13536		Man, standing, right arm raised in front, touching a wall, written vertically, with multiple battlements on the long sides (O36C), left leg raised, touching the wall near the bottom, left arm hanging beside the body.
13537		Man, standing, both arms raised in front, touching a wall, written vertically, with a wide base and battlements on the front, back and top (O36D).
13538		Man, standing, both arms raised in front, touching a wall, written vertically, with multiple battlements on the long sides (O36C).
13539		Man, standing, both arms raised in front, touching a wall, written vertically, with battlements on the front, back and top, left leg raised, touching the wall near the bottom.
1353A		Man, seated on a chair, at a potters wheel with a vessel on top of it, both arms forward, near the top of the wheel, right leg extended, right foot at the base of the potters wheel.
1353B		Man, standing at a potters wheel with a lump of clay on top of it, right leg extended, right foot at the base of the potters wheel, both arms extended in front, right hand above the lumb of clay, left hand at the top of the potters wheel.
1353C		Man, kneeling, right foot with the toes on the ground, left foot flat on the ground, back bend forwards, arms extended forward, hands at the rim of a vessel on a low hand operated potters wheel.
1353D		Man, standing, back bend forwards, arms extended forward, hands at the rim of a vessel on a low hand operated potters wheel.
1353E		Man, standing, arms at either side of the body, extended outward, with the hands at the hight of the waist, with a line running between the hands.
1353F		Man, standing in a beer-jug (W22/W125), arms at either side of the body, holding the rim of the beer-jug.
13540		Man, seated, both kees down, back bend forwards, both arms extended forwards, hands at a brick or block which is at the level of the knees.
13542		Man, standing, back bend forward, both arms extended towards the front, forearms horizontal at the hight of the waist, holding a bib or garment (V60).
13543		Man, standing, bend slightly forward, extending the arms forward, forearms horizontal at the level of the waist, holding a piece of clay or mud.
13545		Man, standing, back bend forward, both arms forward, hands inside a W10 type vessel, which is at the level of the feet.
13546		Man, seated on heel, right knee raised, right arm in front of body, holding two sticks at the base forming a V shape, left arm in front of the body.
13547		A man, seated, right knee raised, both arms forwards, holding a bookroll with the first page unrolled.
13548		Man, seated, right knee raised, right arm forward, holding a scribe's kit, consisting of a palette, ink or paint pouch and a reed pen (Y3), at the reed pen, left arm in front of the body.
13549		Man, seated on heel, right knee raised, right arm raised in front, holding a stick at a backwards angle, left arm in front of the body.
1354B		Man, standing, both arms forward, forearms at the hight of the waist, holding a stick, striking a wick of twisted flax.
1354C		Man, standing on top of a branch, back bend forwards, both arms forward, forearms at the hight of the waist, hands holding the branch which bend upwards.
1354D		Man, standing, back bent forwards, both arms forward, towards a table or block with an oryx on its back on it, head towards the front, towards the belly of the oryx, legs bundled together, the right hand of the man extended towards the horns of the oryx, the left hand holding a knife near the forelegs of the oryx.
1354E		Man, seated, both knees up, with covered legs and arms, with short straight beard and short hair/wig, holding a crook (HqA, S38), with the opening outwards.
1354F		Man (shepherd), seated, both knees up, with covered legs and arms, with a short straight beard and short hair/wig, holding a stick which curves outwards with a band of string or fabric, with the ties towards the body.
13550		Man (shepherd), seated, both knees up, with covered arms and legs, with a short straight beard and long head-cloth, holding a straight stick, with a loop near the bottom of the stick, loop outwards, with strokes on top of the loop.
13551		Man (foreigner), seated, both knees raised, with covered legs and arms, wearing foreign headgear, with a bushy beard, holding an axe with a round blade and a curved handle (T7).
13552		Man, seated, both knees raised, with covered legs and arms, with a short straight beard and short hair/wig, holding two sticks in a V shape.
13553		Man, seated, both knees raised, with covered legs and arms, with short hair/wig, holding two sticks in a V shape.
13554		Man (foreigner), seated, both knees raised, with covered legs and arms, with short hair/wig, with a band of cloth around the head, with a vertical feather on the head, holding two sticks with bulbs at the ends.
13555		Man, seated, both knees raised, with covered arms and legs, with a short straight beard and long wig, holding a coil of rope (V1), with the coil towards the body.
13556		Man, seated, both knees raised, with covered legs and arms, with short hair/wig, holding an object consisting of one long vertical line a one end, with multiple shorter vertical lines connected to a horizontal line, with the shorter vertical line section away from the body.
13557		Man, seated, both knees raised, with covered legs and arms, with short hair/wig, holding an object consisting of one long vertical line a one end, with multiple shorter vertical lines connected to a horizontal line, with the shorter vertical line section towards the body.
13558		Man, seated, both knees raised, with covered legs and arms, with a short, straight beard and short hair/wig, holding a key that resembles a plan of a winding wall (O5).
13559		Man, seated, both knees raised, with covered legs and arms, with short hair/wig, holding an key for a tumbler lock consisting of a slightly bend vertical line with two shorter horizontal lines attached to the top akin to a flag, pointing outwards.
1355A		Man, seated, both knees raised, with covered legs and arms, with a short straight beard, short hair/wig, holding an key for a tumbler lock consisting of a slightly vertical line with two shorter horizontal lines attached to the top akin to a flag, pointing outwards.
1355B		Man, seated, both knees raised, with covered legs and arms, with a short straight beard and short hair/wig, holding an object consisting of one long vertical line a one end, with multiple shorter vertical lines connected to a horizontal line, with the shorter vertical line section towards the body.
1355C		Man, seated, both knees up, with covered legs and arms, with short straight beard and short hair/wig, holding an unknown object (mace/knife/stick?)
1355D		Man, seated, both knees up, with covered legs and arms, with a short bushy beard and short hair/wig, holding a stick which angles slightly towards the body.
1355E		Man, seated, both knees up, with covered legs and arms, with a short beard and short hair/wig, with a yoke around his neck.
13560		Man (foreign), seated, both knees up, with covered legs and arms, with a bushy beard, with a feather (H6) on his head angling backwards,  holding a feather (H6), angling forwards.
13561		Man (foreign), seated, both knees up, with covered legs and arms, with a bushy beard,  holding a feather (H6), angling forwards.
13562		Man (foreign), seated, both knees up, with covered legs and arms, with a bushy beard, with short hair/wig, with a band of cloth around the head.
13563		Man, seated, with both knees raised, with covered legs, both arms forward, holding a stick with a bundle or a mat horizontally over his shoulders, bundle behind the back.
13564		Man, standing, right arm forward, holding a stick with a bundle or mat horizontally over his shoulders, bundle behind the back, left arm hanging beside the body.
13565		Man, standing, arms at either side of the body, holding either side of a stick that lies nearly horizontally over his shoulders.
13566		Man, standing, both arms forward, holding a stick with a bundle or a mat horizontally over his shoulders, bundle behind the back, with the ties at the top.
13567		Man, seated, right knee raised, right arm extended forward, holding a bag with one tie, oriented away from the body, left arm in front of the body.
13568		Man, standing, right arm forward, holding a stick with a bundle or mat horizontally over his shoulders, bundle behind the back, with the ties downwards, left arm forwards at 45° downwards, holding a stick which curves slightly inwards.
13569		Man, standing, right arm forward, holding a stick horizontally on the shoulders, with a long bundle or sack at the front, and a vessel hanging by a loop of string at the back, left arm hanging beside the body.
1356B		Man, standing, right arm forward, forearm horizontal, holding a sedge plant (M23), which curves towards the front, left arm hanging beside the body.
1356C		Man, standing, right arm forward, forearm horizontal, holding a sedge plant (M23), which curves towards the front, with a half circle as the base of the sedge, left arm hanging beside the body.
1356E		Man, standing, both arms forward, left arm at the hight of the shoulder, right hand at the hight of the waist, holding a wick of twisted flax, consisting of a singular loop at the top (V28A).
1356F		Man, standing, both arms forward, right hand on top of a two-pronged mast of a ship with multiple horizontal lines between the prongs (P6), left hand below the two prongs.
13570		Man, standing, right arm forward, hand at the hight of the waist, left arm back, hand a the hight of the waist, holding a rope in both hands which is connected to a djed pillar (R11) written at 45° angle forward.
13571		Man, standing, both arms forward, hand at the hight of the waist, hands touching a Djed pillar (R11) written horizontally.
13572		Man, standing, facing backwards, both arms raised in front of the man, below the top of a column resembling a stem of papyrus with a bud (M13), without an abacus, angled forwards.
13573		Man, standing, right arm forward, hand at the hight of the waist, holding the sign for longs and windpipe (F36) of the same size as the man, left arm hanging beside the body.
13574		Man, standing, both arms forwards, holding a windpipe and lungs (F36) of the same length of the man.
13575		Man, standing, head facing towards the back, right arm towards the front, hand held horizontally at the hight of the shoulder, holding a triangular object, left arm towards the front, left hand nearly vertically, between the triangle and the head.
13576		Man, standing, wearing a lion/leopard skin, tail between the legs, right arm hanging beside the body, holding a scroll horizontally, left arm raised towards the front, left hand held horizontally at shoulder hight, handpalm upwards.
13577		Man, standing, wearing a lion/leopard-skin, right hand holding up the tail of the skin angling at the front, left hand raised in front.
13578		Man, standing, both arms forward, holding a foreleg of an ox (F23), near the hoof, with the wide part near the top, as if to strike.
13579		Man, standing, right arm extended forwards, arm horizontal, holding the sky (N1) on his hand, left arm hanging beside the body.
1357A		Man, standing, right arm forward, right hand held horizontally at the hight of the waist, holding an eye with the markings of the head of a falcon (D10), left arm hanging beside the body.
1357C		Man, standing, right arm forward, right hand held horizontally at the hight of the waist, holding a loop of cord with the ties upward (V6), left arm hanging beside the body.
1357E		Man, standing, arms extended at either side of the body, hands at the hight of the shoulders, holding a tenoned door (O31), written vertically, at the top tenon, door-leaf inwards, with the sky (N1) on top of the tenons.
1357F		Man, standing, both arms forward, right forearm above the left forearm, holding a door written horizontally, with the long side toward the body.
13580		Man, standing, with a line coming from his back at the waist, going down to the feet, right arm in front, right hand held horizontally at the hight of the shoulders, holding D28 (two arms, raised upwards, with the forearms and hands vertical, handpalms inwards), left arm forwards, at a 45° degree downwards.
13582		Man, standing, right arm towards the front, hand held horizontally at the hight of the shoulder, holding two arms, raised upwards, with the forearms and hands vertical, handpalms inwards (D28), left arm towards the front, left hand nearly vertically, between the two arms and the head.
13583		Man, standing, right arm extended forwards, arm horizontal,  holding a rectangular sail, on a mast of a ship with two prongs, connected by vertical lines (P6), written behind the sail (P5), left arm hanging beside the body.
13584		Man, standing, bend slightly forwards, right arm towards the front, holding a round vessel with an upstanding rim (W24) at the hight of the shoulder, left arm hanging beside the body.
13585		Man, standing, right arm towards the front, holding a round vessel with an upstanding rim (W24) at the hight of the shoulder, left arm over the body.
13586		Man, standing, arms extended at either side of the body, holding a stem of papyrus with a flowering bud (M127) in the right hand, and a stem of papyrus with a bud (M13) in the left hand.
13587		Man, standing, with a bulls tail, arms extended on either side of the body, arms horizontal at the hight of the shoulder, holding wings that angle downwards from the hands at a right angle.
13588		Man, standing, back slightly bend forward, both arms forward, holding a herb, with three flowers or leaves, curving upwards, with the middle flower/leaf higher than the other two (M2).
13589		Man, standing, back slightly bend forward, both arms forward, holding a flower by the stem.
1358A		Man, walking, with a line (hairlock?) coming from the head, right arm forward, forearm horizontal, holding a leaf, stem (bend forwards) and root of a lotus plant (M12B), left arm hanging beside the body.
1358B		Man, standing, right arm forward, holding a leaf, stem (bend forwards) and root of a lotus plant (M12B), left arm hanging beside the body.
1358C		Man, standing, right arm forward, holding a leaf, stem (bend forwards) and root of a lotus plant (M12B), left arm hanging beside the body, holding a knife (T30A) horizontally.
1358D		Man, standing, right arm forward, holding the sun with rays of sunlight coming from it (N8) at the sun disk, at the hight of the shoulder, left arm is raised, upper arm horizontal, forearm vertical, left handpalm inwards.
1358E		Man, standing, right arm forwards, holding a bow, with the bowstring towards the front, left arm hanging beside the body holding an arrow without fletching, with the arrowhead in front of the body.
1358F		Man, standing, both arms towards the front, holding a knife (T30), as if to strike with it.
13590		Man, standing, right arm forward, forearm horizontal at the hight of the waist, holding a knife (T30A) at an forward angle, left arm hanging beside the body, holding a knife (T30A) horizontally.
13591		Man, standing, arms extended at either side of the body, forearms horizontal at the height of the waist, holding a knife in both hands, angling away from the body.
13592		Man, standing, right arm forward, hand at the level of the wist, holding a knife vertically, left arm raised, forearm vertically, holding a knife nearly horizontally.
13593		Man, standing, holding an harpoon with the point down, diagonally over the body, from the tip of the right foot beyond the left shoulder, right arm lowered, right hand holding the harpoon between the waist and knee, left arm raised, holding the shaft near the end.
13594		Man, standing, right arm forward, hand at the level of the wist, holding a stick vertically, left arm raised, forearm vertically, holding a stick nearly horizontally.
13595		Man, standing, right arm forward, hand at the level of the wist, holding a curved rattle stick vertically, curving outwards, left arm raised, forearm vertically, holding a curved rattle stick nearly horizontally, curving upwards.
13596		Man, standing, right arm forward, holding a stick horizontally, left arm hanging beside the body.
13597		To men, standing, facing each other, arms forward, holding sticks which cross each other.
13598		Man, standing, looking backwards, right arm forward, forearm and hand at the level of the waist, handpalm upwards, holding a star (N5), left arm raised, forearm vertically, holding a stick nearly horizontally.
13599		Man, standing, both arms forwards, holding a scimitar (khopesh), written vertically, blade curving forwards, with the head of a falcon with a sun-disk (N5) on top of it (T73B).
1359C		Man, seated, right knee raised, right arm forward, holding a mace (T3), angling backwards, left arm over the body, holding a mace (T3), angling backwards against the left shoulder.
1359D		Man (foreign), seated, right knee raised, wearing a foreign headdress, right arm raised, left arm in front of the body, holding a spear against the shoulder, point upwards.
1359E		Man, standing, upper body in profile, arms crossed in front of the chest, with a covered upper body resembling a circle.
1359F		Man, standing, arms in front of the body, forearms at waist hight, covered by cloth.
135A1		Man (foreign), standing, with a long dress, right arm forwards, holding a small, peanut shaped shield, seen from the back, with the top near the hight of the shoulder, left arm in front of the body, holding a spear which rests against the shoulder, speartip at the top, at the hight of the head, spearshaft continuing up to the hight of the shin, crossing over the body.
135A2		Man (foreign), standing, with a foreign headdress, right arm forward, holding a shield, seen from the back, left arm hanging beside the body, holding a sword, with the tip just under the shield.
135A4		Man (king) running, wearing the red crown (S3) right arm in front, hand at the hight of the waist, holding a vertical spear with the point at the top, left arm raised, forearm nearly vertical, holding a stick which runs from the head to slightly beyond the hand.
135A5		Man, seated on heel, right knee raised, with a feather on the head (angled backwards). right arm raised, left arm in front of body.
135A6		Man, seated on heel, right knee up, with two feathers (H6) on his head, arranged in a V shape, straight side inwards, right arm forwards, forearm horizontal, holding a composite bow (T10), left arm in front of the body, holding a stick which angles backwards, leaning against the left shoulder.
135A7		Man, seated on heel, right knee up, with a palm branch, stripped of leaves and notched with a round notch (M4) on its head, right arm forwards, holding a composite bow (T10), left arm in front of the body, holding three arrows, leaning against the shoulder.
135A8		Man, seated on heel, right knee raised, right foot extending beyond the left knee, with two feathers (H6) on his head, arranged in a V shape, straight side inwards, right arm forwards, forearm horizontal, holding a bow, left arm in front of the body, holding 3 arrows, arrowheads upwards, which angle backwards, leaning against the left shoulder.
135A9		Man, seated on heel, right knee raised, right foot extending beyond the left knee, with two feathers (H6) on his head, arranged in a V shape, straight side inwards, right arm raised in fromnt, holding a vertical arrow with the fletching upwards and arrowhead downwards, left arm in front of the body, holding a bow which angles backwards, leaning against the shoulder.
135AA		Man, seated, right knee raised, right arm in front, forearm horizontal, holding a shield, left arm in front of the body, holding a mace (T3) which angles backwards, leaning against the shoulder, macehead above the shoulder.
135AB		Man, seated on heel, right knee raised, right foot in front of the left knee, right arm forward, forearm horizontal, holding a bow without bowstring, left arm in front of the body, holding an arrow at the fledging, which goes horizontally beyond the bow (as if the arrow is drawn).
135AC		Man, seated on heel, right knee raised, right foot in front of the left knee, right arm forward, forearm horizontal, holding a bow, left arm in front of the body, elbow raised above the shoulder, holding an arrow at the fledging, which goes horizontally beyond the bow, with the bowstring behind the fletching.
135AD		Man, seated, both knees down, right arm forward, forearm horizontal, holding a bow or shield, left arm in front of the body, holding multiple sticks or arrows, angling backwards. Leaning against the left shoulder.
135AE		Man, seated on heel, right knee raised, right foot in front of the left knee, right arm forward, forearm horizontal, holding a bundle of arrows in the right hand at the bottom of the shaft, with the arrowheads up, left arm in front of the body, holding a bow, angling backwards, leaning against the left shoulder, bowstring towards the front.
135AF		Man (enemy), seated on heel, both knees down, with a line of liquid coming from the top of the head, towards the front, curving downward, both arms behind the back, bound together with rope.
135B0		Man, seated on heel, both knees down, with a line of liquid coming from the face/neck, towards the front, curving downward, both arms behind the back, bound together with rope.
135B1		Man, seated on heel, both knees down, right arm forward, hanging beside the body, left arm hanging beside the body, with a knife (T30A) written at a 45° degrees over the elbow of the right arm, point at the groin, blade downwards.
135B2		Man, seated on heel, both knees down, right arm raised in front, hand at the hight of the face, left arm hanging beside the body, with a rope coming from the neck at the back, curving downwards.
135B3		Man, seated on heel, arms behind the back, bound with rope, with a knife (T30A) with the point against the chest, handle at the hight of the face, blade downwards.
135B4		Man, seated on heel, right knee raised, right foot in front of the left knee, with a line of liquid coming from the neck/chest, towards the front, curving downward, both arms behind the back, bound together with rope.
135B5		Man (foreigner), seated on heel, right knee raised, with a foreign headdress and bushy beard, arms behind the back, bound with rope, with a rope coming from the neck, at the front, curving upward, with a triangle at the end of the rope, at about the hight of the top of the head.
135B6		Man (foreigner), seated on heel, right knee raised, with foreign headdress and busyh beard, both arms behind the back, bound with rope.
135B7		Man (enemy), with hands bound to feet at the back.
135B8		Man (enemy), without head, with hands bound to feet at the back.
135B9		Man (enemy), without head, with hands bound to feet at the back, with a knife with a triangular blade and straight handle (T30A) in its breast (blade downwards).
135BA		Man (enemy), seated on heel, both knees down, without head, both arms behind the back, bound together with rope.
135BB		Man, seated on heel, right knee raised, right foot in front of the left knee, without head, both arms behind the back, bound with rope.
135BC		Man, with hands bound to feet at the back, in front of a pole which is forked at the top, to which the man is bound.
135BE		Man, seated on heel, both knees down, both arms behind the back, bound with rope, behind a pole which is forked at the top, to which the man is bound with a line coming from the neck.
135BF		Man, without head, with hands bound to feet at the back, in front of a pole which is forked at the top, to which the man is bound.
135C1		Man, seated on heel, both knees down, with a line of liquid coming from the neck/chest, towards the front, curving downward, both arms behind the back, bound together with rope, in front of a pole which is forked at the top, to which the man is bound.
135C2		Man, seated on heel, right knee raised, both arms behind the back, bound together with rope, in front of a pole which is forked at the top, to which the man is bound.
135C3		Man, seated on heel, right knee raised, with a line of liquid coming from the neck/chest, towards the front, curving downward, both arms behind the back, bound together with rope, in front of a pole which is forked at the top, to which the man is bound.
135C4		Man, seated on heel, right knee raised, right foot in front of the left knee, a short line of rope or liquid coming from the neck, curving downwards, both arms behind the back, bound with rope, in front of a pole which is forked at the top, to which the man is bound.
135C5		Man, seated on heel, right knee raised, right foot in front of the left knee, both arms behind the back, bound with rope, in front of a pole which has a triangular top (or papyrus reed bud), to which the man is bound.
135C6		Man, seated on heel, right knee raised, right foot in front of the left knee, a short line of rope or liquid coming from the neck, curving downwards, both arms behind the back, bound with rope, in front of a pole which has a triangular top (or papyrus reed bud), to which the man is bound.
135C7		Man, seated on heel, right knee raised, right foot in front of the left knee, right arm raised in front, left arm hanging beside the body, holding the left ankle, in front of a pole which is forked at the top, to which the man is bound, with rope connecting the left arm and the pole.
135C8		Man, seated on heel, both knees down, both arms behind the back, bound together with rope, in front of a pole which is forked at the top, to which the man is bound.
135C9		Man, seated on heel, both knees down, with a knife at 45°, with the tip on the thighs, blade down, both arms behind the back, bound together with rope, in front of a pole which is forked at the top, to which the man is bound.
135CA		A pole which is forked at the top, with a man, bound hand to feet at either side of it, facing outwards, bound to the pole.
135CB		Man, standing, both arms behind the back, bound together with rope, in front of a pole which is forked at the top, to which the man is bound.
135CC		Man, curled up, right and left leg separated, with a band of cloth around the middle, both arms forward, holding an axe with the axeblade close/in the forehead.
135CD		Man, seated on heel, right knee raised, right foot in front of the left knee, back bend forwards, both arms forward, holding an axe with the axeblade in the head of the man.
135CF		Man, seated on heel, both knees down, both arms forward, holding an axe with the axeblade in the head of the man.
135D0		Man, curled up, right and left lower legs separated, back nearly horizontal, both arms forward, holding and axe with the axeblade in the head of the man.
135D3		Man, curled up, right and left lower legs separated, back nearly horizontal, with a band of cloth around the middle, both arms forward, holding and axe with the axeblade in the head of the man.
135D4		Man (foreigner), standing, wearing a foreign headcloth and bushy beard, arms behind the back, bound with rope.
135D5		Man, standing, arms behind the back, bound with rope.
135D7		Two men, standing, back to back, arms behind the back, bound to each other by the arms with rope.
135D8		Man, standing, looking backwards,, both arms in front, upper arms nearly horizontal, bound with rope.
135D9		Man, running, with short wig/coif and uraeus, in front of a smaller naked man, running, both hands behind the back, bound with rope, larger man right arm forward, hand near the bound arms of the smaller man, left arm in font of the body, holding rope which connects to the rope binding the arms of the smaller man.
135DA		Man (foreigner) seated on heel, both knees down, with foreign headdress, with a rope around the neck, with the ends in the front, making one curl with 1 rope end towards the top, with triangular end, 2 rope ends downward.
135DC		3 men, kneeling/walink on knees, gound together around the neck. First man, right knee before left knee, both arms raised in front, handpalms outward, second man, right knee just in front of left knee, both arms behind the back, as if bound, third man, right knee and foot in front of the left knee, left lower leg raised at 45°, right arm in front, hand against the head of the second man, left arm behind the back, hand visible in front of the waist.
135DD		3 men, seated on heel, both knees down, arms behind the back, bound with rope, connected by the neck by a horizontal line (rope), which makes an upward tick at the front.
135DE		Man, naked, dancing, right leg vertical, left leg raised, knee in front of the right leg, left foot behind the right leg, right arm raised in front, extended forward, hand at the hight of the head, left arm raised at the back, hand at the hight of the head.
135DF		Man, naked, dancing, right leg vertical, left leg raised, knee in front of the right leg, left foot behind the right leg, right arm towards the front, angling downwards towards the left knee, hand at the hight of the left knee, left arm raised at the back, hand at the hight of the head.
135E0		Man, dancing, right leg straight, left knee in front of the right leg, left foot behind the right leg, lower leg horizontal, right arm in front of the body, left arm raised at the back, hand at the hight of the head.
135E1		Man, naked, dancing, right leg vertical, left leg forwards, thigh horizontal, lower leg bend downwards, right arm raised in front, left arm hanging beside the body.
135E2		Man, naked, dancing, right leg vertical, left leg forwards, thigh horizontal, lower leg bend downwards, right arm towards the front, angling downwards towards the left knee, hand at the hight of the left knee, left arm raised at the back, hand at the hight of the head.
135E3		Man, dancing, knees bend as if seated on nothing, right arm extended in front, forearm horizontal, left arm raised in back, forearm vertical.
135E4		Man, standing, right arm in front of body, left arm raised in back, forearm vertical.
135E5		Man, naked, dancing, right leg vertical, left leg raised, left knee in front of the right leg, left foot behind the right leg, right arm raised in front, left arm hanging beside the body.
135E6		Man, dancing, knees bend as if seated on nothing, right arm forwards, angling downwards, left arm raised in back, hand at the hight of the neck.
135E7		Man, naked, face forwards, left leg backwards as if walking, right arm forwards, forearm horizontal at the hight of the waist, left arm raised in back, hand at the hight of the head.
135E8		Man, naked, face backwards, left leg backwards as if walking, right arm forwards, forearm horizontal at the hight of the waist, left arm raised in back, hand at the hight of the head.
135E9		Man, dancing, right leg vertical, left knee in front of the right leg, left foot behind the right leg, right arm extended in front, forearm horizontal, handpalm upwards, left arm raised in back, left hand above the head, handpalm inwards, with a dotted line going from handpalm to handpalm.
135EB		Child, naked, with a side-lock, standing, right arm extended in front, held horizontally, holding a sistrum (Y18), left arm hanging beside the body.
135ED		Child, naked, with a side-lock and wearing the double-crown (S5), standing, right arm extended in front, held horizontally, holding a sistrum (Y18), with a line running from the wrist to the ankle, left arm hanging beside the body.
135EE		Child, naked, with a side-lock, standing, right arm extended in front, held horizontally, holding a sistrum (Y18), left arm hanging beside the body, holding a bead-necklace with a counterweight (S18) at the counterweight.
135EF		King, standing, wearing the red crown (S3) and a long straight beard, right arm forward, on a 45° downward angle, left arm towards the front, left hand in front of the body, at the hight of the shoulder, holding a sceptre (S42).
135F0		Child, naked, with a side-lock, wearing the double crown (S5), standing, right arm extended in front, held horizontally, holding a sistrum (Y18), left arm hanging beside the body, holding a bead-necklace with a counterweight (S18) at the counterweight.
135F1		Man, standing, wearing kilt, right arm extended in front, held horizontally, holding a sistrum (Y18), left arm forwards on a 45° degree angle, holding a bead-necklace with counterweight at the counterweight.
135F2		Child, naked, with a side-lock, standing, right arm extended in front, held horizontally, holding a sistrum (Y18), left arm forwards on a 45° degree angle, holding a bead-necklace with counterweight at the counterweight.
135F3		Child, naked, with a side-lock, standing, right arm extended in front, held horizontally, holding a sistrum (Y8), left arm forwards on a 45° degree angle, holding a bead-necklace with counterweight at the counterweight.
135F4		The king, wearing the white crown (S1),  standing, right arm in front, right hand at the hight of the shoulder, holding a sistrum (Y8), left arm hanging beside the body, holding a flagellum and crook (S38) horizontally.
135F5		Child, naked, with a side-lock, standing, right arm extended in front, held horizontally, holding a sistrum (Y8), with a line running from the wrist to the ankle, left arm hanging beside the body.
135F6		Child, naked, with a side-lock and wearing the double-crown (S5), standing, left arm extended in front, held horizontally, holding a sistrum (Y8), right arm forward, below the left, holding a bead-necklace with counterweight at the counterweight.
135F7		Child, naked, with a sidelock and wearing the double crown (S5), standing, right arm extended in front, held horizontally, holding a sistrum (Y8), left arm hanging beside the body.
135F8		Child, naked, with side-lock and wearing the hmhm crown (S61), standing, right arm extended in front, held horizontally, holding a sistrum (Y8), left arm hanging beside the body.
135F9		Child, naked, with side-lock and wearing the hmhm crown (S61), standing, right arm extended in front, held horizontally, holding a sistrum (Y8), left arm hanging beside the body, holding a bead-necklace with a counterweight (S18) at the counterweight.
135FA		Child, naked, with side-lock and wearing the hmhm crown (S61), standing, right arm extended in front, held horizontally, holding a sistrum (Y18), left arm forwards on a 45° degree angle, holding a bead-necklace with counterweight at the counterweight.
135FB		Man, standing, wearing a long sash-kilt, both arms forward, holding a triangular harp (Y16) at the hight of the chest.
135FC		Man, seated, right knee raised, both arms forwards, holding a rounded harp (Y7) at an backwards angle.
135FD		Man, seated, right knee raised, right arm raised, holding a sistrum (Y8) at an angle towards the head, left arm behind the back, holding a stick with extrudes beyond the left shoulder.
135FE		Man, seated, right knee raised, arms at either side of the body, holding a flute, which goes from the mouth to the left foot.
135FF		Man, standing, arms at either side of the body, holding a oval drum between the waist and the knees.
13600		Man, standing, right arm raised, hand horizontal at the hight of the shoulder, handpalm upwards, with a drum/tambourine slightly above the hand, left arm hanging beside the body.
13601		Man, seated, right knee raised, both arms extended forwards, holding a drum/tambourine at the hight of the shoulder.
13602		Man, standing, both arms forwards, right  hand at the hight of the shoulder, holding a tambourine, left arm raised below it, as if to beat the tambourine.
13603		Man, standing, arms at either side of the body, right ahand at the hight of the waist, left land at the hight of the head, playing a lute which crosses over the chest.
13604		Man, standing, arms at either side of the body, holding the necks of two emblematic animals with panther heads, which runs under the feet of the man, overlapping as a two ties.
13605		Man, standing, right arm forward, holding a staff, left arm hanging beside the body, holding a rope/leash connected to a standing hunting hound (E14), behind the man.
13606		Man, seated, right knee raised, right arm in front of the body, holding a throwing stick (T14), which angles towards the front, left arm hanging beside the body, holding a rope/leash connected to a standing hunting hound (E14), behind the man.
13607		Man, seated on a horse, right hand forward, holding the reins, left arm hanging beside the body.
13608		Man, seated on a horse, right arm raised forward, holding a vertical dagger with a rounded tip above the head of the horse, left arm forward, holding the reins.
13609		Man, seated on a combination of two front-parts of a horse, facing forward and backwards, arms extended at either side of the body, holding the reins of both horseheads.
1360A		Man, seated on a rearing horse, body of the horse mostly horizontal, rear legs nearly vertical, right arm forwards, holding the horse by the neck, left arm hanging beside the body, holding a stick over the thigh of the horse.
1360B		Man, seated on a rearing horse, body of the horse mostly horizontally, right arm forward, holding the reins of the horse, left arm raised at the back, holding a stick, angled towards the head.
1360C		Man, standing in a chariot, drawn by a span of 4 rearing horses, both arms forward, one holding a stick, the other holding the reins.
1360D		Man, standing, right arm forward, holding a spear with the point downwards, left arm hanging beside the body, on top of an bovid, lying on the ground, legs folded under the body, tail down.
1360F		Man, standing behind a bovid, lying on the ground, legs folded under the body, tail down, left leg raised, with the foreleg of the man on top of the back of the bovid, right arm forward, holding the forward horn, left arm hanging beside the body, hand touching the back of the bovid.
13610		A span of oxen with a plough, which is held by a man, standing, bent forward, and with another man, standing, bent forward, holding a stick in his right hand, as if to strike, with the left arm hanging beside the body, behind the plough, between the span of oxen and the man holding the plough.
13611		Child, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) upon his head, seated between the horns of a bull (bovid), standing, arms at either side of the child, holding the horns of the bull.
13612		Child, seated between the horns of a bull (bovid), lying on the ground, legs folded under the body, tail down, arms at either side of the child, holding the horns of the bull.
13613		The king, standing on top of a bovid, lying down, legs folded under the body, tail down, wearing the red crown (S3), right arm forward, holding a staff and a piece of rope which loops in front of the staf, connecting to the head of the bovid, left arm in front of the body, holding the flagellum and crook, resting against the left shoulder.
13614		A child, seated on an elephant, left leg visible, right arm raised with hand to mouth, left arm hanging beside the body.
13615		Man, standing behind a hippotamus, right arm forward, holding the tail of the hippotamus, left arm hanging beside the body.
13616		Man, standing on top of a hippopotamus, right arm forward, holding a horizontal staff or spear (if spear, speartip down), left arm hanging beside the body.
13617		Man, standing behind a pig, right arm forward, holding the tail of the pig, left arm hanging beside the body.
13618		Man, standing, arms hanging at either side of the body, behind a lion, standing, tail down.
13619		Man, standing, arms hanging on either side of the body, behind a hunting hound, standing, tail curling upwards (E14).
1361A		Man, standing, wearing a lion/leopard skin, tail between the legs, facing away from the reading direction, right arm hanging beside the body, left arm forward, holding the hand of a hamadryas baboon (Papio hamadryas), standing on the legs, facing the man, tail down.
1361B		Man, standing, both arms forward, holding a fishing rod at 45°, fishing line vertical, connecting to a fish, between the fishing line and the toes of the man, slightly tilted upwards.
1361D		A child, seated on nothing, forelegs separated, with a side-lock written behind the head, right arm raised with hand to mouth, left arm hanging beside the body.
1361E		Child, seated on nothing, forelegs spread, right arm raised in front, hand vertical, handpalm outwards, left arm hanging beside the body.
1361F		Child, seated on nothing, forelegs spread, right arm forward, holding a knife (T30A), angled toward the body, left arm hanging beside the body.
13620		Child, seated on nothing, forelegs spread, right arm forward, holding a sistrum (Y18), angled toward the body, left arm hanging beside the body.
13621		Child, seated on nothing, one leg visible, right arm extended in front, hand horizontal, holding a round vase with a upstanding rim (nw-vessel, W24), left arm hanging beside the body.
13622		Child, seated on nothing, forelegs spread, right arm extended forward, holding a clump with one papyrus stem and bud, with two buds bent down (M15Avar), at the stem, below the bud, left arm hanging beside the body.
13623		Child, seated on top of a horn, forelegs spread, right arm extended forwards, forearm horizontal, holding the tip of the horn, left arm hanging beside the body.
13625		Child, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) upon his head, seated on nothing, forelegs spread, right arm raised in front with hand to mouth, left arm hanging beside the body.
13626		Child, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) upon his head, seated on nothing, one leg visible, right arm raised in front with hand to mouth, left arm hanging beside the body.
13627		Child, with a cobra in repose (Naja haja), standing up (I80) on his head, seated on nothing, forelegs spread, right arm raised in front with hand to mouth, left arm hanging beside the body.
13628		Child, with the head of a cobra (Naja haja), standing up, with expanded hood (Uraeus) on his head, seated on nothing, forelegs spread, right arm raised in front with hand to mouth, left arm hanging beside the body.
13629		Child, wearing the white crown (S1), seated on nothing, forelegs spread, right arm raised in front with hand to mouth, left arm hanging beside the body.
1362A		Child, wearing the red crown (S3), seated on nothing, one leg visible, right arm raised in front with hand to mouth, left arm hanging beside the body.
1362C		Child, seated on nothing, forelegs spread, both arms forward, left arm angling downwards, holding a cone vertically at the hight of the waist, right arm raised, angled up, right hand above the cone, handpalm down.
1362D		Child, seated on nothing, one leg visible, both arms forward, left arm angling downwards, holding a cone vertically at the hight of the waist, right arm raised, angled up, right hand above the cone, handpalm down.
1362E		Child, seated on nothing, forelegs spread, right arm forward, holding a flagellum (S45), left arm hanging beside the body.
1362F		Child, seated on nothing, forelegs spread, right arm extended forward, hand horizontal, palm upwards, holding a feather (H6), left arm hanging beside the body.
13630		Child, seated on nothing, forelegs spread, right arm extended forward, hand horizontal, palm upwards, holding a hill with a halo of sun rays behind it (N28), left arm hanging beside the body.
13632		A god, with a long curved beard, seated on nothing, forelegs spread, rigth arm extended forwards, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40) of the hight of the seated god, left arm hanging beside the body.
13633		Child, seated on a stool, forelegs separated, right hand raised in front with hand to mouth, left arm hanging beside the body.
13634		A child, seated on a mat, wearing a coif with uraeus, covered in cloth, legs bend in front, feet at the same level as the mat, right arm raised in front with hand to mouth, left arm in front, hand upon the knee.
13636		Child, seated on nothing, one leg visible, right arm raised in front with hand to mouth, left arm hanging beside the body.
13637		Child, seated, both knees raised, feet horizontal, heels touching ties, right arm raised in front with hand to moth, left arm hanging beside the body, on top of a lotus flower, facing upwards, with a short stalk, and a bud at either side (M224A).
13638		Child, naked, standing, right arm raised in front with hand to mouth, left arm hanging beside the body.
13639		Child, naked, standing, right arm raised in front with hand to mouth, with a curved line running from the right wrist/elbow the the left foot, left arm hanging beside the body.
1363B		Child seated on nothing, one leg visible, both arms raised in front, hands at the hight of the face, handpalms outwards.
1363C		A child, seated on nothing, forelegs separated, right arm raised with hand to mouth, left arm hanging beside the body, under a curved dotted line.
1363D		A dwarf, naked, standing, arms hanging at either side of the body.
1363E		A man, with short straight beard, seated on a chair, both arms extended forwards, forearms horizontal, one arm above the other, hands above the knee.
1363F		A man, with short straight beard, seated on a chair, right arm in front of the chest, left arm extended forwards, hand above the knee.
13640		A man, with short straight beard, seated on a block throne, both arms extended forwards, forearms horizontal, one arm above the other, hands above the knee.
13641		God, with long curved beard and long wig, seated on a block throne, both arms extended forwards, forearms horizontal, one above another, holding a flagellum (S45).
13642		God, with long curved beard and long wig, seated on a block throne, on top of a base, both arms extended forwards, forearms horizontal, one above another, holding a flagellum (S45), with a spear, with a straight shaft, topped with the head of a falcon, with a triangular spearhead above the head of the falcon, in front of the throne, on the base.
13643		Man, with short straight beard, seated on a block throne, without visible arms, with a flagellum (S45) and crook, opening inwards (S38) on his knee.
13644		Man, with short straight beard, seated on a chair, both arms extended forwards, forearms horizontal, one above the other, holding a flagellum (S45) and crook, opening inwards (S38).
13645		God, with long curved beard and long wig, seated on a block throne, both arms extended forwards, forearms horizontal, one above another, holding a sceptre with a straight shaft, topped with the head of the Seth animal.
13646		God, with a long curved beard and long wig, seated on a block throne, both arms extended forwards, forearms horizontal, one above the other, top hand holding a tie or strap, used with sandals (ankh-sign, S34) in one hand, horizontally, with the bottom hand opened with the palm facing upwards.
13647		God, with a long curved beard and long wig, seated on a block throne, on top of a base, right arm extended forwards, hand at the hight of the waist, extended beyond the knee, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40), and a tie or strap, used with sandals (ankh-sign, S34), at the loop, horizontally, left arm forward, hand above the lap, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop, horizontally.
13648		Man, seated on a chair, right arm extended forwards, holding a staff at an angle, left arm in front of the body, with the hand on the lap.
1364A		God, with long curved beard and uraeus on the forehead, seated on a block throne on a base, no arms visible, with a flagellum (S45) on top of its knee.
1364B		God, with long curved beard and uraeus on the forehead, seated on a block throne, both arms in front of the chest, right hand holding a crook (S38), opening inward, angled over the right shoulder, left hand holding a flagellum (S45), angled over the left shoulder.
1364C		Man, without beard, short hair/wig, seated on a block throne, no arms visible.
1364D		God, with long curved beard,seated on a chair, with a two feather headdress mounted on the horns of a ram on its head, both arms extended forwards, forearms horizontal, one above the other, holding a flagellum (S45) and crook, opening inwards (S38).
1364E		Man, with a short straight beard, seated on nothing, both arms extended forward, forearms horizontal, one above another, holding a flagellum (S45).
13650		Man, with short straight beard, seated on a chair, no arms visible, with a tie or strap, used with sandals (ankh-sign, S34), which angles forward, on the knee.
13651		Child, naked, wearing the hmhm crown (S61), seated upon a block throne on a base, right arm raised in front with hand to mouth, left arm in front, forearm horizontal, holding a tie or strap, used with sandals (ankh-sign, S34), which angles forward.
13652		God, seated, both knees up, with covered legs and arms, with long wig, without a beard.
13653		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing a coif with uraeus.
13654		God, seated, both knees up, with covered legs and arms, with a long wig, without a beard, holding a sceptre with a straight shaft, topped with the head of the Seth animal.
13655		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig, holding a holding a flagellum (S45).
13656		King, seated, both knees up, with covered legs and arms, with a long straight beard and long wig, holding a flagellum (S45).
13657		God/king, seated, both knees up, with covered legs and arms, with a long wig, without a beard, holding a crook (S38) with the opening outwards.
13659		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing a coif with uraeus, holding a crook (S38) with the opening inwards.
1365B		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig, holding a tie or strap, used with sandals (ankh-sign, S34), which angles forward.
1365C		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing a coif with uraeus, holding a tie or strap, used with sandals (ankh-sign, S34), which angles forward.
1365D		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig, holding a knife (T30A), angled slightly forward.
1365E		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig, holding a sceptre (S42) vertically.
1365F		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig, holding a club used by washer-men for beating laundry (U36) vertically.
13660		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing a coif with uraeus, holding a sedge (M23), with the tip curving outwards.
13661		Man, seated, both knees up, with covered legs and arms, with long, curved beard, short hair/wig, holding a dung beetle (scarab, L1).
13662		Man, standing, right arm forward, holding a crook (S38) at the same size as the man vertically, left arm hanging beside the body.
13663		Man, standing, right arm forward, holding a crook (S38) at the same size as the man vertically, left arm hanging beside the body, standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13664		Man, standing, right arm forward, holding a crook (S38) at the same size as the man vertically, left arm hanging beside the body, holding a flagellum (S45) horizontally.
13665		King, standing, with a long wig or headcloth, with uraeus, right arm in front of the body, holding a stick angled against the right shoulder, left arm hanging beside the body.
13666		Man, standing, right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40), left arm hanging beside the body.
13667		God, standing, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) upon his head, with long curved beard and long wig, right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40), left arm hanging beside the body.
13668		The king, standing, with a long wig and uraeus, right arm forward, holding a stick/staff which is angled towards the body, left arm hanging beside the body, holding a piece of cloth.
13669		Man, standing, right arm forward, holding a staff/stick, left arm hanging beside the body, holding a flagellum (S45) horizontally.
1366C		Man, standing, wearing a long sash-kilt, right arm forward, hand below the waist, holding a one handed fan (S37), with the curve outward, left arm hanging beside the body.
1366D		A statue of the king, standing, with a long straight beard, uraeus and coif, with the left arm vertically over the body.
1366E		Man, standing, with uraeus on the forehead, both arms raised in front, hands held vertically, handpalms outwards.
1366F		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown (S1).
13670		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the white crown (S1).
13671		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown with uraeus (S1A), holding a tie or strap, used with sandals (ankh-sign, S34) vertically.
13672		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the white crown (S1), holding a flagellum (S45).
13673		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown with uraeus (S1A), holding a flagellum (S45).
13674		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown (S1), holding a crook (S38) vertically, with the opening inwards.
13675		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the white crown with uraeus (S1A), holding a crook vertically, with the opening inwards.
13676		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown with uraeus (S1A), holding a crook (S38) vertically, with the opening outwards.
13677		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown (S1), holding a crook (S38) vertically, with the opening outwards.
13678		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown with uraeus (S1A), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13679		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the white crown (S1), holding a flagellum (S45) and crook (S38), with the opening inwards.
1367A		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the white crown with uraeus (S1A), holding a flagellum (S45) and crook (S38), with the opening inwards.
1367B		King, seated, both knees up, with covered legs and arms, with a long straigth beard, wearing the white crown with uraeus (S1A), holding a flagellum (S45) and mace (T3).
1367C		King, seated, both knees up, with covered legs and arms, with a long straigth beard, wearing the white crown with uraeus (S1A), holding a sedge (M23), with the tip curving outwards.
1367D		King, seated, both knees up, with covered legs and arms, with a long straigth beard, wearing the white crown with uraeus (S1A), holding a club used by washer-men for beating laundry (U36) vertically.
1367E		God, seated, both knees up, with covered legs and arms, wearing the white crown (S1), holding a column with a base, with a tenon at the top (O28B).
1367F		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the white crown (S1), holding the white crown (S1).
13680		God, seated, both knees up, foot flat on the ground, heels touching the tighs, with a long curved beard, wearing the white crown (S1), both arms forward, holding a flagellum (S45).
13681		God, seated on a block throne, with a long curved beard, wearing the white crown (S1), no arms visible.
13682		King, seated on a block-throne, with a long straight beard, wearing the white crown (S1), no arms visible, with a flagellum on his knee.
13683		God, seated on a block-throne, with a long curved beard, wearing the white crown (S1), no arms visible, with a flagellum (S430 and the crook (S38), opening upwards, on his knee.
13684		King, standing, with a long straight beard, wearing the white crown with uraeus (S1A), right arm forward, holding a crook (S38) of the size of the king vertically, opening outwards, left arm hanging beside the body, holding a flagellum (S45) horizontally.
13685		King, standing, with a long straight beard, wearing the white crown with uraeus (S1A), right arm forward, holding a crook (S38) of the size of the king vertically, opening outwards, and a flagellum (S45) in front of the crook, left arm hanging beside the body, holding a flagellum (S45) horizontally.
13686		King, standing, with a long straight beard, wearing the white crown (S1), right arm forward, holding a stem of papyrus with a flowering bud (M127), of the hight of the king, left arm hanging beside the body, holding a flagellum (S45) horizontally.
13687		King, standing, with a long straight beard, wearing the white crown (S1), right arm forward, holding a stem of papyrus with a flowering bud (M127), of the hight of the king, left arm hanging beside the body.
13688		King, standing, with a long straight beard, wearing the white crown (S1), right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40) of the hight of the king, left arm hanging beside the body, holding a flagellum (S45) horizontally.
13689		God, in mummyform, standing upright on a platform, with a long curved beard, wearing the white crown (S1), both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, with a tie or strap, used with sandals (ankh-sign, S34), on the middle of the headpiece of the staff, angled forwards.
1368A		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the red crown with (S3).
1368B		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the red crown with (S3), holding a flagellum (S45).
1368C		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the red crown with (S3), holding a flagellum (S45) and a crook (S38) with the opening inwards.
1368D		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the red crown with (S3), holding  a crook (S38) vertically, with the opening inwards.
1368E		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the red crown with (S3), holding  a crook (S38) vertically, with the opening outwards.
1368F		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the red crown with (S3), holding  a tie or strap, used with sandals (ankh-sign, S34) vertically.
13690		King, standing, with a long straight beard, wearing the red crown (S3), right arm forward, right hand at the hight of the shoulder, holding a sceptre (S42), left arm forward, left hand at the hight of the waist, holding a stick/staff which runs from the right toe to the left hand, and a mace (T3), angled forward.
13691		King, seated on a block-throne, with a long straight beard, wearing the red crown (S3), no arms visible.
13692		King, seated on a block-throne, with a long straight beard, wearing the red crown (S3), both arms forward, forearms horizontal, one above the other, holding a flagellum (S45).
13693		King, standing, with a long straight beard, wearing the red crown (S3), right arm forward, holding a stem of papyrus with a bud (M131), of the hight of the king, left arm hanging beside the body, holding a flagellum (S45) horizontally.
13694		King, standing, with a long straight beard, wearing the red crown (S3), right arm forward, holding a stem of papyrus with a bud (M131), of the hight of the king, left arm hanging beside the body.
13695		King, standing, with a long straight beard, wearing the red crown (S3), right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40) of the hight of the king, left arm hanging beside the body, holding a flagellum (S45) horizontally.
13696		King, running, with a long straight beard, wearing the red crown (S3), right arm extended forwards, hand at the hight of the shoulder, holding a vertical hourglass-shaped shaped document holder (R88), left arm in front of the body.
13697		King, seated on heel, right knee raised, with a long straight beard, wearing the red crown (S3), both arms forward, holding a loaf of bread on a reed mat (R4) horizontally.
13698		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the double crown (S5).
13699		God, seated, both knees up, with covered legs and arms, with a long, curved beard, wearing the double crown (S5).
1369A		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the double crown (S5), holding a tie or strap, used with sandals (ankh-sign, S34), angled forwards.
1369B		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the double crown (S5), holding a flagellum (S45) and a crook (S38) with the opening inwards.
1369C		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the double crown (S5), holding a club used by washer-men for beating laundry (U36) vertically.
1369D		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the double crown (S5), holding a crook (S38) vertically, with the opening inward.
1369E		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing the double crown (S5), holding a flagellum (S45).
1369F		King, seated, both knees up, with covered legs and arms, with a long straight beard, wearing the double crown (S5), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
136A0		God, seated, both knees up, with covered legs and arms, with a long, curved beard, wearing the double crown (S5), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
136A1		King, seated on a block-throne, with a long straight beard, wearing the double crown (S5), both arms forward, forearms horizontal, one above the other, holding a flagellum (S45).
136A2		King, standing, with a long straight beard, wearing the double crown (S5), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the king, vertically, left arm hanging beside the body holding a flagellum (S45) horizontally.
136A4		The king, wearing the blue crown, prostrating on a base, right knee forward, left leg nearly horizontal, arm forward, holding a vessel with a pellet of incence upon it.
136A5		Man, standing, wearing headdress consisting of a double plume mounted on rams horns, right arm forward, holding a staff/stick that angles towards the man, left arm hanging besides the body.
136A6		Man, standing, wearing headdress consisting of a double plume mounted on rams horns, right arm forward, holding a crook (S38) of the size of the man, vertically, left arm hanging besides the body, holding a flagellum (S45) horizontally.
136A7		Man, standing, wearing headdress consisting of a double plume mounted on rams horns, right arm forward, holding a staff/stick that angles towards the man, left arm hanging besides the body, holding a flagellum (S45) horizontally.
136A9		Man, standing, wearing headdress consisting of a double plume mounted on rams horns, right arm forward, holding a staff/stick that angles towards the man, left arm hanging besides the body, holding a mace with a pear-shaped head (T3) horizontally, mace-head sticking out in front of the legs.
136AA		Man, standing, wearing headdress consisting of a double plume mounted on rams horns, right arm forward, holding a crook (S38) of the size of the man, vertically, left arm hanging besides the body.
136AC		Man, standing upon a standard used for carrying religious symbols (R12), wearing headdress consisting of a double plume mounted on rams horns, right arm forward, holding a crook (S38) of the size of the man, vertically, left arm hanging besides the body.
136AD		Man, standing on a standard used for carrying religious symbols (R12), wearing headdress consisting of a double plume mounted on rams horns, right arm forward, holding a staff/stick that angles towards the man, left arm hanging besides the body.
136AE		God (Shu) seated, both knees up, with covered legs and arms, with a long, curved beard, with a feather (H6) on the head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
136AF		God, seated, both knees up, with covered legs and arms, with a long, curved beard, wearing the red crown (S1)/Double crown (S5), with 2 plumes on top of the horns of a ram on top/within the red crown/double crown, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
136B1		Man, seated, both knees down, with short straight beard, without arms, with a flagellum (S45) on its knees.
136B2		Man, seated, both knees down, with short straight beard, without arms, with a tie or strap, used with sandals (ankh-sign, S34), angled forwards, on its knees.
136B4		Man, seated, both knees down, with short straight beard, without arms, with a flower on the knee, which curves towards the face, with the flower in front of the face.
136B5		A mummy with a long curved beard, wearing the white crown (S1), standing upright.
136B6		A mummy with a long curved beard, wearing the white crown (S1), standing upright, covered arms crossed in front of the chest.
136B7		A mummy with a long curved beard, standing upright, both arms forward, forearms horizontally, one above another.
136B8		A mummy with a long curved beard, lying horizontally (A54) on top of a bed, with a leonid head, legs and tail (Q19), with the head towards the front.
136B9		A statue of the king, standing on a base, with a long straight beard, uraeus and coif, right arm forward, holding a staff/stick, left hand hanging beside the body, holding a piece of cloth.
136BB		A block statue of a seated man, knees raised with arms on the knees, while covered, without beard.
136BD		A statue of a man, standing on a base, with a support at the back, with the left arm vertically over the body.
136BE		A statue of a mummy with a long curved beard, standing upright on a base, with a support at the back.
136BF		A god, with slightly curved beard and straight wig, recumbent on a rectangular carrying chair, covered by cloth.
136C0		A god, with a long curved beard and short hair/wig, with a forwards and backwards curving line on the head, recumbent on a rectangular carrying chair, covered by cloth.
136C1		A statue of a woman,  wearing a long dress, standing on a base, with a support at the back, with the left arm vertically over the body.
136C2		A mummy, with a long curved beard, standing on a platform in front of an obelisk.
136C4		A man, lying down on a bed, facing upwards, knees slightly bend, right arm under the body, left arm over the body.
136C5		Man, written horizontally, facing down, right arm in front, left arm over the body, on top of a bed.
136C7		A mummy with a long straight beard and ureaus, lying belly down, head raised, on top of a bed, with a leonid legs and tail.
136C8		A man, mummyform, without beard, lying on a bed with short triangular bed-posts, facing upwards.
136C9		The king, seated on heel, both knees down, with a long straight beard, uraeus and coif/long wig, both arms forward, at the top of a sealed ointment jar (W2).
136CA		The king, seated on heel, both knees down, with a long straight beard, uraeus and coif/long wig, back bend forward, arm forward, hand  at the hight of the  waist, holding a cup or vessel (W10).
136CB		The king, seated on heel, right knee raised, with long straight beard and ureaeus, with arms raised in front of him, hand-palms facing outwards.
136CC		Man/king, seated, right knee raised, arms raised at either side of the body, right hand holding a tie or strap, used with sandals (ankh-sign, S34) of the same hight as the man, left hand holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40) in the left hand.
136CD		Two men, standing, facing each other, arms extended forwards, hands at the hight of the waist, holding one side of a loop of cord with the ends upwards (V6).
136CE		Two men, standing, both facing forward, back bend forward, arms down at either side of the body, holding a rope at he hight of the knees, which curves downward at either end.
136CF		Man and woman, standing, facing each other, man, left arm raised, extended towards the face of the woman, right arm hanging beside the body, woman, arms hanging at either side of the body.
136D0		Man and woman, copulating on a bed.
136D1		Man, kneeling, both knees down, arms extended forward, facing another man in the same position, holding hands.
136D2		Man, seated, right knee raised, arms extended forwards, handpalms forwards, facing another man in the same position, hands touching.
136D3		God, seated, both knees up, with covered legs and arms, with a long curved beard and long wig, with two heads with long curved beard and long wig written overlapping behind the head.
136D4		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a stem of papyrus with a bud (M131) or flower vertically.
136D5		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a club used by washer-men for beating laundry (U36) vertically.
136D6		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a diadem.
136D7		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a diadem and ureaus.
136D9		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a diadem and ureaus, holding a tie or strap, used with sandals (ankh-sign, S34), which angles forward.
136DA		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a diadem, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
136DB		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a diadem, holding a stem of papyrus with a bud (M131) or flower vertically.
136DC		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a diadem, holding a flagellum (S45).
136DE		Woman, seated, both knees up, with covered legs and arms, with long hair, wearing a cup (W10) on her head, holding a stem of papyrus with a bud (M13).
136DF		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a flower which curves forward, with the flower orientated downwards.
136E0		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a flower which curves towards the face, with the flower in front of the face.
136E1		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a round vessel with an upstanding rim (W24).
136E2		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a feather (H6) vertically.
136E3		Woman, seated, both knees up, with covered legs and arms, with long hair, holding two sticks in a 'v' shape.
136E4		Woman, seated, both knees up, with covered legs and arms, with long hair, with a line coming from the back of the head at 45° upwards.
136E5		Woman, seated, both knees up, with covered legs and arms, with long hair, and uraeus on the forehead.
136E6		Woman, seated, both knees up, with covered legs and arms, with long hair, with a dotted line coming from the face which angles downwards.
136E7		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a tie or strap, used with sandals (ankh-sign, S34), which angles forward.
136E8		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
136E9		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a flagellum (S45).
136EA		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a cup (W10).
136EB		Woman, seated, both knees up, with covered legs and arms, with long hair, holding a stem of papyrus with a bud (M13).
136ED		Woman, seated, both knees up, with covered legs, with long hair, with a cup/vessel (W10) on her head, rigth arm forward, supporting the cup/vessel at the front, left arm hanging beside the body.
136EE		Woman, seated, both knees up, with covered legs, with long hair, arms raised in front, hands near the hight of the face, handpalms outwards.
136EF		Woman, seated, both knees up, with covered legs, with long hair, left arm forward, with the wrist on the knee, right arm forward, upper arm horizontal, forearm vertical, right hand grasping the wrist of the left arm.
136F0		Woman, seated, both knees down, no arms visible.
136F2		Woman, seated, both knees down, no arms visible, with a lotus bud and stem on the knee, which curves upwards, with the bud pointing upwards at the hight of the face.
136F3		Woman, seated, both knees down, wearing a diadem, no arms visible, with a flower on the knee, which curves towards the face, with the flower in front of the face.
136F4		Woman, seated, both knees down, with long hair, with a spindle or spinning tool coming from her waist at 45°.
136F6		Woman, seated, both knees down, long hair, arms extended at either side of the body, angling downwards.
136F7		Woman, seated, both knees down, with long hair, arms extended forwards, towards the knees.
136F8		Woman, seated, both knees down, with long hair, arm raised in front , hand at the hight of the face, hand horizontal, hand palm up.
136F9		Woman, seated, both knees down, with long hair, arms extended forwards, towards the knees, handpalms upwards, with a riple of water (N35) above the hands.
136FA		Woman, seated, both knees down, with long hair, arms raised in front, hands vertical, handpalms outwards.
136FB		Woman, seated, both knees down, with long hair, arm raised in front, hand at the face, handpalm inwards.
136FC		Woman, seated, both knees down, with long hair, right arm in front of the body, left arm extended forward, hand on the knee.
136FE		Woman, seated, both knees down, with long hair, both arms forward, with a ball or other round object (tambourine?) between the two hands.
136FF		Woman, seated, both knees down, arms raised in front, upper arms horizontal, hands in a clapping motion, on top of a cover of a quiver without a loop on top.
13700		Woman, seated on a chair, with long hair, lower legs separated, no arms visible.
13701		Woman, seated on a chair, with long hair, lower legs separated, no arms visible, with a flagellum (S45) on her knee.
13702		Woman, seated on a chair, with long hair, both arms forward, forearms horizontal, one above the other, holding a flagellum (S45).
13703		Woman, seated on a chair, with long hair, lower legs separated, no arms visible, with a flower on the knee, which curves towards the face, with the flower in front of the face.
13704		Woman, seated on a chair, with long hair, lower legs separated, no arms visible, with a flower on the knee, which curves forward, flower hanging downwards near the knee.
13705		Woman, seated on a block throne, with long hair, no arms visible.
13706		Woman, seated on a block throne, with long hair, with an uraeus on the forehead, no arms visible.
13708		Woman, seated on a block throne without internal detail, on a base, with long hair, right arm in front of the body, left arm extended forward, hand on the knee.
13709		Woman, seated on a block throne, with long hair, both arms forward, forearms horizontal, one above the other, holding a flagellum (S45).
1370A		Woman, seated on a block throne on a base, with long hair, both arms forward, forearms horizontal, one above the other, holding a flagellum (S45).
1370C		Woman, seated on a chair, with long hair, both arms forward, forearms horizontal, one above the other, holding a stem of papyrus with a bud (M131) or flower, of the length of the seated figure, vertically.
1370D		Woman, seated on nothing, arms at either side of the body, hanging beside the body.
1370E		A girl, with long hair, seated on nothing, forelegs separated, right arm raised with hand to mouth, left arm hanging beside the body.
1370F		Woman, standing, right arm forward, arm bend, hand slightly higher than the shoulder, holding a flower with a long stem which angles forwards and downwards, just under the flower, which is in front of the face, left arm hanging beside the body.
13710		Woman, standing, back slightly bend forwards, arms extended forwards, hands at the hight of the waist, handpalms upwards, with a riple of water (N35) above the hands.
13711		Woman, standing, wearing a diadem, back slightly bend forwards, arms extended forwards, hands at the hight of the waist, handpalms upwards, with a riple of water (N35) above both hands.
13713		Woman, standing, with long hair, arms raised in front, hands to the face, handpalms outwards.
13714		Woman, standing, with long hair, arms raised in front, hand to mouth.
13715		Woman, standing, with long hair, arms hanging at either side of the body.
13716		Woman, standing, with long hair, right arm forward, hand at the hight of the waist, left arm over the body, left hand on the waist.
13717		Woman, standing, with long hair, arms raised at either side of the body, hands horizontal at the hight of the top of the head, handpalms upwards.
13718		Woman, standing, with long hair, arm in front, hand horizontal at the hight of the face, handpalm upwards.
13719		Woman, standing, with long hair, right arm forward, hand at the hight of the waist, holding a sedge (M23) of the hight of the woman, with the tip curving forward, left arm hanging beside the body.
1371A		Woman, standing, with long hair, right arm forward, hand at the hight of the waist, holding a crook (S38) of the hight of the woman, with the opening outward, left arm hanging beside the body.
1371B		Woman, standing, with long hair, bend forward, back nearly horizontal, arms extended forward, hands in an object which resembles a sand covered mountain over the edge of the cultivated areas (N26), in front of the feet.
1371C		Woman, standing, with long hair, arms raised in front, upper arms horizontal, forearms vertical, holding a snake (or wavy stick) over their head.
1371D		Woman, standing, with long hair, arms raised at either side of the body, hands horizontal at the hight of the top of the head, handpalms upwards, holding up the sky (N1).
1371E		Woman, standing, with long hair, right arm forward, holding a stick/staff, left arm hanging beside the body.
1371F		Woman, standing, with long hair, right arm forward, holding a stick/staff, left arm hanging beside the body, holding a flagellum (S45) horizotally.
13720		Woman, standing, with long hair, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13721		Woman, standing, with long hair, right arm forward, hand at the hight of the shoulder, holding a long staff or pole with a two-pronged end piece for holding down snakes (U116), left arm hanging beside the body.
13722		Woman, standing, with long hair, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13723		Woman, standing, with long hair, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
13724		Woman, standing, with long hair, right arm forward, hand at the hight of the waist, holding a A palm branch, stripped of leaves, with multiple sharp notches (M4A), tip curving outwards, notches towards the body, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13725		Woman, standing, back bend froward, with a line coming from the head, curving forward, arms extended forward, forearms horizontal at the hight of the waist.
13726		Woman, standing, back slight bend forwards, with two lines coming from the head, one bending forward, one bending backwards, both arms forward, twisted together.
13727		Woman, standing, with long hair, wearing a diadem, both arms raised at either side of the body, hands a the hight of the face, holding a stick in each hands, which is angled inwards.
13728		Woman standing, arms extended forwars, holding a child, with the arms and child covered in a cloth, only the head of the child visible.
13729		Woman, standing, with long hair, arms extended in front, hands at the hight of the shoulder, with a vertical line descending from each hand, which end at the hight of the bottom of the dress.
1372B		Woman, seated, both knees up, with covered arms and legs, with long hair, wearing a headdress consisting of a sundisk in horns, with two plumes on top of the sundisk (S65) and a vulture headdress, holding a stem of papyrus with a bud (M131) or flower vertically.
1372C		Woman, seated, both knees up, with covered legs and arms, with long hair, with two plumes on her head, holding a flower which curves forward, with the flower orientated downwards.
1372F		Woman, standing, wearing the white crown (S1), right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the size of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13730		Woman, standing, wearing the red crown (S3), arms hanging at either side of the body.
13731		Woman, standing, with long hair, both arms forward, hands at the hight of the shoulder, holding a plan of a crossroads in a village (O49).
13732		Woman, standing, with long hair, both arms forward, hands at the hight of the shoulder, holding a tambourine.
13733		Woman, standing, with long hair, both arms forward, hands at the hight of the shoulder, with a circle (tambourine?) floating above the hands.
13734		Woman, seatead, both knees down, with long hair, both arms forward, holding a tambourine.
13735		Woman, seated, both knees down, wearing a diadem, right arm extended forward, held horizontally, holding a a tie or strap, used with sandals (ankh-sign, S34) vertically, left arm extended forwards, between the legs and the right arm, holding a bead-necklace with counterweight at the counterweight.
13736		Woman, standing, with long hair, both arms forward, hands at the hight of the forehead, nearly horizontal, hands as if clapping.
13737		Woman, pregnant, seated, both knees down, no arms visible.
13738		Woman, pregnant, seated, both knees down, right arm forward, angling down towards the knee, left arm hanging beside the body, with 3 lines coming from the legs, central one vertical, the outer two curving from the same point as the central line.
13739		Woman, pregnant, seated, both knees down, right arm forward, angling down towards the knee, left arm hanging beside the body, with four vertical lines coming from the legs.
1373A		Woman, seated, both knees down, wearing a fillet with uraeus on her head, without arms, with four vertical lines coming from the legs.
1373B		Woman, seated, both knees down, wearing a fillet with uraeus on her head, arms hanging on either side of the body, with multiple backwards angled lines below the legs.
1373C		Woman, pregnant, seated, both knees down, with a hairlock coming from either side of the head, arms hanging on either side of the body, with the head and arms of a child coming from the legs.
1373D		Woman, seated, both knees down, both arms forward, hands in front of the belly, holding a child, seated on the arms of the woman, facing towards her, both arms beside the body.
1373F		Woman, seated, both knees down wearing a the red crown (S3), nursing a child, seated on her lap, left arm forward, hand over the chest of the woman, right arm hanging beside the body, right arm forward, embracing/holding the child, left arm over the chest.
13740		Woman, seated, both knees down wearing a the white crown (S1), nursing a child, seated on her lap, left arm forward, hand over the chest of the woman, right arm hanging beside the body, right arm forward, embracing/holding the child, left arm over the chest.
13742		Woman, seated on nothing, with a child on her lap, facing her, both arms down, both arms forward, right arm behind the child, left arm in front of the child.
13743		Woman, seated on a chair, both arms forward, holding a child, seated, both arms at either side of the body.
13744		Woman, bend backwards, facing up, arms and feet on the ground, hair hanging downwards.
13745		Woman, standing, bend forward arms extended forward, hands towards an rectangular object in front of the feet.
13747		Woman, standing, bend forwards, back  horizontal, hair hanging forwards in front of the face, arms extended towards the ground,  hands horizontal at the hight of the middle of the shins, handpalms downwards.
13748		Woman, seated, with both knees down, with the head and arms of a child coming from the legs, both arms raised at either side of the body, hands at the hight of the top of the head, supporting a duck or goose (G38-G39) on each hand, looking outwards, with a half round loaf of bread (X1) over the shoulder of each bird.
13749		God, seated, both knees up, with covered legs and arms, with a long curved beard and long hair/wig, with a crescent moon with the entire moon disk (N62) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1374A		God, seated, both knees up, with covered legs and arms, with a long curved beard and long hair/wig, with a crescent moon with the entire moon disk (N62) on its head.
1374B		God, seated, both knees up, with covered legs and arms, with a long curved beard and long hair/wig, with a crescent moon with the entire moon disk (N62) on its head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
1374D		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with a uraeus on his forehead, with a star upon his head (N14).
1374E		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with a uraeus on his forehead, with a star upon his head (N14), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1374F		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with an eye (D4) upon his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13751		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with an elephants tusk (F18) upon his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13752		God, standing, with long curved beard and long wig, with an elephants tusk (F18) upon his head, right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), left arm hanging beside the body, holding tie or strap, used with sandals (ankh-sign S34) at the loop.
13753		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with a rectangular piece of cloth, with fringes on one of its short sides (S32) upon his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13754		God, standing, with long curved beard and long wig, with a rectangular piece of cloth, with fringes on one of its short sides (S32) upon his head, right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), left arm hanging beside the body, holding tie or strap, used with sandals (ankh-sign S34) at the loop.
13758		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, wearing the hmhm crown (S61), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13759		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with a dung beetle (scarab (L1)) upon his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1375A		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with two arms, raised, with the palms of the hands towards each other (D28) upon his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1375B		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with the hind-quarters of a seated lion or leopard (F22) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1375C		God, seated, both knees up, with covered legs and arms, with a long curved beard and short hair/wig, with a cobra in repose (Naja haja (I10)) upon his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1375D		God, seated, both knees up, with covered legs and arms, with a long curved beard and long hair/wig, with an uraeus on the forehead, wearing a headdress consisting of two feathers and a sun-disk on rams horns, with an uraeus at either side of the feathers, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1375F		God, both knees up, with covered legs and arms, with the head of a vulture, holding a knife (T30A).
13760		God, both knees up, with covered legs and arms, with the head of a vulture, with a Atef crown without horns (S8A) on its head.
13761		God, both knees up, with covered legs and arms, with the head replaced by a dung beetle (scarab, L1).
13762		God, both knees up, with covered legs and arms, with the head replaced by a dung beetle with outstretched wings, resembling the wings of a bird (L12A).
13763		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), with a line running from the crown down the back.
13764		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A).
13765		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), with a line running from the crown down the back, holding a feather (H6).
13766		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), with a line running from the crown down the back, holding a tie or strap, used with sandals (ankh-sign, S34), vertically.
13767		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), holding a tie or strap, used with sandals (ankh-sign, S34), vertically.
13768		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), with a line running from the crown down the back, holding a sceptre (S42).
13769		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the double-plume headdress (S72A), with a line running from the crown down the back, holding a tie or strap, used with sandals (ankh-sign, S34), written over a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1376C		God, seated on a block throne on a base, with a ong curved beard, wearing the double plume headdress (S72A), with a line running from the crown down the back, with a child seated on the his lap, facing the god, both arms down, both arms forward, right arm behind the child, left arm in front of the child.
1376D		God, standing, with a long curved beard, wearing the double plume headdress (S72A), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
1376E		God, seated, knees up, with covered legs and arms, with the head of a jackal, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13770		God, seated, knees up, with covered legs and arms, with the head of a jackal, holding a knife (T30A).
13771		God, seated, knees up, with covered legs and arms, with the head of a jackal, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
13772		God, seated on heel, right knee raised, with the head of a jackal, right arm angled in front of the body, left arm raised, hand held verically, handpalm inwards (hnw gesture).
13773		God, seated on nothing, with the head of a jackal and tail, right arm forwards, hand at the hight of the face, left arm raised at the back, forearm vertical, holding a stick.
13774		God, seated on nothing, with the head of a jackal and tail, right arm forwards, holding a shield, left arm raised at the back, forearm vertical, holding a stick.
13776		God, seated on nothing, with the head of a jackal and tail, right arm forwards, hand at the hight of the shoulder, holding a shield/buckler resembling a half circle, left arm raised at the back, forearm vertical, holding a knife with a triangular blade and straight handle (T30A) horizontally, blade downwards.
13777		God, seated on nothing, with the head of a jackal and tail, right arm forwards, hand at the hight of the shoulder, holding a knife (T30A) vertically, left arm raised at the back, forearm vertical, holding a stick.
13778		God, seated on nothing, with the head of a jackal and tail, right arm forwards, hand at the hight of the shoulder, holding a stick vertically, running from the hand to the knee, left arm raised at the back, forearm vertical, holding a stick.
13779		A mummy, with the head of a jackal, standing upright.
1377A		God, standing, with the head of a jackal, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
1377B		Goddess, standing, with the head of a jackal, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the goddess, vertically, left arm hanging beside the body.
1377C		God, standing, with the head of a jackal, right arm forward, hand at the hight of the shoulder, holding the top of a staff, which angles towards the body, left arm hanging beside the body
1377D		God, standing, with the head of a jackal, both arms forward, right hand at the hight of the shoulder, left hand at the hight of the waist, both holding a knife (T30A).
1377E		God, standing, with the head of a jackal, right arm forward, hand at the hight of the shoulder, holding a knife (T30A), left arm hanging beside the body, holding a knife horizontally.
1377F		God, standing, with the head of a jackal, right arm forward, hand at the hight of the waist, holding a knife (T30A), left arm raised at the back, forearm vertical, holding a knife (T30A) with the blade upwards.
13780		God, standing, with the head of a jackal, right arm forward, hand at the hight of the waist, holding an oval shield, left arm raised at the back, forearm vertical, holding a stick horizontally.
13784		God, seated, right knee raised, with the head of a jackal, right arm forward, forearm horizontal, hand in front of the chest, left arm raised, forearm vertical, holding a stick which extends over the head.
13785		God, seated on nothing, with the head of a jackal and tail, right arm forwards, hand at the hight of the shoulder, holding a knife (T30A) vertically, left arm raised at the back, forearm vertical, holding a knife (T30A) with the blade downwards.
13786		God, standing, with the head of a jackal, right arm forward, hand at the hight of the waist, holding a knife (T30A), left arm raised at the back, forearm vertical, left hand as a fist.
13788		God, standing, with the head of a jackal, with a tail, both arms forward, holding a stick with a bundle or a mat horizontally over his shoulders, bundle behind the back, with the ties at the top.
13789		God, standing, with the head of a jackal, right arm extended forwards, holding a stick  with a bundle or a mat at a slight angle over his shoulder, bundle behind the back, with the ties at the top, left arm hanging beside the body.
1378A		God, standing, with the head of a jackal, right arm forward, hand at the hight of the waist, holding a stick, left arm raised at the back, forearm vertical, holding a stick.
1378B		God, seated, both knees up, foot flat on the ground, with the head of a jackal, both arms raised at either side of the body, forearms vertical, handpalms inwards.
1378C		God, standing, with the head of a jackal, arms hanging on either side of the body, standing behind a mummy with a long curved beard, lying horizontally (A54) on top of a bed, with a leonid head, legs and tail (Q19), with the head towards the front (A55A).
1378D		God/Goddess, seated, knees up, with covered legs and arms, with the head of a bovid, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1378F		God, seated, knees up, with covered legs and arms, with the head of a bovid.
13790		God, standing, with the head of a bovid, right arm forward, hand at the hight of the shoulder, holding a knife (T30A), left arm hanging beside the body.
13791		God, standing, with the head of a bovid, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
13793		God, seated, knees up, with covered legs and arms, with a long curved beard, with a feather (H6) on his head.
13794		God, seated, knees up, with covered legs and arms, with a long curved beard, with a feather (H6) on his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13795		God, seated, knees up, with covered legs and arms, with a long curved beard, with a feather (H6) on his head, holding a feather (H6), vertically.
13796		God, seated, knees up, with covered legs and arms, with the head of a falcon.
13797		God, seated on heel, right knee raised, with the head of a falcon, right arm angled in front of the body, left arm raised, hand held verically, handpalm inwards (hnw gesture).
13798		God, seated, knees up, with covered legs and arms, with the head of a falcon, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13799		God, seated, knees up, with covered legs and arms, with the head of a falcon, holding a mace with a pear-shaped head (T3), angled forwards.
1379A		God, seated, knees up, with covered legs and arms, with the head of a falcon, holding a club used by washer-men for beating laundry (U36) vertically.
1379B		God, seated, knees up, with covered legs and arms, with the head of a falcon, holding a sceptre (S42) vertically.
1379C		God, seated, knees up, with covered legs and arms, with the head of a falcon, holding a knife (T30A) vertically.
1379D		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head.
1379E		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a sun disk (N5) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1379F		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137A0		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a harpoon head made of bone, with one curl (T19), with the curl facing forward.
137A1		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
137A2		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a sun disk (N5) on its head, holding a crook (S38) vertically, with the opening inwards.
137A4		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a sun disk (N5) on its head, holding a feather (H6) vertically.
137A5		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a sun disk (N5) on its head, holding a sceptre (S42) vertically.
137A6		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a sceptre (S42) vertically.
137A7		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding an eye, with the markings of the head of a falcon (wdA.t, D10).
137A8		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a round vessel with an upstanding rim (W24).
137A9		God, seated, knees up, with covered legs and arms, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding an egg (H8).
137AA		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5).
137AB		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137AC		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
137AD		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding an eye, with the markings of the head of a falcon (wdA.t, D10).
137AE		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a sceptre (S38), vertically.
137AF		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a club used by washer-men for beating laundry (U36) vertically.
137B0		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding two arms, raised, with the palms of the hands towards each other (D28).
137B1		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a scepter with the head of a falcon with a sun disk on top of the head.
137B2		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a sistrum (Y18).
137B3		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically, upon a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
137B4		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the double crown (S5), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically, upon a canal (N36).
137B5		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the white crown (S1), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137B6		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the Atef crown without horns (S8A), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137B7		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing the Atef crown without horns (S8A),
137B8		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing a headdress consisting of two plumes with a sun disk (S63A/S70).
137B9		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing a headdress consisting of two plumes with a sun disk (S63A/S70), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137BA		God, seated, knees up, with covered legs and arms, with the head of a falcon, wearing a headdress consisting of two plumes with a sun disk on rams horns, with an uraeus with a sun disk on its head, at either side of the plumes (S75).
137BC		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a crescent moon with the entire moon disk (N62) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137BD		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a crescent moon with the entire moon disk (N62) on its head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
137BF		God, seated, knees up, with covered legs and arms, with the head of a falcon, with the sun rising over a sand covered mountain over the edge of the cultivated areas (N27) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137C1		God, seated, knees up, with covered legs and arms, with the head of a falcon, with the hind-quarters of a seated lion or leopard (F22) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137C2		God, seated, knees up, with covered legs and arms, with the head of a falcon, with the hind-quarters of a seated lion or leopard (F22) on its head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
137C3		God, seated, knees up, with covered legs and arms, with the head of a falcon, with the hind-quarters of a seated lion or leopard (F22) on its head, holding a A cobra (Naja haja), standing up, with expanded hood (Uraeus, I64).
137C4		God, seated, knees up, with covered legs and arms, with the head of a falcon, with the hind-quarters of a seated lion or leopard (F22) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically, upon a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
137C5		God, seated on a block throne, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40) of the lenght of the god, left arm forward, hand on the knee, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
137C6		God, seated on a block throne, with the head of a falcon, with the sun rising over a sand covered mountain over the edge of the cultivated areas (N27) on its head , right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40) of the lenght of the god, left arm forward, hand on the knee.
137C7		God, seated on a block throne on a base, with the head of a falcon, wearing the double crown (S5), right arm forward, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40) of the lenght of the god, left arm forward, hand on the knee.
137C8		God, seated on a block throne on a base, with the head of a falcon, wearing the double crown (S5), right arm forward, hand a the hight of the face, holding a tie or strap, used with sandals (ankh-sign, S34) horizontally at the base, left arm forward, hand at the knee, holding a tie or strap, used with sandals (ankh-sign, S34) horizontally at the loop.
137C9		God, seated on a block throne on a base, with the head of a falcon, wearing the double crown (S5), left arm forward, hand near the knee, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (wAs, S40), right arm forward, holding a tie or strap, used with sandals (ankh-sign, S34) horizontally at the base, above the wAs scepter.
137CB		God, seated, knees up, with covered legs and arms, with the head of a falcon, with a star (N14) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137CC		God, standing, with the head of a falcon, wearing the double crown (S5), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34) vertically at the loop.
137CD		God, standing, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34) vertically at the loop.
137CE		God, standing, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, right arm forward, hand at the hight of the waist, holding a spear made into a standard (R15A), left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34) vertically at the loop.
137D0		God, standing, with the head of a falcon, with a headdress consisting of two plumes and a sun disk (S63A/S70) and an uraeus at the front on its head, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
137D1		God, standing, with the head of a falcon, with the sun rising over a sand covered mountain over the edge of the cultivated areas (N27) on its head, arms raised at either side of the body, hands at the hight of the shoulders, holding a stick/staff topped with a sand covered mountain over the edge of the cultivated areas (N26) sign.
137D2		God, standing, with the head of a falcon, with the sun rising over a sand covered mountain over the edge of the cultivated areas (N27) on its head, arms extended at either side of the body, hands at the hight of the waist, holding a wick of twisted flax, consisting of three loops (V28), below the third loop.
137D3		God, standing, with the head of a falcon, arms raised at either side of the body, hands at the hight of the shoulders, holding a stick/staff topped with a sand covered mountain over the edge of the cultivated areas (N26) sign.
137D4		God, standing, with the head of a falcon, with a sun disk (N5) on its head, arms raised at either side of the body, hands at the hight of the shoulders, holding a stick/staff topped with a sand covered mountain over the edge of the cultivated areas (N26) sign.
137D5		God, standing, with the head of a falcon, wearing the double crown (S5), right arm forward, hand at the hight of the waist, holding a spear/harpoon of the same size as the god, vertically, point towards the top, left arm hanging beside the body.
137D6		God, standing, with the head of a falcon, right arm forward, hand at the hight of the waist, holding a spear/harpoon of the same size as the god, vertically, point towards the top, left arm hanging beside the body.
137D7		God, standing, with the head of a falcon, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
137D8		God, standing, with the head of a falcon, holding an harpoon with the point down, diagonally over the body, from the tip of the right foot beyond the left shoulder, right arm lowered, right hand holding the harpoon between the waist and knee, left arm raised, holding the shaft near the end.
137DA		God, standing, with the head of a falcon, wearing the double crown (S5), right arm extended forwards, arm horizontal, holding a scimitar (khopesh), written vertically, blade curving forwards, with the head of a falcon with a sun-disk (N5) on top of it, left arm forwards, hand at the hight of the waist, holding a tie or strap, used with sandals (ankh-sign, S34) at an downwards angle at the loop.
137DB		God, standing, with the head of a falcon, wearing the double crown (S5), right arm extended forwards, arm horizontal, holding a scimitar (khopesh), written vertically, blade curving forwards, with the head of a falcon with a sun-disk (N5) on top of it, left arm hanging beside the body.
137DC		A mummy, with the head of a falcon, standing upright.
137DE		God, standing, with the head of a falcon, with the sign for the hill country over the edge of the cultivated areas (N25) on its head, arms hanging downwards on either side of the body, holding a cobra in repose (Naja haja (I10)), with the horizontal part of the body at the hight of the waist.
137DF		God, standing, with the head of a falcon, right arm forward, holding a spear with the point downwards, left arm hanging beside the body, on top of an bovid, lying on the ground, legs folded under the body, tail down.
137E0		God, standing, with the head of a falcon, wearing the double crown (S5), right arm forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically , left arm hanging beside the body, on top of an bovid, lying on the ground, legs folded under the body, tail down.
137E2		God, standing, with the head of a falcon, wearing the double crown (S5), right arm forward, holding a spear with the point downwards, left arm hanging beside the body, on top of an bovid, lying on the ground, legs folded under the body, tail down.
137E3		God, standing, with the head of a falcon, right arm forward, holding a spear with the point downwards, left arm hanging beside the body, on top of an crocodile (I3).
137E4		God, standing, with the head of a falcon, wearing the double crown (S5), holding an harpoon with the point down, diagonally over the body, from the tip of the right foot beyond the left shoulder, right arm lowered, right hand holding the harpoon between the waist and knee, left arm raised, holding the shaft near the end, on top of an crocodile, head looking backwards (I3E).
137E5		God, standing, with the head of a falcon, wearing the double crown (S5), holding an harpoon with the point down, diagonally over the body, from the tip of the right foot beyond the left shoulder, right arm lowered, right hand holding the harpoon between the waist and knee, left arm raised, holding the shaft near the end, on top of an crocodile (I3).
137E6		God, standing, with the head of a falcon, wearing the double crown (S5), right arm forward, holding a spear with the point downwards, left arm hanging beside the body, on top of an crocodile (I3).
137E7		God, seated, both knees up, with covered legs and arms, with a long curved beard, wearing a headdress consisting of a combination of the Atef crown and the red crown (S53vara?), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
137E8		God, seated, right knee raised, with long curved beard and long wig, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
137E9		God, seated, right knee raised, with long curved beard and long wig, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
137EA		God, seated, right knee raised, with long curved beard and coif, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, raised arms at either side of the body, hands held vertically, with the handpalms outwards.
137EB		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
137EC		Man/god, seated on heel, both knees down, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
137ED		God, seated, right knee raised, with long curved beard and long wig, with a tie or strap, used with sandals (ankh-sign, S34) vertically on its head, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
137EE		Man/god, seated, right knee up, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, raised arms at either side of the body, forearms vertical, holding a tie or strap, used with sandals (ankh-sign, S34) vertically at the base in each hand.
137EF		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, arms extended at either side of the body, holding a palm branch, stripped of leaves with multiple notches (M4A), with the notches outwards.
137F0		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, arms extended at either side of the body, holding a palm branch, stripped of leaves with multiple notches (M4A), with the notches outwards, on top of a base.
137F2		Man, seated on heel, right knee raised, with coif/short hair, without beard, arms extended at either side of the body, holding a palm branch, stripped of leaves with multiple notches (M4A), with the notches outwards.
137F3		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, arms extended at either side of the body, holding a palm branch, stripped of leaved with multiple notches (M4A), with the notches outwards, on top of frogs, looking inwards, on top of a basket (V30).
137F5		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head (M4), notch forward, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards at the top, connecting at the top of the M4 sign on the head.
137F6		Man/god, seated, right knee raised, with coif/short hair, without beard, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards at the top, connecting to the top of the head.
137F7		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, raised arms at either side of the body, hands held vertically, with the handpalms inwards.
137F8		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves with multiple notches (M4A), with the notches outwards.
137F9		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves with multiple notches (M4A), with the notches outwards, on top of frogs, looking inwards.
137FA		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a lotus bud with a long stem, with the buds orientated towards the sun disk, with a tie or strap, used with sandals (ankh-sign, S34) around each elbow.
137FC		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards at the top, connecting at the sun disk on the head, on a base.
137FD		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, with multiple notches (M4A), with the notches outwards, connecting at the sun disk on the head.
137FE		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards at the top, connecting at the sun disk on the head, on top of a collar of beads, without detail (S12A).
137FF		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards at the top, connecting at the sun disk on the head, branches on top of frogs, looking inwards.
13800		Man/god, seated on heel, right knee raised, with coif/short hair, without beard, with a sun disk (N5) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards at the top, connecting at the sun disk on the head, on top of a basket (V30).
13801		Man/god, seated, right knee up, with long hair, without beard, with a sun disk with an uraeus at either side (N6B) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards, with a forked top end, on top of frogs, looking inwards.
13802		Man/god, seated, right knee up, with long hair, without beard, with a sun disk with an uraeus at either side (N6B) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards, on top of a basket (V30).
13803		Man/god, seated, right knee up, with long hair, without beard, with a sun disk with an uraeus at either side (N6B) on his head, arms extended at either side of the body, holding a palm branch, stripped of leaves, curving inwards.
13804	13805	Youth, standing, right arm raised in front, with hand to mouth, with a line running from the elbow to the ankle, left arm hanging beside the body.
13806		Youth, standing, wearing the double crown (S5), right arm raised in front, with hand to mouth, with a line running from the elbow to the ankle, left arm hanging beside the body.
13808		Youth, seated, both knees up, feet on the ground, right arm raised in front, with hand to mouth, left arm in front of the body, holding a flagellum (S45) which angles against the left shoulder.
13809		Youth, seated, right knee up, wearing the red crown (S3), right arm raised in front, with hand to mouth, left arm in front of the body.
1380A		Youth, seated, both knees up, feet on the ground, wearing the double crown (S5), right arm raised in front, with hand to mouth, left arm in front of the body, holding a flagellum (S45) which angles against the left shoulder.
1380B		God, seated, knees up, with covered legs and arms, with the head of a ram, with forward curling horns.
1380C		God, seated, knees up, with covered legs and arms, with the head of a ram with horizontal twisted horns, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1380D		God, seated, knees up, with covered legs and arms, with the head of a ram with horizontal twisted horns, with the head of a cobra in the middle of the horns, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1380E		God, seated, knees up, with covered legs and arms, with the head of a ram with horizontal twisted horns, holding a stone jug with a large handle connecting to the rim, with small handle (W9), with the large handle forward.
1380F		God, seated, knees up, with covered legs and arms, with the head of a ram, with a headdress consisting of two feathers with a sun disk on rams horns, with an uraeus at either side of the feathers, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13810		God, seated, knees up, with covered legs and arms, with the head of a ram, with a headdress consisting of two feathers with a sun disk on rams horns, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13811		God, seated, knees up, with covered legs and arms, with the head of a ram, with a headdress consisting of two plumes with a sun disk on rams horns, with an uraeus at either side of the plumes, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13812		God, seated, knees up, with covered legs and arms, with the head of a ram, wearing the atef crown (without sun disk), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13813		God, seated, knees up, with covered legs and arms, with the head of a ram with horizontal twisted horns, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13815		God, seated, knees up, with covered legs and arms, with the head of a ram, with forward curling horns, with a sun disk (N5) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13816		God, seated on a block, with the head of a ram with horizontal twisted horns, in front of a potters wheel with a vessel on it, right arm forward, hand inside the vessel, left arm forward, hand at the potters wheel.
13817		God, seated on a block throne on a base, with the head of a ram with horizontal twisted horns, in front of a potters wheel with a lump of clay on it, right leg forward, with the foot at the base of the potters wheel, right arm forward, hand on top of the lump of clay, left arm forward, hand at the potters wheel.
13818		God, with the head of a ram with horizontal twisted horns, standing at a potters wheel with a round vessel with an upstanding rim (W24) on it, right leg extended, right foot at the base of the potters wheel, both arms extended in front, right hand above the vessel, left hand at the top of the potters wheel.
13819		God, with the head of a ram with horizontal twisted horns, with a sun disk (N5) on its head, seated on nothing, in front of a potters wheel with a lump of clay on it, right leg extended, right foot at the base of the potters wheel, both arms extended in front, right hand above the lump of clay, left hand at the top of the potters whieel.
1381A		God, standing, with the head of a ram with forwards curving horns, both arms raised in front, touching a vertically written wall with battlements near the top, left leg raised, touching the wall near the bottom.
1381B		God, standing, with the head of a lion, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
1381C		God, standing, with the head of a lion, with a feather (H6) on its head, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
1381D		God, standing, with the head of a lion, right arm forward, forearm horizontal, holding a knife (T30A) vertically, with the blade outwards, left arm hanging beside the body, holding a stick horizontally.
1381E		An ithyphallic god, with long, curved beard, standing on a platform, wearing a double plume headdress, with a line coming from the back of the head, running down to the base, arm raised in back, with a flagellum (S45) written over the arm, in front of an altar/table with two conical shapes (tree or bread) and a flower (M16A?).
13820		An ithyphallic god, with long, curved beard, standing, wearing a double plume headdress, arm raised in back, with a flagellum (S45) written over the arm.
13824		God, seated, both knees down, with a long curved beard and long wig/hair, with a clump of three papyrus flowers (M16A) on its head, both arms forward, hands at the hight of the chest, holding a tray or reed mat, with a tall water-pot, with spout, with line of fluid coming from the spout (W15A) on top of it, with the spout at the front.
13825		God, seated, both knees down, with a long curved beard and long wig/hair, with a clump of three papyrus flowers (M16A) on its head, both arms forward, hands at the hight of the chest, holding a tray or reed mat, with a tall water-pot, with spout, with line of fluid coming from the spout (W15A) on top of it, with the spout at the front, with a lotus flower and stem (M9/rotated M133) written over the water pot.
13826		God, seated, both knees down, with a long curved beard and long wig/hair, with a clump of three papyrus flowers (M16A) on its head, both arms forward, hands at the hight of the waist, holding a tall waterpot (W14).
13827		God, seated, both knees down, with a long curved beard and long wig/hair, with a clump of three papyrus flowers (M16A) on its head, both arms forward, hands at the hight of the waist, holding a tall waterpot, with spout, with line of fluid coming from the spout (W15A), spout at the front.
13829		God, standing, with a long curved beard and long wig/hair, with a clump of three papyrus flowers, with two buds bent down (M15) on its head, both arms forward, hands at the hight of the waist, holding a tray or reed mat, with two tall waterpots (W14) on it.
1382A		God, standing, with a long curved beard and long wig/hair, with a clump of three papyrus flowers, with two buds bent down (M15) on its head, both arms forward, hands at the hight of the waist, holding a tray or reed mat, with two tall waterpots (W14) on it, with a lotus flower and stem (M9/rotated M133) written over the water pot, with the stem of the flower extending beyond the mat, with a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40) between the two pots, with the lower half of the sceptre extending below the mat.
1382B		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing a headdress consisting of four plumes on rams horns (S68), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1382D		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing a headdress consisting of two plumes on rams horns (S62).
1382E		God, walking, looking backwards, with a long curved beard and long hair/wig, right arm forward, hand at the hight of the waist, holding a stick/staff, left arm backwards, hand at the hight of the shoulder, holding a star (N14).
1382F		God, walking, wearing the double crown, right arm forward, angling down, left arm raised in back, forearm vertical, holding a stick.
13830		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the Atef crown without horns (S8A).
13831		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the Atef crown without horns (S8A), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13833		God, seated, knees up, with covered legs and arms, with a long curved beard, wearing the Atef crown without horns (S8A), holding a flagellum (S45) and a crook (S38), opening inwards.
13835		God, seated on a block throne on a base, with a long curved beard, wearing the Atef crown without rams horns (S8A), both arms in front of the chest, right hand holding a crook (S38), opening inward, angled over the right shoulder, left hand holding a flagellum (S45), angled over the left shoulder, on top of a base.
13836		God, seated on a block throne on a base, with a long curved beard, wearing the Atef crown with rams horns (S8), both arms in front of the chest, right hand holding a crook (S38), opening inward, angled over the right shoulder, left hand holding a flagellum (S45), angled over the left shoulder, on top of a base.
13837		God, in mummyform, standing upright, with a long curved beard, wearing the Atef crown with rams horns (S8), both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically.
13838		God, in mummyform, standing upright on a platform, with a long curved beard, wearing the Atef crown without rams horns (S8A), both arms in front of the chest, right hand holding a flagellum (S45), angled over the right shoulder, left hand holding a crook (S38), opening inward, angled over the left shoulder.
13839		God, seated, knees up, with covered legs and arms, with a long straight beard, wearing a cap.
1383A		God, seated, knees up, with covered legs and arms, with a long straight beard, wearing a cap, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1383B		God, seated, knees up, with covered legs and arms, with a long straight beard, wearing a cap, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
1383C		God, seated, knees up, with covered legs and arms, with a long straight beard, wearing a cap, holding a papyrus scroll vertically.
1383D		God, in mummyform, standing upright, with a long straight beard, wearing a cap, both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, adorned with an ankh-sign (S34) and a djed-pillar (R11).
1383E		God, in mummyform, standing upright, with a long straight beard, wearing a cap, both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically.
1383F		God, in mummyform, standing upright on a platform, with a long straight beard, wearing a cap, both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, inside a shrine without the pillar at the front.
13840		God, in mummyform, standing upright on a platform, with a long straight beard, wearing a cap, both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, adorned with an ankh-sign (S34) and a djed-pillar (R11), inside the facade of a shrine (O21), inside a shrine supported by pillars resembling a stem of papyrus with a bud (M13).
13841		God, seated on a block-throne on a base, with a long straight beard, wearing a cap, both arms forwards, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13842		God, seated on a block-throne on a base, with a long straight beard, wearing a cap, both arms forwards, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically, adorned with an ankh-sign (S34) and a djed-pillar (R11).
13843		God, seated, knees up, with covered legs and arms, with a long curved beard and long wig/hair, with a sun-disk (N5) on his head.
13844		God, seated, knees up, with covered legs and arms, with a long curved beard and long wig/hair,  with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13845		God, seated, knees up, with covered legs and arms, with a long curved beard and long wig/hair, with a sun-disk (N5) on his head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13846		God, seated, knees up, with covered legs and arms, with a long curved beard and long wig/hair, with a sun-disk (N5) on his head, holding a dung-beetle (scarab, L1).
13847		God, seated, knees up, with covered legs and arms, with a long curved beard and long wig/hair,  with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a sceptre (S42).
13849		God, seated, knees up, with covered legs and arms, with the head of a Seth animal, holding a crook (S38), vertically, with the opening inwards.
1384A		God, seated, knees up, with covered legs and arms, with the head of a Seth animal, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
1384B		God, seated, knees up, with covered legs and arms, with the head of a Seth animal, on top of a collar of beads (S12).
1384E		Humanoid, with the head of a donkey/ass, with a knife (T30A) at the face, blade towards the back, with hands bound to feet at the back, in front of a pole which is forked at the top, to which the humanoid is bound.
1384F		Humanoid, with the head of a donkey/ass, with hands bound to feet at the back, in front of a pole which is forked at the top, to which the humanoid is bound.
13850		God, in mummyform, standing upright, with the head of a donkey/ass, with a knife (T30A) at the face, blade downwards.
13851		God, seated, knees up, with covered legs and arms, with the head of a crocodile.
13852		God, seated, knees up, with covered legs and arms, with the head of a crocodile, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13853		God, seated, knees up, with covered legs and arms, with the head of a crocodile, with rams horns and a N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13855		Goddess, seated, both knees down, with the head of a crocodile, wearing the red crown (S3), arms extended as either side of the body, angling downwards, with a crocodile (I3) under each arm.
13856		An archaic image of a falcon, with a thorn, point upwards in front of it.
13857		God, seated, knees up, with covered legs and arms, with the head of a baboon/monkey, holding a mace (T3) angling forward.
13858		God, seated, knees up, with covered legs and arms, with the head of a baboon/monkey, holding a knife (T30A), vertically.
13859		God, seated, knees up, with covered legs and arms, with the head of a baboon/monkey, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1385A		God, in mummyform, standing upright, with the head of a baboon/monkey.
1385B		God, seated, knees up, with covered legs and arms, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers on rams horns (S77).
1385C		God, seated, knees up, with covered legs and arms, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers on rams horns (S77), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1385D		God, seated, knees up, with covered legs and arms, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers on rams horns (S77), holding a pestle (U33), vertically, top curving outwards.
1385E		God, seated, both knees down, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers on rams horns (S77), both arms forward, holding a flagellum (S45).
1385F		God, seated, both knees down, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers on rams horns, with a uraeus at either side of the feathers, both arms forward, holding a flagellum (S45).
13860		God, seated, both knees down, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers and a sun disk on rams horns, with a uraeus with a sun disk on the head at either side of the feathers, both arms forward, holding a flagellum (S45) and a crook (S38), opening inwards.
13861		God, seated, both knees down, with a long curved beard and long hair/wig, wearing a headdress consisting of two feathers on rams horns (S77), both arms forward, holding a flagellum (S45) and a crook (S38), opening inwards.
13862		God, standing, wearing a headdress consisting of two feathers and a sun disk on rams horns (S73), right arm forward, forearm horizontal, holding a sedge plant (M23), which curves towards the front, left arm hanging beside the body.
13863		God, in mummyform, standing upright, with a long curved beard, wearing a headdress consisting of two feathers and a sun disk on rams horns (S73).
13864		God, seated, knees up, with covered legs and arms, with the head of an ibis, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13865		God, seated, with knees up, feet flat on the ground, with the head of an ibis, arm forward, forearm following the angle of the upper leg, hand on the knee, horizontally, handpalm up, supporting a human face (D2).
13866		God, seated, knees up, with covered legs and arms, with the head of an ibis, holding a heart (F34).
13867		God, seated, knees up, with covered legs and arms, with the head of an ibis, with a crescent moon with the entire moon disk (N62) on its head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13868		God, seated, knees up, with covered legs and arms, with the head of an ibis, with a crescent moon with the entire moon disk (N62) on its head.
1386A		God, seated, knees up, with covered legs and arms, with the head of an ibis, wearing the Atef crown with ram's horns, without sun disk, holding holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1386B		God, seated, knees up, with covered legs and arms, with the head of an ibis, wearing the Atef crown without rams horns (S8A), holding a human face (D2).
1386C		God, seated, knees up, with covered legs and arms, with the head of an ibis, wearing the Atef crown without rams horns (S8A), holding a heart (F34).
1386D		God, standing, with the head of a ibis, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
1386F		God, standing, with the head of a ibis, arms hanging at either side of the body.
13871		God, standing, with a long sash-kilt, no beard and short hair/wig, wearing a headdress consisting of four plumes and a sun disk, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
13872		An oval of sand, with two human heads on top, at either end, facing outwards.
13873		God/man, standing on a base, with coif/short hair, without beard, with a palm branch, stripped of leaves and notched on his head, notch and curve backwards, arms raised at eithere side of the body, hands above the hight of the head, handpalms inwards.
13874		Man/god, with the legs replaced by a star, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically, left arm hanging beside the body.
13875		Man/god, standing, with a sand covered mountain over the edge of the cultivated areas (N26) on his head, right arm forward, hand at the hight of the waist, holding a cobra in repose (Naja haja), standing up (I80), left arm hanging beside the body.
13876		Goddess, seated, knees up, with covered legs and arms, wearing a headdress of outwards waving plumes, holding a holding a stem of papyrus with a bud (M131) or flower vertically.
13877		Goddess, seated, knees up, with covered legs and arms, wearing a headdress of outwards waving plumes, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
13878		Goddess, seated, knees up, with covered legs and arms, with the head of a snake, holding a stem of papyrus with a bud (M13).
1387A		Goddess, seated, knees up, with covered legs and arms, with the head of a snake.
1387B		Goddess, seated, knees up, with covered legs and arms, with the head of a snake, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1387C		Goddess, seated, knees up, with covered legs and arms, with the head of a snake, holding a knife (T30A) at an forward angle.
1387D		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), and an uraeus.
1387E		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102) and the vulture headdress, holding a stem of papyrus with a bud (M131) or flower vertically.
1387F		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a holding a stem of papyrus with a bud (M131) or flower vertically.
13880		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13881		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a holding a sceptre (S42), vertically.
13882		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a club used by washer-men for beating laundry (U36) vertically.
13883		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a knife (T30A), angling forward.
13884		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a stem of papyrus with a bud (M13).
13885		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a tie or strap, used with sandals (ankh-sign, S34), vertically at the base.
13886		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a chisel (U22).
13888		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a sistrum, with the top piece in the form of a shrine, with the human face on the handle having hair (Y28)
13889		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding  sistrum, with horizontal wavy lines over the top piece (Y18).
1388A		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a string wound on a stick (V24) vertically.
1388B		Goddess, seated, knees up, with covered legs, with a headdress of bovine horns with a sun disk (F102), with one visible arm, extended forwards, holding a flagellum (S45).
1388C		Goddess, seated, both knees down, with a headdress of bovine horns with a sun disk (F102), both arms forward, right hand at the hight of the shoulder, holding a tambourine, left hand on the tambourine.
1388D		Goddes, standing, with a headdress of bovine horns with a sun disk (F102), both arms forward, right hand at the hight of the shoulder, holding a tambourine, left hand on the tambourine.
1388E		Goddess, seated, both knees down, with a headdress of bovine horns with a sun disk (F102), nursing a child, seated on her lap, left arm forward, hand over the chest of the goddess, right arm hanging beside the body, right arm forward, embracing/holding the child, left arm over the chest.
1388F		Woman, seated, both knees down wearing a headdress consisting of a sundisk between bovid-horns, with a seat on top of the sun disk, nursing a child, seated on her lap, left arm forward, hand over the chest of the woman, right arm hanging beside the body, right arm forward, embracing/holding the child, left arm over the chest.
13890		Goddess, standing, with a headdress of bovine horns with a sun disk (F102), right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
13891		Goddess, standing, with a headdress of bovine horns with a sun disk (F102), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13892		Goddess, standing, with a headdress of bovine horns with a sun disk (F102), right arm forward, hand at the hight of the waist, holding a crook (S38) of the size of the king vertically, opening outwards, left arm hanging beside the body.
13893		Goddess, standing, with a headdress of bovine horns with a sun disk (F102), with a seat (Q1) on the sun disk, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13894		Goddess, seated on a block throne, wearing a headdress of bovine horns with a sun disk (F102), without visible arms.
13895		Goddess, seated on a block throne with a base, wearing a headdress of bovine horns with a sun disk (F102), right arm forward, hand ad the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the length of the seated figure, vertically, left arm forward, hand upon the knee.
13896		Goddess, seated on a block throne with a base, with a seat (Q1) on her head, right arm forward, hand ad the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the length of the seated figure, vertically, left arm forward, hand upon the knee.
13897		Goddess, seated on a block throne with a base, wearing a headdress of bovine horns with a sun disk (F102), with a seat (Q1) on the sun disk, right arm forward, hand at the hight of the shoulder, holding a tie or strap, used with sandals (ankh-sign, S34), horizontally at the base, left arm forward, hand upon the knee, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13899		Goddess, seated on a block throne with a base, wearing a headdress of bovine horns with a sun disk (F102), with a seat (Q1) on the sun disk, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the length of the seated figure, vertically, left arm forward, hand upon the knee, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
1389B		Goddess, seated on a block throne with a base, wearing a headdress of bovine horns with a sun disk (F102), right arm forward, hand at the hight of the shoulder, holding a tie or strap, used with sandals (ankh-sign, S34), horizontally at the base, left arm forward, hand upon the knee, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
1389C		Goddess, seated on a block throne with a base, wearing a headdress of bovine horns with a sun disk (F102), right arm forward, hand ad the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the length of the seated figure, vertically, left arm forward, hand above the hight of the bud of the papyrus, holding a tie or strap, used with sandals (ankh-sign, S34), horizontally at the base.
1389D		Goddess, seated, knees up, with covered legs and arms, with a seat (Q1) on her head.
1389E		Goddess, seated, knees up, with covered legs and arms, with a seat (Q1) on her head, holding a stem of papyrus with a bud (M131) or flower vertically.
1389F		Goddess, seated, knees up, with covered legs and arms, with a seat (Q1) on her head, holding a holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
138A0		Goddess, seated, knees up, with covered legs and arms, with a seat (Q1) on her head, holding a stem of papyrus with a bud (M13).
138A1		Goddess, seated, knees up, with covered legs and arms, with a seat (Q1) on her head, holding a two-barbed arrow-head, with the point facing downwards, in the ground, with only the barbs visible (T22A).
138A4		Goddess, seated, knees up, with covered legs and arms, with a seat (Q1) on her head, holding a
138A5		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102) with a seat (Q1) on the sun disk, holding a holding a stem of papyrus with a bud (M131) or flower vertically.
138A6		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102) with a seat (Q1) on the sun disk, holding a stem of papyrus with a bud (M13).
138A7		Goddess, seated, both knees down, back bend forward, with a seat (Q1) on her head, and a band of cloth around her forehead, both arms forward, hands at the hight of the waist.
138A8		Goddess, standing, with a seat (Q3) on her head, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
138A9		Goddess, standing, wearing a vulture headdress, with a seat (Q3) on her head, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
138AA		Goddess, standing, with a sun disk (N5) on her head, arms covered by wings, right arm raised in front, hand at the hight of the shoulder, holding a feather (H6), left arm extended forward, hand at the hight of the waist, holding a feather (H6).
138AB		Goddess, standing, with a sun disk (N5) on her head, arms covered by wings, right arm raised in front, hand at the hight of the shoulder, holding a feather (H6), left arm extended forward, hand at the hight of the waist, holding a flagellum (S45) outwards.
138AC		Goddess, standing, with a headdress of bovine horns with a sun disk (F102), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), on top of a collar of beads (S12).
138AD		Goddess, seated, knees up, with covered legs and arms, with a headdress of bovine horns with a sun disk (F102), holding a holding a stem of papyrus with a bud (M131) or flower vertically, on top of a collar of beads, without detail (S12A).
138AE		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness.
138AF		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, holding a stem of papyrus with a bud (M13).
138B0		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, holdign a flagellum (S45).
138B2		God, seated, knees up, with covered legs and arms, with the head of a lion, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
138B4		God, seated, knees up, with covered legs and arms, with the head of a lion, holding a knife (T30A), angled forwards.
138B6		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
138B7		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head, holding a stem of papyrus with a bud (M13).
138B8		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with a plan of a crossroads in a village (O49) on her head, holding a stem of papyrus with a bud (M13).
138B9		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head, holding a tie or strap, used with sandals (ankh-sign, S34), angled forwards.
138BA		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head, holding a holding a stem of papyrus with a bud (M131) or flower vertically.
138BB		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with a sun disk (N5) on her head, holding a holding a stem of papyrus with a bud (M131) or flower vertically.
138BE		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, holding a stem of papyrus with a bud (M13), on top of a stairway (O40).
138BF		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with two feathers (H6) on her head, forming a v shape, holding a stem of papyrus with a bud (M13)
138C0		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with two feathers (H6) on her head, forming a v shape.
138C1		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with two feathers (H6) on her head, forming a v shape, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
138C2		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness, with a feather (H6) on her head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
138C3		Goddess, seated, knees up, with covered legs and arms, with the head of a lion/lioness,  with the head of a cobra on her head.
138C4		Goddess, standing, with the head of a lion/lioness, arms hanging on either side of the body, handpalms towards the back.
138C5		Goddess, standing, with the head of a lion/lioness, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head, right arm raised in front, hand at the hight of the face, handpalm outwards, left arm hanging beside the body.
138C6		Goddess, standing, with the head of a lion/lioness, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
138C7		God, standing, with the head of a lion, , right arm extended forward, right hand at the hight of the shoulder, holding a sealed perfume jar with handles, and strings (W27), left arm forward, hand at the hight of the waist, holding a sealed ointment jar without ties (W2)
138C8		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding a stem of papyrus with a bud (M131) or flower vertically.
138C9		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding a crook (S38), vertically, with the opening outwards.
138CA		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding a feather (H6), vertically.
138CB		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding the head and neck of a canine animal (F12), vertically.
138CC		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding two maces (T3), both angled forward.
138CD		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
138CE		Goddess, standing, with a cloth band around her head, with a feather (H6) on her head, arms hanging beside the body, handpalms towards the back.
138CF		Goddess, standing, with a cloth band around her head, with a feather (H6) on her head, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34).
138D0		Goddess, standing, with a cloth band around her head, with a feather (H6) on her head, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), on top of a canal (N36).
138D1		Goddess, standing, with a cloth band around her head, with a feather (H6) on her head, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), on top of a platform (Aa11).
138D2		Goddess, seated on a block throne, with a cloth band around her head, with a feather (H6) on her head, right arm forward, hand at the hight of the waist, holding the head and neck of a canine animal (F12) of the length of the figure, vertically, left arm forward, hand on the knee, holding a tie or strap, used with sandals (ankh-sign, S34), angled forwards.
138D3		Goddess, seated on a block throne, with a cloth band around her head, with a feather (H6) on her head, right arm forward, hand at the hight of the waist, holding the head and neck of a canine animal (F12) of the length of the figure, vertically, left arm forward, hand on the knee, holding a tie or strap, used with sandals (ankh-sign, S34), horizontally at the loop.
138D4		Goddess, standing, with a cloth band around her head, with a feather (H6) on her head, right arm forward, hand at the hight of the waist, holding the head and neck of a canine animal (F12) of the length of the figure, left arm hanging beside the body.
138D5		Goddess, seated, knees up, with covered legs and arms, with a cloth band around her head, with a feather (H6) on her head, holding a feather (H6), vertically, on top of a gateway (O32B), with a conical loaf of bread (X8) written inside.
138D6		Goddess, seated, knees up, with covered legs and arms, with two feathers (H6) on her head, in a v shape, holding a stem of papyrus with a bud (M13).
138D7		Goddess, seated, both knees down, with long hair, with a clump of three papyrus flowers (M16A) on her head, arms raised in front, hand horizontally at the hight of the shoulder, hand palm up.
138D8		Goddess, seated, both knees down, with long hair, with a clump of three papyrus flowers, with two buds bent down (M15) on her head, arms raised in front, hand horizontally at the hight of the shoulder, hand palm up.
138D9		Goddess, seated, one knee raised, with long hair, with a clump of three papyrus flowers, with two buds bent down (M15) on her head, arms raised in front, hand horizontally at the hight of the shoulder, hand palm up.
138DA		Goddess, seated, both knees down, with long hair, with a clump of three papyrus flowers (M16A) on her head, both arm raised in front, handpalms outwards.
138DC		Goddess, standing, with long hair, with a clump of three papyrus flowers (M16A) on her head, arms raised in front, hand horizontally at the hight of the shoulder, hand palm up.
138DD		Goddess, standing, with long hair, with a clump of three papyrus flowers (M16A) on her head, arms raised in front, hand horizontally at the hight of the shoulder, hand palm up, on top of a collar of beads (S12).
138DE		Goddess, seated, knees up, with covered legs and arms, wearing the double crown (S5) and a vulture headdress.
138DF		Goddess, seated, knees up, with covered legs and arms, wearing the double crown (S5) and a vulture headdress, holding a stem of papyrus with a bud (M131) or flower vertically.
138E0		Goddess, seated, knees up, with covered legs and arms, wearing the double crown (S5), holding a stem of papyrus with a bud (M131) or flower vertically.
138E1		Goddess, standing, wearing the double crown (S5), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34)
138E2		Goddess, standing, wearing the double crown (S5), right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
138E3		Goddess, seated on a block throne on a base, wearing the double crown (S5) and vulture headdress, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40), which has a horizontal tie or strap, used with sandals (ankh-sign, S34) at the head of the sceptre, left arm forward, hand on knee.
138E4		Goddess, seated on a block throne on a base, wearing the double crown (S5) and vulture headdress, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm forward, hand on knee.
138E5		Goddess, seated, knees up, with covered legs and arms, wearing the red crown (S3).
138E6		Goddess, seated, both knees down, wearing the red crown (S3), no visible arms.
138E7		Goddess, seated, knees up, with covered legs and arms, wearing the red crown (S3), holding a stem of papyrus with a bud (M131) or flower vertically.
138E8		Goddess, seated on a block throne on a base, wearing the red crown (S3), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40), left arm forward, hand on knee.
138E9		Goddess, seated, knees up, with covered legs and arms, wearing the red crown (S3), holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically, with one arrow, fletching upwards in front of the sceptre, and two arrows, fletching upward behind the scepter.
138EA		Goddess, standing, wearing the red crown, left arm forward, forearm horizontal, hand at the hight of the waist, holding an eye with the markings of the head of a falcon (D10), right arm forward, hand above the eye.
138EB		Goddess, standing, wearing the red crown (S3), right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
138EC		Goddess, standing, wearing the red crown (S3), right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
138ED		Goddess, standing, wearing the red crown (S3), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34).
138EE		Goddess, seated, knees up, with covered legs and arms, with two crossed arrows, fletching towards the back, on top of a standard resembling a half circle on top of a long and short stick (T61) on her head.
138EF		Goddess, standing, wearing the red crown, right arm forward, hand at the hight of the waist, holding a griffon vulture (Gyps fulvus, G14), left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34).
138F0		Goddess, seated, knees up, with covered legs and arms, with a enclosed hill, with a shrub at the top (N30A) on top of a conical loaf of bread (X8) on her head, holding a stem of papyrus with a bud (M131) or flower vertically.
138F1		Goddess, seated, knees up, with covered legs and arms, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head.
138F2		Goddess, seated, knees up, with covered legs and arms, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, holding a stem of papyrus with a bud (M131) or flower vertically.
138F3		Goddess, seated, knees up, with covered legs and arms, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, holding a two-barbed arrow-head, with the point facing downwards, in the ground, with only the barbs visible (T22A).
138F4		Goddess, standing, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, arms hanging on either side of the body.
138F5		Goddess, seated, knees up, with covered legs and arms, with round vessel with upstanding rim (W24) on her head, holding a stem of papyrus with a bud (M131) or flower vertically.
138F6		Goddess, seated, knees up, with covered legs and arms, with round vessel with upstanding rim (W24) on her head, holding a stem of papyrus with a bud (M13).
138F7		Goddess, seated, knees up, with covered legs and arms, with round vessel with upstanding rim (W24) on her head.
138F8		Goddess, standing, with round vessel with upstanding rim (W24) on her head, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
138F9		Goddess, standing, with a circle on her head, arms hanging beside either side of the body.
138FA		Goddess, seated, knees up, with covered legs and arms, with a  plan of a crossroads in a village (O49) on her head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
138FB		Goddess, seated, knees up, with covered legs and arms, with a  plan of a crossroads in a village (O49) on her head.
138FC		Goddess, standing, back horizontal, arms vertical, hands  at the same hight as the toes, head between the two arms.
138FD		Goddess, standing, back horizontal, arms vertical, hands  at the same hight as the toes.
138FE		Goddess, standing, back horizontal, arms vertical, hands  at the same hight as the toes, on top of an a strip of land (N17), with a child, seated on nothing, forelegs separated, right arm raised with hand to mouth, left arm hanging beside the body (A17) beside a sun disk with rays of sunlight coming forth from it (N8), written between the arms and legs.
138FF		Goddess, seated, knees up, with covered arms and legs, wearing a headdress consisting of the white crown and two bovid horns, holding a stem of papyrus with a bud (M131) or flower vertically.
13900		Goddess, seated, knees up, with covered arms and legs, with a flower enclosed by two horns (R20) on her head.
13902		Goddess, standing, with a flower enclosed by two horns (R20) on her head, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
13903		Goddess, standing, with a flower enclosed by two horns (R20) on her head, right arm forward, hand at the hight of the waist, holding a club, angled forward, with the head of the club on top a vertical stick held by the left hand.
13904		Goddess, standing, with a flower enclosed by two horns (R20) on her head, right arm forward, hand at the hight of the waist, holding a club, angled forward, with the head of the club on top a vertical stick held by the left hand, facing a god, standing, with a long curved beard, wearing the Atef crown with horns,  right arm forward, hand at the hight of the waist, holding a club, angled forward, with the head of the club on top a vertical stick held by the left hand, both sticks within an oval of rope.
13905		Goddess, standing, with a star (N14) on her head, both arms hanging beside the body.
13907		Goddess, seated, knees up, with covered arms and legs, with the head of a bovid.
13908		Goddess, standing, with the head of a bovid, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop.
13909		Goddess, seated, knees up, with covered arms and legs, with the pole of a balance, resembling a column with a base, with a tenon at the top (O28B), with a plummet resembling a heart (F34) hanging from the cross-beam (U39N) on her head.
1390A		Goddess, standing, with a headdress consisting of two feathers on rams horns (S77) on her head, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
1390B		Goddess, standing, with a headdress consisting of two feathers on rams horns (S77) on her head, right arm forward, hand at the hight of the waist, holding stick of the hight of the woman, vertically, left arm hanging beside the body.
1390C		Goddess, standing, with a headdress consisting of two feathers on rams horns (S77) on her head, right arm forward, hand at the hight of the waist, holding stick of the hight of the woman, vertically, left arm hanging beside the body, holding a flagellum (S45) horizontally.
1390D		Goddess, standing, with a headdress consisting of two feathers on rams horns (S77) on her head, right arm forward, hand at the hight of the waist, holding crook (S38) of the hight of the woman, vertically, left arm hanging beside the body.
1390E		Goddess, standing, with a headdress consisting of two feathers on rams horns (S77) on her head, right arm forward, hand at the hight of the waist, holding crook (S38) of the hight of the woman, vertically, left arm hanging beside the body, holding a flagellum (S45) horizontally.
1390F		Goddess, standing, with a headdress consisting of two feathers with a sun disk on rams horns (S73) on her head, right arm forward, hand at the hight of the waist, holding crook (S38) of the hight of the woman, vertically, left arm hanging beside the body.
13910		Goddess, seated, knees up, with covered arms and legs, with the head of a desert hare.
13911		Goddess, seated, knees up, with covered arms and legs, with the head of a desert hare, holding a stem of papyrus with a bud (M131) or flower vertically.
13912		Goddess, seated, knees up, with covered arms and legs, with the head of a desert hare, wearing the red crown (S3), holding a stem of papyrus with a bud (M131) or flower vertically.
13913		Goddess, standing, with the head of a desert hare, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
13914		Goddess, standing, with the head of a desert hare, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34).
13915		Goddess, seated, knees up, with covered arms and legs, wearing the Atef crown with horns, holding a tie or strap, used with sandals (ankh-sign, S34), vertically.
13916		Goddess, seated, knees up, with covered arms and legs, with long hair, with a fish, with its tail orientated downwards (K4A) on her head.
13917		Goddess, seated, knees up, with covered arms and legs, with long hair, with a fish, with its tail orientated downwards (K4A) on her head, holding a stem of papyrus with a bud (M131) or flower vertically.
13918		Goddess, seated, knees up, with covered arms and legs, with the head of a frog.
13919		Goddess, seated, knees up, with covered arms and legs, with the head of a frog, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
1391A		Goddess, seated, knees up, with covered arms and legs, with a feather (H6) angled forwards and a falcon (G5) on her head, holding a feather (H6) horizontally.
1391B		Goddess, seated, knees up, with covered arms and legs, with a scorpion (L19) on her head, holding a stem of papyrus with a bud (M13).
1391C		Goddess, seated, knees up, with covered arms and legs, with long hair, with a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal, with a fillet and feather (R19), on top of a canal (N36) on her head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward.
1391D		Goddess, standing, with a palm branch, stripped of leaves and notched on her head (M4), notch forward, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the hight of the woman, vertically, left arm hanging beside the body.
1391E		Goddess, seated, knees up, with covered arms and legs, with long hair, with a crescent moon with the entire moon disk (N62) on her head .
1391F		Woman/goddess, standing, right arm forward, hand at the hight of the waist, holding a duck or goose at the wings, left arm hanging beside the body.
13920		Goddess, seated, knees up, with covered arms and legs, with long hair, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head.
13921		Goddess, seated, knees up, with covered arms and legs, with long hair, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on her head, holding a stem of papyrus with a bud (M13).
13922		Goddess, seated, knees up, with covered arms an legs, with long hair, with a rectangular piece of cloth, with fringes on one of the short sides (S32) on her head, holding a sceptre with a straight shaft, topped with the head of the Seth animal vertically.
13923		Goddess, standing, with a seat (Q3) on her head, both arm forward, facing a goddess, standing, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, both arm forward, both goddesses have one hand above and one hand below a sun disk with rays of sunlight coming forth from it (N8), with a star written below the arms.
13925		Goddess, standing, with a seat (Q3) on her head, both arm forward, facing a goddess, standing, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, both arm forward, both goddesses have one hand above and one hand below a sun disk with rays of sunlight coming forth from it (N8).
13926		Goddess, standing, with a seat (Q3) on her head, both arm forward, facing a goddess, standing, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, both arm forward, both goddesses have one hand above and one hand below a sun disk with rays of sunlight coming forth from it (N8), with a spear made into a standard (R15) written below the arms.
13927		Goddess, standing, with a seat (Q3) on her head, both arm forward, facing a goddess, standing, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, both arm forward, both goddesses have one hand above and one hand below a sun disk (N5).
13928		Goddess, standing, with a seat (Q3) on her head, both arm forward, facing a goddess, standing, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, both arm forward, one hand below a sun disk (N5), one hand lower, with a horizontal line.
13929		Goddess, seated, knees up, with covered legs and arms, with a seat (Q1) on her head, facing a goddess, seated, knees up, with covered legs and arms, with a basket (V30) on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6) on her head, with a group of a sand covered mountain over the edge of the cultivated areas (N26) sign and two half round loafs of bread (X1) written above their knees.
1392A		Four women/goddesses, standing, first two looking towards the back, last two looking forwards, arms raised, supporting the sky (N1).
1392B		Goddesss, seated on a block throne, wearing a headdress consisting of the double crown on horns with a feather in the front, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the length of the seated figure, vertically, left arm forward, hand upon the knee, facing a god, seated on a block throne, with the head of a falcon, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on his head, right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40), of the length of the seated figure, vertically, left arm forward, hand upon the knee, both on top of a base.
1392C		Goddesss, seated on a block throne, wearing a headdress consisting of the double crown on horns, right arm forward, hand at the hight of the waist, holding a stem of papyrus with a bud (M131) or flower, of the length of the seated figure, vertically, left arm forward, hand upon the knee, facing a god, seated on a block throne, with the head of a falcon, wearing the double crown (S5), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40), of the length of the seated figure, vertically, left arm forward, hand upon the knee, both on top of a base.
1392D		The head of a human woman, seen in profile.
1392E		The head of a human man (foreigner), seen in profile, with a band of cloth around the head, with a bushy beard (or rather distinctive chin).
1392F		The head of a human man, seen in profile, with the mouth open.
13930		The head of a human man, seen in profile, with a line coming from the mouth, angling downwards.
13931		The head of a human man, seen in profile, with the  uterus of a cow (F45) on his head.
13932		The head of a goddess, seen in profile, wearing a headdress of bovine horns with a sun disk (F102), wearing the vulture headdress, on top of a staff with a forked end.
13933		The head of a goddess, seen in profile, wearing a headdress of bovine horns with a sun disk (F102), wearing the vulture headdress, on top of a staff.
13934		The head of a human man, seen in profile, with the horns of a bovid (F13) on top of the head.
13935		The face of a human female, with the ears of an ox.
13936		The face of a human female, with hair in two curling sidelocks.
13937		The head of human male in profile, with two arms, on raised in front, forarm at 45°, hand at the hight of the face, handpalm outwards, other arm with the hand below the head.
13938		The head of human male in profile, with two arms, on raised in front, forarm at 45°, hand at the hight of the face, other arm with the hand below the head, both hands as fists.
13939		The head of human male in profile, with two arms, on raised in front, hand at the hight of the forehead, handpalm outwards, other arm hanging downwards.
1393A		The head of human woman in profile, with two arms, on raised in front, hand at the hight of the forehead, handpalm outwards, other arm hanging downwards.
1393D		The head of a human male in profile, with two arms, one raised in front, hand to mouth, other arm hanging downwards, handpalm facing outwards.
1393E		The head of a human male in profile, with one arm, raised in front, hand to mouth.
1393F		The head of a human male in profile, with a sidelock, with two arms, one raised in front, hand to mouth, other arm hanging downwards, handpalm facing outwards.
13940		The head of a human male in profile, with two arms, one hanging downwards in front, handpalm inwards, other arm with the hand below the head, handpalm upwards.
13941		The head of a human male in profile, with two arms, both arms hanging downwards.
13942		The head of a human male in profile, with two arms, both arms forward, hands horizontal at the hight of the face, handpalms inwards.
13943		The head of a human male in profile, with two arms, both arms extended forwards, at a downwards angle.
13944		The head of a human male in profile, with one arm, raised in front, forearm vertical, hand horizontal, handpalm upwards.
13945		The head on a human male in profile, with one arm, raised at the back, forearm vertical, handpalm inwards.
13946		The head on a human male in profile, with two arms, raised at the back, forearm vertical, handpalm inwards.
13948		The head of a human male in profile, with two arms, arms raised at either side, upper arms horizontal.
13949		The head of a human male in profile, with beard and coif, with two arms, arms raised at either side, upper arms horizontal.
1394A		The head of a human male in profile, with two arms, arms raised at either side, upper arms horizontal, upside down.
1394B		The head of a human male in profile, with two arms, one raisde in front, hand at the hight of the shoulder, upper arm of the other on top of an horizontal line, facing backwards.
1394D		The head of human male in profile, with two arms, in a swimming motion, on top of a canal, with drops of liquid above the head and arms.
1394F		The head of a human male in profile, with two arms, one arm forward, forearm horizontal, holding a stick, other arm arm hanging downwards.
13950		The head of a human male in profile, with two arms, one arm forward, hand at the hight of the shoulder, holding a feather (H6), other arm hanging downwards.
13951		The head of a human male in profile, with two arms, one arm forward, hand at the hight of the shoulder, holding a sceptre (S42), other arm hanging downwards, holding a basket at the handle.
13952		The head of a human male in profile, with two arms, one arm forward, hand at the hight of the shoulder, holding a feather (H6) three triangles of skin with a tail (F27A), other arm hanging downwards.
13954		The head of a human male in profile, with two arms, one arm forward, hand at the hight of the shoulder, holding a staff or pole with a two-pronged end piece for holding down snakes (U116), other arm hanging downwards.
13955		The head of a human male in profile, with a long curved beard an coif, one arm forward, hand at the hight of the shoulder, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40), other arm hanging downwards.
13956		The head of a human male in profile, with two arms, one arm forward, hand to mouth, with a dotted line coming from the mouth, angling downwards, other arm hanging downwards, handpalm outwards.
13957		The head of a human male in profile, with a vessel on his head, with two arms, one arm forward, hand at the vessel, other arm hanging downwards.
13958		The head of a human male in profile, with two arms, one arm forwards, hanging downwards, other arm raised at the back, forearm vertical, holding a stick.
13959		The head of a human male in profile, with two arms, one arm forwards,forearm backwards, hand under the head, other arm raised at the back, forearm vertical, holding a stick.
1395A		The head of a human male in profile, with two arms, one arm forward, angling downwards, holding a phallus with a scrotum (D52) horizontally, other arm extended towards the back, forearm upwards.
1395B		The head of a human male in profile, with two arms, one arm extended forwards, forearm horizontal, handpalm upwards, other raised in back, forearm vertical, handpalm inwards, with a dotted line between the hands.
1395D		The head of a human male in profile, with two arms, one arm extended forwards, forearm horizontal, holding a stick, other raised in back, forearm vertical, holding a stick.
1395E		The head of a human male in profile, with one arm, extended forward, hand at the hight of the shoulder, holding some type of weapon/mace/stick.
1395F		The head of a human male in profile, with one arm, extended forward, hand at the hight of the shoulder, holding a vessel horizontally to the mouth.
13960		The head of a human male in profile, with one arm, forward, forearm horizontal, holding a bundle of three papyrus flowers.
13962		The head of a human male in profile, with two arms, extended forwards, forearms separated, holding a stick.
13963		The head of a human male in profile, with two arms, extended forwards, forearms separated, holding a pestle in a mortar (U32).
13964		The head of a human male in profile, with two arms, extended forwards, holding a stick with a bundle or a mat horizontally over his shoulders, bundle behind the back.
13965		The head of a human male in profile, with two arms, one arm forward, upper arm horizontal, hand on top of a lump of clay on a potters wheel, other arm forward, hand at the potters wheel.
13966		The head of a human male in profile, with two arms, both arms forwards, hands touching a vertical wall with battlements (O36C) near the top and the bottom.
13967		The head of a human male in profile, with two arms, both arm forward, holding a reed pipe ending in a tuyère at his mouth.
13969		The head of a human male in profile, with two arms, both hands below the head, holding a stick angling over the back shoulder.
1396A		A lock of hair with three strands, with curly endings.
1396B		A lock of hair with three strands, with curly endings (D3A), with a knife with a triangular blade and straight handle (T30A) written over the rightmost strand.
1396D		A lock of hair, resembling a half circle, with three strands of hair with a curly ending, starting in the middle, to the back.
1396E		A lock of hair with four wavy strands.
1396F		A lock of hair with three vertical strands, and multiple short horizontal strands at the front.
13970		A lock of hair, resembling a half circle, with three strands of hair with a curly ending, one left, one middle, one right.
13971		An oval lock of hair with 3 strands
13972		A braided hairlock.
13973		Two eyes, as seen from the front.
13974		An eye without pupil.
13975		An eye (D4), written on top of a butcher's block (T28).
13976		An eye, with a rippled line of eye-paint above the eye.
13977		An eye with a painted lower lid. And two lines at the top forming a V shape.
13978		An eye with three touches of eye paint above the eye, without pupil.
13979		An eye with many touches of eye paint above the eye.
1397A		An eye with an eyebrow above it.
1397B		Two eyes, as seen from the front, with eyebrows above them.
1397C		An closed eye.
1397D		An eye with and eyebrow above it, upside down.
1397E		An eye with two touches of eye paint above the eye, and a painted lower lid.
1397F		A closed/half closed eye.
13980		An eye, with a line of eye-paint above the eye, and a painted lower lid.
13982		An eye, with a wavy line of eye-paint above the eye, inside an oval.
13984		An eye with an eyebrow above it, upside down, inside an oval.
13985		An eye, with with two vertical lines below it, each starting at the white of the eye.
13986		An eye with two touches of eye paint above the eye, with three lines coming from the bottom.
13987		An eye, with a line of eye-paint above the eye, with three lines coming from the bottom.
13988		An eye, with a rippled line of eye-paint above the eye, with three lines coming from the bottom.
13989		An eye with three vertical lines of overlapping triangles coming from the bottom.
1398A		An eye with a line of eye paint above the eye, with three vertical lines of overlapping triangles coming from the bottom.
1398B		An eye, with a rippled line of eye-paint above the eye, with three vertical lines of overlapping triangles coming from the bottom.
1398C		An eye, with the markings of the head of a falcon, with a vertical line at the back with a triangle at the curl.
1398D		An eye without a pupil, with the markings of the head of a falcon, with a vertical line at the back with a triangle at the curl.
1398E		An eye, with the markings of the head of a falcon, with a vertical line at the back with a triangle at the curl (D10A), written inside a circle.
1398F		Two circles representing the eyes or pupils.
13990		The hearpiece of human.
13991		The nose, eye, mouth and cheek of a human.
13992		The head of a human in profile, with a breast (D27) at the mouth.
13994		A mouth with one vertical line at the bottom of the sign.
13995		A mouth with two vertical lines at the bottom of the sign, the first longer than the second.
13996		A human mouth, seen from the side.
13997		The upper lip of a human with teeth, represented as a half circle.
13998		The upper and lower lip of a human, with teeth, represented as a half circle.
13999		A mouth with a downwards sloping dotted line above the front short end.
1399A		A mouth with a downwards sloping line above the front short end.
1399B		A mouth with a downwards sloping line at the front short end.
1399C		A human mouth, seen from the side, with a downwards sloping line coming from the lips.
1399D		A human mouth, seen from the side, with a dotted downwards sloping line coming from the lips.
1399E		A long beard, curling up.
1399F		The lower half of a human, seated on heel, both knees down.
139A0		Two arms, raised upwards, with the forearms and hands vertical, represented by circles.
139A1		Two arms, unconnected, raised upwards, with the forearms and hands vertical, handpalms inwards.
139A2		Two arms, raised upwards, with the forearms and hands vertical, handpalms inwards, on top of a base, which might represent a torso.
139A3		Two arms, raised upwards, with the forearms and hands vertical, handpalms inwards, with a sedge (M23) written between the arms, upon a standard used for carrying religious symbols (R12).
139A4		A roll of bread, with oval lines within it (HG X4A), written horizontally with two forearms coming from the top of the bread, forearms and hands vertical, handpalms inwards, with a vertical stroke (Z1) written between the arms.
139A5		Two arms, raised upwards, with the forearms and hands vertical, handpalms inwards, on top of a roll of bread without internal detail.
139A6		Two arms, lowered, arms straight, handpalms facing towards each other.
139A7		Two arms, lowered, with the elbows bent inwards, palms of the hands oriented outwards.
139A9		Two arms, lowered, arms straight, handpalm oriented outwards.
139AA		Two arms, lowered, with the elbows bent outwards, and the palms of the hands facing towards each other (D32), with a stool made out of reed matting (Q3) written between the hands.
139AB		Two arms, lowered, with the elbows bent outwards, and the palms of the hands facing towards each other (D32), with a vertical line of dots written between the arms.
139AC		Two arms, lowered, not connected at the shoulders, with the elbows bent outwards, the palms of the handds facing towards each other.
139AD		Two arms, lowered, not connected at the shoulders, with the elbows bent outwards, the palms of the handds facing towards each other (D197), with a line of angled strokes between the arms.
139AE		Two arms, lowered, not connected at the shoulders, with the elbows bent outwards, the palms of the handds facing towards each other (D197), with a club used by washer-men for beating laundry (U36) written between the arms.
139AF		Two arms, lowered, not connected at the shoulders, elbows bent inwards, connected by a horizontal line, arms making a X shape, handpalms inwards.
139B0		Two arms, holding an oar, in a rowing motion, below a cover resembling the mouth (D21).
139B1		Two arms, holding an oar, in a rowing motion, with the upper arms overlapping.
139B2		Two arms, elbows bent, upper arms overlapping, forearms separated, lower forearm nearly horizontal, holding an oar which tapers out towards the bottom.
139B3		Two arms, holding an oar at the blade, below a cover resembling a ring-stand for jars (W11).
139B4		The torso of a man, right arm downwards, forearm horizontal over the waist, left arm raised, both arms holding an oar.
139B5		A forearm, with the palm of the hand facing upwards, with an oar at an backwards angle, with a oval blade, written with the handle over the hand.
139B6		A forearm, with the palm of the hand facing upwards, with an oar at an forwards angle, with a rectangular blade, written with the handle over the hand.
139B7		Two arms, connected at the shoulders, one arm forwards, with a horizontal forearm, holding a round topped shield, other arm downwards, forearm vertical, holding a mace (T3) horizontally.
139B8		Two arms, connected at the shoulders, one arm forwards, with a horizontal forearm, holding a triangular shield with rounded top, other arm downwards, forearm angled slightly forwards, holding a mace (T3) at an upwards angle.
139B9		Two arms, connected at the shoulders, one arm forwards, with a horizontal forearm, holding a triangular shield with rounded top, other arm downwards, forearm angled slightly forwards, holding a mace (T3) horizontally.
139BA		Two arms, connected at the shoulders, one arm forwards, with a horizontal forearm, holding a shield as seen in profile, top curving inwards, other arm downwards, forearm vertical, holding a mace with a pear-shaped head, with a round blade attached to the mace-head (T3C), horizontally, blade upwards.
139BC		Two arms, connected at the shoulders, one arm forwards, with a horizontal forearm, holding a oval shield with a cross-type internal decoration, other arm downwards, forearm angled slightly forwards, holding a lance or spear.
139BE		Two arms, connected at the shoulder, one arm forward, forearm straight, holding a mace (T3), other arm downwards, forearm at 90° of the upper arm, holding a stick.
139BF		Two arms, connected at the shoulder, one arm forward, forearm horizontal, holing a wand or brush, other arm downwards, forearm at 90° of the upper arm, holding a sceptre (S42).
139C0		Two arms, connected at the shoulder, one arm forward, holding a round shield, seen in profile, other arm downwards, forearm at 90° of the upper arm, holding a mace (T3) vertically.
139C1		The torso of a man, right arm forwards, forearm horizontal, holing a bow, left arm in front of the body, forearm horizontal, holding an arrow at the fletching which lies with the arrowhead on top of the right hand.
139C2		Two arms, connected at the shoulders, spread sideways, forearms horizontal, with the palms of the hands facing downwards.
139C3		A forearm, with the palm of the hand facing upwards, and the upper arm represented by a loop.
139C4		A forearm, with the hand angled downwards.
139C5		Two forearms with the hand holding a conical loaf of bread (X8), overlapping each other.
139C6		A forearm with the hand holding an egg (H8).
139C7		A forearm with the hand held as a fist.
139C8		A forearm with the hand holding a loop of cord with the ends upwards (V6).
139C9		A forearm, with the palm of the hand downwards, with the upper arm represented by a loop, which is angled forwards.
139CA		A forearm, with the hand held like a fist, and the upper arm represented by a loop.
139CB		A forearm with the hand holding a wide cup (W10)
139CC		A forearm with the hand holding two arms, raised upwards, with the forearms and hands vertical, handpalms inwards (D28).
139CD		A forearm with the hand holding 3 vertical strokes.
139CE		A forearm with the hand holding the sun upon a butchers block (N7).
139CF		A forearm with the hand holding the sun, with thee beams of sunlight coming from it (N8E).
139D0		A forearm with the hand holding a feather (H6).
139D1		A forearm, with the palm of the hand facing upwards, with a feather (H6) on top of the hand, vertically.
139D3		A forearm with the hand holding a knife with a triangular blade and straight handle (T30A).
139D5		A forearm, with the palm of the hand facing upwards, with a forward downwards angling line coming from the hand.
139D6		A forearm, hand angled downwards, with the palm of the hand facing upwards, with a downwards curved line over the hand.
139D7		A forearm with the palm of the hand facing upwards, with a downwards curving wavy line above the arm, with the lower end ending in the hand-palm.
139DA		A forearm with the hand holding the leaf stem and root of a lotus plant (M12).
139DB		An arm, bend at the elbow, hand horizontal, handpalm upwards.
139DC		An arm, bend at the elbow, hand horizontal, handpalm upwards, with a forward, downwards angling dotted line coming from the hand.
139DD		An arm, bend at the elbow, hand horizontal, handpalm downwards.
139DE		An arm, bend at the elbow, with the hand held as a fist, orientated backwards.
139DF		An arm, bend at the elbow, upper arm horizontal, handpalm outwards.
139E0		An arm, bend at the elbow, upper arm horizontal, hand held horizontal, handpalm upwards.
139E1		An arm, bend at the elbow, forearm nearly horizontal, holding a flagellum (S45).
139E2		An arm, bend at the elbow, forearm nearly horizontal, holding a tall wand or brush, vertically.
139E6		An arm, bend at the elbow, forearm nearly horizontal, holding a sceptre (S42).
139E7		An arm, bend at the elbow, forearm nearly horizontal, holding a feather (H6).
139E8		An arm, bend at the elbow, forearm nearly horizontal, holding a wick of twisted flax, consisting of 3 loops (V28).
139E9		An arm, bend at the elbow, forearm nearly horizontal, holding a conical loaf of bread (X8).
139EA		An arm, bend at the elbow, forearm nearly horizontal, holding a neck and head of a gazelle
139EB		An arm, bend at the elbow, forearm nearly horizontal, holding a knife with a triangular blade and straight handle.
139EC		An arm, bend at the elbow, in a V shape, hand holding a stick  at the top.
139ED		An arm, bend at the elbow, in a V shape, hand holding a brush.
139EE		An arm, bend at the elbow, in a V shape, hand on top of a wall or shrine.
139EF		An arm, bend at the elbow, in a V shape, hand holding a staff or pole with a two-pronged end piece for holding down snakes (U116).
139F0		Two arms, elbows bend, upper arms overlapping, in a V shape, forearms separated, handpalms outwards.
139F1		Two arms, elbows bent, upper arms overlapping, forearms separated, lower forearm nearly horizontal, holding a wand or brush, resembling a loop.
139F2		Two arms, elbows bent, upper arms overlapping, forearms separated, lower forearm nearly horizontal, holding a wand or brush.
139F3		Two arms, elbows bent, upper arms overlapping, forearms separated, lower forearm nearly horizontal, holding a wand or brush, resembling a long fan-like object.
139F4		Two arms, elbows bent, upper arms overlapping, forearms separated, lower forearm nearly horizontal, holding a stick.
139F5		Two arms, elbows bent, upper arms overlapping, forearms separated, lower forearm nearly horizontal, holding a wick of twisted flax, consisting of 3 loops (V28).
139F6		An arm, bend at the elbow, forearm horizontal, holding a stick, facing its mirror, sticks crossing.
139F7		A human hand with depicted fingers.
139F8		A human hand with a forwards downwards line of dots coming from the thumb.
139F9		A spiral, winding counter-clockwise away from its central point, ending at the right lower corner after about 1,5 turns (Z7), written over a human hand (D46).
139FA		A human hand with the thumb upwards and the palm curved upwards, with visible fingers.
139FB		A human hand with the thumb upwards and the palm curved upwards, with a forwards, downwards line of dots coming from the handpalm.
139FC		A human hand, angled downwards, with visible fingers.
139FD		A human hand with the thumb upwards and the palm curved upwards (D47), holding a snake with the head of a ram with horizontal horns (I27).
139FE		A human hand with the thumb upwards and the palm curved upwards (D47), holding an egg (H8).
139FF		A human hand, holding an oval, with the fingers and the thumb being the same length.
13A00		A human hand, held as a fist, with the thumb on top (D49), holding a folded piece of cloth (V60).
13A01		A human hand, held as a fist, with the thumb on top (D49), holding a vertical stick.
13A02		A human hand, held as a fist, with the thumb on top (D49), holding a short vertical stick, which barely extends beyond the hand.
13A03		A human hand, held as a fist, with the thumb on top (D49), holding a loop of cord with the ends upwards (V6).
13A04		A human hand, held as a fist, with the thumb on top (D49), holding a vertical hourglass-shaped shaped document holder (R88).
13A05		A distal and intermediate phalanges of a finger, written horizontally.
13A06		Two vertical human fingers on top of a standard used for carrying religious symbols (R12).
13A07		A phallus with a scrotum, written with the tip in a vulva (N41).
13A08		A phallus with a scrotum, with a line of liquid issuing from it (D53), pouring into a wide cup (W10).
13A09		A phallus with a scrotum, with a line of liquid issuing from it (D53), pouring into a vulva (N42).
13A0A		A human scrotum with two testicles.
13A0B		A testicle.
13A0C		A scrotum with two testicles.
13A0D		A vulva with a triangular shape at the bottom.
13A0F		A well with three lines coming from the top.
13A10		The uterus of a cow, represented as two separate vertical lines with a curl on the top.
13A11		The uterus of a cow, represented as two separate vertical lines with a curl on the top, on top of a half-circle.
13A12		Legs in a walking posture, feet orientated towards the reading direction, back foot only touches the baseline with the toes.
13A13		Legs in a walking posture, rotated 90° forward, feet towards the bottom.
13A14		Hieratic representation of legs in a walking posture, feet orientated away from the reading direction (D55).
13A15		Two legs, spread out, knees bent, toes orientated away from each other.
13A16		Legs in a walking posture, upside down, toes towards the reading direction.
13A17		Legs in a walking posture, feet orientated towards the reading direction (D54), with a mouth (D21) on top of it.
13A18		Legs in a walking posture, feet orientated towards the reading direction (D54), with a half round loaf of bread (X1) on top of it.
13A19		A leg with a bend knee, with the upper leg written horizontally.
13A1A		A leg with a bend knee (D56), with a knife with a triangular blade and straight handle (T30A) written over it, blade downwards.
13A1B		A leg with a bend knee (D56), written with the upper leg over a knife with a triangular blade and straight handle, orientated with the blade upwards and the handle forwards, at a slight angle.
13A1C		A leg with a bend knee (D56), written with the upper leg over a knife with a triangular blade and straight handle, orientated with the blade downwards and the handle forwards, at a slight angle.
13A1D		A human foot and lower leg (D58), with a forward, downwards dotted line coming from the top.
13A1E		A human foot and lower leg (D58), with a backwards, downwards dotted line coming from the top.
13A1F		A human foot and lower leg (D58), with a vase on its side, with liquid issuing from it (W54) on top of it, with the line of liquid ending on top of a canal (N36).
13A20		A fire-drill in a piece of wood (U28), written on top of a human foot and lower leg (D58).
13A21		A stylised set of two toes, resembling a cloth wound on a pole, an emblem of divinity (R8), on top of a base resembling a tusk of an elephant (F18), with two lines connecting the two emblems, the top line horizontal, the bottom line at an angle, connecting to the top line at the second emblem.
13A22		A stylised set of two toes, resembling a harpoon-head with two horizontal strokes and a single curl on top of the point (HG T19), on top of base resembling the left side of the sky (mirror N44).
13A23		A stylised set of two toes, resembling a harpoon-head with two horizontal strokes and a single curl on top of the point (HG T19), on top of base resembling the right side of the sky (N44).
13A24		stylised set of two toes, resembling a harpoon-head with two horizontal strokes and a two angled strokes as point (T19A), on top of base resembling the right side of the sky (N44).
13A25		A stylised set of two toes, resembling a harpoon-head with two horizontal strokes and a single curl on top of the point (HG T19), on top of a base with a loop at the back end.
13A26		A stylised set of two toes, resembling a harpoon-head with two horizontal strokes on top, and triangular point (T79), on top of a base with a loop at the back end.
13A27		A stylised set of two toes, resembling a harpoon-head with two horizontal strokes on top, and triangular point (T79), on top of base resembling the right side of the sky (N44).
13A28		A stylised set of two toes, resembling a harpoon-head with two horizontal strokes and two angled strokes on top of the point (T19C), on top of base resembling the right side of the sky (N44).
13A29		stylised set of two toes, resembling a harpoon-head with two horizontal strokes and a two angled strokes as point (T19A), on top of base resembling the sky (N1), with a star (N14) written between the two harpoon-heads.
13A2A		A stylised set of two toes, resembling a harpoon-head with two horizontal strokes and a single curl on top of the point (HG T19), on top of base resembling a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13A2B		A stylised set of three toes, resembling a harpoon-head with two horizontal strokes on top, and triangular point (T79), on top of base resembling the right side of the sky (N44).
13A2D		Two harpoon-heads with two horizontal strokes on top and a single curl on top of the point (HG T19), on top of a wickerwork basket with a handle, orientated with the handle to the back (V31).
13A2F		Two harpoon-heads with two horizontal strokes on top and a single curl on top of the point (HG T19), on top of a forward bend feather (H27)/a wide tongue (F20A), or a wide band of string or fabric (V12A).
13A32		A trachea (windpipe).
13A33		A shape resembling the modern 3, facing its mirror.
13A34		A trachea (windpipe).
13A35		A child, seated on nothing, forelegs separated, right arm raised with hand to mouth, left arm hanging beside the body (A17), written within the pupil of an eye (D12).
13A36		A donkey/ass, standing, carrying a load on its back, consisting of a round bag with bound bundle with ears of corn on top of the bag.
13A37		A donkey/ass, lying down, legs folded beneath the body, ears angled backwards, tail down.
13A38		A donkey/ass, seated as a Sethian animal, ears in a V shape, tail up.
13A39		A donkey/ass, lying down, legs folded beneath the body, ears in a V shape, tail down.
13A3B		A donkey/ass, legs bound together, ears in a V shape, tail up.
13A3C		A donkey/ass, lying down, with the ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A) vertically on the back, blade towards the front.
13A3D		A donkey/ass, lying down, with the ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A), angled on the nose, blade downwards.
13A3E		A donkey/ass, lying down, with the ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A) vertically on the back, blade towards the back.
13A3F		A donkey/ass, lying down, with the ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A) vertically on the back, blade towards the front, and a knife angled on the nose, blade downwards.
13A40		A donkey/ass, lying down, legs folded under the body, ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A), angled on the nose, blade upwards.
13A41		A donkey/ass, lying down, legs folded under the body, ears in a V shape, without tail, with a knife with a triangular blade and straight handle (T30A), angled on the nose, blade downwards.
13A42		A donkey/ass, lying down, legs folded under the body, ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A), vertically on the back, blade towards the front.
13A43		A donkey/ass, lying down, legs folded under the body, ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A), vertically on the back, blade towards the back.
13A44		A donkey/ass, lying down, legs folded under the body, ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A), angled on its ass, blade towards the front and a knife between the ears, blade towards the front, on top of a standard used for carrying religious symbols (R12).
13A45		A donkey/ass, legs bound together, ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A) vertically on its back, blade forwards, and a knife angled on its nose, blade upwards.
13A47		A donkey/ass, legs bound together, ears in a V shape, tail up, with a knife with a triangular blade and straight handle (T30A) vertically on its back, blade towards the back.
13A49		A donkey/ass, lying on its back, legs bound together, tail upwards, with a knife with a triangular blade and straight handle (T30A) vertically on top of the hooves, blade towards the back.
13A4C		The animal of Seth, standing, tail up.
13A4E		The animal of Seth, lying down, tail up (E21), on top of a wickerwork basket (V30).
13A4F		The animal of Seth, seated, tail up (E20), on top of a wickerwork basket (V30), with a knife with a triangular blade and straight handle (T30A) angled at the nose, blade downwards.
13A50		The animal of Seth, lying down, tail up (E21), with a knife with a triangular blade and straight handle (T30A) angled at the nose, blade downwards.
13A54		The animal of Seth, lying down, tail up (E21), on top of a standard used for the carrying of religious symbols (R12).
13A55		The animal of Seth, lying down, tail up (E21), with a wavy line of liquid running upwards and backwards over the head of the animal, ending above the head.
13A56		The animal of Seth, lying down, tail up (E21), with a wavy line of liquid running from the tail to the front paw.
13A57		The animal of Seth, seated, tail up, wearing the double crown (S5).
13A58		The animal of Seth, lying down, tail up (E21), on top of the sky, with lines of moisture coming from it (N4).
13A59		The animal of Seth, lying down, tail up (E21), on top of the sky (N1).
13A5A		An oryx, standing (E28), with a falcon (G5) on its back.
13A5B		An oryx, standing (E28), with a falcon (G5) on its back, on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13A5C		A basket with four pieces of grain or fruit (M39) in front of an oryx, standing (E28), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13A5E		An oryx, lying down, legs folded under the body.
13A5F		A gazelle, lying down, legs folded under the body.
13A60		A gazelle, lying down, 3 legs folded under the body, right front leg extended forwards.
13A62		An oryx, lying down, legs folded under the body, head looking backwards.
13A63		An oryx, lying on its back, tail in front, legs held to the belly, head looking forwards, with a knife with a triangular blade and straight handle at the back, blade towards the back..
13A64		A gazelle, standing, with two horns visible.
13A65		A gazelle, standing, with two horns visible, with a loop of cord with the ends downwards (V7) around its neck.
13A66		A gazelle, with two horns visible, about to rear.
13A67		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a flagellum (S45) on its back.
13A68		Three rams (Ovis longipes palaeo-aegyptiacus), standing, overlapping each other.
13A69		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on top of the horns.
13A6A		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard, with a headdress consisting of two feathers and a sun-disk (S76) on top of the horns.
13A6B		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard, with a cobra (Naja haja), standing up, with expanded hood (Uraeus, I64) on its horns.
13A6C		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard, with the white crown (S1) on its horns.
13A6D		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard, with the red crown (S3) on its horns.
13A6E		A ram (Ovis longipes palaeo-aegyptiacus), standing, without a beard, with a cobra (Naja haja), standing up, with expanded hood (Uraeus)(I64) on its head, with the wings of a bird on its back, spread in a v-shape.
13A70		A cult image of a ram (Ovis longipes palaeo-aegyptiacus), lying down, with a beard.
13A71		A ram, standing, without horns, without beard.
13A72		A ram, standing, without horns, without beard, wearing the red crown (S3).
13A74		A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard, with a headdress consisting of two feathers and a sun-disk (S76) on top of the horns, with a flagellum (S45) on its back, in front of a A ram (Ovis longipes palaeo-aegyptiacus), standing, with a beard, with a flagellum (S45) on its back, on top of a standard used for carrying religious symbols (R12).
13A75		A bubalis antilope, standing, head looking backwards.
13A76		A bubalis antilope, standing, head looking forwards.
13A77		A bubalis antilope, standing, head looking forwards (E103), with a cobra in repose (Naja haja), with its tail coiled up, in a single coil (I80A).
13A78		A bubalis antilope, standing, head looking backwards (E103A), with an African sacred ibis (Threskiornis aethiopicus) (G26A) on its back.
13A7A		A newborn bubalis antilope, lying down, three legs folded beneath the body, right front leg extended forwards.
13A7B		A newborn bubalis antilope, lying down, legs folded beneath the body, short tail upwards.
13A7C		A jackal, running, tail down.
13A7D		Two jackals, standing, tail down, overlapping each other.
13A7E		A jackal, standing, tail down, with a feather (H6) angled forwards at its front paw.
13A7F		A jackal, standing, tail down, with a feather (H6) angled backwards on its rear.
13A80		A desert plant with two branches, with a downwards hanging flower at the top, written over a jackal, standing, tail down (E17).
13A81		A flowering sedge (M23A), written ovar a jackal, standing, tail down (E17), on top of a standard used for carrying religious symbols (R12).
13A82		A jackal, standing, tail down, head looking backwards, with a band of string or fabric around its neck, ties towards the front.
13A83		A jackal, standing, tail down, with a band of string or fabric around its neck, ties towards the front, hanging downwards.
13A84		A jackal, standing, tail down (E17), on top of a standard used for the carrying of religious symbols, with an uraeus and SdSd-pretuberance at the front of the standard.
13A85		Three jackals, standing, tail down, overlapping each other, bound with rope to a boat/ship with a prow and stern resembling the handle of a handle of a sickle (U1), on top of a rectangle resembling water, with an oar/rudder at the back (P28), with a god, seated, knees up, with covered legs and arms, with the head of a falcon, with a sun disk (N5) on its head, holding a tie or strap, used with sandals (ankh-sign, S34), angling forward (C2) within it.
13A86		A dog or hound, seated, tail up.
13A87		A jackal, lying down, tail downwards, on top of a platform.
13A88		A jackal, lying down, tail downwards, with a flagellum (S45) on its back, on top of a platform.
13A89		A jackal, lying down, tail downwards, on top of a platform, with a dagger with a rounded tip (T8) angled forward on its front paw.
13A8A		A jackal, lying down, tail downwards, with a flagellum (S45) on its back.
13A8B		A jackal, lying down, tail downwards, with a flagellum (S45) on its back, with a sceptre (S42) on its front paws, vertically.
13A8C		A jackal, lying down, tail downwards, with a feather (H6) angled backwards on its back.
13A8D		A jackal, lying down, tail downwards, wearing a wig, on top of a shrine which widens near the top.
13A8E		A jackal, lying down, tail downwards, on top of a vessel with a small base and a wide rim (W77).
13A8F		A jackal, lying down, tail downwards, on top of a vessel with a small base and a wide rim (W77), with an uraeus on its front legs.
13A91		A jackal, lying down, tail downwards, on top of a standard used for the carrying of religious symbols, consisting of a horizontal bar, vertical pole with a shorter vertical line at either side of the pole.
13A92		A jackal, lying down, tail downwards, with an uraeus on its front paw, on top of a standard used for the carrying of religious symbols, consisting of a horizontal bar, vertical pole with a shorter vertical line at either side of the pole.
13A93		A jackal, lying down, tail down (E15), on top of a standard used for the carrying of religious symbols, with an uraeus and SdSd-pretuberance at the front of the standard, with a mace (T3) written horizontally over the vertical pole of the standard, macehead towards the back.
13A94		A jackal, lying down, tail downwards, with a feather (H6) angled backwards on its back (E143), on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13A95		A jackal, lying down, tail downwards, with a feather (H6) angled backwards on its back (E143), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A) on top of a parcel of land with irrigation ditches (N24).
13A97		A jackal, lying down, tail downwards (E15), on top of the sky (N1).
13A98		A cult image of a jackal.
13A99		A cat, seated, tail upwards.
13A9A		A cat, seated, tail curled over the body towards the back, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on top of its head.
13A9B		A cat, seated, tail curled over the body towards the back, with its right paw on top of a triangle.
13A9C		A cat, seated, tail curled over the body towards the back, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on top of its head, with a tie or strap, used with sandals (ankh-sign, S34), angling forward on top of its front paw.
13A9E		A cat, seated, tail curled over the body towards the back, with a flagellum on its back.
13A9F		A cat, seated, tail curled over the body towards the back, right paw raised in front, holding a round loaf with four curved imprints (X6B).
13AA0		A cat, seated, tail curled over the body towards the back, right paw raised in front, holding an oval vertically.
13AA1		A cat, seated, tail curled over the body towards the back, right paw raised in front, holding a roll of bread, with oval lines within it (X4A) vertically.
13AA2		A cat, standing, tail down.
13AA3		A cat, standing, tail down, on top of a shepherds crook with a package lashed to it, with a tie above and below the package (T18), written horizontally.
13AA4		A cat, seated, tail upwards (F1504), on top of a stairway (O40).
13AA5		A horse, standing, right front leg raised.
13AA6		A horse, standing.
13AA7		A horse, rearing, front legs nearly horizontal.
13AA8		A horse, rearing, front legs nearly horizontal, with a plume on its head.
13AAA		A goat, standing.
13AAB		A goat, standing, without horns, without a beard, with a seal, attached to a neck string (S20), around its neck
13AAE		A griffin with the head of a jackal.
13AAF		A griffin.
13AB0		A hippopotamus (Hippopotamus amphibius) (E25), with a griffon vulture (Gyps fulvus) (G4) on its back.
13AB1		A hippopotamus (Hippopotamus amphibius) (E25), in front of a stem of papyrus with a bud (M13), on top of a base.
13AB3		A hippopotamus (Hippopotamus amphibius), with a forwards, downwards line of liquid coming from the mouth, with a knife with a triangular blade and straight handle (T30A) vertically on its back, blade forwards.
13AB4		A hippopotamus (Hippopotamus amphibius), with a knife with a triangular blade and straight handle (T30A) on its head, blade towards the back.
13AB5		A hippopotamus, with the limbs of a feline, head and tail of a crocodile, standing on its hind legs, front leg extended forwards.
13AB6		A hippopotamus, with the limbs of a feline, head and tail of a crocodile, standing on its hind legs, front leg extended forwards (E134), upon a rolled up herdsman's shelter of papyrus (V17).
13AB7		A hippopotamus, with the limbs of a feline, head and tail of a crocodile, standing on its hind legs, front leg extended forwards (E134), with a headdress consisting of two plumes and a sun-disk (S63A), paw upon a rolled up herdsman's shelter of papyrus (V17).
13AB8		A hippopotamus, with the limbs of a feline, head and tail of a crocodile, standing on its hind legs, front leg extended forwards (E134), with a headdress of bovine horns with a sun disk (F102), paw upon a rolled up herdsman's shelter of papyrus (V17).
13AB9		A hippopotamus, with the limbs of a feline, head and tail of a crocodile, with a headdress consisting of two plumes and a sun-disk (S63A), standing on its hind legs, front legs raised in front of the head, holding a tambourine or drum.
13ABA		A hippopotamus, with the limbs of a feline, head and tail of a crocodile, standing on its hind legs, front leg extended forwards, holding a knife with a triangular blade and straight handle (T30A), blade forwards.
13ABB		A hyena.
13ABD		A desert hare, lying down, with a upwards tail with a forked end.
13ABE		A desert hare, lying down, with whiskers.
13ABF		A desert hare, lying down (E34), on top of a standard used for the carrying of religious symbols (R12)
13AC1		A lion, standing, tail downwards (E22), written within an dotted oval.
13AC2		A lion, standing, tail upwards.
13AC3		A lion, seated, tail curled over the body towards the back.
13AC4		A lion, lying down, tail upwards.
13AC5		A lioness, lying down.
13AC7		A lion, standing, tail upwards (E22A), with a platform on its back, with a falcon with its wings spread out on top of it.
13AC8		A lion, standing, tail upwards (E22A), with a sun-disk on its head.
13ACA		A lion, standing, tail downwards, right front leg raised, holding a tie or strap, used with sandals (ankh-sign, S34).
13ACB		A lion, standing, tail downwards, with a tie or strap, used with sandals (ankh-sign, S34) on its front paw.
13ACC		A lion, standing, tail downwards, right front leg raised, holding a knife with a triangular blade and straight handle (T30A).
13ACD		A lion, standing, tail downwards, right front leg raised, holding a tall water pot (W14).
13ACE		A lion, standing, tail downwards (E22), with the headpiece of a sistrum resembling a shrine on top of a hathorian face, on top of its back.
13ACF		A lion, standing, tail downwards (E22), on top of a shepherds crook with a package lashed to it, with a tie above and below the package (T18), written horizontally.
13AD0		A lion, standing, tail downwards (E22), wearing the double crown (S5).
13AD1		A lion, standing, tail downwards (E22), with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head.
13AD2		A lioness, lying down, with a stick with an circular inset (P11A) on its back.
13AD3		A lioness, lying down, tail raised, on top of a shrine which widens near the top.
13AD4		A lion, lying down, tail curled over the body towards the back, with a forwards, downwards line of liquid coming from the mouth.
13AD5		A lion, lying down, wearing the red crown (S3), tail curled over the body towards the back.
13AD6		A lion, standing, tail in the form of a cobra (Naja haja), standing up, with expanded hood (Uraeus).
13AD7		A lion, lying down, tail curled over the body towards the back, with a wide cup (W10) on the front paw.
13AD8		A lion, lying down, tail curled over the body towards the back, with a knife with a triangular blade and straight handle (T30A) on the front paw.
13AD9		A lion, lying down, tail curled over the body towards the back, with a feather (H6) on the front paw.
13ADA		A lion, lying down, tail curled over the body towards the back, with a head-cloth (S56) on the front paw.
13ADB		A lion, lying down, tail curled over the body towards the back, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head, with a wide cup (W10) on the front paw.
13ADD		A lion, lying down, tail curled over the body towards the back, on top of a standard used for the carrying of religious symbols (R12).
13ADE		A lion, lying down, tail curled over the body towards the back, rear half enclosed in an rectangle which is open at the front.
13ADF		A lion, lying down, tail curled over the body towards the back, with the sun-disk (N5) on its back.
13AE0		A lion, lying down, tail curled over the body towards the back, on top of a shrine which widens towards the top, with a cloth wound on a pole, an emblem of divinity (R8) behind the lion, angling slightly backwards.
13AE1		A lion, lying down, tail curled over the body towards the back, with a pole/staff with the head of a falcon with the sun-disk on the head, on the front paw of the lion.
13AE2		A lion, standing, tail down, on some structure resembling the letter I with downwards sloping extentions at the top.
13AE3		A lion, standing on its hind legs, foreleg extended forwards, holding a a knife with a triangular blade and straight handle (T30A).
13AE4		A lion, standing on its hind legs, forelegs extended forwards, holding a stick in each paw, tail raised upwards.
13AE5		A panther or leopard, in a running posture.
13AE6		A panther or leopard, standing, head angled downwards, looking towards the reader.
13AE7		A pig, standing on its hind legs, with a forwards, downwards angling dotted line coming from the mouth, ending in a human mouth, seen from the side, on top of a sand covered mountain over the edge of the cultivated areas (N26).
13AE8		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail upwards.
13AE9		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail folded under the rear, with a feather (H6) on its knee, angled forwards.
13AEA		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding an eye, with the markings of the head of a falcon (D10).
13AEB		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a bowl for incense with smoke rising from the bowl (R7).
13AEC		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a bowl for incense, with two pellets of incence, with smoke rising from the bowl (R7C).
13AED		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a vertical stick which is forked at the top.
13AEE		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a stick, angled towards the face.
13AEF		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a palm branch, stripped of leaves and notched with a round notch, with the notch towards the body.
13AF0		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a staff (S43).
13AF1		A hamadryas baboon (Papio hamadryas), seated, wearing the red crown (S3), arms forward, holding a staff (S43).
13AF4		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a piece of flesh, point down, curving backwards (F51B).
13AF5		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a round vessel with an upstanding rim (W24).
13AF7		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail folded under the rear (E35), in front of a sheath or receptacle with V shaped indentation at the top, on top of a base, with a forward curved line coming from the indentation (V36K), on top of a basket (V30).
13AF8		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail folded under the rear (E35), in front of a sheath or receptacle with V shaped indentation at the top (V98), on top of a rectangular base.
13AF9		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail folded under the rear (E35), in front of an long, cup-shaped object with a vertical line of dots inside it, on top of a rectangular base.
13AFA		A hamadryas baboon (Papio hamadryas), seated, wearing the red crown (S3), arms forward, handpalm upwards.
13AFB		A hamadryas baboon (Papio hamadryas), seated, hands on knees, tail folded under the rear, wearing the red crown (S3).
13AFE		A hamadryas baboon (Papio hamadryas), seated, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64) on its head.
13AFF		A hamadryas baboon (Papio hamadryas), seated, arms forward, holding a spiral, winding counter-clockwise away from its central point, ending at the right lower corner after about 1,5 turns (Z7).
13B00		A baboon, standing on it hind legs, tail down, arms raised in front, forearms nearly vertical, handpalms outwards.
13B01		A monkey, standing on its hind legs, tail down, arms raised in front, forearms nearly vertical, handpalms outwards.
13B02		A hamadryas baboon (Papio hamadryas), standing on its hind legs, tail down, arms raised in front, forearms nearly vertical, handpalms outwards.
13B03		A baboon, standing on its hind legs, tail down, arms downwards, hands at the hight of the waist, handpalms downwards.
13B05		A baboon, standing on its hind legs, tail down, arms extended forwards, holding a knife with a triangular blade and straight handle (T30A), blade facing outwards.
13B07		A baboon, standing on its hind legs, tail down, arms extended forwards, left forearm horizontal, holding the face of a human man (D2), with the right arm angling over it.
13B08		A baboon, standing on its hind legs, tail down, arms extended forwards, left forearm horizontal, holding a palm branch, stripped of leaves and notched with a sharp notch (M4B), with the notch inwards, right hand between the head and the palm brance, handpalm outwards..
13B0A		A baboon, standing on its hind legs, tail down, arms extended forwards, holding a palm branch, stripped of leaves, with multiple sharp notches (M4A), on top of a half round loaf of bread (X1), with the curve outwards.
13B0B		A baboon, standing on its hind legs, tail down, arms extended forwards, left forearm horizontal, holding a windpipe and heart, with a single horizontal stroke at the top (F35), with the right arm angled upwards with the hand near the tops stroke.
13B0C		Two baboon, standing on its hind legs, tail down, facing each other, arms extended forwards, left forearm horizontal, holding the same windpipe and heart, with a single horizontal stroke at the top (F35), with the right arm angled upwards with the hand near the tops stroke.
13B0D		A baboon, wearing the white crown (S1), standing on its hind legs, tail down, arms extended forwards, left forearm horizontal, holding a windpipe and heart, with a single horizontal stroke at the top (F35), with the right arm angled upwards with the hand near the tops stroke.
13B0E		A baboon, standing on its hind legs, tail down, arms extended forwards, forearm horizontal, hand horizontal, handpalm upwards, holding a windpipe and heart, with a single horizontal stroke at the top (F35).
13B0F		A baboon, standing on its hind legs, tail down, arms extended forwards, left forearm horizontal, holding a lotus flower with a bud at either side, with a long vertical stem, with the right hand above the flower.
13B11		A baboon, standing on its hind legs, tail down, right arem extended forwards, forearm horizontal, handpalm upwards, holding a column imitating a bundle of stalks tied together (R11), with a folded piece of cloth, short section outwards below the right hand, left arm extended backwards, hand at the hight of the shoulder, handpalm upwards, supporting a quail chick (G43).
13B12		A baboon, dancing, right leg vertical, left leg raised, left knee in front of the right leg, left foot behind the right leg, right arm extended forwards and downwards, holding an ornamental chevaux de frise on top of walls, with a circle dividing the top and base (Aa30), of nearly the size of the baboon near the base of the ornament.
13B13		A baboon, seated on a block, tail down, arms extended forward.
13B14		A mouse.
13B15		A lion, lying down, with the face of a man, with a long, curved beard and coif.
13B16		A lion, lying down, with the face of a man, with a long, curved beard and coif, with an uaeus on its forehead.
13B17		A lion, lying down, with the face of a man, with a long, curved beard and coif, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)) on its head.
13B18		A lion, lying down, with the face of a man, with a long, curved beard and coif, with the sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side (N6B), on its head.
13B19		A lion, lying down, with the face of a man, with a long, curved beard and coif, wearing the double crown (S6) and with ureaus on the forehead.
13B1A		A lion, lying down, with the face of a man, with a long, curved beard and coif (E151A), with a tie or strap, used with sandals (ankh-sign, S34) on its front paw.
13B1B		A lion, lying down, with the face of a man, with a long, curved beard and coif, with an uaeus on its forehead (E151), with a tie or strap, used with sandals (ankh-sign, S34) on its front paw.
13B1C		A lion, lying down, with the face of a man, with a two-barbed harpoon, with a handle on the shaft (T21A), vertically on its front paw, handle inwards.
13B1D		A lion, lying down, with the face of a man, with a long, curved beard and coif, with an uaeus on its forehead (E151), with a two-barbed harpoon, with a handle on the shaft (T21A), vertically on its front paw, handle outwards.
13B1E		A lion, lying down, with the face of a man, with a sceptre with a straight shaft, topped with the head of the Seth animal vertically on the front paw.
13B20		A lion, lying down, with the face of a man, with a long, curved beard and coif, with an uaeus on its forehead (E151), with a vessel on its front paw.
13B21		A lion, lying down, with the face of a man, with a long, curved beard and coif (E151A), on top of a wickerwork basket (V30).
13B22		A lion, lying down, with the face of a man, with a long, curved beard and coif, with an uaeus on its forehead, on top of a base, front paw holding a sealed ointment jar, without ties (W2) in its front paw, with the jar resting on the base.
13B23		A cult image of a lion, lying down, on a base, with the head of a man, with a long curved beard and coif, with an uraeus on its forehead, wearing the double crown (S5).
13B25		A lion, lying down, with the face of a man, with a long, curved beard and coif, with an uaeus on its forehead (E151), on top of a shrine.
13B26		A lion, standing, tail upwards, with the head of a human man, with a coif or headdress running partly over its back.
13B27		The forepart of a sphinx.
13B28		Two foreparts of sphinxes, connected back to back.
13B2B		A bovid (bull), running.
13B2C		A bovid (bull), standing, right front leg raised.
13B2D		Two bovids (oxen), standing, overlapping each other, in front of a plough (U13).
13B2E		A bovid (bull), stanidng, with the red crown (S3) between the horns.
13B2F		A bovid (bull), standing, with a folded piece of cloth around its neck (V60), with a headdress consisting of a sun-disk with two feathers (S76) between its horns.
13B30		A bovid (bull), standing, with a forwards, downwards line coming from the mouth.
13B31		A bovid, standing, with a feather (H6) on its back, angled backwards.
13B32		A bovid (bull), lying down, legs folded beneath the body, tail downwards.
13B33		A bovid (cow), lying down, legs folded beneath the body, right front leg extended forwards.
13B34		A bovid (bull), lying down, legs folded beneath the body, tail upwards.
13B35		A bovid (bull), lying down, legs folded beneath the body, tail upwards, with a knife with a triangular blade and straight handle (T30A) vertically on its back, blade forwards.
13B36		A bovid (bull), lying down, legs folded beneath the body, tail downwards, with a knife with a triangular blade and straight handle (T30A) vertically on its back, blade forwards.
13B37		A bovid (bull), lying down, legs folded beneath the body, tail downwards, with a knife with a triangular blade and straight handle (T30A) vertically on its back, blade backwards.
13B38		A bovid (bull), lying down, legs folded beneath the body, tail upwards, head looking backwards, with a knife with a triangular blade and straight handle (T30A) vertically on its back, blade forwards.
13B39		A bovid (bull), legs bound together, tail downwards.
13B3B		A butchered bovid without head or legs, tail coiling upwards, with a knife with a triangular blade and a straight handle (T30A) angled at the top, blade backwards, and a knife angled at the bottom, blade backwards.
13B3C		A bull, standing, head lowered, tail down, left front leg bend, with a line running from the left front leg hoof over the head of the bull.
13B3E		A pustule or gland, or a packet of mummy bandages (Aa2), in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12).
13B3F		The hill country over the edge of the cultivated areas (N25) in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12).
13B40		The hill country over the edge of the cultivated areas (N25) in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12) on top of a parcel of land with irrigation ditches (N24).
13B41		A cover of a quiver, with a vertical loop on top (Aa18A), in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12).
13B42		The head of a human man, seen in profile, with a short straight beard, in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12).
13B43		A bovid (bull), standing (E1), on top of a sledge (U15).
13B44		A cow, looking backwards.
13B45		A bovid (cow), lying down, legs folded beneath the body, tail down, with the head and front legs of a calf protruding from the cow.
13B46		A cult image of a bovid (cow), lying down.
13B47		A bovid (cow), standing, head facing backwards, looking over its back, with a calf below the cow, suckling.
13B48		A bovid (cow), standing, tail raised with the head and front legs of a calf protruding from the cow.
13B49		A bovid, standing, tail down, with the sundisk (N5) between the horns.
13B4A		A bovid, standing, tail down, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)), between the horns, with the uraeus occuring in front of the horns.
13B4C		A bovid (bull?), standing, with a headdress consisting of a base with multiple plumes coming from it between the horns.
13B4D		A bovid, standing, tail down, with the double crown (S5) between the horns.
13B4E		A bovid (cow), lying down, legs folded beneath the body, with a flagellum (S45) on its back.
13B4F		A bovid, lying down, legs folded beneath the body, with N6 (The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus)), between the horns, with the uraeus occuring in front of the horns.
13B50		A cult image of a bovid (cow), lying down, with a flagellum (S45) on its back.
13B51		A cult image of a bovid (cow), lying down, with a sun-disk (N5) between the horns, with a flagellum (S45) on its back.
13B53		A bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12).
13B54		A piece of crocodile skin with horizontal spines, or tail of a crocodile (I6), in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12).
13B56		A newborn bubalis antilope, lying down, legs folded beneath the body, long tail downwards (E9), in front of a bovid (bull), standing (E1), on top of a standard used for the carrying of religious symbols (R12).
13B57		A bovid (bull), standing, without horns.
13B58		A bovid (bull), legs bound together, tail downwards.
13B59		Some type of mouse-like animal.
13B5A		An mythological animal with the head of a crocodile, the foreparts of a lion, and the hindquarters of a hippopotamus.
13B5B		A rhinoceros.
13B5C		The head of a donkey/ass, with the ears in a v shape.
13B5D		The head of a donkey/ass, with the ears angled backwards.
13B60		The head of a ram, without horns or forward curving horns.
13B61		The head of a ram, without horns or forward curving horns, with the neck curving downwards.
13B62		The head of a ram (Ovis longipes palaeo-aegyptiacus), without beard.
13B63		The head of a ram (Ovis longipes palaeo-aegyptiacus), with beard, with a sun-disk (N5) on top of the horns.
13B64		The head of a ram (Ovis longipes palaeo-aegyptiacus), with beard, with two feather-like plumes (S78) on top of the horns.
13B65		The head of a ram (Ovis longipes palaeo-aegyptiacus), with beard, with the Atef crown (S8A) on top of the horns.
13B66		The forepart of a ram (Ovis longipes palaeo-aegyptiacus), with beard, with a second head looking backwards.
13B67		The forepart of a lion with the head of a ram (Ovis longipes palaeo-aegyptiacus), with beard.
13B68		The forepart of a ram (Ovis longipes palaeo-aegyptiacus), with beard, with a uraeus on top of the horns.
13B69		Four heads of rams (Ovis longipes palaeo-aegyptiacus) with necks, connected two-by-two to each other, vorming an cross-shape.
13B6A		The head of a bovid (ox) with outwards curving horns, with a short vertical stroke under the chin (beard?).
13B6B		A head of a bovid (ox), as seen from the front.
13B6C		The head of a bovid (bull), angled forward as if to attack with a cobra (Naja haja), standing up, with expanded hood (Uraeus), between its horns.
13B6D		The head of a bovid (ox), with a forwards, downwards line coming from the mouth.
13B6E		The head of a bovid (ox), with a rope around the nose, angling downwards and backwards.
13B6F		The forepart of a bovid (ox).
13B70		The head of a bovid (ox), cnnected to a foreleg of an bovid (ox), hoof orientated away from the reading direction (F24).
13B71		The head of a bovid (cow), without horns, with an ear, with a short vertical line under the chin (beard?).
13B72		The head of a bovid (cow), without horns, without an ear.
13B73		Two heads of a bovid (cow), without horns, without an ear, facing each other.
13B74		The foreparts of a bovid, connected at the back, standing.
13B75		Two foreparts of a bovid, connected at the back, lying down, legs extended forwards, horizontally.
13B76		The head of a jackal, with a loop of rope around the neck, ties at the front.
13B77		The head of a jackal, looking backwards, with the neck curving towards the back, over a horizontal line.
13B78		The head of a jackal, neck curving forwards, with line of string around the neck, with a loop above ties at the back.
13B79		The head of a jackal, looking backwards, with the neck curving towards the back, written over a band of string or fabric (V12).
13B7B		A mace (T3) written horizontally over the neck of a head of a jackal, looking backwards, with the neck curving towards the back, with the macehead towards the back.
13B7C		A mace (T3), written over the neck of a head of a jackal, neck curving forwards, macehead towards the back.
13B7D		The head of a jackal.
13B7F		The head of a jackal on a cup-like base, with two arms, raised in front, handpalms forwards.
13B81		The head and neck of a canine animal, with a forked end to the neck.
13B82		The head and neck of a long-necked horned animal, with a forked end to the neck.
13B83		The head and neck of a long-necked horned animal, with a horizontal line written over the neck, near the bottom.
13B85		The head and neck of a canine animal, on top of a stick.
13B86		The head of a canine, on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13B87		The head of an antilope.
13B88		A mouth (D21), written over the neck of the head of an antilope (F81).
13B89		A representation of two ribs, written parallel (Aa15), written over the neck of the head of an antilope (F81).
13B8A		The head of an oryx.
13B8B		A mouth (D21), written over the neck of the head of a bubalis antilope.
13B8C		The head of a Nubian ibex (Capra nubiana).
13B8E		The head of a gazelle.
13B8F		A simplified head of a hippopotamus.
13B90		Head of a lion/lioness
13B91		The forepart of a lion, lying down (F4), wearing a headdress consisting of two feathers on top of the horns of a ram, with a cobra (Naja haja), standing up, with expanded hood (Uraeus), with sun disks its head on either side of the feathers, orientated outwards (S74).
13B92		The forepart of a lion, standing.
13B93		Two foreparts of lions, lying down, connected back to back.
13B94		Two foreparts of lions, lying down, connected back to back, with a sun-disk (N5) on their back.
13B95		The head of a leopard with a uraeus on top of it.
13B97		A sledge with the forepart of a donkey/ass, with one leg.
13B98		A sledge with the forepart of a donkey/ass, with one leg and only one ear, angled forwards.
13B9A		A sledge with the forepart of a horse, with one leg, with a rectangular block on the sledge.
13B9B		A sledge with the forepart of a horse, with two legs.
13B9C		A sledge with the forepart of a donkey/ass, with two legs, one raised, one bend backwards.
13B9D		A sledge with the head of a horse.
13B9F		A sledge with the head of a donkey/ass, with only one ear, angled forwards.
13BA0		A sledge with the head of a horse, with the vertical lines of the sledge extending beyond the top line.
13BA1		The head of an elephant.
13BA2		Head of an Hyena.
13BA3		The head of an animal with the ears backwards.
13BA4		A palm branch, stripped of leaves and notched with a round notch (M4), written within the horns of a bovid (F13), on top of an alabaster basin with a diamond shaped inner marking (W3).
13BA5		A palm branch, stripped of leaves, with multiple sharp notches (M4A), written within the horns of a bovid (F13).
13BA6		A large sun-disk (N5), written within the horns of a bovid (F13).
13BA9		The horns of a bovid (ox), upside down, enclosing a fire-drill in a piece of wood (U28).
13BAA		The horns of a goat.
13BAB		The horns of a goat, on a base resembling a half round loaf of bread (X1).
13BAE		A tusk, with vertical lines over the tusk.
13BB2		A lower jaw-bone of an animal with pointed teeth.
13BB3		A tongue, with the stroke at the back vertical.
13BB4		A tongue, without a stroke at the back.
13BB5		A wide tongue.
13BB6		The ear of a bovid (ox), with internal detail.
13BB7		The hind-quarters of a seated lion or leopard, on top of a standard used for carrying religious symbols (R12).
13BB8		The leg and hoof of a bovid (ox), bend at the top.
13BB9		The leg and hoof of a bovid (ox), foreleg nearly horizontal, thigh muscle widened.
13BBB		The tigh of a bovid (ox).
13BBC		The leg and hoof of a bovid (ox), bend at the ankle.
13BBD		The leg and hoof of a bovid (ox), with a cross (Z9) written above it.
13BBE		The paw and foreleg of a lion.
13BBF		The leg of a chair or table, resembling the paw and foreleg of a lion.
13BC0		A schematic representation of a lions paw.
13BC2		A lions paw and foreleg, horizontally.
13BC3		A lions paw.
13BC4		The tigh of a bovid (ox), on top of a standard used for the carrying of religious symbols (R12).
13BC5		A gray heron (Ardea cinerea) (G31), on top of the skin of a goat (F26).
13BC6		A human headed bird, with a flagellum (S45) on its shoulder (G53C), on top of the skin of a goat (F26).
13BC7		Three triangles of skin with a tail.
13BC8		The skin of a cow, with the top of the skin a lot wider than the bottom, with the tail curved backwards.
13BC9		The skin of a cow, with downwards ties at the sides, with the tail angled forwards.
13BCA		The skin of an leopard.
13BCB		A hoof.
13BCC		The skin of a cow, with the tail hanging straight down, about to be pierced by an arrow (T11A).
13BCD		The skin of a cow, with the tail hanging straight down, pierced by an arrow with rectangular fletching (T11A).
13BCE		The body of a bird with the head replaced with a flower and stem, the stem curling backwards around a flower and long stem in a vase.
13BCF		The skin of a goat, wrapped around a pole.
13BD0		A water skin without ties at the opening.
13BD1		A water skin without ties at the opening, and a loop at the back.
13BD2		A water skin with an carying strap.
13BD3		A simplified representation of three skins of foxes, tied together at the top.
13BD4		The udder of a cow, seen from the side.
13BD5		The udder and tail of an animal, seen from below, represented as a mace or spindle-like object.
13BD6		A simpliefied form of the tail of an animal.
13BD7		A heart (F34), written inside a circle/oval.
13BD8		A heart with downwards hanging loops on either side.
13BDA		An oval, enclosed by a band which curves outwards at the ends.
13BDB		A heart (F34), written inside a brazier (Q7E).
13BDC		A heart, with two downwards hanging bands coming from the top.
13BDD		A windpipe and heart, with a double horizontal stroke at the top.
13BDE		A windpipe and heart, without a horizontal stroke at the top.
13BDF		A windpipe and lungs, with a broad horizontal bar on top.
13BE0		A windpipe and lungs (F36), with a flower written under the horizontal bar, on each side, with the stems crossing over the windpipe.
13BE1		A windpipe and lungs (F36), with a wide cup on each side of the lungs, from which two flowers come, one vertical, the other crossing the windpipe, to angle downwards on top of the vertical flower on the other side.
13BE2		A windpipe and lungs (F36), with a stem of papyrus with a flowering bud (M127) left of the windpipe and a stem of papyrus with a bud (M13) right of the windpipe, both beneath the horizontal bar.
13BE3		A vertebra with a tall spine on top, seen lenghtwise.
13BE4		A piece of spine with four vertebrae, angling forwards.
13BE5		A piece of spine with four vertebrae, angling forwards, with a horizontal line over the spines.
13BE6		A spine with three ribs, connected to each other.
13BE7		A spine with four ribs, written at an angle, with a circle at either end of the spine.
13BE8		A spine with three ribs.
13BE9		A spine with five ribs, written at an angle.
13BEA		A spine with six ribs, written at an angle.
13BEB		A spine with four ribs, connected to each other.
13BEC		A spine, with the spinal cord hanging out at the backside, with three blocks at the bottom.
13BEE		A spine with the spinal cord hanging out at eiter side, the vertebrae represented as a rectangle.
13BEF		A spine, with the spinal cord hanging out at either side, from the middle of the vertebrae.
13BF0		A stack of ribs, on top of an upside down cone.
13BF1		A stack of ribs, written over an upside down cone.
13BF2		A stack of ribs, written behind an upside down cone.
13BF3		A stack of ribs.
13BF5		A representation of two ribs, written parallel (Aa15), written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13BF6		A representation of two ribs, written parallel (Aa15) with the opening to the front, written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13BF7		A leg-bone with meat, with sections of meat missing at the back.
13BF8		A haunch of meat with an indentation at either side of the back.
13BFA		A leg-bone with meat (F44) on top of a wide cup (W10).
13BFB		A piece of meat, angled backwards, tapering of near the top.
13BFC		A piece of meat, with a tapered end on either side.
13BFD		An intestine, in a rectangular form, opening at the front, with the internal line running from the left bottom corner to the top right corner.
13BFE		An intestine, with the opening to the top left (F46A), written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13BFF		An intestine, with the opening to the top right (F47A), written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13C00		An intestine, with the opening to the bottom left (F47), written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13C01		A pustule or gland, with a forward, downwards dotted line of fluid issuing forth.
13C02		A pustule or gland without blobs at the side, with a forward, downwards dotted line of fluid issuing forth.
13C03		A human placenta (a circle with multiple horizontal lines written within, which do not connect to the circle.
13C04		A circle with a broad dot inside.
13C05		A liver.
13C07		A talon or fang.
13C08		A vertebra, seen lenghtwise.
13C09		Some type of mat or bag, or woven material, representing a oval with two vertical lines coming from the top, which curve downwards.
13C0A		A loop in which the lines do not overlap, with conical end-pieces.
13C0B		An ostrich (Struthio camelus), with one wing downwards and one wing upwards.
13C0D		A long-legged buzzard (Buteo rufinus), with a lappet.
13C0E		A pintail duck (Anas acuta), with a dotted circle around its head.
13C0F		A pintail duck (Anas acuta), written within a dotted oval/circle.
13C10		A pintail duck (Anas acuta), with a lappet.
13C11		A pintail duck (Anas acuta), with a lappet, with a flagellum over its shoulder.
13C13		A pintail duck (Anas acuta), with the beak open.
13C14		A feather (H6), in front of a pintail duck (G39)/goose (G38), on top of a base.
13C15		A greater white-fronted goose (Anser albifrons), flying, with a wing at either side of its body (G40), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C16		A greater white-fronted goose (Anser albifrons), standing, with a wing at either side of its body.
13C17		A greater white-fronted goose (Anser albifrons), flying, with a wing at either side of its body (G40), written within a dotted oval/circle.
13C18		A greater white-fronted goose (Anser albifrons), standing, with an open beak, with a wing at either side of its body, seen from behind.
13C19		A greater white-fronted goose (Anser albifrons), head looking downwards, legs extended, both wings behind the body.
13C1A		A greater white-fronted goose (Anser albifrons), standing, head looking downwards, open beak, wings behind the body, spread apart.
13C1B		A greater white-fronted goose (Anser albifrons), standing, wings behind the body.
13C1C		A feather (H6), in front of a greater white-fronted goose (Anser albifrons), alighting or flying up, with both wings behind the body (G41), on top of a strip of land, with three grains of sand written beneath it (N16).
13C1D		A feather (H6), in front of a greater white-fronted goose (Anser albifrons), alighting or flying up, with both wings behind the body (G41), on top of a strip of land, with three grains of sand written beneath it, arranged horizontally, and a tongue  of land (N21) written on either side of the grains of sand (N16A).
13C1F		Three ducks or geese, without wings, standing, overlapping each other vertically, rotated 90°
13C20		Three ducks or geese, both wings above the body, standing, overlapping each other vertically, rotated 90°
13C21		A greater white-fronted goose (Anser albifrons), alighting or flying up, with both wings behind the body (G41), with a downwards angling rope around the neck, connected to a pole resembling a throwing-stick, or a club used by foreign people (T14).
13C22		A greater white-fronted goose (Anser albifrons), standing, wings behind the body (G74), with a downwards angling rope around the neck, connected to a pole resembling a throwing-stick, or a club used by foreign people (T14).
13C23		A greater white-fronted goose (Anser albifrons), standing (G38), with a downwards angling rope around the neck, connected to a pole resembling a throwing-stick, or a club used by foreign people (T14).
13C25		A greater white-fronted goose (Anser albifrons), with an protruding feather at the back of the bird.
13C27		Three ducks or geese, overlapping each other, front duck or goose looking forwards, back ducks or geese looking backwards.
13C28		A greater white-fronted goose (Anser albifrons), head angled downwards, with open beak.
13C29		A greater white-fronted goose (Anser albifrons), with an protruding feather on top of the head.
13C2A		A duck or goose, seated upon a bed with leonine legs and tail.
13C2B		A greater white-fronted goose (Anser albifrons), head angled downwards, with the neck making a loop, with open beak.
13C2D		An owl, legs drawn towards the body, with a human face.
13C2E		A schematic representation of an owl, legs drawn towards the body, with the head replaced by two upwards lines.
13C2F		A schematic representation of an owl, legs drawn towards the body, with the head replaced by an upwards line and one forward line.
13C30		A schematic representation of an owl, legs drawn towards the body, with the head represented by a blob with two upwards lines.
13C31		A great cormorant (Phalacrocorax carbo), with two ticks on its head.
13C33		A black ibis (Plegadis falcinellus), head down, tip of the beak at the level of the toes.
13C34		A saddle-billed stork (Ephippiorhynchus senegalensis), without lappet.
13C36		A saddle-billed stork (Ephippiorhynchus senegalensis), with lappet (G29), with a bowl for incense with smoke rising from the bowl (R7) written in front of the legs.
13C37		A saddle-billed stork (Ephippiorhynchus senegalensis), without lappet (G29A), with a flagellum (S45) on its back.
13C38		A saddle-billed stork (Ephippiorhynchus senegalensis), with lappet (G29), with a flagellum (S45) on its back.
13C3A		Three saddle-billed stork (Ephippiorhynchus senegalensis), overlapping each other, without lappets.
13C3C		A saddle-billed stork (Ephippiorhynchus senegalensis), without lappet (G29A), on top of a hoe with a rope connecting the two pieces, written horizontally (U7).
13C40		A cattle egret (Bubulcus ibis) with a lappet.
13C43		A vulturine guineafowl (Acryllium vulturinum), without internal detail.
13C45		A cattle egret (Bubulcus ibis), standing, with a fish in its beak.
13C46		A bird, standing, its neck making a loop, head looking down.
13C47		A cattle egret (Bubulcus ibis), with a bent neck, with the head facing towards the body.
13C48		A cattle egret (Bubulcus ibis), with a bent neck, with the head facing towards the body, with a line over its neck (mutilation).
13C49		A cattle egret (Bubulcus ibis), with a bent neck, with the head facing towards the body, and legs drawn towards the body.
13C4A		A gray heron (Ardea cinerea) with a lappet.
13C4B	13C4C	Three gray herons (Ardea cinerea), overlapping each other.
13C4D		A crested ibis (Ibis comata), with a crest of two long feathers on the head.
13C4E		A gray heron (Ardea cinerea) (G31), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C4F		A gray heron (Ardea cinerea) (G31), with a flagellum (S45) on its back.
13C50		A gray heron (Ardea cinerea) (G31), with a flagellum (S45) on its back, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C51		A gray heron (Ardea cinerea), legs drawn towards the body.
13C52		A gray heron (Ardea cinerea), legs drawn towards the body, with a flagellum (S45) on its back, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C54		A gray heron (Ardea cinerea), legs drawn towards the body (G215), with the claws on top of the hill country over the edge of the cultivated areas (N25).
13C55		A gray heron (Ardea cinerea), legs drawn towards the body (G215), with the claws on top of a sand covered mountain over the edge of the cultivated areas (N26).
13C56		A gray heron (Ardea cinerea), legs drawn towards the body (G215), with a flagellum (S45) on its back, with the claws on top of a sand covered mountain over the edge of the cultivated areas (N26).
13C57		A gray heron (Ardea cinerea), legs drawn towards the body (G215), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C58		A gray heron (Ardea cinerea) with a single straight feather on its head and a lappet.
13C59		A gray heron (Ardea cinerea) with a single feather on its head and a lappet.
13C5A		A gray heron (Ardea cinerea) with two feather on its head.
13C5B		A crested ibis (Ibis comata), with a spiral, winding counter-clockwise away from its central point, ending at the right lower corner after about 1,5 turns (Z7) on its claws.
13C5C		An African sacred ibis (Threskiornis aethiopicus) (G26A), with a flagellum (S45) on its shoulder, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C5D		An African sacred ibis (Threskiornis aethiopicus) (G26A), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C5E		An African sacred ibis (Threskiornis aethiopicus) (G26A), with a feather (H6), angled forward, on its toe.
13C5F		A feather (H6), in front of an African sacred ibis (Threskiornis aethiopicus) (G26A), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C60		A feather (H6), in front of an African sacred ibis (Threskiornis aethiopicus) (G26A), with a flagellum (S45) on its back, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C62		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body.
13C63		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body (G195), with a flagellum (S45) on its back.
13C64		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body, with a flagellum on its back (G195A), with a feather (H6), angled forwards, on its toe.
13C65		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body (G195), with a feather (H6), angled forwards, on its toe.
13C66		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body, with the wings extended on either side of the body.
13C67		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body (G195), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C68		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body, with a flagellum on its back (G195A), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C69		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body (G195), with a feather (H6), angled forwards, on its toe, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C6A		An African sacred ibis (Threskiornis aethiopicus), legs drawn towards the body (G195), with a flagellum (S45) on its back, with a feather (H6), angled forwards, on its toe, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C6C		A falcon (G5), with a cobra (Naja haja), standing up, with expanded hood (Uraeus), wearing the white crown (I55) at its toe.
13C6D		A falcon (G5) with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on its head.
13C6E		A falcon (G5) with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on its head, with a flagellum (S45) on its shoulder.
13C6F		A falcon (G5) with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on its head, with a tie or strap, used with sandals (ankh-sign, S34) angled forwards, on its toe.
13C70		A falcon (G5), wearing the double crown (S5).
13C71		A falcon (G5), wearing the double crown (S5), with an uraeus on its toe.
13C72		A falcon (G5), wearing the double crown (S5), with an flagellum (S45) on its back.
13C73		A falcon (G5), wearing the double crown (S5), on top of a standard used for carrying religious symbols (R12).
13C74		A falcon (G5), wearing the red crown (S3).
13C75		A falcon (G5), wearing the red crown (S3), with a flagellum (S45) on its shoulder.
13C76		A falcon (G5), wearing the red crown (S3), on top of a standard used for carrying religious symbols (R12).
13C77		A falcon (G5), wearing the white crown.
13C78		A falcon (G5), wearing a headdress consisiting of a sun-disk between the horns of a bovid, with two plumes on top of the sun-disk (S65/S65A).
13C79		A falcon (G5), wearing a headdress consisiting of the horns of a ram with two plumes on top of it (S62).
13C7A		A falcon (G5), wearing a headdress consisting of a sun-disk between the horns of a bovid (F102).
13C7B		A falcon (G5), wearing a headdress consisting of a sun-disk between the horns of a bovid (F102), with a flagellum (S45) on its shoulder.
13C7C		A falcon (G5), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C7D		A loop of cord with the ends upwards (V6), in front of a falcon (G5), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C7F		A feather (H6), in front of a falcon (G5), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C80		A feather (H6), in front of a falcon (G5), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13C81		A falcon (G5), on top of a standard used for carrying religious symbols (R12), with a band of string or fabric (V12) written over the pole of the standard.
13C82		A falcon (G5) with a flagellum (S45) on its shoulder, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C83		A falcon (G5), wearing a headdress consisting of a sun-disk between the horns of a bovid (F102), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C84		A falcon (G5), wearing a headdress consisting of a sun-disk between the horns of a bovid (F102), with a flagellum (S45) on the shoulder, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C85		A falcon (G5), with a headdress consisting of two plumes (S9), on top of a standard used for carrying religious symbols (R12).
13C86		A  tie or strap, used with sandals (ankh-sign, S34), written vertically, in front of a falcon (G5), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C87		A club used by washer-men for beating laundry (U36), in front of a falcon (G5), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C88		A club used by washer-men for beating laundry (U36), in front of a falcon (G5), with a flagellum (S45) on its shoulder, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13C8A		A falcon (G5), on top of a windpipe and lungs (F36), with a stem of papyrus with a flowering bud (M127) left of the windpipe and a stem of papyrus with a bud (M13) right of the windpipe, both beneath the horizontal bar.
13C8B		A falcon (G5), with a flagellum (S45) on its shoulder, on top of a windpipe and lungs (F36), with a stem of papyrus with a flowering bud (M127) left of the windpipe and a stem of papyrus with a bud (M13) right of the windpipe, both beneath the horizontal bar.
13C8C		Two falcons (G5), arranged horizontally, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13C8D		A pole of a balance, with the pole resembling a seat (U39A), in front of a falcon (G5), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13C8E		The pole of a balance, with the pole resembling a seat (Q1), with a plummet resembling a heart (F34) hanging from the cross-beam (U39F), in front of a falcon (G5), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13C8F		A pole of a balance, with the pole resembling a seat (U39A), in front of a falcon (G5), with a flagellum (S45) on its shoulder, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13C90		A pole of a balance, with the pole resembling a seat (U39A), in front of a falcon (G5), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A), on top of a parcel of land with irrigation ditches (N24).
13C91		A falcon (G5), on top of a façade of a palace or tomb, with a doorway (O33A).
13C92		An obelisk, with base (O25), in front of a falcon (G5), on top of a façade of a palace or tomb, with a doorway (O33A).
13C93		A falcon (G5) with a flagellum (S45) on its back (G6), on top of a façade of a palace or tomb, with a doorway, without internal detail (O129).
13C94		A falcon (G5) with a flagellum (S45) on its back (G6), with a tie or strap, used with sandals (ankh-sign) in front of it, vertically, on top of a façade of a palace or tomb, with a doorway, without internal detail (O129).
13C95		A façade of a palace or tomb, with a doorway, without internal detail (O129), with a falcon (G5) written inside it.
13C96		A falcon (G5), with a feather (H6), angled forward, on its toe.
13C97		A falcon (G5) on a rounded support, resembling a half round loaf of bread (X1), with a feather (H6), angled forwards, attached to the support in front of the falcon.
13C98		A falcon (G5), on top of a standard used for carrying religious symbols (R12), on top of a base with a butcher's block (T28) at the front and a sandy hill slope (N29) at the back.
13C99		A falcon (G5), on top of a standard used for carrying religious symbols (R12), on top of a butcher's block (T28).
13C9B		A falcon (G5), on top of the head of a bovid (ox) with outwards curving horns (F1).
13C9C		A falcon (G5), with a flagellum (S45) on its shoulder, on top of a head of a bovid (ox), as seen from the front (F1A).
13C9D		A falcon (G5), on top of a teardrop sized perch with the point down.
13C9E		A falcon (G5), on top of a piece of flesh, point down, curving forwards.
13C9F		A falcon (G5), on top of a long upside down cone.
13CA0		A falcon (G5), standing on top of a rectangle.
13CA1		A falcon (G5), standing on top of a ring-stand for jars (W11).
13CA2		A falcon (G5), with a flagellum (S45) on its shoulder, on top of a butcher's block (T28).
13CA3		A falcon (G5), on top of a shrine resembling a rectangular coffin with a rounded lid, and upstanding sides, without legs (Q6A).
13CA4		A falcon (G5), with a flagellum (S45) on its shoulder, on top of a wickerwork basket (V30).
13CA5		A falcon (G5), on top of a collar of beads, without detail (S12A).
13CA6		A falcon (G5), with a flagellum (S45) on its shoulder, on top of a collar of beads, without detail (S12A).
13CA7		A falcon (G5), on top of a moon sickle shaped boat, on top of a sledge that resembles a parcel of land with irrigation ditches (N24).
13CAA		A falcon (G5), on top of a standard used for carrying religious symbols (R12), with a rectangular piece of cloth with fringes on the top long side, resembling a game board (of Senet) with game pieces (Y5), attached to the back of the pole.
13CAB		A falcon (G5), on top of a standard used for carrying religious symbols (R12), with a horizontal strip of cloth with four fringes (S116) attached to the back of the pole.
13CAC		A falcon (G5), on top of a standard used for carrying religious symbols (R12), on top of a horizontal object resembling a mace or wooden column (O29).
13CAD		A falcon, standing, with extended wings at either side of the body, at a 45° angle.
13CAF		A falcon, legs drawn towards the body, with its wings extended at either side of the body.
13CB0		A falcon, standing, with extended wings at either side of the body, at a 45° angle, A block-throne with a rectangle in the far bottom corner, on top of a base (Q12).
13CB2		A falcon, with horizontally spread wings and spread legs, head looking towards the reading direction, claws on top of cartouches in a round form (V9), with a sceptres with a straight shaft and the head of the Seth animal going from the claw over the wing, vertically, the head of the staff above the wings, facing outwards.
13CB3		A falcon, with horizontally spread wings and spread legs, head looking towards the reading direction, claws on top of cartouches in a round form (V9), with one handed fans (S37) going from the claw over the wing, at an outwards angle, the head of the fan above the wing, fan curving outwards.
13CB4		A falcon, with spread wings that angle downwards and spread legs, head looking towards the reading direction, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on its head, claws on top of cartouches in a round form (V9), with one handed fans (S37) going from the claw over the wing, at an outwards angle, the head of the fan above the wing, fan curving outwards.
13CB5		A falcon, with horizontally spread wings and spread legs, head looking towards the reading direction, claws on top of cartouches in a round form (V9), with one handed fans (S37) going from the claw over the wing, at an outwards angle, the head of the fan above the wing, fan curving outwards.
13CB6		A falcon, with horizontally spread wings and spread legs, head looking towards the reading direction, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on its head, claws on top of cartouches in a round form (V9), with one handed fans (S37) going from the claw over the wing, at an outwards angle, the head of the fan above the wing, fan curving outwards, on top of a sand covered mountain over the edge of the cultivated areas (N26).
13CB7		A falcon, with spread wings that angle downwards and spread legs, head looking towards the reading direction, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on its head, claws on top of cartouches in a round form (V9), with one handed fans (S37) going from the claw over the wing, at an outwards angle, the head of the fan above the wing, fan curving outwards, written inside the sun-disk of the sun rising over a sand covered mountain over the edge of the cultivated areas (N27).
13CB8		A falcon, with spread wings that angle downwards and spread legs, head looking towards the reading direction, written inside the sun-disk of the sun rising over a sand covered mountain over the edge of the cultivated areas (N27).
13CBA		A falcon, with spread wings that angle downwards, without legs, with one handed fans with the top replaced by a feather, written over the wing, angling outwards.
13CBC		A falcon, standing, with extended wings at either side of the body, at a 45° angle, on top of a standard used for carrying religious symbols (R12).
13CBE		A falcon, seen from the side, one wing forward, horizontal, other wing forward, at a 45° downwards angle, one claw visible below the lower wing.
13CBF		A falcon, legs drawn towards the body, body nearly horizontal.
13CC0		A falcon, legs drawn towards the body, body nearly horizontal (G158), with a flagellum (S45) on its back.
13CC1		A falcon, legs drawn towards the body, body nearly horizontal (G158), with a flagellum (S45) on its back, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13CC2		A falcon, legs drawn towards the body, body nearly horizontal (G158), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13CC3		An archaic image of a falcon/cult image of a falcon, with a counter weight of a necklace (S18A) on its back.
13CC4		An archaic image of a falcon/cult image of a falcon, with a counter weight of a necklace (S18A) and a flagellum (S45) on its back.
13CC5		An archaic image of a falcon/cult image of a falcon, wearing the white crown (S1).
13CC6		An archaic image of a falcon/cult image of a falcon, wearing the white crown (S1), with a counter weight of a necklace (S18A) on its back.
13CC7		An archaic image of a falcon/cult image of a falcon, wearing the white crown with an uraeus (S1A), with a flagellum (S45) on its back.
13CC8		An archaic image of a falcon/cult image of a falcon, wearing the white crown (S1), with a counter weight of a necklace (S18A) and a flagellum (S45) on its back.
13CC9		An archaic image of a falcon/cult image of a falcon, wearing a headdress consisting of two plumes (S9), with a flagellum (S45) on its back.
13CCA		An archaic image of a falcon/cult image of a falcon, wearing a headdress consisting of two plumes on top of the horns of a ram (S62), with a flagellum (S45) on its back.
13CCC		A falcon, lying down on a bed, with a leonid head, legs and tail (Q19).
13CCF		A bird with a small tail.
13CD0		A swallow (Hirundo rustica savignii) (G36), on top of a mouth (D21).
13CD1		A house sparrow (Passer domesticus niloticus), with upwards curling line coming from the tail.
13CD3		A hoopoe (Upupa epops), without distinctive crest.
13CD4		A hoopoe (Upupa epops) (G22), upon a rectangular stone slab or brick (O39).
13CD5		A human headed bird.
13CD6		A human headed bird, with a flagellum (S45) on its shoulder.
13CD9		A human headed bird (G53A), wearing a headdress consisting of two feathers on top of the horns of a ram (S77).
13CDA		A human headed bird (G53A), wearing a headdress consisting of two feathers on top of the horns of a ram (S77), with a flagellum (S45) on its shoulder.
13CDC	13CDD	A human headed bird (G53A), wearing a headdress consisting of a sun-disk between the horns of a bovid (F102), with a flagellum (S45) on its shoulder.
13CDE		A bird with the head of a ram.
13CE0		A vulture chick, outstretched wings on either side of the body, and the legs drawn towards the body.
13CE1		A duckling, with open beak, outstretched wings above the body, and the legs drawn towards the body.
13CE2		A duck, beak closed, outstretched wingsd above the body, legs drawn towards the body.
13CE3		A duckling, with open beak, outstretched wings on either side of the body, and the legs drawn towards the body (G47), above a sickle moon shaped nest.
13CE5		Two heads of a pintail duck (Anas acuta) (H1), protruding from an oval pool or nest.
13CE6		Two heads of a pintail duck (Anas acuta) (H1), protruding from a pool or nest resembling the sky (N1).
13CE7		Two ducklings, with open beak, outstretched wings on either side of the body, and the legs drawn towards the body (G47), above a sickle moon shaped nest.
13CE9		Three heads and upper bodies of ducks protruding from a nest without internal detail.
13CEA		Three heads and upper bodies of ducks protruding from a nest with netting as internal detail.
13CEB		Three heads of a pintail duck (Anas acuta) (H1), protruding from a pool or nest resembling a wickerwork basket (V30).
13CEC		Three heads and upper bodies of ducks protruding from a nest resembling a wickerwork basket (V30).
13CED		Three heads of a pintail duck (Anas acuta) (H1), protruding from an oval pool or nest.
13CEE		Three heads of a pintail duck (Anas acuta) (H1), protruding from an oval pool or nest, with internal netting as detail.
13CEF		Four heads of a pintail duck (Anas acuta) (H1), protruding from an oval pool or nest
13CF0		A pelican.
13CF1		A saharan helmeted guinea fowl (Numida m. meleagris), without a lappet.
13CF2		A saharan helmeted guinea fowl (Numida m. meleagris), with a lappet, without the two protruding feathers on its head.
13CF4		A saharan helmeted guinea fowl (Numida m. meleagris), without a lappet, without the two protruding feathers on its head.
13CF5		A vulturine guineafowl (Acryllium vulturinum), without lappet.
13CF6		A vulturine guineafowl (Acryllium vulturinum), with lappet.
13CF7		A lapwing (Vanellus cristatus), legs drawn towards the body.
13CF8		Three lapwings (Vanellus cristatus), overlapping each other.
13CF9		A lapwing (Vanellus cristatus), standing, wings extended above the body, on top of a wickerwork basket (V30).
13CFA		A lapwing (Vanellus cristatus), legs drawn towards the body, wings extended above the body, with two human arms, extending from the breast, raised, with handpalms outwards.
13CFB		A lapwing (Vanellus cristatus), standing, wings extended above the body, with two human arms, extending from the breast, raised, with handpalms outwards.
13CFC		A griffon vulture (Gyps fulvus), without internal detail or visible wings, without a lappet.
13CFE		A griffon vulture (Gyps fulvus), without internal detail or visible wings, with a lappet.
13D00		A griffon vulture (Gyps fulvus), wearing the Atef crown (S8A).
13D02		A griffon vulture (Gyps fulvus), with a flagellum (S45) on its back, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13D03		A griffon vulture (Gyps fulvus), with a flagellum (S45) on its back, right claw extended forwards, holding a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), angling forwards, with a cartouche in a round form (V9) with the loop around the staff.
13D04		A griffon vulture (Gyps fulvus), on top of a wickerwork basket (V30).
13D05		A griffon vulture (Gyps fulvus), with a flagellum (S45) on its back, on top of a wickerwork basket (V30).
13D06		A griffon vulture (Gyps fulvus), wearing the white crown (S1), on top of a wickerwork basket (V30).
13D07		A griffon vulture (Gyps fulvus), wearing the white crown (S1), with a flagellum (S45) on its back, on top of a wickerwork basket (V30).
13D08		A griffon vulture (Gyps fulvus), with the horns of a bovid (F13) on its head, on top of an alabaster basin with a diamond shaped inner marking (W3).
13D09		A griffon vulture (Gyps fulvus), with the wings extended forward, as if to envellop someone or something.
13D0A		A griffon vulture (Gyps fulvus), with the wings extended forward, as if to envellop someone or something, without internal detail.
13D0B		A griffon vulture (Gyps fulvus), with the wings extended forward, as if to envellop someone or something, with a flagellum (S45) on its back.
13D0E		An Egyptian vulture (Neophron percnopterus), with two strokes (feathers?) in a V shape on top of its head.
13D0F		A griffon vulture (Gyps fulvus) (G14), facing a falcon (G5), on top of a wickerwork basket (V30).
13D10		A staff, with a straight shaft, topped with the head of a falcon, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on top of the head.
13D11		A staff, with a straight shaft and forked end, topped with the head of a falcon, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on top of the head.
13D12		A staff, with a straight shaft, topped with the head of a falcon, with a crescent moon with the entire moon disk (N62) on its head.
13D13		A staff, with a straight shaft and forked end, topped with the head of a falcon, with a crescent moon with the entire moon disk (N62) on its head.
13D14		A spear, with a straight shaft and forked end, topped with the head of a falcon, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) on top of the head, tail descending behind the head, with a triangular spearhead above the snake.
13D15		A spear, with a straight shaft and forked end, topped with the head of a falcon, with a triangular spearhead above the head.
13D16		A tiller of a ship, topped with the head of a falcon, angled forwards.
13D17		The eye of a falcon with the markings of the head of a falcon.
13D18		The markings of the head of a falcon, without the eye / a very schematic representation of a spine, with the spinal cord hanging out at the backside.
13D19		The head of a bird, with its mouth open, neck curling backwards.
13D1A		The head of a common crane (Grus grus).
13D1B		The head of a bird, with its mouth open, neck straight.
13D1C		The head of a bird, with its mouth open, neck straight, with a loop of cord with the ends forwards around its neck.
13D1D		The head of a crested ibis (Ibis comata), neck curving backwards.
13D1E		The head of a crested ibis (Ibis comata), with a beak with a downwards angled tip, with a crest of two long feathers on the head, neck curving backwards,
13D1F		The head of a griffon vulture (Gyps fulvus), with a straight neck.
13D23		A feather (H6), angled forwards, on top of a wide cup (W10).
13D24		A feather (H6), angled forwards, on top of strip of land, with three grains of sand written beneath it (N16).
13D25		Two feathers, connected back to back.
13D26		The leg of a bird, as if drawn towards the body.
13D28		The schematic representation of the claw of a bird.
13D29		A duckling, with open beak, outstretched wings on either side of the body, and the legs drawn towards the body (G47), inside an egg (H8).
13D2A		An egg (H8), on top of a standard used for carrying religious symbols (R12).
13D2B		The head and upper body of an owl.
13D2C		The head and upper body of a quail chick (Coturnix coturnix).
13D2D		The head and upper body of an Egyptian vulture (Neophron percnopterus).
13D2E		The head and upper body of a greater white-fronted goose (Anser albifrons).
13D2F		A snake with human arms, extended at the sides, with the forearms angling forward.
13D30		A hardun lizard (laudakia stellio), seen from the side.
13D31		A freshwater turtle, with a knife with a triangular blade and straight handle (T30A) at either leg, blade inwards.
13D32		A cult image of a crocodile (I5A), on top of a wickerwork basket (V30).
13D33		A cult image of a crocodile, wearing a headdress consisting of two feathers, connected back to back (H41) on its head.
13D34		A Nile crocodile (Crocodylus niloticus), with a feather (H6), angled forwards, on its head.
13D35		A Nile crocodile (Crocodylus niloticus), with a flagellum (S45) on its back.
13D36		A Nile crocodile (Crocodylus niloticus), with a feather (H6), angled forwards, on its head, with an spear, arrow without fletching or a harpoon without handle, written vertically, tip downwards on its back, on top of a standard used for carrying religious symbols, with a short vertical pole, with a loop under the horizontal beam, running over the vertical pole (R56).
13D37		A Nile crocodile (Crocodylus niloticus), with a feather (H6), angled backwards, on its head, on top of a standard used for carrying religious symbols (R12).
13D38		A Nile crocodile (Crocodylus niloticus), with a feather (H6), angled backwards, on its head, and a knife with triangular blade and straight handle (T30A) vertically on its back, blade towards the front, on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
13D39		A Nile crocodile (Crocodylus niloticus), with a two-barbed harpoon, with a handle on the shaft, written vertically (T21A), on its nose and on its back, near the hip.
13D3A		A Nile crocodile (Crocodylus niloticus), with a two-barbed harpoon, with a handle on the shaft, written vertically (T21A), on its nose.
13D3B		A Nile crocodile (Crocodylus niloticus), with a knife with a triangular blade and straight handle (T30A) on its nose, blade outwards.
13D3C		A Nile crocodile (Crocodylus niloticus), with a two-barbed harpoon, with a handle on the shaft, written vertically (T21A), on its back.
13D3D		A Nile crocodile (Crocodylus niloticus), with four two-barbed harpoon, with a handle on the shaft, written vertically (T21A), two on the head, two on the back.
13D3E		A Nile crocodile (Crocodylus niloticus), with its head facing backwards.
13D3F		A Nile crocodile (Crocodylus niloticus), with its head facing backwards, with a two-barbed harpoon, with a handle on the shaft, written vertically (T21A),on its head.
13D40		A Nile crocodile (Crocodylus niloticus) with its tail curved forward, below the body (I5), wearing a headdress consisting of two feathers on top of the horns of a ram (S77).
13D41		A Nile crocodile (Crocodylus niloticus) with its tail curved forward, below the body, with a two-barbed harpoon, with a handle on the shaft, written vertically (T21A),on its head and its back, near the hip.
13D42		A Nile crocodile (Crocodylus niloticus) on top of a shrine (I4), wearing the Atef crown (S8).
13D43		A Nile crocodile (Crocodylus niloticus) on top of a shrine (I4), with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on top of its head.
13D44		A Nile crocodile (Crocodylus niloticus), wearing a headdress consisting of two feathers on top of the horns of a ram (S77).
13D45		A Nile crocodile (Crocodylus niloticus), wearing a headdress consisting of two feathers on top of the horns of a ram, with a cobra (Naja haja), standing up, with expanded hood (Uraeus), with sun disks its head on either side of the feathers, orientated outwards.
13D46		Nile crocodile (Crocodylus niloticus), with the horns of a goat (F107) on its head.
13D47		Nile crocodile (Crocodylus niloticus), with the horns of a goat (F107) on its head, with a flagellum (S45) on its back.
13D48		Nile crocodile (Crocodylus niloticus), on a base, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on top of its head.
13D49		Two Nile crocodiles (Crocodylus niloticus), crossed over each other at an angle, facing outwards.
13D4A		The head of a crocodile.
13D4B		A piece of crocodile skin with horizontal spines, or tail of a crocodile (I6), written inside a plan of a crossroads in a village (O49).
13D4C		A piece of crocodile skin with vertical spines.
13D4D	13D4E	A round piece of crocodile skin with spines.
13D50		A curved piece of crocodile skin with horizontal spines.
13D51		A frog or toad (I7), on top of a cartouche in a round form (V9).
13D52		A frog or toad (I7), on top of a wickerwork basket (V30).
13D53		A tadpole (I8), on top of a cartouche in a round form (V9).
13D54		A snake.
13D55		A snake entering into an open rectangle, with the long sides of equal length.
13D56		A snake entering into an open rectangle, with the top line shorter than the bottom line.
13D57		A snake coming out of a open rectangle.
13D58		A snake with a coiled tail, coming out of an open oval.
13D59		A snake, written vertically.
13D5A		A snake with a coiled tail, written vertically
13D5B		A snake, upon two sets of legs in a walking posture, feet orientated towards the reading direction (D54).
13D5C		A horned desert viper (Cerastes cerastes) (I9), written inside an oval with a rippled line (water).
13D5D		A horned desert viper (Cerastes cerastes), written vertically.
13D5E		A horned desert viper (Cerastes cerastes) (I9), with a round loaf of bread in a wide cup (X2D) on its back.
13D5F		A horned desert viper (Cerastes cerastes) (I9), coming out of something resembling a representation of two ribs, written parallel (Aa15).
13D61		A horned desert viper (Cerastes cerastes) (I9), written inside an oval.
13D62		A horned desert viper (Cerastes cerastes) (I9), coming out of a house or enclosure (O1), written on its side.
13D63		A horned desert viper (Cerastes cerastes) (I9), upon two sets of legs in a walking posture, feet orientated towards the reading direction (D54).
13D64		A horned desert viper (Cerastes cerastes) (I9), coming out of an open rectangle, with the top line shorter than the bottom line (I24), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13D65		A snake with the head of a ram with horizontal horns.
13D67		A cobra in repose (Naja haja), with its tail curved forward below the body.
13D68		A cobra in repose (Naja haja), with human arms, extended forwards, one arm horizontal, holding a vessel with something in it, other arm above the vessel.
13D69		A cobra in repose (Naja haja) (I10), with a feather (H6) on its back, angling backwards, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13D6A		A cobra in repose (Naja haja) (I10), with a feather (H6) on its back, angling backwards, on top of a standard used for carrying religious symbols (R12).
13D6B		A cobra in repose (Naja haja) (I10), upon a rounded stick and a ?stela?, with a feather (H6) on its back, angled backwards.
13D6C		A cobra in repose (Naja haja) (I10), upon a standard resembling a folded piece of cloth (S29), with a feather (H6) vertically on its back, in line with the standard.
13D6D		A cobra in repose (Naja haja) (I10), written below a cobra in repose (Naja haja) (I10), on top of standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13D6E		A cobra in repose (Naja haja) (I10), written below a cobra in repose (Naja haja) (I10) with a feather (H6) on its back, angling backwards (I31), on top of standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13D6F		A mummy with a long curved beard, standing upright (A53), encircled by a cobra in repose (Naja Haja), with its head above the head of the mummy, with the tip of the tail in front of the mummy.
13D70		A cobra in repose (Naja haja) (I10), wearing the red crown (S3).
13D71		A house or enclosure (O1), with two long vertical lines on top of it, with a cobra in repose (Naja haja) (I10) written over the two vertical lines.
13D72		A cobra in repose (Naja haja) (I10), written over the white crown (S1), with a sun disk (N5) written between the crown and the vertical section of the tail of the cobra.
13D73		A cobra in repose (Naja haja), with its tail curved forward below the body, with a forwards downwards line of liquid coming from the mouth, which ends in a vessel in front of the tip of the tail.
13D74		A cobra in repose (Naja haja), with a forwards downwards line of liquid coming from the mouth, which ends in a vessel.
13D75		A cobra in repose (Naja haja), with its tail curved forward below the body, with a vessel in front of the tip of its tail.
13D76		A cobra in repose (Naja haja), tail facing forward, with a forwards, downwards line of liquid issuing from the mouth, on top of the right side of the sky (N44).
13D77		A cobra in repose (Naja haja), tail facing forward, with a forwards, downwards line of liquid issuing from the mouth, on top of the left side of the sky (N44-mirror).
13D78		A cobra in repose (Naja haja), tail facing forward, with a forwards, downwards line of liquid issuing from the mouth, on top of the right side of the sky (N44), facing its mirror, with the two lines of liquid ending in a vessel which sits between the two signs.
13D79		A cobra (Naja haja), standing up, with expanded hood (Uraeus).
13D7A		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), on top of a wickerwork basket (V30).
13D7B		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), with a forward, downwards dotted line coming from the mouth.
13D7C		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), with three forward, downwards lines coming from the mouth.
13D7D		Two cobras (Naja haja), standing up, with expanded hood (Uraeus) (I64), overlapping each other.
13D7E		Three cobras (Naja haja), standing up, with expanded hood (Uraeus) (I64), overlapping each other.
13D7F		Nine cobras (Naja haja), standing up, with expanded hood (Uraeus), in a row, with overlapping bodies.
13D82		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13D83		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), in front of a half round loaf of bread (X1) above an egg (H8), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13D84		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), on top of standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13D87		Three cobras (Naja haja), standing up, with expanded hood (Uraeus) (I64), overlapping each other, on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13D88		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing the white crown (S1).
13D89		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing the white crown (S1), on top of a wickerwork basket (V30).
13D8A		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing the red crown (S3).
13D8B		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing the red crown (S3), on top of a wickerwork basket (V30).
13D8C		Two cobras (Naja haja), standing up, with expanded hood (Uraeus) (I64), overlapping each other, the first cobra wearing the white crown (S1), the second cobra wearing the red crown (S3).
13D8D		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing a headdress consisting of a sun-disk between the horns of a bovid (F102).
13D8E		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing a headdress consisting of a sun-disk between the horns of a bovid (F102), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
13D8F		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing a headdress consisting of two plumes (S9).
13D90		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail, resembling a sideways number 8.
13D91		Two cobras (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), overlapping each other.
13D92		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), written over a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), angling forwards, with a cartouche in a round form (V9) with the loop around the staff.
13D93		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), wearing the red crown (S3), upon a wickerwork basket (V30).
13D94		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), wearing the red crown (S3), with a flagellum (S45) on its back, upon a wickerwork basket (V30).
13D95		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), wearing the white crown (S1), upon a wickerwork basket (V30).
13D96		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail (I12), wearing a headdress consisting of two plumes (S9).
13D97		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with its tail going forwards, over its head.
13D98		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with its tail coiled in an elaborate pattern.
13D99		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with its tail coiled, one coil horizontally, with the tail going forwards, over the head after this coil.
13D9A		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with its tail coiled, with two coils, with the tail going forwards, over the head after the second coil.
13D9B		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with its tail going backwards, over its head.
13D9D		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail, consisting of three horizontal coils, arranged vertically.
13DA0		A cobra, with the head of a bovid.
13DA1		A cobra, with the head of a bovid, with a sun disk between the horns.
13DA2		A cobra with the head of a leonid.
13DA3		A cobra with the head of a female.
13DA4		A cobra with the head of a female, with a coiled tail.
13DA5		A cobra with the head of a female, with a coiled tail, wearing a headdress consisting of the horns of a ram with two plumes on top of it (S62).
13DA7		A cobra with the head of a female, with a coiled tail, wearing a headdress consisting of two plumes (S9).
13DA8		A cobra with the head of a falcon, with a feather (H6) on its head, vertically, with a coiled tail, consisting of four coils.
13DA9		Two bows, strung together as a package, with the four tips of the bow represented as uraei.
13DAA		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), with a forward, downwards dotted line coming from the mouth (I43), on top of the corner of a wall (O38), facing its mirror.
13DAB		A cobra in repose (Naja haja), standing up.
13DAD		A cobra in repose (Naja haja), with its tail coiled up, in a single coil.
13DAE		A horned desert viper (Cerastes cerastes), with its tail coiled up in a single coil.
13DAF		A snake, with its head facing its tail, with a knife with a triangular blade and straight handle (T30A) at its neck, and two over its body in the horizontal section.
13DB1		A horned viper (Cerastes cerastes) with a coiled tail, consisting of two coils that are lower than the head.
13DB2		A horned viper (Cerastes cerastes) with a coiled tail, consisting of two coils that are of the same hight as the head.
13DB3		A snake with a coiled tail, consisting of two coils (I14), wearing the red crown (S3).
13DB4		A snake with a coiled tail, consisting of two coils, head down, with a knife with a triangular blade and straight handle (T30A), written vertically on its head, blade inwards.
13DB6		A snake, with a coiled tail, with it head in a hole.
13DB7		A cobra (Naja haja), standing up, with expanded hood (Uraeus), with a coiled tail, consisting of three coils.
13DB8		A snake with a coiled tail, consisting of three coils.
13DB9		A snake with the head of a bird, with its tail coiled around itself, with the tip of the tail below the neck.
13DBB		A snake with a coiled tail, consisting of three coils, with a knife with a triangular blade and straight handle (T30A), written vertically on each coil, blade forwards.
13DBC		A snake with a coiled tail, consisting of three coils, with the tail towards the front, below the body after the third coil, head down.
13DBD		A snake with a coiled tail, consisting of three coils, head downwards, with a knife with a triangular blade (T30A) written on each coil of the tail, at an forwards angle, blade down.
13DBE		A snake with a coiled tail, consisting of three coils, head upward, with a knife with a triangular blade (T30A) written on each coil of the tail, at an forwards angle, blade down.
13DBF		A snake with a coiled tail, consisting of three coils, head down.
13DC0		A snake with a coiled tail, consisting of three coils, head down, with a knife with a triangular blade and straight handle (T30A) written vertically on the head of the snake, blade inwards.
13DC1		A snake with a coiled tail, consisting of three coils, head down, with a knife with a triangular blade and straight handle (T30A) written vertically on the head of the snake and the tip of the tail, blade towards the back.
13DC3		A snake with a coiled tail, with three coils, head vertically down, with the tail towards the front, below the body after the third coil, head down, with a knife with a triangular blade and straight handle (T30A) written vertically on each coil, blade towards the back.
13DC4		A snake with a coiled tail, consisting of four coils.
13DCA		Two cobras (Naja haja), standing up, with expanded hood (Uraeus), facing outwards, tails coiled together, with a child, seated on nothing, forelegs separated, right arm raised with hand to mouth, left arm hanging beside the body (A17) written between the two heads.
13DCB		Two snakes, standing up, facing outwards, tails coiled together, with a plan of a crossroads in a village written between the two heads.
13DCC		A catfish with two upstanding barbels.
13DCD		A fish, at a 45° angle, with a V shaped tail, and a long and short fin on either side of the body.
13DCE		Two mullets (Mugil cephalus), arranged vertically, on top of a standard used for carrying religious symbols (R12).
13DCF		A fish with two small fins above and below the body, and one small fin on its side, near the head.
13DD0		A fish, at a 45° angle, written in an oval, at a 45° angle.
13DD1		A fish with a small fin on the top and a small and long fin on the bottom (K14), written inside an oval consisting of a wavy line.
13DD2		A fish with a small fin on the top and a small and long fin on the bottom (K14), written inside an oval.
13DD3		A fish, with its tail orientated downwards.
13DD4		A fish, with its tail orientated downwards (K4A), on top of standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13DD5		A fish, with its tail orientated downwards, with two small fins on either side of its body.
13DD6		A fish with two small fins above the body, written horizontally, on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13DD7		A fish, with its tail orientated downwards (K4A), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13DD8		A fish, with its tail orientated downwards, with two small fins on either side of its body (K4B), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13DD9		A fish with a small fin on top, angled upwards, on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13DDA		A fish with a small fin on the top and a small and long fin on the bottom.
13DDC		A fish without a head, with a tail-fin with a deep V like shape.
13DDD		A fish without head, with a tail-fin with a triangular shape.
13DDF		A fish without a head, with a tail-fin with a V like shape, with a fin at either side of the body.
13DE1		A dung beetle, scarab (scarabaeus sacer), with multiple vertical lines over the carapace.
13DE2		A dung beetle, scarab (scarabaeus sacer), with the red crown (S3) on its head.
13DE3		A dung beetle, scarab (scarabaeus sacer) (L1), holding a cartouche in the round form (V9) in its hind legs.
13DE4		A dung beetle, scarab (scarabaeus sacer) (L1), with the fully extended wing of a bird (H5) on either side of the body.
13DE5		A dung beetle, scarab (scarabaeus sacer) (L1), with the fully extended wing of a bird (H5) on one side of the body.
13DE6		A dung beetle, scarab (scarabaeus sacer) (L1), with outstretched wings.
13DE7		A dung beetle, scarab (scarabaeus sacer) (L1), with outstretched wings, which are curved upwards, connecting, making a circular form.
13DE9		A dung beetle, scarab (scarabaeus sacer), with outstretched wings (L11), on top of the sun rising over a sand covered mountain over the edge of the cultivated areas (N27).
13DEA		A dung beetle, scarab (scarabaeus sacer), with upwards pointing outstretched wings, holding a circle in its front legs.
13DEB		A dung beetle, scarab (scarabaeus sacer), with the head of a falcon.
13DEC		A dung beetle, scarab (scarabaeus sacer) with the head of a ram, wearing the red crown (S3).
13DED		The head and forelegs of a dung beetle, scarab, (scarabaeus sacer).
13DEE		A fly.
13DEF		A wasp.
13DF0		A locust, depicted as a bee without wings.
13DF1		A praying mantis.
13DF2		A scorpion.
13DF4		A scorpion (L19), on top of an arrow-head, point facing downwards, in the ground, resembling a cartouche in the round form (T22D).
13DF5		A scorpion (L19), on top of a standard used for carrying religious symbols (R12).
13DF6		A covered or winged scorpion, holding a seal, attached to a neck string (S20).
13DF7		A louse, flea or tiny insect.
13DF8		A butterfly.
13DF9		A click beetle (Lanelater Notodonta).
13DFA		A tree, with the trunk and foliage separated.
13DFB		A tree upon a wide base with upstanding edges.
13DFD		A tree, with a rippled line of water coming from both sides of the top, angling downwards.
13DFE		A shield with a round top, with a line coming from either side, angled upwards, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R56/R12A).
13DFF		A tree (M1), written over a horned desert viper (Cerastes cerastes) (I9), in front of the hind-quarters of a seated lion or leopard (F22), on top of a standard used for carrying religious symbols (R12).
13E00		A branch, horizontally written (M3), written over a tree (M1), on top of a standard used for carrying religious symbols (R12).
13E01		A tree (M1), on top of a standard used for carrying religious symbols (R12).
13E02		A tree (M1), written over a horned desert viper (Cerastes cerastes) (I9), in front of a schematic representation of three sealed water pots in a rack (W17A), on top of a standard used for carrying religious symbols (R12).
13E03		A tree without foliage.
13E05		A coniferous tree.
13E06		A tree without foliage, or a branch with many side branches.-
13E07		A branch or tree with three vertically arranged clumbs of branches.
13E08		A palm tree.
13E09		A dom palm tree (Hyphaene thebaica) with sprouts a the bottom, with dom palm dates haning from the top.
13E0A		A dom palm tree (Hyphaene thebaica) with sprouts a the bottom, with two trunks.
13E0B		A herb, with three flowers or leaves, curving upwards, all flowers/leafs at the same hight.
13E0D		A bundle of three flowers, with one flower between two buds, written horizontally.
13E0F		A branch, vertically written.
13E10		A dead tree or a branch, vertically written,  with multiple side-branches on either side.
13E11		A branch, horizontally written, with two branches in a V shape at the far end.
13E13		A branch, horizontally written, with only one side-branch near the tip.
13E14		A branch of laurel leaves.
13E15		A seed or branch, resembling an angled oval, with two bulbs on top and one at the bottom.
13E16		A clump of three reed-stalks with florets.
13E17		A flower or herb, with a side branch on either side, flowers resemble a fork.
13E19		A palm branch, stripped of leaves, without notch, curved forwards.
13E1A		A palm branch, stripped of leaves, with multiple sharp notches.
13E1B		A palm branch, stripped of leaves and notched with a sharp notch.
13E1C		A palm branch, stripped of leaves and notched with a sharp notch, with additional perpendicular lines over the section above the notch.
13E1D		A palm branch, stripped of leaves and notched with a sharp notch (M4B), on top of a half round loaf of bread (X1).
13E1E		A palm branch, stripped of leaves, with multiple sharp notches (M4A), on top of a half round loaf of bread (X1).
13E1F		A palm branch, stripped of leaves, with multiple sharp notches (M4A), on top of a circle.
13E20		A palm branch, stripped of leaves and notched with a sharp notch (M4B), on top of a stool made of reed matting (Q3).
13E21		A palm branch, stripped of leaves, with multiple sharp notches (M4A), on top of a stool made of reed matting (Q3).
13E22		A palm branch, stripped of leaves and notched with a sharp notch (M4B), on top of a plan of a crossroads in a village (O49).
13E23		A palm branch, stripped of leaves and notched with a sharp notch (M4B), on top of a circle.
13E26		A palm branch, stripped of leaves and notched with a round notch (M4), on top of a horizontal line.
13E27		A palm branch, stripped of leaves and notched with a sharp notch (M4B), on top of a stool made of reed matting, with internal detail (HGx Q3A).
13E2A		A strip of land or pond, with a lotus-bud and a lotus flower.
13E2D		A pool with lotus flowers, consisting of three vertical flowers.
13E2E		A pool with lotus flowers, consisting of three vertical flowers (M8E), written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13E2F		A pool with papyrus flowers, consisting of three vertical flowers.
13E30		A pool with papyrus flowers, consisting of three vertical flowers and two buds.
13E31		A pool with papyrus flowers, consisting of three vertical flowers and two buds, with a ripple of water written inside the oval pool.
13E32		A strip of land or pool, with lotus flowers, with three vertical flowers resembling a forked pole for supporting a roof (O30) and two buds.
13E33		A pool with lotus flowers, consisting of three vertical flowers and two stalks.
13E34		A pool with papyrus flowers, consisting of three vertical flowers and three buds, starting with the bud.
13E35		A pool with papyrus flowers, consisting of three vertical flowers and four buds.
13E36		A pool with lotus flowers, consisting of four vertical flowers and three buds.
13E38		A lotus bud, on a tree pronged stalk, with a emty stalk below and above the bud, and the main long stalk folded under itself.
13E39		A lotus bud with a long twisted stalk, which is folded under itself.
13E3A		A lotus bud, with the bud facing upwards, and the stem facing backwards.
13E3C		A lotus flower.
13E3F		A lotus flower with internal decoration, facing upwards, with a short stalk.
13E40		A lotus flower, facing upwards, with a short stalk.
13E41	13E42	A lotus flower with internal detail, facing upwards, with a long stalk.
13E43		A flower with 7 petals, on top of a long stalk.
13E44		A lotus flower with internal detail, facing forwards, with a long stalk, folded under itself.
13E45		A lotus flower with internal detail, facing forwards, with a long twisted stalk, which is folded under itself.
13E47		A lotus flower with a extended central point/a papyrus flower, facing forwards, with a long stalk, folded under itself.
13E48		A flower, seen seen from above, with seven distinct petals visible inside a wavy circle.
13E4A		A flower, seen from above, with seven petals.
13E4B		A flower with four petals in a circle.
13E4C		A flower with four petals in a circle, with the petals extending beyond the circle, with four branches with a flower in an x shape coming from the same center.
13E4E		A flower with eight petals.
13E4F		A lotus flower, facing upwards, with a short stalk, and a bud at either side.
13E51		A lotus flower, facing upwards, with a long stalk, with a bud at either side, on top of a canal (M36).
13E52		A rush or reed with two side branches, all tipped by flowers.
13E53		A flowering rush or reed with a long backwards curving stem, with a leaf on each side of the stem.
13E54		A bouquet of four lotus flowers, stacked on top of each other, with a vertical stem.
13E55		A bouquet of three lotus flowers, stacked on top of each other, with a vertical stem.
13E58		A bouquet of two lotus flowers, stacked on top each other, with a forward curving stem.
13E59		A plant with a bud, facing forwards, with a crossbar behind the bud, with a long twisted stalk, which is folded under itself.
13E5A		A plant with a bud, with a crossbar behind the bud, facing forwards, with a long stalk, folded under itself.
13E5B		A lotus flower, facing forwards, with a long stalk, folded under itself.
13E5C		A leaf, stem and root of a lotus plant, with the stem before the leaf bend forwards.
13E5D		A leaf, stem and root of a lotus plant, with a forwards, downwards line coming from the leaf.
13E5E		A leaf, stem and root of a lotus plant, with the stem before the leaf bend forwards, with the root resembling a cartouche in a round form (V9), with two angled strokes on either side of the stem.
13E5F		A leaf, stem and root of a lotus plant, with the root simplified to a horizontal line.
13E60		A leaf and stem of a lotus plant, with two branches on either side of the stem.
13E61		A leaf, stem and root of a lotus plant, with the leaf written vertically, and with the root resembling a cartouche in a round form (V9).
13E62		A leaf, stem and root of a lotus plant, with the leaf written vertically.
13E63		The leaf and stem of a lotus, with the stem curving downwards.
13E64		The leaf of a lotus.
13E65		The root of a lotus, with four stems with leaves, with the leaf written vertically.
13E66		A stem of papyrus with a bud, with two shoots at the bottom.
13E67		A thin stem of papyrus with a bud, with two shoots at the bottom.
13E6A		A stem of papyrus with a bud (M13), on top of a plan of a crossroads in a village (O49).
13E6B		A round vessel with an upstanding rim (W24), on top of a stem of papyrus with a bud (M13).
13E6C		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), on top of a stem of papyrus with a bud (M13), with the tail curling around the stem.
13E6D		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing the white crown (S1), on top of a stem of papyrus with a bud (M13), with the tail curling around the stem.
13E6E		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), wearing the red crown (S3), on top of a stem of papyrus with a bud (M13), with the tail curling around the stem.
13E6F		A pectoral in the form of three stems of papyrus with a bud (M13), written over the sky (N1).
13E70		A stem of papyrus with a flowering bud (M127), on top of a plan of a crossroads in a village (O49).
13E73		A rounded clump of three papyrus flowers.
13E74		A clump with one papyrus flower, with two buds bent down.
13E75		A stem of papyrus with a bud (M13), with a bud bend down on either side of the stem, on top of a plan of a crossroads in a village (O49).
13E76		A stem of papyrus with a bud (M13), with a downwards hanging flower coming from the bottom of the bud, on either side of the stem.
13E77		A strip of land (N17), written over a clump with one papyrus flower with a thin stem, with two buds bent down.
13E78		A stem of papyrus with a flowering bud (M129), with a bud bend down on either side of the stem, on top of a plan of a crossroads in a village (O49).
13E79		A clump with stem of papyrus with a flowering bud, with two buds bent down.
13E7A		A rounded clump of three papyrus flowers, with two buds bent down.
13E7B		A clump of three stems of papyrus with flowering buds, with the two flowers at the side angled outwards, with two buds bent down.
13E7C		Three thin stems of papyrus with a flowering bud, with two buds bend down, on top of a plan of a crossroads in a village (O49).
13E7D		A clump of three stems of papyrus with flowering buds, with two buds bent down.
13E7E		A clump of three stems of papyrus with flowering buds.
13E80		A clump with three papyrus flowers, with two buds bent down.
13E81		A clump of three papyrus flowers, with the flowers at the side orientated to the sides, with an upwards bud at either side of the group of three flowers, on a base.
13E82		A flower with a long stem, making a curve turning the flower upside down, between two stems of papyrus with a bud (M13) on top of a circle.
13E83		A bouquet of three times a lotus flower, facing upwards, with a short stalk, and a bud at either side (M224A), arranged vertically.
13E84		A flowering reed without the vertical line at the front.
13E85		A flowering reed, with two shoots at the bottom, at a 45° angle forwards.
13E86		A flowering reed (M17) and a club used by washer-men for beating laundry as part of the cleaning process (U36), connected by a network consisting of three horizontal lines and three lines going from top corner to bottom corner.
13E87		A club used by washer-men for beating laundry as part of the cleaning process (U36) and a flowering reed (M17), connected by a network consisting of four horizontal lines and three lines going from top corner to bottom corner.
13E88		A flowering reed (M17) and a mace with a pear-shaped head (T3), connected by a network consisting of four horizontal lines and three lines going from bottom corner to top corner.
13E89		A mace with a pear-shaped head (T3) and a flowering reed (M17), connected by a network consisting of four horizontal lines and three lines going from top corner to bottom corner.
13E8A		A pool or strip of land with three lotus buds, consisting of three vertical buds.
13E8B		A clump of reeds.
13E8C		A rush with two shoots, on top of a horizontal line.
13E8E		A rush with two shoots, with split ends.
13E8F		A flowering sedge (M23A), on top of a mouth (D21).
13E90		A desert plant, with two branches, with flowers on every branch, on top of a mouth (D21).
13E91		A forearm, with the palm of the hand facing upwards (D36), written over a rush with two shoots (M22).
13E92		A sedge on a horizontal base.
13E93		A sedge (M23), on top of a half round loaf of bread (X1).
13E95		A sedge (M23), written on top of a plan of a crossroads in a village (O49).
13E96		A sedge (M23), writen on top of a human placenta (a circle with lines written within horizontally) (Aa1).
13E97		A flowering sedge.
13E98		A desert plant, with four branches, with flowers on every branch.
13E99		A desert plant, with two branches, with flowers on every branch.
13E9A		A flowering sedge (M23A), on top of a half round loaf of bread (X1).
13E9B		A forearm, with the palm of the hand facing upwards (D36), written over a desert plant with a flower, without branches,
13E9D		A sedge (M23), written on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13E9E		A pod of a carob (Ceratonia siliqua) tree, with a leaf at the top.
13E9F		A root, with the point curving backwards, with a loop horizontally at the front.
13EA0		A root, with the point curving backwards, with a loop vertically at the top.
13EA1		A rhizome of a lotus, with one shoot at the bottom, curved forwards, and three lines at the top.
13EA2		A rhizome of a lotus, with one shoot at the bottom, curved forwards, and a wavy top.
13EA3		A round rhizome of a lotus.
13EA5		A grain of corn.
13EA6		An ear of corn with two long sprouts at the top.
13EA8		A bundle of flax, bound together by string with the loop and tie at the back, with a pointy bottom, angled forwards, with the bolls visible on top.
13EA9		A bundle of flax, bound together by string with the loop and tie at the front, with a pointy bottom, angled forwards, with the bolls visible on top.
13EAA		A bundle of flax stems, bound together with string near the top of the bundle, with the bolls visible on the top.
13EAB		A bundle of flax, bound together with string near the bottom of the bundle, without internal detail.
13EAC		A wide bundle of flax stems, bound together with string, loop and tie at the back, with bolls visible on the top, squared at the bottom.
13EAF		A bundle of flax stems, bound together by string, loop and tie at the back, with a widened base resembling the mirror of the letter L.
13EB0		A bundle of flax stems, bound together by string, loop and tie at the back, with a widening base and top, with the bolls visible on the top.
13EB1		A basket with three pieces of grain or fruit.
13EB2		A basket with two roes of four pieces of grain or fruit.
13EB3		A vertical bundle of reeds, with a sloped top towards the front, tied together, with the ties at the back.
13EB4		A vertical bundle of reeds, with a sloped top towards the back, tied together, with the ties at the back.
13EB5		A vertical bundle of reeds, with a sloped top towards the back, tied together, with the ties at the back, with the tops of the reeds visible.
13EB7		A vine on two forked supports.
13EB8		A vine with fruits on two forked supports, on top of a base, with the back of the vine descending to the base.
13EB9		A fig.
13EBA		A vertical head of lettuce.
13EBB		A bundle of unions, on top of a wide cup (W10).
13EBC		A head of lettuce.
13EBE		A head of lettuce, without internal detail, bottom curving backwards.
13EC0		A fig.
13EC2		The sky, upside down.
13EC3		A heap of grain on a raised floor, without internal detail, or enclosing line at the top.
13EC4		The sky with a curved line running from the tips of the top.
13EC5		Right side of the sky (or a cavern in profile).
13EC6		Three round vases with upstanding rim (W24), on top of the sky (N1).
13EC7		The sky with a star below it, connected with a vertical line.
13EC8		A circle with two horizontal lines at either side, attached to a line which curves over and back down over the sky (N1).
13EC9		A star, attached to a line which curves over and back down over the sky (N1).
13ECA		A star, attached with a vertical line to the sky, with a second shorter vertical line beside it.
13ECB		A heart (F34), attached with a vertical line to the sky, with a second shorter vertical line beside it.
13ECC		A circle with two horizontal lines at either side, attached with a vertical line to the sky, with a second shorter vertical line beside it.
13ECD		The sky, with three stars written inside it.
13ECE		The sun-disk (N5) on top of the sky.
13ECF		The sky, with ribbled lines of water coming from it.
13ED0		The sky, with three lines coming from it resembling a wick of twisted flax, consisting of 3 loops (V28).
13ED1		The sky, with three lines coming from it, resembling triangles.
13ED2		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus), written inside it, with a tie or strap, used with sandals (ankh-sign) (S34) written below the sun.
13ED3		The sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus), head below the sun-disk, with a tie or strap, used with sandals (ankh-sign, S34) with the loop around the body of the cobra, in front of the sun-disk.
13ED4		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side.
13ED5		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, with the left cobra wearing the red crown (S3) and the right cobra wearing the white crown (S1).
13ED6		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, with a tie or strap, used with sandals (ankh-sign, S34) with the loop around the body of the cobras.
13ED7		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, with a tie or strap, used with sandals (ankh-sign, S34) with the loop around the body of the cobras, with the left cobra wearing the white crown (S1), and the right cobra wearing the red crown (S3).
13ED8		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, heads below the sun-disk.
13ED9		The sun with three straight lines of sunlight coming from it (N8), with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, heads below the sun-disk, with the left cobra wearing the red crown (S3) and the right cobra wearing the white crown (S1).
13EDA		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, heads below the sun-disk, with the left cobra wearing the red crown (S3) and the right cobra wearing the white crown (S1).
13EDB		The sun, encircled by a circle of dots.
13EDD		A small circle (sun-disk) within a larger circle of dots.
13EDE		The sun, within a halo, with three beams of sunlight coming from it.
13EDF		The sun, within a halo, with three beams of sunlight coming from it, resemling lines of triangles.
13EE0		The sun, with three beams of sunlight coming from it, resemling lines of triangles.
13EE1		The sun, with a broad beam of light coming from below.
13EE2		The sun, within a halo, with two sets of three beams of light coming from it, ending in hands.
13EE3		A dotted oval.
13EE4		The sun, with two short horizontalal strokes on either side of the sun-disk, on top of an irrigation canal (N23).
13EE5		The sun, written above two horizontal strokes.
13EE6		The sun, written above a horizontal stroke.
13EE7		The sun, with the fully extended wing of a bird (H5) on either side, arranged horizontally.
13EE8		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side (N6B), with the fully extended wing of a bird (H5) on either side, arranged horizontally.
13EE9		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side (N6B), with the fully extended wing of a bird (H5) on either side, arranged horizontally, with the horns of a ram on top of the sun disk, with the white crown (S1), facing left to right, overlapping the red crown (S3), facing right to left, on top of the horns.
13EEA		The sun, with the fully extended wing of a bird (H5) on the front side, arranged horizontally.
13EEB		A sun-disk with wings at a 90° angle.
13EEC		A sun-disk with wings at a 90° angle, with an uraeus coming from the sun-disk, filling the space between the two wings.
13EED		A sun-disk with wings, angled downwards.
13EEE		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, heads below the sun-disk (N6D), with wings, angled downwards.
13EEF		The sun, with three beams of sunlight coming from it, resemling lines of triangles (N8A), with wings, angled downwards.
13EF0		The sun, with a broad beam of light coming from below (N8B), with wings, angled downwards.
13EF1		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side, heads below the sun-disk (N6D), with a broad beam of light coming from below, between the cobras, with wings, angled downwards.
13EF2		The sun, with a cobra (Naja haja), standing up, with expanded hood (Uraeus) at either side (N6B), with wings, angled downwards.
13EF3		A crescent moon, upside down, with the full moon disk upon it.
13EF5		The front half of a crescent moon.
13EF6		A crescent moon (N11), written over a star (N14).
13EF8		Three half-circles of dots written on top of each other.
13EF9		The white crown (S1), on top of a plan of a crossroads in a village (O49), written over a strip of land, with three grains of sand written beneath it (N16).
13EFA		The red crown (S3), on top of a plan of a crossroads in a village (O49), written over a strip of land, with three grains of sand written beneath it (N16).
13EFB		A strip of land, with two grains of sand written beneath it.
13EFC		A strip of land, with four grains of sand written beneath it.
13EFD		A strip of land, with three grains of sand written beneath it, arranged horizontally, and a tongue  of land (N21) written on either side of the grains of sand.
13EFE		A strip of land, with three grains of sand written beneath it, arranged as a triangle, with one grain at the top, and a tongue  of land (N21) written on either side of the grains of sand.
13EFF		A strip of land, with three grains of sand written beneath it, arranged as a triangle, with one grain at the bottom, and a tongue  of land (N21) written on either side of the grains of sand.
13F00		An oval of sand (N18), written on top of a conical loaf of bread (X8).
13F01		A tongue of land with a vertical line at the front, and the horizontal lines extending beyond the vertical line.
13F02		An irrigation canal, represented by a single line.
13F03		A parcel of land with irrigation ditches, consisiting of three horizontal lines.
13F04		A parcel of land with irrigation ditches, vertical lines at an angle.
13F05		A parcel of land with irrigation ditches, vertical lines at an angle, extending beyond the top and bottom horizontal line.
13F06		A parcel of land with irrigation ditches, consisting of three by three lines, extending beyond each other.
13F08		A standard used for carrying religious symbols, with a short vertical pole, with a loop under the horizontal beam, running over the vertical pole (R56), written on top of a parcel of land with irrigation ditches (N24).
13F0A		A parcel of land with irrigation ditches, consisiting of four horizontal lines of blocks.
13F0C		Two vertical human fingers on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F0D		The head of a man on a stick, wearing a double feather headdress (S78), with a band of string around its head, with the ties at the back, with arms coming from the head, right arm forwards, holding a crook (S38) vertically, left arm hanging beside the stick, holding a flagellum (S45) horizontally, on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F0E		A forearm, with the palm of the hand facing upwards (D36), holding a loop of cord with the ends downwards (V7), written over a tree (M1),  on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F0F		A tree (M1), in front of a piece of flesh, point down, curving forwardswards (F51), in front of hind-quarters of a seated lion or leopard (F22), arranged horizontally, on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F10		A tree (M1), in front of hind-quarters of a seated lion or leopard (F22), arranged horizontally, on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F11		A horned desert viper (Cerastes cerastes) (I9), written over a tree (M1), in front of hind-quarters of a seated lion or leopard (F22), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F12		A schematic representation of three sealed water pots in a rack (W17A), in front of a spear made into a standard, resembling a fire-drill in a piece of wood (R15A), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F13		Child, wearing the white crown (S1), seated on nothing, forelegs spread, right arm raised in front with hand to mouth, left arm hanging beside the body (A274), in front of the head of a bovid (cow), without horns, with an ear (F63), on top of a half round loaf of bread (X1), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F14		The hill country over the edge of the cultivated areas (N25), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13F15		The hill country over the edge of the cultivated areas (N25), on top of a standard used for carrying religious symbols (R12).
13F16		The hill country over the edge of the cultivated areas (N25), with a throwing-stick, or a club used by foreign people (T14), written on top of the middle hill.
13F17		The hill country over the edge of the cultivated areas, with a reed or plant on each hill.
13F18		A tie or strap, used with sandals (ankh-sign) (S34), written on top of the middle hill of the hill country over the edge of the cultivated areas (N25).
13F19		A tie or strap, used with sandals (ankh-sign) (S34), written on top of a sand covered mountain over the edge of the cultivated areas (N26).
13F1A		A throwing-stick, or a club used by foreign people (T14), written on top of a sand covered mountain over the edge of the cultivated areas (N26).
13F1B		A knife with a triangular blade and a straight handle (T30A), written vertically, blade forwards, on top of a sand covered mountain over the edge of the cultivated areas (N26).
13F1C		A horned desert viper (Cerastes cerastes) (I9), above a sand covered mountain over the edge of the cultivated areas (N26), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
13F1D		A sand covered mountain over the edge of the cultivated areas (N26), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13F1E		A sand covered mountain over the edge of the cultivated areas (N26), on top of a standard used for carrying religious symbols (R12)
13F1F		Three grains of sand, arranged in a triangle, one grain at the top.
13F20		A falcon (G5), written inside the sun-disk of the sun rising over a sand covered mountain over the edge of the cultivated areas (N27).
13F21		A knife with a triangular blade and straight handle (T30A), on top of the sun rising over a sand covered mountain over the edge of the cultivated areas (N27).
13F23		The sun rising over a sand covered mountain over the edge of the cultivated areas, with three beams of sunlight coming from it, resemling lines of triangles.
13F24		A hill, with a halo of sun rays behind it, represented as separate rays.
13F25		An enclosed hill, with a shrub at the top.
13F26		An enclosed hill, without a shrub at the top.
13F28		An enclosed hill, with a shrub at the top (N30A), on top of a conical loaf of bread (X8).
13F29		A oval road, with three shrubs resembling flowers written over it, with the middle flower oriented with the flower downwards.
13F2A		A road consisting of two horizontal lines, with three shrubs resembling flowers written over it, with the middle flower oriented with the flower downwards.
13F2C		A curving path in a V shape, with three shrubs resembling flowers written over it, with the middle flower oriented with the flower downwards.
13F2D		A curving path in a V shape, with three shrubs resembling flowers written over it, with the middle flower oriented with the flower downwards, with a loop and tie at the right side.
13F2E		A road with three shrubs at both sides.
13F2F		A rectangular road, with two ties at the top right corner, with three shrubs resembling flowers written over it, all flowers at the top.
13F30		A downwards curving rippled line.
13F31		A ripple of water (N35), written within a rectangle.
13F32		A horizontal line written within an oval.
13F33		A ripple of water (N35), written within an ova (N102), on top of a conical loaf of bread (X8).
13F34		A ripple of water (N35), written inside a canal (N36).
13F36		A canal, with vertical ripples of water written within it.
13F37		A canal, with multiple rectangles resembling battlements written on top of the sign.
13F38		A wavy canal.
13F39		A canal with a rectangular block on the top and the bottom.
13F3A		A canal (M36), written over legs in a walking posture, feet orientated towards the reading direction (D54).
13F3B		A folded piece of cloth (S29), mirrored, written over a canal (N36), with the base of the cloth and the base of the canal being at the same hight.
13F3C		A rectangular pool, with three diagonal strokes written inside.
13F3E		A row of dots, written inside a horizontal rectangle.
13F3F		A row of dots, written inside an oval.
13F40		An oval of sand, with a vertical line in the middle.
13F41		A rectangular pool, with three diagonal strokes written inside (HG N37A), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
13F43		A plan of a prehistoric building at the town of Nekhen, written as an oval, with the angled lines touching the oval (O47), on top of a conical loaf of bread (X8).
13F44		A well with a straight line of water, with a vertical line in the middle.
13F45		A well with multiple ripples of water.
13F46		An ingot of metal, with a wide round bottom.
13F47		A diamond shaped ingot of metal.
13F48		A metal T shaped basin, with shapes on the short side resembling trees.
13F4A		A lightning bolt, represented by ha horizontal line with arrows at each end.
13F4C		Two houses or enclosures, connected to each other horizontally.
13F4D		An oar, written vertically (P8), intruding with the top inside a a house or enclosure (O1).
13F4F		A feather (H6), written vertically on top of a house or domain (O1).
13F50		Three feathers (H6), written angled forwards, on top of three houses or enclosures, connected together, horizontally (O262).
13F51		Three feathers (H6), written vertically, on top of three houses or enclosures, connected together, horizontally (O262).
13F52		Three houses or enclosures, connected together, horizontally.
13F53		An enclosrue wall without battlements, with an enclosed square in the bottom right corner.
13F54		A house or domain, with only one of the two lines at the bottom.
13F56		Two rectangles, written vertically, connected together.
13F57		Three plans of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction, connected to each other.
13F59		A plan of a rectangular enclosure, with an internal rectangle in both upper corners.
13F5A		A façade of a palace or tomb, represented by two rectangles, with a horizontal line with two crossing vertical lines in it, left horizontal line lower than the top.
13F5B		Three times a half round loaf of bread (X1), written inside a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6), connected horizontally.
13F5C		Three times a half round loaf of bread (X1), written inside a plan of a rectangular enclosure, with an internal rectangle in the top corner away from the reading direction (O6), connected horizontally.
13F5D		A falcon (G5), written inside a plan of a rectangular enclosure, with an internal rectangle in the top corner away from the reading direction, on top of a wickerwork basket (V30).
13F5E		A plan of a rectangular enclosure with battlements on all sides, with an internal rectangle in the lower corner away from the reading direction
13F5F		A plan of a rectangular enclosure with battlements on three sides, with a wide base, with an internal rectangle in the top corner away from the reading direction.
13F60		A wickerwork basket (V30), on top of a plan of a rectangular enclosure, with an internal rectangle in the lower corner away from the reading direction (O6).
13F61		A wickerwork basket (V30), on a small vertical rectangle, with a wide base.
13F62		A wall of the palace, with ornamental chevaux de frise on top of the wall, with internal decoration of two rectangles, with an angled line inside, forming a point at the back.
13F63		A wall of the palace, without the ornamental chevaux de frise on top of the wall, with internal decoration of two rectangles, with an angled line inside, forming a point at the back.
13F64		a wall of the palace, without the ornamental chevaux de frise on top of the wall, with internal decoration of two rectangles, with an angled line inside, both angling downwards towards the back.
13F65		A wall of the palace, with ornamental chevaux de frise on top of the wall, without internal decoration.
13F66		a wall of the palace, with ornamental chevaux de frise on top of the wall, with two sections, the front section without internal decoration, the back with internal decoration.
13F67		A wall of the palace, with ornamental chevaux de frise on top of the wall, with internal decoration resembling two rectangles with an angled line inside, going from the front bottom corner to the top corner in the back, on a wide base, with a rectangle at the front with horizontal chevaux de frise at the front.
13F69		A wall of the placace, with ornamental chevaux de frise on top of the wall, in two sections, the front section with internal decoration, with a cloth wound on a pole, an emblem of divinity (R8) written inside the second section.
13F6A		An enclosure wall with battlements on the front, back and top, and on the vertical line of the opening.
13F6B		An enclosure wall with battlements on the front, back and top.
13F6C		An enclosure wall with battlements on the front, back and top, with blocks on the corners.
13F6D		An enclosure wall with battlements on the front, back and top, with an additional horizontal line partly covering the open section.
13F6E		A man, standing, both arms raised in front, holding a mat with a loaf of bread on each side of a vessel, written inside an enclosure wall with battlements on the front, back and top.
13F70		An enclosure wall with battlements on the front, back and top, without an internal section.
13F71		A wall, written vertically, with battlements on the front, back and top, and blocks on the corners.
13F72		A wall, written vertically, with multiple battlements on the long sides.
13F73		A wall, with multiple battlements on its long sides, written at a forward angle.
13F74		A wall, with multiple battlements on its long sides, written at a forward angle, with a forwards, downwards line coming from the top.
13F75		A wall, written vertically, with battlements on all edges.
13F76		An vertical oval, with rounded battlements.
13F78		A wall, written vertically, with a wide base and battlements on the front, back and top.
13F79		A wall, written vertically, with multiple battlements on the long sides (O36C), in front of a mace with a pear-shaped head, written vertically (T3), written on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
13F7B		A façade of a palace or tomb, with a doorway, without internal detail.
13F7C		A façade of a palace or tomb, with a doorway, open at the bottom, without internal detail.
13F7F		A façade of a palace or tomb, with a doorway.
13F80		A façade of a palace or tomb, with a doorway, without a line at the bottom.
13F81		A façade of a palace or tomb, with a doorway, with internal detail in the doorway.
13F82		Two eyes, with the markings of the head of a falcon (D10), arranged horizontally, mirroring each other,w ritten inside a wall of the palace, with internal detail in the lower half.
13F83		Two eyes, with the markings of the head of a falcon (D10), arranged horizontally, mirroring each other, written inside a wall of the palace, with ornamental chevaux de frise on top of the wall, without internal decoration.
13F84		A façade of a palace or tomb, simplyified to a horizontal line with a short crossbar in the middle, and three vertical lines at the bottom.
13F85		A façade of a palace or tomb, with an interior decoration pattern consisting of three thin horizontal beams, separated by two broad horizontal beams consisting of vertical strokes.
13F86		A façade of a palace or tomb, with a doorway, with a horizontal beam of vertical lines on top of it.
13F87		A façade of a palace or tomb, represented by a rectangle with a angled line, running from the bottom left to the top right corner, with vertical stokes below the rectangle.
13F88		The pylon of a temple.
13F89		The two towers of a pylon of a temple, divided, with a rectangular opening in each tower on the inside.
13F8A		A façade of a shrine with a flat roof, with a angled line written from the front bottom corner to the left top corner inside.
13F8B		A façade of a shrine with a flat roof, with oblique sides, with a large doorway.
13F8C		written over a façade of a shrine with a flat roof, with vertical sides, with a small rectangle in the bottom corner away from the reading direction (O21D).
13F8D		A shrine with vertical sides and a flat roof, without internal detail, with an uraeus at the front tip of the roof.
13F8E		A shrine with vertical sides and a flat roof, with a doorway, with an uraeus at the front tip of the roof.
13F8F		A façade of a shrine with a flat roof, with uraei on top, facing forwards, with oblique sides, with a large doorway.
13F90		A façade of a shrine with a flat roof, with vertical strokes on top, with oblique sides, with a small doorway.
13F91		A façade of a shrine with a flat roof, with uraei on top, facing forwards, with oblique sides, with two doorways.
13F92		A gateway, with the bottom fully closed.
13F93		A gateway, with the bottom fully closed, with internal detail.
13F94		A star (N14), on top of a gateway, with the bottom fully closed (O32B).
13F95		A gateway, with the bottom fully closed (O32B), with feathers (H8) on top, angled forwards.
13F96		A broad gateway.
13F98		A plan and façade of a hall, with a cornice with four Uraei on top, orientated forwards.
13F99		A thin plan and façade of a hall, with a cornice with four Uraei on top, orientated forwards.
13F9A		A plan and façade of a hall, with an entrance, with a cornice with Uraei on top, with half orientated forwards and half orientated backwards.
13F9C		A shrine, seen from the side, with a downwards sloping roof, on top of a sledge.
13F9D		A shrine, seen from the side, with a downwards sloping roof, with an uraeus at the front of the roof.
13F9E		A shrine, seen from the side, with a downwards sloping roof, with an uraeus at the front of the roof, with a rectangle attached to the front.
13F9F		God, in mummyform, standing upright on a platform, with a long straight beard, wearing a cap, both arms forward, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the god, vertically (C19), written inside a shrine, seen from the side, with a downwards sloping roof (O18).
13FA1		A façade of a shrine with a flat roof, with oblique sides, with a large doorway, with a pole in front, with a horizontal line over the roof, connected to the roof with three vertical lines.
13FA2		A façade of a shrine with a flat roof, with oblique sides, with a large doorway, with a pole in front, with a horizontal line over the roof, connected to the roof with two vertical lines.
13FA3		The façade of a shrine with a flat roof, with oblique sides, with a large doorway, with a long horizontal line coming from the front of the base, the front tip connected with a curved line going to the front tip of the roof, resembling a representation of two ribs, written parallel (Aa15).
13FA5		A shrine, seen from the side, with a downwards sloping roof, with an uraeus at the front of the roof (O18A), with a horizontal line over it.
13FA6		An archaic shrine, seen from the side, with a downwards sloping roof, with two vertical poles forming the front, with horizontal lines extending from the top of the roof, with short vertical lines in front of the poles.
13FA7		The emblem erected outside the temple of Min, resembling the horns of a bovid (F13) on top of a stem of papyrus with a bud (M13), with a coil of rope (V1) between the horns, connected to a cone, with a spike on top, from which a shrine with a flat roof comes forth.
13FA8		The head of a bovid, coming from an cone, with a forwards, downwards line of liquid coming grom the mouth, on top of a base.
13FA9		An archaic shrine, seen from the side, with a downwards sloping roof, with vertical poles in front of the shrine itself, connected to the shrine (O19), on top of a sledge.
13FAB		A low shrine with a round roof, with two vertical lines enclosing the top, on top of a broad base, with a vertical line on either side of the shrine, with three horziontal strokes over it.
13FAC		A tall shrine with a round roof, with two vertical lines enclosing the top, on a base, with internal decoration consisting of two rectangles, with an angled line inside, forming a point at the front.
13FAD		A tall shrine with a round roof, with two vertical lines enclosing the top, on a base, without internal decoration.
13FAE		A tall shrine with oblique sides, a triangular roof and a doorway.
13FB1		A façade of a shrine with a flat roof, with oblique sides, with a entryway resembling a smaller façade of a shrine with a flat roof, with oblique sides, with a large gateway, with a vertical rectangle added to the far side.
13FB3		A pronaos or kiosk, with a rounded roof, supported on two colums with a flower-like capital on the outside of the lower walls.
13FB4		A pronaos or kiosk, with a rounded roof, supported on four colums with a flower-like capital, connected to the lower walls.
13FB5		A schematic representation of a pronaos or kiosk, with the two columns represented by a vertical line with a crossbar.
13FB6		?The front wall and roof of a shrine?
13FB7		A booth, with the roof curving gradually.
13FB8		A booth, with the roof curving gradually, with a circle on top of the roof.
13FB9		A booth, with the roof curving gradually after a sharp corner.
13FBA		A booth with a pointed roof.
13FBB		A cloth wound on a pole, an emblem of divinity (R8), written inside a booth, with the roof curving gradually, with a circle in the center of the roof.
13FBC		A booth, with the roof curving gradually, supported by a forked pole for supporting a roof (O30), with a circle on top of the roof.
13FBD		A booth, with the roof curving gradually, with a vertical support in the middle, with a circle on top of the roof.
13FBE		An open booth supported by a forked pole for supporting a roof (O30).
13FBF		An open booth supported by a pole with an circular inset (P11A).
13FC0		A double pavilion, with flower-topped capitals on the columns, with curved roofs, without seats.
13FC1		A carrying chair (Q2), written inside a booth or pavillion, with two columns resembing a stem of papyrus with a bud (M13), with a curved roof.
13FC3		An open booth supported by a central column with a flower-like capital, with two carrying chairs (Q2) written inside, facing away from one another.
13FC4		An open booth supported by a forked pole for supporting a roof (O22C), on top of a wickerwork basket (V30).
13FC5		A booth with a round roof, supported by three columns with a flower-like capital, on top of an alabaster basin with a diamond shaped inner marking (W3).
13FC6		A booth with a wavy roof, supported by three forked poles for supporting a roof (O30), on top of a wickerwork basket (V30).
13FC9		An open booth supported by a forked pole for supporting a roof (O22C), on top of a wickerwork basket (V30), with a horizontal line, intersected in the middle by a coil of rope (V2) written left of the central pole.
13FCA		An open booth supported by a pole, on top of an alabaster basin with a diamond shaped inner marking (W4), with a horizontal line, intersected in the middle by a coil of rope (V2) written left of the central pole.
13FCB		A horizontal line, intersected in the middle by a coil of rope (V2), written within a booth, with the roof curving gradually after a sharp corner (O175D), on top of a wickerwork basket (V30).
13FCC		A booth with a round roof with a circle on top, supported by three columns with a flower-like capital, with two carrying chairs (Q2) facing away from one another, on top of an alabaster basin with a diamond shaped inner marking (W3).
13FCD		A booth with a round roof, supported by three columns, with the outer two having a flower-like capital, with the middle column resembling a rolled up herdsman's shelter of papyrus (V18), on top of an alabaster basin with a diamond shaped inner marking (W3).
13FCE		A booth with a round roof, supported by three forked poles for supporting a roof (O30), with two carrying chairs (Q2) facing away from one another, on top of a wickerwork basket (V30).
13FCF		A booth with a round roof, supported by three forked poles for supporting a roof (O30), on top of a wickerwork basket (V30).
13FD0		An open booth supported by a pole (O22), with two carrying chairs (Q2) facing away from one another, on top of an alabaster basin with a diamond shaped inner marking (W3).
13FD2		A double pavilion, with columns topped with a triangle, with two carrying chairs (Q2), facing away from one another, on top of an alabaster basin with a diamond shaped inner marking (W3).
13FD3		A double pavilion, with columns without capitals, with the rounded roofs curving upwards at the far sides, with two carrying chairs (Q2), facing away from one another, on top of an alabaster basin with a diamond shaped inner marking (W3).
13FD4		A double pavilion, with columns without capitals, with the rounded roofs curving upwards at the far sides, on top of an alabaster basin with a diamond shaped inner marking (W3).
13FD5		Two booths, with the roof curving gradually (O169A), on top of a plateau with a staircase at either side.
13FD6		A booth with a round roof, supported by two columns resembling a stem of papyrus with a bud (M13), with an abacus, on top of a base (O210A).
13FD7		A ram (Ovis longipes palaeo-aegyptiacus), standing, without a beard (E11), with a round vessel with an upstanding rim (W24) above its back, written inside a booth consisiting of the sky (N1), supported by two columns resembling a stem of papyrus with a bud (M13), with an abacus, on top of a base (O210A), on a broad base.
13FD8		A mastaba with an enclosure wall.
13FD9		A truncated pyramid.
13FDA		An obelisk, without a base.
13FDB		The benben stone.
13FDC		An obelisk of an Old Kingdom sun temple.
13FDD		A narrow shrine or stela on a base.
13FDE		A snake with a coiled tail, written vertically (I114), written inside a narrow shrine or stela on a plan of a crossroads in a village (O49).
13FDF		A column with a tenon at the top.
13FE0		A column with a base, with a tenon at the top, with a spiral, winding counter-clockwise away from its central point, ending at the right lower corner after about 1,5 turns (Z7) attached to the top, towards the front.
13FE1		A column with a tenon at the top, resembling a half circle.
13FE2		A column with a base, with a tenon at the top.
13FE4		A column resembling a stem of papyrus with a bud (M13), with an abacus, on top of a base.
13FE5		A column resembling a stem of papyrus with a bud (M13), without an abacus, on top of a base.
13FE7		A column with oblique sides, with a capital resembling a round vessel with an upstanding rim.
13FEA		The capital of a column, in the form of a lotus flower, with an abacus on top.
13FEB		A column resembling a stem of papyrus with a bud (M13), on a base, with a
13FEC		The capital of a column, in the form of the face of a human female, with hair in two curling sidelocks, with the ears of a bovid.
13FEE		A rectangular door bolt, resembling the forepart of a lion, with a rope used for locking attached to the paw of the lion, with a plummet attached to the rope.
13FEF		A lion, lying down, tail curled over the body towards the back (E23), on top of a shrine, facing its mirror, connected with a rope running from paw to paw.
13FF0		A door bolt, which curves upwards and backwards at the top.
13FF1		A door-bolt (O34), written over a circle of dots.
13FF2		A door-bolt (O34), written over two human feet and lower legs (D58), arranged horizontally.
13FF3		The corner of a wall, with battlements at the top, with the opening in the bottom right corner.
13FF4		A stairway, with a rectangle in the bottom corner.
13FF5		A fence, consisting of a base resembling a representation of two ribs, written parallel (Aa15), with four forward angled lines with a bulb on top of it, with a horizontal line through the bulbs, with a loop at the far end.
13FF6		A fence, consisting of a base resembling a representation of two ribs, written parallel (Aa15), with four forward angled lines with a bulb on written over it, starting at the bottom line.
13FF7		A fence, consisting of a base resembling a representation of two ribs, written parallel (Aa15), mirrored, with four forward angled lines with a bulb on written over it, starting at the bottom line.
13FFA		A fence, consisting of a base resembling a representation of two ribs, written parallel (Aa15), with four forward angled lines with a bulb on top of it, with a horizontal line through the bulbs, which angles downwards at both sides.
13FFB		A fence, consisting of a base resembling a representation of two ribs, written parallel (Aa15), with four forward angled lines with a bulb over it, starting at the bottom line, with a horizontal line through the bulbs, with a loop at the far end.
13FFD		A fence, consisiting of three forwards angled lines with a bulb at the end, on top of a horizontal base.
13FFE		A fence, consisiting of four forwards angled lines with a bulb at the end, on top of a horizontal base.
13FFF		A heap of grain on a raised floor, without internal detail.
14000		A granary on a raised floor, with upstanding edges, with a doorway.
14002		A rounded heap of grain on a raised floor, with lines of dots (grain?) written inside it.
14003		A rounded heap of grain on a raised floor, without internal detail.
14005		Two rounded heaps of grain on a raised floor, without internal detail, with a small bit of space between the heaps.
14006		An ornamental chevaux de frise on top of walls, with a circle dividing the top and base, with the top spreading like a flower.
14007		An ornamental chevaux de frise on top of walls, with a circle dividing the top and base, with the top spreading like a flower, with a circle on top of the tip.
14008		A tower of a fortress, with battlements on the top resembling a sand covered mountain over the edge of the cultivated areas (N26), with a forwards, downwards line coming from the front.
14009		A mastaba with two compartments under a single beam, with a doorway in both compartments.
1400A		A cult emblem of Min, a standard consisting of a coil on top of a pole, with a triangular shape topping the coil.
1400B		The emblem erected outside the temple of Min, resembling the horns of a bovid (F13) on top of a stem of papyrus with a bud (M13).
1400C		The emblem erected outside the temple of Min, resembling the horns of a bovid (F13) on top of a stem of papyrus with a bud (M13), with a sun disk (N5) between the horns, connected with a line to a cone with a spike on top.
1400D		A domed building, in the form of a quarter of a circle, with a smaller quarter circle inside the corner away from the reading direction, with an extended pole at the back, with multipe perpendicular lines on top of the curved line.
1400E		A domed building, in the form of a quarter of a circle, with a smaller quarter circle inside the corner away from the reading direction, with an extended pole at the back.
1400F		A bird trap, with two vertical poles at the back, with a line running from the front to the vertical poles, with multipe perpendicular lines on top of the curved line.
14010		A plan of a prehistoric building at the town of Nekhen, written as an oval, with the angled lines touching the oval.
14011		A plan of a prehistoric building at the town of Nekhen, written as an oval, with multiple angled lines used as internal decoration.
14012		A plan of a prehistoric building at the town of Nekhen, written as an oval, with four angled lines, touching the oval.
14014		A rectangular stone slab or brick, on top of two stone slaps or bricks, arragned horizontally, with a  bit of space between them.
14017		A plan of a crossroads in a village, with multiple lines, angled towards the back, coming from the top.
14018		A wall of a fortress, represented by a rectangle with horizontal lines as internal decoration, with battlements resembling the hill country over the edge of the cultivated areas (N25), with a loop and tie at the back.
14019		An arial view of a platform with a stairway/walkway attached to the left side.
1401A		A boat/ship, resembling a crescent moon, on top of a rectangle representing water.
1401B		A boat/ship, resembling a crescent moon, with a low cone shape inside the boat/ship: on top of a base of two blocks, on a longer horizontal line.
1401C		A boat/ship with a upwards curved prow and a horizontal stern, with a low cone shape inside the boat/ship, on top of a base of two blocks, on a longer horizontal line.
1401D		A boat/ship, resembling a crescent moon with the ends bending horizontally, with a low cone shape inside the boat/ship.
1401E		A boat/ship, resembling a crescent moon, on top of a canal (N36), with a low cone shape inside the boat/ship.
1401F		A boat/ship, resembling a crescent moon, on top of a rectangle representing water, with a vertical rectangle inside the boat/ship.
14020		An upside down boat/ship, resembling a crescent moon, on top of a rectangle representing water, with a vertical rectangle inside the boat/ship.
14021		A boat/ship, resembling a crescent moon, on top of a rectangle representing water, with a horizontal line extending front and back from the rectangle, with a carrying chair (Q2) inside the boat/ship, with an oar/rudder at the back.
14022		A boat/ship, resembling a crescent moon, on top of a canal (N36), with a carrying chair (Q2) inside the boat/ship.
14023		A boat/ship, resembling a crescent moon, with a carrying chair (Q2) inside the boat/ship, with an oar/rudder at the back.
14024		A boat/ship, resembling a crescent moon, with a tall cone inside the boat/ship.
14025		An upside down boat/ship, resembling a crescent moon, with an oar/rudder at the back.
14026		A boat/ship, resembling a crescent moon, on top of a rectangle representing water, with a carrying chair (Q2) inside the boat/ship.
14027	14028	A boat/ship, resembling a crescent moon, on top of a rectangle of water, with an oar/rudder at the back.
14029		A boat/ship, resembling a crescent moon, on top of a canal (N36), with an oar/rudder at the back.
1402B		A boat/ship with a vertical prow, and an inward curved stern, on top of a rectangle representing water, with a carrying chair (Q2) inside the boat/ship.
1402D		A boat/ship with a vertical prow, and a stern resembling the handle of a sickle (U1), on top of a rectangle representing water, with a carrying chair inside the boat/ship, with two oars/rudders at the back.
1402F		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water.
14030		A boat/ship with a prow and stern resembling the handle of a sickle (U1).
14031		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with an oar/rudder at the back.
14032		A boat/ship with an inward curved prow and stern, on top of a rectangle representing water, with an oar/rudder at the back.
14033		A boat/ship with a vertical prow and stern, on top of a rectangle representing water, with a rectangle inside the boat/ship.
14034		A boat/ship with a prow and stern resembling the handle of a handle of a sickle (U1), on top of a rectangle resembling water, with a low cone shape inside the boat/ship, with an oar/rudder at the back.
14035		A boat/ship with a vertical prow and a stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with two throwing-sticks, or clubs used by foreign people (T14) inside the boat/ship.
14036		A boat/ship with a vertical prow with a forwards, downwards rope coming from it, with a vertical stern, on top of a rectangle resembling water, with a low cone shape inside the boat/ship.
14037		A papyrus float, seen from the side, with a cross-like structure on it.
14038		A boat/ship with a vertical prow and a simple stern, on top of a rectangle resembling water, with an oval with rounded corners in the boat/ship, with an oar/rudder at the back.
14039		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a vertical pole and an oval with rounded corners inside the boat/ship, with an oar/rudder at the back.
1403A		A sedge (M23), written inside a boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water.
1403C		A boat/ship with a vertical prow and a stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a throwing-stick, or club used by foreign people (T14), two sceptres with a straight shaft, topped with the head of the Seth animal, facing backwards, and a rectangle, arranged horizontally, inside the boat/ship.
1403F		A boat/ship, resembling a crescent moon, on top of a rectangle of water, with a sail, with a cross-bar over the mast, with the mast supported by ropes on either side, with an oar/rudder at the back.
14040		A boat/ship, resembling a crescent moon, on top of a rectangle of water, with a sail, with an oar/rudder at the back.
14041		A boat/ship, resembling a crescent moon, with a sail, with a cross-bar over the mast.
14042		A boat/ship, resembling a crescent moon, on top of a rectangle of water, with a sail on top of a rectangle, with an oar/rudder at the back.
14043		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a façade of a shrine with a flat roof, with oblique sides, with a large doorway (O21B) inside the boat/ship, with an oar/rudder at the back.
14044		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a façade of a shrine with a flat roof, with oblique sides, with a large doorway (O21B) inside the boat/ship.
14045		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a shrine, seen from the side, with a downwards sloping roof, with an uraeus at the front of the roof (O18A) inside the boat/ship.
14046		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a canal (N36), with a shrine, seen from the side, with a downwards sloping roof, with an uraeus at the front of the roof (O18A) inside the boat/ship.
14048		A boat/ship, resembling a crescent moon, with the head of a falcon, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on top of the head, on top of the prow and stern, on top of a rectangle of water, with a shrine, seen from the side, with a downwards sloping roof, with an uraeus at the front of the roof (O18A) inside the boat/ship, with an oar/rudder at the back.
14049		A boat/ship, resembling a crescent moon, with the head of a falcon, with the sun, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6) on top of the head, on top of the prow and stern, prow lower than the stern, on top of a rectangle of water, with a shrine, seen from the side, with a downwards sloping roof, with an uraeus at the front of the roof (O18A) inside the boat/ship, with an animal (leonid?) standing, wearing headgear resembling two feathers on top of goat horns, on top of a standard used for carrying religious symbols (R12) in front of the shrine, with a god with the head of a falcon, standing in front of the rudders, controlling the rudders with a horizontal rope with an ureaus at the front.
1404B		A boat/ship, resembling a crescent moon, with the head of a woman, with a headdress of bovine horns with a sun disk (F102), on top of the prow and stern, on top of horizontal carrying poles, on top of a table, with a shrine within the boat/ship, with an oar/rudder at the back.
1404C		A boat/ship, resembling a crescent moon, with the head of a woman, with a headdress of bovine horns with a sun disk (F102), on top of the prow and stern, on top of horizontal carrying poles, being carried by four men, standing, two in front, two in back, with a shrine within the boat/ship, with an oar/rudder at the back.
1404D		A boat/ship, resembling a crescent moon, with the head of a woman, with a headdress of bovine horns with a sun disk (F102), on top of the prow and stern, on top of a rectangle resembling water, with a façade of a shrine with a flat roof, with oblique sides, with a large doorway (O21B) inside the boat/ship.
1404E		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a circle inside the boat/ship, with an oar/rudder at the back.
1404F		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a circle inside the boat/ship.
14050		A boat/ship with a prow and stern curved inwards, ending with a flower-like shape, with a circle inside the boat/ship.
14051		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of ripple of water, written inside a canal (N36C), with a circle inside the boat/ship.
14052		A boat/ship with a prow and stern resembling the handle of a sickle (U1), with a circle inside the boat/ship, with an oar/rudder at the back.
14054		A boat/ship with a vertical prow and stern, on top of a rectangle representing water, with a shepherds crook with a package lashed to it, with a tie above and below the package (T18) inside the boat/ship, with an oar/rudder at the back.
14056		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with two falcons (G5), arranged horizontally, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A) inside the boat/ship.
14057		A boat/ship with a prow and stern resembling the handle of a sickle (U1), with a falcon (G5) inside the ship.
14058		A boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a rectangle resembling water, with a falcon (G5) inside the ship, with a rudder at the back.
14059		A bovid, lying down, written within a boat/ship with a prow and stern resembling the handle of a sickle (U1) (P106), with a star (N14) above the back of the bovid.
1405A		Man, running, wearing the white crown (S1), looking backwards, right arm extended in front, forearm nearly horizontal, hand at the hight of the waist, left arm raised at the back, forearm nearly veritcal, handpalm inwards, with a star (N14) written above the right arm, at the hight of the head, inside a boat/ship with a prow and stern resembling the handle of a sickle (U1) (P106), with an oar/rudder at the back.
1405C		God, standing, with the head of a falcon, with with a sun-disk (N5) on his head, facing towards the back, both arms raised in front, hands held vertically, handpalms outwards, in front of a goddess, standing, with a headdress of bovine horns with a sun disk (F102), right arm forward, hand at the hight of the waist, holding a sceptre with a straight shaft, forked bottom and head of the Seth animal (S40) of the same size as the woman, left arm hanging beside the body, holding a tie or strap, used with sandals (ankh-sign, S34), at the loop (C228), inside a boat/ship with a prow and stern resembling the handle of a sickle (U1), on top of a canal (N36).
1405E		A boat/ship, resembling a crescent moon, on top of a rectangle of water, with a feather (H6) inside the boat/ship, with an oar/rudder at the back.
1405F		A boat/ship, resembling a crescent moon, on top of a rectangle of water, with a mirrored feather (H6) inside the boat/ship, with an oar/rudder at the back.
14061		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow.
14062		A moon-sickle shaped boat with an higher stern than prow, with the head of an antilope (F81) on top of the stern, on top of a rectangle representing water, with loop of cord with the ends upwards (V6) writen inside the boat.
14063		A moon-sickle shaped boat with an higher stern than prow, with the head of an antilope (F81) on top of the stern, with a low cone shape inside the boat/ship.
14065		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a rectangle representing water, with a falcon (G5) standing on top of a façade of a shrine with a flat roof, with oblique sides, with a large doorway (O21B), inside the boat/ship, with an oar/rudder at the back.
14066		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a rectangle with a broad base, with a falcon (G5) standing on top of a low cone shape, written inside the boat/ship.
14067		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a rectangle representing water, with a falcon (G5) standing on top of a tall cone shape, inside the boat/ship, with an oar/rudder at the back.
14068		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a sledge with two vertical internal strokes, with a falcon (G5) staninding on top of a rectangular shrine, inside the boat/ship, with an oar/rudder at the back.
14069		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a sledge with two vertical internal strokes, with a falcon (G5) staninding on top of a low cone shape, inside the boat/ship, with an oar/rudder at the back.
1406A		A moon-sickle shaped boat with an higher prow than stern, with a rudder, with the head of an antilope (F81) on top of the prow, facing inwards, with a downwards line with multiple horizontal lines written over it coming from the head, on top of a sledge.
1406B		A moon-sickle shaped boat with an higher stern than prow, with the head of an antilope (F81) on top of the stern, on top of a rectangle representing water, with a tall shrine with a round roof, with two vertical lines enclosing the top, on a base, without internal decoration (GID O20D), written inisde the boat/ship.
1406C		A moon-sickle shaped boat with an higher stern than prow, with the head of an antilope (F81) on top of the stern, with lines crossing the prow, on top of a sledge with two vertical internal strokes, with a tall shrine with a round roof, with two vertical lines enclosing the top, on a base, without internal decoration (GID O20D), written inisde the boat/ship.
1406D		A boat/ship, resembling a crescent moon, on top of a rectangle representing water, with a falcon with spread wings, on top of a low cone shape inside the boat/ship, embracing the cone with its wings, with an oar/rudder at the back.
1406E		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a rectangle representing water, with a falcon (G5) standing on top of a low cone shape inside the boat/ship, with an oar/rudder at the back.
1406F		A moon-sickle shaped boat with an higher stern than prow, with the head of an antilope (F81) on top of the stern, on top of a rectangle representing water, with a falcon (G5) standing on top of a low cone shape inside the boat/ship, with an oar/rudder at the front.
14070		A moon-sickle shaped boat with an higher prow than stern, with a rudder, with the head of an antilope (F81) on top of the prow, facing inwards, with a downwards line with multiple horizontal lines written over it coming from the head, on top of a sledge, with a falcon (G5), standing on top of a rectangular shrine, with vertical lines as internal detail, written inside the boat/ship.
14071		A moon-sickle shaped boat with an higher stern than prow, with the head of an antilope (F81) on top of the stern, on top of a block shaped shrine or table, with a falcon (G5) standing on top of a low cone shape inside the boat/ship, with an oar/rudder at the front.
14073		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a rectangle representing water, with a falcon with spread wings, on top of a low cone shape inside the boat/ship, embracing the cone with its wings, with an oar/rudder at the back.
14074		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, on top of a sledge represented by four vertical lines between two horizontal lines, with a falcon with spread wings, on top of a low cone shape inside the boat/ship, embracing the cone with its wings, with an oar/rudder at the back.
14075		A moon-sickle shaped boat with an higher prow than stern, with the head of an antilope (F81) on top of the prow, with a downwards line with multiple horizontal lines written over it coming from the head, on top of a sledge represented by four vertical lines between two horizontal lines, with a falcon with spread wings, on top of a low cone shape inside the boat/ship, embracing the cone with its wings, with an oar/rudder at the back.
14076		A fishersman's boat, with a net inside the boat, with internal detail.
14077		A fishersman's boat, with a net inside the boat, without internal detail.
14078		A fishersman's boat, on top of a rectangle representing water, with a net inside the boat, with internal detail.
14079		A boat/ship, on top of a rectangle representing water, with a horizontal rectangle inside the ship, with an oar/rudder at the back.
1407A		A boat/ship, on top of a rectangle representing water, with a horizontal rectangle inside the ship with vertical lines as internal decoration, with three vertical poles on top of the rectangle, with a line running from the top of the middle pole to the front and stern, forming a triangular shape, with an oar/rudder at the back.
1407C		A rectangular sail, with the mast over the sail, with two loops of rope coming from the lower beam of the sail, looping over the mast.
1407D		A sail with a shorter top than bottom, with the mast behind the sail, with a line of rigging running from the tip of both bottom corners of the sail to the mast.
1407E		A sail resembling a half circle, with the mast over the sail, with mulitple lines of rigging coming from the top beam of the sail to the mast.
14080		A rectangular sail with the mast behind the sail, with multiple lines of rigging coming from the bottom beam of the sail, running to the mast.
14081		A rectangular sail, with the mast in front of the sail, with a cross on the sail at either side of the mast.
14082		A rectangular sail with the mast behind the sail, with one line of rigging coming from the bottom corners of the sail, running to the mast.
14084		A sail, with a point at the bottom, angling backwards, with a distinct beam at the top, with a mast in front of the sail.
14085		A sail, with a point at the bottom, angling backwards, with a mast in front of the sail.
14086		A rectangular sail, with the mast over the sail, with a cross-beam over the sail, with rigging coming from the top of the sail to the mast.
14088		A sail, with a point at the bottom, angling backwards, with a distinct beam at the top, with a mast in front of the sail, with a cross-beam over the mast, with rigging coming from the top of the sail to the mast.
14089		A sail, with a point at the bottom, angling forwards, with a distinct beam at the top, with a mast in front of the sail, with a cross-beam over the mast.
1408A		A rectangular sail, with the mast in front of the sail.
1408B		A mast of a ship, with two prongs, with filling between the prongs.
1408C		A mast of a ship, with two prongs,  resembling a sceptre (S42).
1408D		A mast of a ship, with two prongs.
14090		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), written over a mast of a ship with two prongs, connected by vertical lines (P6).
14093		A stilistic representation of an arm holding an oar.
14094		An upwards bent line, with the ends broadening.
14095		A part of the steering gear of ships, with the lines horizontal and vertical.
14097		A part of the steering gear of ships, consisiting of one beam.
14099		A seat on top of a base.
1409A		A seat, with a rectangle in the bottom corner.
1409B		A seat, with a rectangle in the bottom corner, on top of a base, with an upwards tick on top of the seat.
1409C		A block-throne on a base.
1409E		A block-throne with a rectangle in the far bottom corner, on top of a base.
1409F		A seat with legs, resembling the legs of a cow.
140A0		A carrying chair with the seat in a bowl-like structure.
140A1		A carrying chair with a broad base, and open internal section.
140A3		A stool made of reed matting, with internal detail.
140A4		A carrying chair consisting of a rectangular mat, with horizontal poles.
140A5		A seat with a backwards angled backrest.
140A6		A headrest on a block.
140A8		A bed with leonid legs, inclined backwards.
140A9		An inclined bed, with a single leonid leg.
140AA		A carrying chair, seen from the top, with the legs as seen from the side.
140AB		A bed, with a leonid head, legs and tail.
140AC		A bed, with a leonid head, legs and tail, with a triangular shape on the bed.
140AD		A bed with a leonid head and leg on either side, oriented outwards.
140AF		A bed, with a leonid head, legs and tail, on top of a base, with a block-throne on the bed.
140B0		A mirror.
140B1		A mirror with a handle with curves downwards at the top.
140B2		A chest with a downwards sloping lid, with legs.
140B3		A wide cup (W10), with two maces with a pear-shaped head (T3), and a stick/staff with a cone on top with a crosbar below the cone.
140B4		A wide cup (W10), with two maces with a pear-shaped head (T3), surrounding a sceptre (S42) in the middle.
140B5		A wide cup (W10), with three feathers (H6) inside it, angling forwards.
140B6		A box or chest, on top of a sledge, with a lid that encloses the chest, with feathers (H6) on top, two angled forwards, two angled backwards.
140B7		A box, resembling a façade of a shrine with a flat roof, with oblique sides, with a large doorway (O21B), with four feathers on top of the lid, two angling forwards, two angling backwards.
140B8		An offering table or altar, with a rounded top.
140B9		A portable shrine.
140BA		A portable chest, sealed with a tie.
140BB		A rectangular coffin with a rounded lid, and upstanding sides, with legs at the sides and in the middle.
140BC		A rectangular coffin with a rounded lid, and upstanding sides, with three legs in the middle, with the rectangular section divided into four blocks.
140BD		A rectangular coffin with a rounded lid, and upstanding sides, with legs at the side, with the rectangular chest divided by a vertical line.
140BE		A rectangular coffin with a rounded lid, and upstanding sides, with legs at the side, with the top piece of a sistrum in the form of a shrine encosed by two curls on top of the lid.
140BF		A box or shrine, with a rounded lid and upstanding edges, with the head of a bubalis antilope on top of the lid.
140C1		A rectangular coffin with a rounded lid, and upstanding sides, without legs.
140C2		A rectangular coffin with a rounded lid, and upstanding sides, on a broad base.
140C3		A rectangular coffin with a rounded lid with a triangular shape on top, and upstanding sides, with legs at the side.
140C4		A rectangular chest with a round lid and upstanding sides, with legs at the side, with a band around the chest, with the loop and tie at the far side.
140C5		A rectangular chest with legs at the side and the middle, with four rectangles arranged as a square on top of it.
140C6		A brazier resembling a wide cup (W10), with a flame rising from it, flame angling towards the front.
140C7		A brazier with a flame rising from it, flame angling towards the front, represented as dots.
140C8		A brazier with a flame rising from it, flame curving forwards.
140CA		A brazier.
140CB		A low, broad brazier.
140CC		A brick-lined fireplace, on a base.
140CD		A brick-lined fireplace.
140CE		A ladder, with the uprights connecting at the bottom.
140CF		An one legged table.
140D0		Four circular items, arranged horizontally, on a one legged table with a bowl-shaped surface.
140D6		Three round vessels with an upstanding rim (W24), arranged horizontally, on top of a table resembling a gateway, with the bottom fully closed (O32B).
140D8		Four circular items, arranged horizontally, on a block table, with a cone at the far side, with a cross inside the block.
140DA		Three circular items, arranged horizontally, on a block shaped table on a base, with a cone at the far side, with internal decoration of sequentially smaller rectangles.
140DB		A table with an inwards incline of the front legs and vertical legs at the back, with a horizontal beam connecting the legs, with a vertical beam connecting the surface and horizontal beam, with a cone at the far side.
140DC		Four circular items, arranged horizontally, on top of a table with an inwards incline of the front legs and vertical legs at the back, with a horizontal beam connecting the legs, with a vertical beam connecting the surface and horizontal beam, with a cone at the far side (R3M).
140DD		Three circular items, arranged horizontally, on top of a table with an inwards incline of the legs, with a horizontal beam connecting the legs, with a vertical beam connecting the surface and horizontal beam, with a cone at the far side.
140DE		A round loaf of bread (X6), a conical loaf of bread (X8) and a round loaf of bread (X6), arranged horizontally, on top of a table with an inwards incline of the legs, with a horizontal beam connecting the legs, with a vertical beam connecting the surface and horizontal beam, with a cone at the far side.
140E1		A cluster of offerings, consisiting of a conical loaf (X8), two oval loafs (X4), a round loaf (X6), a piece of flesh (F51), a wide cup (W10) and two circular items, arranged on top of a table with an inwards incline of the legs, with a horizontal beam connecting the legs, with a vertical beam connecting the surface and horizontal beam, with a cone at the far side.
140E2		Five circular items, arranged horizontally, on top of a table with an inwards incline of the legs, with a horizontal beam connecting the legs, with multiple vertical beams connecting the surface and horizontal beam, with a cone at the far side.
140E3		Two circular items, on a table with the back leg extending beyond the surface, with a horizontal beam connecting the legs, with beams running from corner to corner, forming a cross.
140E4		Three cones, on top of a table with the legs extending beyond the surface, with a horizontal beam connecting the legs, with vertical beams between the surface and horizontal beam.
140E5		Five tall water jars, on top of a table or stand with the surface at two levels, highest at the back, with legs, three on the lower level, two at the higher level.
140E6		A cluster of offerings, consisting of a horziontal oval loaf of bread on top of two round loafs of bread, between two vertical oval loafs of bread, with a conical loaf of bread behind the vertical oval loaf, with a head of lettuce on top of the horizontal loaf, on top of a table with an inwards incline of the legs, with a horizontal beam directly under the surface, with a cone at the back.
140E7		A one-legged altar or table with a bowl-shaped surface.
140EA		A one-legged altar or table with a bowl-shaped surface, with three lines of smoke coming from the surface.
140EC		A one legged offering table with surface resembling a sand covered mountain over the edge of the cultivated areas (N26).
140ED		A one legged offering table with surface resembling a sand covered mountain over the edge of the cultivated areas (N26), with smoke/flame coming from the surface.
140EE		A one legged offering table with surface resembling a sand covered mountain over the edge of the cultivated areas (N26), with smoke/flame coming from the surface, with a pellet of incense at either side of the flame.
140EF		A one legged offering table with surface resembling a sand covered mountain over the edge of the cultivated areas (N26), with a flame in the middle, with a pellet of insence at either side of the flame, with an oval shape, angling inwards at the tips of the table surface.
140F0		A one legged offering table with surface resembling a sand covered mountain over the edge of the cultivated areas (N26), with a flame in the middle, with a pellet of insence at either side of the flame.
140F1		An one legged offering table, with upstanding rims, resembling a bowl, with multiple pellets of incense inside the bowl.
140F3		A diamond shaped censer for fumigation, written horizontally, with a forwards, downwards line at the front.
140F4		A censer for fumigation, written vertically, with a triangular shape at the top and the bottom.
140F6		A censer for fumigation, resembling the forearm and hand of a man, holding a bowl for incense with smoke rising from the bowl (R7), with the head of a falcon, with a sun disk (N5) on top of its head, at the back.
140F9		A bowl for incense, with two pellets of incense, with smoke rising from the bowl.
140FC		Three times a cloth wound on a pole, an emblem of divinity (R8), arranged horizontally.
140FD		Three times a cloth wound on a pole, an emblem of divinity (R8), arranged horizontally, on top of a wickerwork basket.
140FE		Nine times a cloth wound on a pole, an emblem of divinity (R8), arranged horizontally.
140FF		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), connected at the base of a cloth wound on a pole, an emblem of divinity (R8).
14100		Three times a cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), connected at the base of a cloth wound on a pole, an emblem of divinity (R8), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
14101		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), connected at the base of a cloth wound on a pole, an emblem of divinity (R8), in front of three vertical strokes in a triangular orientation with one stroke a the top (Z2A), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
14102		A cobra (Naja haja), standing up, with expanded hood (Uraeus) (I64), connected at the base of a cloth wound on a pole, an emblem of divinity (R8), on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
14104		A cloth wound on a pole, an emblem of divinity (R8), on top of a circle.
14105		A cloth wound on a pole, an emblem of divinity (R8), written over a butchers block with a rounded bottom, and a broad horizontal line on top, with the pole at the bottom of the block.
14106		A cloth wound on a pole, an emblem of divinity (R8), written over a butchers block with a broad base, with the pole at the bottom of the block.
14107		A cloth wound on a pole, an emblem of divinity (R8), written on top of a butchers block (T28).
14109		A cloth wound on a pole, an emblem of divinity (R8), written over top of a butchers block, with the pole at the bottom of the block, at the back of a rectangular base, with a feather (H6), angled forwards on the tip of the base.
1410A		A cloth wound on a pole, an emblem of divinity (R8), written over top of a butchers block, with the pole at the bottom of the block, on top of a rectangular base, with a feather (H6), angled forwards on the tip of the base and a sandy hill slope (N29) behind the butchers block.
1410B		A cloth wound on a pole, an emblem of divinity (R8), written between a sandy hill slope (N29) and a butchers block.
1410C		A cloth wound on a pole, an emblem of divinity (R8), written between a butchers block and a sandy hill slope (N29), with a feather (H6), angled forwards at the front tip of the butchers block.
1410D		A cloth wound on a pole, an emblem of divinity (R8), written between a butchers block and a sandy hill slope (N29), with a feather (H6) on the butchers block and hill.
1410E		A cloth wound on a pole, an emblem of divinity (R8), connected at the back of the hill country over the edge of the cultivated areas (N25), with a feather (H6) on the first hill, and a butchers block on the second hill.
14110		A feather (H6), angled forwards on top of a butchers block, on the front of a base, with a cloth wound on a pole, an emblem of divinity (R8) at the back of the base, with a half round loaf of bread (X1), above the hill country over the edge of the cultivated areas (N25), written between the butchers block and the emblem of divinity.
14111		Two times a cloth wound on a pole, an emblem of divinity (R8), arranged horizontally, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
14112		Two arms, raised upwards, with the forearms and hands vertical, handpalms inwards (D28), written over a column imitating a bundle of stalks tied together (R11), supporting a ripple of water (N35), written within an oval (N102).
14113		A column imitating a bundle of stalks tied together with a sun disk (N5) on top of it.
14114		A head-dress consisting of a sun disk with two feathers on top of the horns of a ram (S73), on top of a column imitating a bundle of stalks tied together (R11).
14115		A standard used for carrying religious symbols, with a short vertical pole, with a loop under the horizontal beam, running over the vertical pole.
14116		A pole, held upright through ropes.
14117		A standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole.
14118		A standard used for the carrying of religious symbols with the vertical stick at the far side.
14119		A mirror resembling circle on top of standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
1411B		An unknown object, resembling a piece of cloth with two bands at the back, on top of a standard used for carrying religious symbols (R12).
1411C		A half round loaf of bread (X1), above an egg (H8), in front of a star (N14), written on top of a standard in the shape of a crescent moon, with a loop under the horizontal beam, running over the vertical pole.
1411D		An unknown object, on top of a standard used for carrying religious symbols (R12).
1411E		A cobra in repose (Naja haja) (I10), upon a pole, with a feather (H6) vertically on its back, in line with the standard, on top of a base with a cone at the far side.
1411F		A feather (H6), angled forwards on top of a standard with a round top.
14120		A feather (H6), angled forwards on top of a standard with a round top, on top of a sand covered mountain over the edge of the cultivated areas (N26).
14121		A feather (H6), angled forwards on top of a standard with a round top, with outwards angled lines coming from the tip of the pole.
14122		A feather (H6), written on top of a standard with a rounded top, with a loop over the standard, on top of a sand covered mountain over the edge of the cultivated areas (N26).
14124		A feather (H6), angled forwards on top of a standard with a round top, with outwards angled lines coming from the tip of the pole, on top of the hill country over the edge of the cultivated areas (N25).
14126		A feather (H6), written on top of a standard with a rounded top, with a loop over the standard.
14128		A spear made into a standard, resembling a fire-drill in a piece of wood (U28).
14129		A spear made into a standard, resembling a fire-drill in a piece of wood (R15A), on top of a sand covered mountain over the edge of the cultivated areas (N26).
1412A		A spear made into a standard, resembling a fire-drill in a piece of wood (U28),  with outwards angled lines coming from the tip of the pole.
1412B		A spear made into a standard, resembling a fire-drill in a piece of wood (U28), with a loop over the standard.
1412C		A spear made into a standard, resembling a fire-drill in a piece of wood (U28), with a loop over the standard (Jsesh R15A), on top of a sand covered mountain over the edge of the cultivated areas (N26).
1412D		A spear made into a standard, resembling a fire-drill in a piece of wood (U28), with a loop over the standard (Jsesh R15A), on top of the hill country over the edge of the cultivated areas (N25).
1412F		A spear made into a standard, resembling a fire-drill in a piece of wood (U28), with two ties or bands of cloth at the far side of the pole.
14131		A spear made into a standard, with a circle on either side of the speartip, with a backwards curving loop over the standard.
14132		A spear made into a standard, resembling a fire-drill in a piece of wood (U28), with a floating dot at eithere side of the cone, with two ties or bands of cloth at the far side of the pole.
14136		A wig, without fillet, with two feathers and a horizontal bar, upon a pole
14137		A wig, without fillet, with a headdress consisting of two feathers on top of the horns of a ram (S77), on top of a standard used for carrying religious symbols (R12).
14138		A wig without a fillet, with a headdress consisting of two plumes (S9), with a cobra (Naja haja), standing up, with expanded hood (Uraeus) written behind the wig, upon a pole.
1413A		A wig without a fillet, with a headdress consisting of two plumes (S9), with a cobra (Naja haja), standing up, with expanded hood (Uraeus) written behind the wig, upon a pole (R18A), on top of a sand covered mountain over the edge of the cultivated areas (N26).
1413B		A wig without a fillet, with a headdress consisting of two plumes (S9), with a cobra (Naja haja), standing up, with expanded hood (Uraeus) written behind the wig, upon a pole (R18A), on top of the hill country over the edge of the cultivated areas (N25).
1413C		A wig without a fillet, with a headdress consisting of two plumes (S9), with a cobra (Naja haja), standing up, with expanded hood (Uraeus) written behind the wig, upon a pole (R18A), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1413E		A standard resembling a lotus flower with two plumes on top (R16C), with a counterweight of a necklace (S18A), hanging down from either side of the plumes and from the top of the standard.
1413F		A standard resembling a lotus flower with two plumes on top (R16C), with a counterweight of a necklace (S18A), hanging down from either side of the plumes.
14140		A standard resembling a lotus flower with two plumes on top (R16C), with a counterweight of a necklace (S18A) hanging down at the far side.
14141		A sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), with a backwards angled feather (H6)on top of the staff.
14142		A sceptre with a straight shaft, topped with the head of the Seth animal, with a fillet and a backwards angled feather (H6) on top of the staff, on top of a plan of a crossroads in a village (R109).
14144		A flower, enclosed by the horns of a bovid, upside down.
14145		Two bones of a fossil squid (belemnite), resembling two arrow-shaped points at each side of a circle.
14147		Two bones of a fossil squid (belemnite) (R22), on top of a standard used for carrying religious symbols (R12).
14148		A vertical hourglass-shaped shaped document holder.
1414C		A pedestal or plinth.
1414D		The white crown of Upper Egypt, with an uraeus.
1414E		The white crown of Upper Egypt, with an uraeus and a fillet around the crown.
1414F		The white crown of Upper Egypt, over a band of string or fabric, which makes a curl at the front.
14150		The white crown of Upper Egypt, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
14151		The white crown of Upper Egypt, witn an uraeus (S1A), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
14152		The white crown of Upper Egypt in front of a flagellum (S45), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
14153		The white crown (S1), on top of a plan of a crossroads in a village (O49).
14154		The white crown (S1), with the curl of the red crown (S89), on top of a wickerwork basket (V30).
14155		The white crown (S1), with the curl of the red crown (S89), on top of a wickerwork basket with a handle, orientated with the handle to the back (V31).
14156		A cloth wound on a pole, an emblem of divinity (R8), in front of the white crown (S1), on top of a wickerwork basket (V30).
14157		The red crown (S3), on top of a plan of a crossroads in a village (O49).
14158		The red crown, without the curl, on top of a wickerwork basket (V30).
14159		Two red crowns (S3), overlapping each other.
1415A		The red crown, without the curl, with a uraeus, with a piece of cloth at the neck.
1415B		The union of the red crown of Lower Egypt and the white crown of Upper Egypt, with the white crown worn within the red crown. (i.e., the double crown), with an Uraeus at the front.
1415C		The base of a crown.
1415D		A head-cloth, tied together at the back, with the loop and ties at the back.
1415E		A head-cloth, with a piece of cloth extending at the back.
1415F		A head-cloth, with an uraeus, extending downwards to the shoulder, with a piece of cloth extending at the back.
14160		A head-cloth, with an uraeus, with a piece of cloth extending at the back.
14161		A head-cloth, extending downwards to the shoulder, with a piece of cloth extending at the back.
14162		A long head-cloth, extending over the back.
14164		A band of cloth, used as a fillet, represented as a half-circle, with a loop and ties at the back.
14165		A band of cloth, used as a fillet, represented as a half-circle, with an uraeus at the front and a loop and ties at the back.
14166		The Atef crown.
14168		The Atef crown with rams horns and a sun disk, with a circle on top of the split top of the Atef crown.
14169		The Atef crown, on top of the horns of a ram, with an uraeus, with a sun disk on the head, facing outwards, at either side of the crown.
1416B		A crown, consisting on three horizontally arranged Atef crowns with a round bulb at the top, and the sun-disk at the base, on top of the horns of a ram, with an uraeus, with a sun disk on the head, facing outwards, at either side of the crown, with two ureai hanging downwards from the horns at either side.
1416C		A crown, consisting on three horizontally arranged Atef crowns with a round bulb at the top, and the sun-disk at the base, on top of the horns of a ram, with an uraeus, with a sun disk on the head, facing outwards, at either side of the crown.
1416D		A crown, consisting on three horizontally arranged Atef crowns, on top of the horns of a ram, with an uraeus, facing outwards, at either side of the crown.
1416E		A headdress consisting of the horns of a ram with two plumes on top of it.
1416F		A headdress consisting of two plumes, with a sun-disk between them, on top of a short vertical line.
14170		A headdress consisting of two plumes, with a sun-disk between them.
14173		A headdress consisting of four plumes on top of a head-cloth (S56).
14174		A headdress consisting of the horns of a ram with two plumes on top of it, with a sun-disk between the plumes.
14175		A headdress consisting of the horns of a ram with two plumes on top of it, angled backwards, with a sun-disk between the plumes.
14176		A headdress consisting of the base of a crown (S55), with two plumes on top of it.
14177		A headdress consisting of the base of a crown (S55), with two plumes on top of it, with a sun-disk between the plumes.
14178		A headdress consisting of two feathers on top of the horns of a ram, with a sun-disk between the feathers, with a cobra (Naja haja), standing up, with expanded hood (Uraeus), with sun disks its head on either side of the feathers, orientated outwards.
14179		A headdress consisting of two feathers on top of the horns of a ram, with a cobra (Naja haja), standing up, with expanded hood (Uraeus), with sun disks its head on either side of the feathers, orientated outwards.
1417A		A head-dress consisting of a sun disk with two feathers on top of the horns of a ram.
1417B		A head-dress consisting of a sun disk with two feathers.
1417C		A headdress consisting of two feathers on top of the horns of a ram.
1417D		Two feathers without quil, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
1417E		A headdress consisting of two feathers on a short vertical line.
1417F		A headdress consisting of two feathers on top of the horns of a ram, with a sun-disk between the feathers, with a cobra (Naja haja), standing up, with expanded hood (Uraeus), on either side of the feathers, orientated outwards, on top of a half-circle shaped base.
14180		A sun disk (N5), between two feathers (H6), orientated outwards, on top of the horns of a ram (F107).
14181		A dung beetle, scarab (scarabaeus sacer) (L1), with the fully extended wing of a bird (H5) on either side of the body (L12A), on top of a sun-disk, on top of the horns of a ram, with a feather (H6), orientated outwards, at either side of the scarab, connected to the tip of the wings.
14182		A band of cloth, used as a fillet, with a flower at the front, and two loops at the back.
14183		A band of cloth, used as a fillet, with an ureaus at the front, and a loop and string at the back.
14184		A band of cloth, used as a fillet, with two uraei at the front, one wearing the white crown (S1), one wearing the red crown (S3).
14185		A band of cloth, used as a filltet, decorated with flowers on the band.
14186		A garland of laurel leaves.
14187		A collar of beads without internal decoration or terminals.
14189		A collar of beads, resembling a basket, with ornamental terminals, resembling the head of a falcon, looking outwards.
1418A		A collar of beads with ornamental terminals, resembling the head of a falcon, looking outwards.
1418B		A collar of beads with ornamental terminals, resembling the head of a falcon, looking outwards, with outwards curving ties on top of the heads.
1418C		A collar of beads with ornamental terminals, resembling the head of a falcon with a sun-disk on its head, looking outwards, with two vertical strings with flower-like end-pieces, curving outwards and downwards, coming from the collar.
1418D		A necklace, consisting of a deep loop, with 7 beads or scircles on the loop, with the two top beads connected with a line with a bead in the center, with the ends of the loops curving outwards and downwards, with flower-like end-pieces, the left one a lotus bud and the right one a flowering lotus bud.
1418E		A necklace, consisting of a deep loop with three horizontal lines over it, with the capital of a column, in the form of the face of a human female, with hair in two curling sidelocks, with the ears of a bovid (O215) at the bottom of the loop, connected with a rectangle to the lowest horizontal line, with ornamental terminals resembling the same capital (O215), with outwards and downwards curving strings with flower-like end-pieces coming from the terminals.
14190		A necklace, consisting of a deep loop, with multiple horizontal strands connecting the two sides of the loop, with the ends of the loops curving outwards and downwards, with flower-like end-pieces.
14191		A necklace, consisting of a deep loop, with multiple horizontal strands connecting the two sides of the loop, connected by vertical lines (beads/netting), with the ends of the loops curving outwards and downwards, with flower-like end-pieces.
14192		Two round vessels with a thin, upstanding rim, arranged horizontally, connected by the rims by an horizontal line of string, curving upwards at both ends.
14194		A necklace, consisting of two eyes, with the markings of the head of a falcon, with a vertical line at the back with a triangle at the curl, as if seen from the front (Jsesh D10C), with an outwards and upwards loop of string, with a flower-like end-piece at the end of the string, orientated towards each other.
14195		A collar of beads, without detail.
14196		A pectoral, resembling the sky (N1), with three vertical lines of short horizontal strokes, with V shaped arrows on top of the sky.
14197		The sky (N1) / a pectoral resembling the sky, with three vertical lines consisting of short horizontal strokes, with upwards arrows at the bottom of the vertical lines, with three triangular shapes on top of the sky.
14199		A pectoral resembling the sky (N1) over three vertical lines with a forked top.
1419A		A pectoral of four glass or faïence beads.
1419C		A pectoral consisting of a big rectangle with six strands of glace or faience beads.
1419D		A pectoral, consisting of a horizontal rectangle with three vertical strands of glass or faience beads.
1419E		A girdle worn by various gods, with three hanging pieces.
1419F		A bead necklace with a counterweight, written horizontally.
141A1		A bead necklace with a counterweight, written horizontally, with a loop at the front, with the head of woman, with a headdress of bovine horns with a sun disk (F102) and the vulture headdres on top of the counterweight.
141A2		A bead necklace with a counterweight, written horizontally, with the head of woman, with a headdress of bovine horns with a sun disk (F102) and the vulture headdres on top of the counterweight.
141A4		An amulet resembling a shrine with a flat roof, with two ties at the top.
141A5		An amulet resembling a shrine with a flat roof, with a rectangle on the roof, with two outwards, downwards curving ties at the top, with an uraeus at either side of the base, with the front uraeus wearing the white crown (S1), and the back uraeus wearing the red crown (S3).
141A6		An amulet resembling a shrine with a flat roof, with a rectangle on the roof, with two outwards, downwards curving ties at the top, with flower-like end-pieces.
141A7		An amulet resembling a shrine with a flat roof, with a rectangle on top, without ties.
141A8		A seal, attached to a neck string, with the sting curving backwards and downwards, without a loop below the seal.
141A9		A seal, attached to a neck string, without ties below the seal.
141AA		A shoulder knot, with a loop at every corner.
141AB		A shoulder knot, with loops a the top ends and ties at the bottom ends, with circular blobs on top of the middle section, and vertical strokes below the vertical section.
141AC		A horizontal rectangular garment (bracelet, armband), with downwards curving ties at both sides.
141AD		An curved rectangular garment (bracelet, armband), with downwards cuving ties at both sides, with triangular end-pieces at the ties.
141AF		A kilt or apron, with the side sections forming a sickle shape.
141B0		A vertical rectangular garment, with fringes at the bottom, and two outwards curving ties at the top.
141B1		A loop of cord with the ends upwards (V6), written between the fringes of cloth resembling forked staffs, horizontal loop of twisted cloth connecting the two fringes, written over the loop of cord.
141B2		A horizontal strip of cloth with one fringe.
141B3		A horizontal strip of cloth with three fringes.
141B4		A horizontal strip of cloth with four fringes.
141B5		A horizontal strip of cloth with five fringes.
141B6		A horizontal strip of cloth with six fringes.
141B7		A bag or a sack of linen, with a tie towards the front, and a (small) loop at the back (V33), written between two staves or poles with a two-pronged end piece for holding down snakes (U116), on top of a standard used for carrying religious symbols, with a short vertical pole, with a loop under the horizontal beam, running over the vertical pole (R56).
141B8		A horizontal strip of cloth with fringes, written over a folded piece of cloth (S29).
141B9		A horizontal strip of cloth with fringes, above a folded piece of cloth (S29).
141BA		A knife sharpener with a rounded body and a loop at the back (T31), on top of legs in a walking posture, feet orientated towards the reading direction (D54), written with the tip over a folded piece of cloth (S29).
141BD		A knife sharpener with a rounded body and a loop at the back (T31), on top of legs in a walking posture, feet orientated towards the reading direction (D54), written with the tip over a folded piece of cloth (S29), with the short end towards the front.
141BE		A knife sharpener with a rounded body, without a loop at the back, on top of legs in a walking posture, feet orientated towards the reading direction (D54), written with the tip over a folded piece of cloth (S29).
141BF		A rectangular piece of cloth, with sloping fringes on its front short side, with a cross shape as internal decoration.
141C0		A square piece of cloth, with long sloping fringes on its front short side, with a cross shape as internal decoration.
141C1		A rectangular piece of cloth, with fringes at the short sides, and a stripe over the middle, written horizontally.
141C2		A rectangular piece of cloth, with fringes at the top and far side.
141C3		An oval piece of cloth, with fringes on the top.
141C5		A tie or strap, used with sandals (ankh-sign, S34), with the sun, within a halo (N5) on top of the loop.
141C6		A vessel with a spout and broad rim, made to resemble a tie or strap, used with sandals (ankh-sign, S34).
141C7		A tie or strap, used with sandals (ankh-sign, S34), with a column imitating a bundle of stalks tied together (R11) replacing the vertical strap.
141C8		A tie or strap, used with sandals (ankh-sign, S34), a column imitating a bundle of stalks tied together (R11) and a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), arranged horizontally, on top of a wickerwork basket.
141C9		A tie or strap, used with sandals (ankh-sign, S34) and a sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), arranged horizontally, on top of a wickerwork basket.
141CB		A sun shade, made of a staff which holds feathers at the top, resembling a half circle, without internal detail, with outwards angled lines coming from the tip of the staff.
141CC		A sun shade, made of a staff which holds feathers at the top,  resembling a half circle, without internal detail, with an additional short staff.
141CD		A sun shade, made of a staff which holds feathers at the top, resembling a half circle, without internal detail, with strokes on top of the feahers, with an additional short staff.
141CE		A line with a straight hook, written over a sun shade, made of a staff which holds feathers at the top, resembling a half circle, without internal detail (S36).
141CF		A one handed fan with a straight handle.
141D0		A one handed fan, with a handle consisting of a bowl-shaped section with a vertical downwards line at either side, with a straight handle section.
141D2		A one handed fan, with a handle resembling a stem of papyrus with a bud (M13), and the fan resembling a feather (H6).
141D4		Two sceptres with a straight shaft, a forked base, topped with the head of the Seth animal (S40), written on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
141D5		A sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), under a crescent moon (N11) shape topped with a feather (H6).
141D6		A sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), under a crescent moon (N11) shape topped with a feather (H6), written on top of a standard used for the carrying of religious symbols with the vertical stick at the far side (R92A).
141D7		A sceptre with a straight shaft, a forked base, topped with the head of the Seth animal (S40), under a double crescent moon (N11), arranged vertically, topped with a feather (H6), written on top of a standard used for the carrying of religious symbols (R12).
141D8		A sceptre with two extensions at the top.
141DA		A sceptre (S42), with a flagellum (S45) at the back of the headpiece.
141DB		A staff which widens towards the bottom, with a vertical stroke just above the base, extending beyond the staff.
141DC		A staff which widens towards the bottom, without internal detail.
141DD		A sceptre or staff, with a cone shape above the handle.
141DE		A sceptre, with a short handle with a crosbar, with a long oval shape above the handle.
141DF		A staff, which widens towards the bottom (S43) with a flagellum (S45) attached to the back.
141E0		A staff, which widens towards the bottom, with a circle on top, with a flagellum (S45) attached to the back.
141E1		A type of garment.
141E2		An object resembling a drill, with a crossbar on the lower section.
141E3		A flagellum (S45), written through a cartouche in a round form (V9).
141E4		A circle with a flagellum (S45) coming from the top.
141E5		An object consisting of a horizontal rectangle with a triangular indentation at the bottom, with a vertical line on top of it, with a horizontal line on top of the vertical line, to the same width of the rectangle.
141E6		An object consisting of a horizontal rectangle with a triangular indentation at the bottom, with a vertical line on top of it, with a half-circle on top of the vertical line.
141E7		An object consisting of a horizontal rectangle with a triangular indentation at the bottom, with a vertical line on top of it, with a short horizontal line crossing the vertical line.
141E8		An object consisting of two connected triangles, arranged horizontally with the points down, with a vertical line on the top, with a half-circle on top of the vertical line.
141E9		A type of ornament, with a tie and a loop at the  top, and fringes at the bottom.
141EA		A type of ornament, with two ties at the top, with a broad base.
141EB		A type of ornament, with two ties at the  top, and fringes at the bottom.
141EC		A type of ornament, with two ties at the top, with two horizontal lines at the bottom, with the vertical lines extending beyond the horizontal lines.
141ED		A cone-shaped object with two ties at the top, with a forwards, downwards dotted line coming from the top
141EE		Some type of cone-shaped ornament, with netting as internal decoration, on top of a base, with a two ties at the top, the front short and angled upwards, the tie at the back forming a curl.
141F0		A type of ornament, with two ties at the top, connected to a circle, with an oval passing over the vertical lines.
141F1		The curl of the red crown.
141F2		A bracelet.
141F3		A rectangular/square piece of cloth with wavy fringes on top.
141F4		A vertical piece of cloth which bends forward towards the top.
141F5		An necklace, resembling the sun, with thee beams of sunlight coming from it, with an arm at either side, holding a staff or line at an outwards angle, connected to a base, with the head of a canine on the front staff.
141F6		A birthing brick, with the head of a female on top of it, in the front, with a headdress of bovine horns with a sun disk (F102).
141F7		A mace with a pear-shaped head, written vertically, with a round blade attached to the front of the mace-head.
141FB		A mace with a pear-shaped head, written vertically, with an upwards tick at the front of the mace-head.
141FC		A mace with a pear-shaped head, written vertically, with an upwards and inwards curving line at the front of the mace-head, and an upwards tick at the back.
141FD		A coil of rope (V1), written behind the macehead of a mace with a pear-shaped head, written vertically (T3).
141FE		A mace with a pear-shaped head, written vertically, on top of a circle, with two branches at either side of the handle.
141FF		A mace with a pear-shaped head, written vertically (T3), on top of a collar of beads (S12).
14200		Three maces with a pear-shaped head, written vertically (T3), arranged horizontally, on top of a standard used for carrying religious symbols, with a short vertical pole, with a loop under the horizontal beam, running over the vertical pole (R56).
14201		A shield with a rounded top, with a boss in the center.
14202		An round shield with a boss in the center, with a veritcal line dividing the two halves of the shield.
14203		An oval shield with a boss in the center, with a vertical line dividing the two halfs of the shield.
14204		An axe with a round blade and a straight handle.
14205		A strainer, written horizontally, with the handle towards the front, curving downwards and backwards.
14206		The broad blade of an axe.
14207		A counterweight of a necklace.
14208		A bow, written horizontally, with a rectangular central section, with the tips of the bow angling downwards.
14209		A bow, written horizontally, with a small rectangle in the middle, with the two limbs angling downwards, with upwards coiled tips.
1420A		A bow, written horizontally, with a rectangular central section, with the tips of the bow forming a bowl-shaped loop (T9), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
1420B		Two horizontal horns, with a curved line of dots running from tip to tip.
1420C		An arrow with rectangular fletching.
1420D		An arrow with a splayed arrowhead, and rounded fletching.
1420E		An arrow with rectangular fletching, written vertically, on top of a plan of a crossroads in a village (O49).
1420F		An arrow or dart, with the tip slowly tapering out towards the back.
14210		An hexagon, with a loop at each corner.
14211		An arrow with rectangular fletching (T11B), written with the fletching towards the front, with the arrowhead inside a hexagon with curling corners.
14213		Two arrows, crossing, with the fletching at the top, on top of a standard used for carrying religious symbols (R12).
14214		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
14215		A shield with a rounded top, on top of a base, with an arrow with the fletching upwards sticking out at either side.
14216		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62), on top of a standard used for carrying religious symbols, with a short vertical pole, with a loop under the horizontal beam, running over the vertical pole (R56).
14217		A shield with a rounded top, with two arrows crossed over it, arrowheads at the top, on top of a standard used for carrying religious symbols, with a short vertical pole, with a loop under the horizontal beam, running over the vertical pole (R56).
14218		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62), in front of a clump of three papyrus flowers, with two buds bent down (M15), on top of a standard used for carrying religious symbols (R12).
14219		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62), in front of a sedge (M23), on top of a standard used for carrying religious symbols (R12).
1421B		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62) in front of a mouth (D21) written over a sedge (M23), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1421C		A shield with a rounded top, with two arrows crossed over it, fletching upwards (T62) in front of a sedge (M23), on top of a standard used for carrying religious symbols (R12), on top of a parcel of land with irrigation ditches (N24).
1421D		A shield with a rounded top, with two arrows crossed over it, fletching upwards
1421F		An oval shield on a pole, with two arrows crossed over the shield, fletching towards the front.
14220		A cover of a quiver, with a vertical loop on top.
14221		A cover of a quiver, resembling two rectangles.
14222		A cover of a quiver, with the rope cuving into a loop.
14223		The cover of a quiver, with a rectangular inset with vertical lines inside, with the rope on top making an sharp corner.
14224		A bow-string, coiled up, with both ends angling downwards.
14225		A bow-string, coiled up, with both ends angling downwards, with a large loop at the ends.
14226		A bow-string, coiled up, with both ends angling downwards, with a small round loop with a short piece of rope continuing at the ends.
14227		A bow-string, coiled up, with both ends angling downwards, with a small round loop with a short piece of rope continuing at end at the back.
14228		A bow-string, coiled up, with both ends angling downwards, with cone-shaped loops at the ends.
14229		Pieces of wood, lessening in width towards the top, lashed together with the top piece at an angle, with a loop at the back.
1422A		A throwing-stick, or a club used by foreign people, on top of a horizontal line.
1422B		A band of string or fabric, which coils at the front, written over three throwing-sticks, or a clubs used by foreign people (T14), arranged horizontally.
1422D		Two throwing-sticks, or a clubs used by foreign people (T14), arranged horizontally, on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
1422E		A fire-drill in a piece of wood (U28), written between two times pieces of wood, lashed together with the top piece at an angle, with a loop and tie at the back (T13), with the first mirrored, on top of a base.
1422F		A scimitar (khopesh), written vertically, blade curving inwards, with the head of a falcon with a sun-disk (N5) on top of it.
14230		A scimitar (khopesh), written vertically, blade curving forwards, with the head of a falcon with a sun-disk (N5) on top of it.
14231		A scimitar (khopesh), written vertically, blade curving inwards, with the head of a falcon with he sun within a halo, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6), on top of the head.
14232		A scimitar (khopesh), written vertically, blade curving forwards, with the head of a falcon with he sun within a halo, encircled by a cobra (Naja haja), standing up, with expanded hood (Uraeus) (N6), on top of the head.
14233		A scimitar (khopesh), written vertically, blade curving inwards, with the head of a falcon, wearing the double crown (S5), on top of it.
14234		A scimitar (khopesh), written vertically, blade curving forwards, with the handle resembling a one-legged table.
14235		A shepherds crook with a package lashed to it, with the top tie replaced by a knife (T30A).
14236		A shepherds crook with a package lashed to it, with a tie above the package.
14237		A shepherds crook with a package lashed to it, with the top tie replaced by a knife (T30A), on top of legs in a walking posture, feet orientated towards the reading direction (D54)
14238		A harpoon-head with two horizontal strokes on top and a two angled strokes as point.
1423A		A harpoon-head with two horizontal strokes on top, and triangular point.
1423B		A harpoon-head with two horizontal strokes on top, and triangular point on top of a circle.
1423C		A harpoon-head with two horizontal strokes on top and a single curl on top of the point .
1423D		A harpoon-head with two horizontal strokes on top and two curls on top of the point .
1423E		A harpoon-head with two horizontal strokes on top and a single curl on top of the point (T19), on top of a circle.
1423F		harpoon-head with two horizontal strokes and two angled strokes on top of the point
14240		A harpoon-head with two horizontal strokes on top and a single curl on top of the point (T19), on top of a plan of a crossroads in a village (O49).
14241		A two-barbed harpoon, with a handle on the shaft, written horizontally.
14242		A two-barbed harpoon, with a handle on the shaft, written vertically, on top of a half-circle.
14243		A two-barbed harpoon, with a handle and curl on the shaft, written horizontally.
14245		A two-barbed harpoon, with a rectangle and and angled line on the shaft, written horizontally.
14246		A two-barbed harpoon, with a rectangle and and a curl on the shaft, curling towards the shaft, written vertically.
14247		A two-barbed harpoon, with a handle and curl on the shaft, with a bowl-like curved section at the end, written horizontally.
14248		An spear, arrow without fletching or a harpoon without handle, written vertically, tip upwards.
14249		An spear, arrow without fletching or a harpoon without handle, written horizontally.
1424A		A two-barbed arrow-head, with the point facing downwards, in the ground, resembled by a half-circle.
1424B		A two-barbed arrow-head, with the point facing downwards, in the ground, resembling a half-circle, with only the barbs visible.
1424C		An two-barbed arrow-head, point facing downwards, in the ground, resembling a cartouche in the round form (V9), with only the barbs visible.
1424D		An arrow-head, point facing downwards, in the ground, resembling a cartouche in the round form (V9).
14250		An spear, arrow without fletching or a harpoon without handle, written horizontally, on top of a  boat/ship, resembling a crescent moon, with an oar/rudder at the back, connected by three lines, on a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
14251		An spear, arrow without fletching or a harpoon without handle, written horizontally, on top of a crescent moon shape, connected by three lines, point towards the front, on a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole (R12A).
14253		An spear, arrow without fletching or a harpoon without handle, written horizontally, on top of a crescent moon shape, connected by four lines, with the point towards the back, in front of a feather (H6), angled forwards on top of a standard with a round top, with outwards angled lines coming from the tip of the pole (R14B), on top of a standard used for carrying religious symbols, with a loop under the horizontal beam, running over the vertical pole.
14256		An spear, arrow without fletching or a harpoon without handle, written horizontally, on top of a crescent moon shape, connected by four lines, in front of a feather (H6), angled forwards on top of a standard with a round top, with outwards angled lines coming from the tip of the pole (R14B), on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
14259		A long net without internal detail, which widens towards the end, with two loops on the front.
1425A		A long oval net with internal netting, with two loops at the front, with a triangular block at the back.
1425B		A long oval net with internal netting, with two loops at the front.
1425C		A round net with internal netting, with ties at the front which angle outwards and backwards.
1425D		A long, oval net  with internal netting, which widens towards the back, with two loops at the front, and a downwards curving rope at the back.
1425E		A long oval net, resembling a loop with vertical lines inside it, with ties at the front which curve outwards and backwards.
1425F		A net used for fishing or fowling, without weights.
14260		A fish-trap, which widens towards the bottom.
14261		A float, made of reed, with a circle with two ties at the top.
14262		A float, made of reed, with a circle with three ties at the top, which resemble triangles.
14263		A float, made of reed.
14265		A float, made out of two inwards curving bundles, connected at the bent with an horizontal line.
14268		A bird trap, with one vertical and horizontal pole, with multiple lines running from the horizontal to the vertical pole, with two lines at 45° coming from the corner.
14269		A bird trap, with one vertical and horizontal pole, with a net running from the front to the vertical pole, with two angled strokes on the middle of the net.
1426A		A bird trap, with two vertical poles at the back, with a net running from the front to the vertical poles, with two upwards ticks at the front of the net.
1426B		A bird trap, with two vertical poles at the back, with a net running from the front to the vertical poles.
1426C		A bird trap, with two vertical poles at the back, with a line running from the front to the vertical poles, with two upwards ticks at the front of the line.
1426D		A bird trap, with two vertical poles at the back, with two lines running from the front to the vertical poles, with two upwards ticks at the front of the line.
1426E		A bird trap, with a horizontal beam and an upstanding beam at 20° clockwise, with a line running from the front to the upstanding beam.
1426F		A bundle of reeds woven together, bending backwards.
14270		A hexagon-shaped net trap, with a rope to pull it close.
14271		An oval shaped net trap, with a triangular piece at either short end.
14272		A hexagon shaped net trap, on top of an oval base, bound between two pillars, the front pillar resembling a flowering reed (M17), the back resembling a stem of papyrus with a bud, with the head of a bird on top of it.
14273		A wickerwork fish trap.
14274		A knife with a triangular blade and a straight handle.
14275		A knife with a triangular blade and a straight handle (T30A), written horizontally, on top of an oval.
14276		A knife with a triangular blade and a straight handle (T30A), written inside an oval.
14277		A knife with a triangular blade and a straight handle (T30A), written at an forward angle, blade forwards, on top of an oval.
14278		A knife with a triangular blade and a straight handle (T30A), written at an forward angle, blade forwards, on top of an oval with three angled lines inside the oval.
1427A		A knife with a triangular blade and a straight handle (T30A), on top of a round-topped butchers block, with a base with upstanding edges.
1427B		A knife with a triangular blade and a straight handle (T30A), on top of a round-topped butchers block, with netting as internal decoration, with a base with upstanding edges.
1427C		A knife with a triangular blade and a straight handle (T30A), on top of a butchers block, with the base of the block resembling the hill country over the edge of the cultivated areas (N25).
1427D		A knife with a triangular blade and a straight handle (T30A), on top of ribs of beef (F43).
1427E		A knife with a triangular blade and a straight handle (T30A), with the tip of the knife on top of stack of ribs, on top of an upside down cone (F41C), angling forwards, blade upwards.
1427F		A knife with a triangular blade and a straight handle (T30A), on top of a round-topped butchers block, with the vertical lines of the base extending to the same hight as the top of the butcher's block.
14280		A knife with a triangular blade and a straight handle (T30A), on top of a wickerwork basket (V30).
14282		A knife with a rounded blade and rounded handle, with a dotted line of liquid coming from the blade.
14283		A knife with a rounded blade and rounded handle (T30), written over a butcher's block (T28).
14285		A knife with a rounded blade and rounded handle (T30), written over a beer-jug (W22).
14287		A knife sharpener, with a thin, curved body, and a loop at the end (T33), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
14289		A fish-tail shaped ritual object (knife?).
1428A		A horizontal line with a crescent moon shape at the front.
1428B		Two harpoon-heads, with a straight line above a angled line as tip, connected to each other.
1428C		A loop of cord with muliple lines with fishhooks hanging from it.
1428D		Assumed to be a curved knife.
1428F		A sickle (U1), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
14290		A hoe with a rope connecting the two pieces, written at an angle (U6), on top of a strip of land (N17).
14291		A hoe without a rope connecting the two pieces (U8), written at an angle, on top of a strip of land (N17).
14292		A hoe with a rope connecting the two pieces, written at an angle (U6), on top of a strip of land, with three grains of sand written beneath it (N16).
14293		A hoe without a rope connecting the two pieces (U8), written at an angle, on top of a circle.
14294		A hoe with a rope connecting the two pieces, with the frontal sections of the pieces crossing each other.
14296		A half-round vessel on its side, with three forward, downwards lines of grain coming from the front.
14297		A half-round vessel on its side, with a forward, downwards lines of grain coming from the front.
14298		A grain measure, resembling a horizontal stick, with a bulb at the front, with a vertical line on top of the horizontal stick, with an upwards stroke behind the vertical line.
14299		A grain measure, resembling a horizontal stick, with a downwards hanging bulb at the front, with a vertical line on top of the horizontal stick, with an upwards stroke behind the vertical line.
1429A		A grain measure, resembling a horizontal stick, with a downwards hanging bulb at the front, with a rectangle on top, with a upwards stroke behind the rectangle.
1429C		A plough, with a cross-bar and two vertical lines on the long beam, without a circle at the front of the long beam.
1429D		A plough, with a cross-bar and two vertical lines on the long beam, with the long beam curving upwards at the front.
1429F		A plough with a cross-bar, with two vertical lines on the tip of the top beam.
142A1		A plough, without a cross-bar and two vertical lines on the end of the long beam.
142A2		A hoe without a rope connecting the two pieces, with a downwards, backwards line of grains coming from the bottom piece.
142A3		A sledge with two ties on top.
142A4		A sledge, with a broad downwards loop at the front, with two forwards angled lines on top of the loop.
142A6		A sledge, with the head of a canine at the front, with a half-circle on the sledge.
142A7		A rectangular coffin with a rounded lid, and upstanding sides, with a three by three block internal decoration, on top of a sledge.
142A8		The sun, within a halo (N5), on top of a sledge (U15).
142A9		A hoe or pick excavating a rectangular pool, with the handle angling upwards, with the handle towards the front.
142AA		A hoe or pick excavating a rectangular pool, resembling the sky (N1), with the handle angling upwards, with the handle towards the back.
142AB		An adze with an attached blade at the front.
142AC		An adze with a triangular blade at the front.
142AD		An adze without the downwards curve at the end of the handle, with three blades.
142AE		An adze, with an attached blade at the front, in a piece of wood, with the end handle below the wood.
142AF		An adze with an attached blade at the front, with multiple horizontal lines.
142B0		An adze, with an attached triangular blade at the front, bound at the upwards piece of the handle, in a piece of wood, with the end handle ending at the hight of the wood.
142B1		A chisel with a conical handle, with a broad base and horizontal line on top, and a blade.
142B2		A chisel with a half-circle handle, vertical line at the top and a long blade.
142B3		A drill for drilling holes in beads, with a rectangular handle which slopes inwards towards the drill-bit, with a vertical line as drill-bit (U27B), on top of a horizontal rectangle.
142B4		A drill for drilling holes in beads, with a rectangular handle which slopes inwards towards the drill-bit, with a vertical line as drill-bit.
142B5		A drill for drilling holes in beads, with a handle resembling an upside down pawn, with a vertical line as drill-bit, on top of a horizontal rectangle.
142B7		A hair-pin or burin, with a horizontal line above and below the circle of the pin.
142B8		A hair-pin or burin, with a horizontal line under the oval of the pin, and a triangular shape on top of the oval.
142B9		A hair pin or burin, resembling a vase on a table.
142BA		A hair-pin or burin, with a straight vertical line as pin, with a round head with a triangular top.
142BB		A drill, consisting of a upside down conical handle which tapers out into the drill-bit.
142BC		A drill, with a conical handle with a dot on top, with a curl in towards the front at the handle, with a round drill-bit with a horizontal line over it.
142BD		A drill, with a conical handle, with a curl in towards the back at the handle, with a forked drill-bit with a horizontal line over it.
142BE		A drill, with a half-circle handle, with a curl towards the front on top of the handle, with a round drill-bit with a horizontal line on top of the circle, with a forwards curl coming from the horizontal line.
142BF		A drill, with a half-circle handle, with a curl towards the front on top of the handle, with a vertical line as drill-bit, on top of a horizontal rectangle.
142C1		A drill, with a half-circle handle, with a vertical line on top, with a round drill-bit.
142C2		A drill, with a half-circle handle, with a vertical line on top, with a forked drill-bit.
142C4		A drill, with a half-circle handle, with a vertical line on top, with a curl towards the front on top of the handle, with a forked drill-bit.
142C5		A drill, with a half-circle handle, with a vertical line on top, with a curl towards the front on top of the handle, with a forked drill-bit, with a horizontal line on top of the forked drill-bit.
142C6		A drill, with a conical handle on a horizontal line, with a outwards curl at either side of the handle, with a forked drill-bit.
142C7		A drill, with a half-circle handle, with a vertical line on top, with a curl towards the front on top of the handle, with a round drill-bit, with a horizontal line on top of the round drill-bit.
142C8		A drill, with a half-circle handle, with two lines on top, in a V shape, with a forked drill-bit.
142C9		A drill, with a half-circle handle, with a feather (H6) on top, with a round drill-bit with a horizontal line on top of the circle, with a short vertical stroke next to the drill, connected to the handle.
142CA		A drill, with a half-circle handle, with a forward angled stroke on top of the handle, with a forked drill-bit, with a horizontal line on top of the forked drill-bit.
142CB		A drill, with a round handle with a horizontal line on top, with a forwards, upwards line at the front of the circle, with a round drill-bit with a horizontal line over it.
142CD		A stoneworker's drill weighted at the top with stones, lashed together with strings, with two right angled bends in the drill-bit.
142CE		A stoneworker's drill, with a round handle with two ties at the top, in a V-shape, with two right angled bends in the drill-bit.
142D0		A drill, with a half-circle handle, with a forwards angled loop and a backwards tick on top of the handle, with a round drill-bit, with a horizontal line above and below the drill-bit.
142D1		A drill with a handle consisting of a horizontal line with a vertical line on top of each side, with a round drill-bit with a horizontal line over it.
142D3		A drill, with a rectangular handle with a forwards curving line on it, with a forked drill-bit with a horizontal line over it.
142D4		A drill, with a rectangular handle with a loop on it, with a round drill-bit with a horizontal line over it.
142D5		A drill, with a rectangular handle with a loop on it, with a forked drill-bit, with a horizontal line on top of the forked drill-bit.
142D6		A drill, with a handle consisting of a horizontal line with a curling line which curves from the back to the front on top of the horizontal line, with a forked drill-bit, with a horizontal line on top of the drill-bit.
142D7		A drill, with a handle consisting of a horizontal line with a circle on top of the horizontal line, with a round drill-bit, with a horizontal line on top of the drill-bit.
142D8		A pottery kiln, with the bottom curving forwards.
142D9		A pottery kiln, with the bottom curving forwards, without the round lid on top.
142DA		A pestle, inside a mortar, with angular corners.
142DB		A pestle, inside a square-bottomed mortar.
142DC		A pestle with a handle/tick at the front, inside a round-bottomed mortar.
142DD		A pestle with a vertical line over the bottom.
142DE		A spindle resembling a dagger with a pointed tip (T8A).
142DF		A razor, with a straight handle, orientated to the front.
142E0		Two razors, handles upwards, inside a pouch.
142E2		A simplified representation of a balance on a pole.
142E3		A balance, with the pole resembling a column with a base, with a tenon at the top (O28B), with a plummet, resembing a heart (F34).
142E4		A balance, without a pole (or a handheld balance).
142E5		A pole of a balance, with the pole resembling a seat (Q1).
142E8		The pole of a balance, with the pole resembling a seat (Q1), with a plummet resembling a heart (F34) hanging from the cross-beam.
142E9		The pole of a balance, with the pole resembling a seat (Q1), with a plummet resembling a heart (F34) hanging from a forwards, downwards curving line.
142EA		The pole of a balance, resembling a column with a base, with a tenon at the top (O28B), with an upwards angling cross-beam.
142EB		The pole of a balance, resembling a column with a base, with a tenon at the top (O28B), with a plummet resembling a heart (F34) hanging from the cross-beam.
142EC		A schematic representation of the pole of a balance.
142EE		The pole of a balance, represented as a vertical line with a curved hook at the front.
142EF		A tool, represented as a veritical line with a horizontal line at the front, with a line with a plummet resembling a heart (F34) hanging from the horizontal line.
142F0		The pole of a balance, represented as a vertical line with a curved hook at the front, with a horizontal base.
142F1		The pole of a balance, on a horizontal base, with a angled line on each side as support, with a line with a plummet resembling a heart (F34) hanging from the hook.
142F2		The pole of a balance, with the cross-bar resembling a feather (H6), written horizontally.
142F3		An astronomical instrument, with a line coming from the top of the rectangle, with a plummet at the end of the line, below the base of the object.
142F4		An astronomical instrument without a line and plummet.
142F6		An astronomical instrument to measure the movements of the stars, consisting of a square block coming from a long horizontal base, with a vertical line going over the middle of the square block, decending below the block, with a plummet in the shape of a heart (F34) hanging below the base.
142F7		An astronomical instrument to measure the movements of the stars, consisting of two vertical ovals at the front of a long horizontal base, with a plummet in the shape of a heart (F34) hanging below the base, connected to a vertical line between the two ovals, with an upwards dot in the middle of the horizontal base.
142F8		An astronomical instrument, resembling a façade of a shrine with a flat roof, with oblique sides, with a large doorway (O21B), on a long base, beside a sandy hill slope (N29), with a line coming from the top of the shrine, with a plummet at the end of the line, below the base of the shrine.
142F9		A level, consisting of three beams in a A shape, with a vertical line with a plummet coming from the top corner.
142FA		A level, consisting of two beams in a triangular formation, with a vertical line with a plummet coming from the top corner.
142FB		A tall potter's wheel, with a forward angling lump of clay on top of the wheel.
142FC		The head of a human man, seen in profile, with a short straight beard (D1), on top of a tall potter's wheel.
142FD		A tall potter's wheel, with a lump of clay on top of the wheel.
142FE		A flat bottomed jar without handles, with a broad rim, on top of a low potter's wheel.
142FF		A potter's wheel, resembling a wickerwork basket (V30), on a small vertical rectangle, with a wide base (O9C).
14300		A potters wheel.
14303		A wine press, consisiting of two rectangular forked poles, with a slightly curved line between the poles, on which a basket hangs, connected by three lines to the curved line.
14304		A wine press, consisiting of two rectangular forked poles, with a horizontal line just below the forks, extending beyond the poles, with two vertical lines of 3 dots coming from the middle of the horizontal line.
14306		A wine press, resembling two masts of a ship with two prongs, connected by vertical lines (P6) on a base, with a wide cup (W10) between the two masts, on top of the base, with two horizontal lines connecting the inner prongs of the masts.
14307		A wine press, resembling two masts of a ship with two prongs, connected by vertical lines (P6) on a base, with a wide cup (W10) between the two masts, on top of the base, with a horizontal line connecting the inner prongs of the mast, with a loop of cord with the ends upwards (V6), written with the ties over the horizontal line.
14308		A wine press, with two forked poles, with two horizontal lines written over the middle of the shaft of the two poles, extending beyond the poles, with a wide cup (W10) written on top of the top horizontal line.
1430B		A wine press, resembling two masts of a ship with two prongs, connected by vertical lines (P6) on a base, with a line connecting the shafts of the poles, with a loop and tie at the far side.
1430F		A warp, stretched between two forked poles with a short shaft below the fork.
14310		A warp, stretched between two poles resembling wide cups (W10), with ties extending downwards at the back.
14312		Two rectangular pieces of cloth, with fringes on the short sides, written vertically, with a loop around the two pieces of cloth.
14313		A circular sieve.
14314		A rectangular cage with netted sides.
14315		A cauldron.
14317		A milling tool, with a long vertical stick under a horizontal oval with vertical lines at the short sides.
14318		A milling tool, with a long vertical stick, under a half-circle, with vertical lines at the short sides.
14319		A diamond shaped plummet of a balance with an backwards and downwards curving string / A diamond shaped brazier with a flame rising from it, flame angling towards the front.
1431A		A broom.
1431B		A saw.
1431C		Upright of an ladder.
1431D		An instrument used by bricklayers, with a triangular extension at the front.
1431F		Three coils of rope (V1), arranged horizontally, on top of a horizontal line.
14320		A Hieratic ligature of a horizontal line, intersected in the middle by a coil of rope (V2), above legs in a walking posture, feet orientated towards the reading direction (D54).
14321		A rope, resembling a plan of a courtyard or reed shelter (O4).
14322		A loop of cord with the ends upwards (V6), combined with a folded piece of cloth (S29).
14323		A leaf, root and stem of a lotus-plant, with the leaf resembling a loop of cord with the ends downwards (V7), and with the root resembling a cartouche in a round form (V9).
14324		An egg (H8), written inside a cartouche in a round form (V9), with three stems of papyrus with a bud (M13) at the front, and three stems of papyrus with a flowering bud (M127) at the back.
14325		An oval cartouche, written vertially, with two horizontal lines inside the cartouche.
14327		A coiled rope, resembling a W shape.
14329		A wide band of string or fabric, written within an oval of dots.
1432A		A looped cord, used as a hobble for cattle, with two horizontal loops and four vertical loops.
1432B		A looped cord, used as a hobble for cattle, with two horizontal loops and six vertical loops.
1432C		A looped cord, used as a hobble for cattle, without horizontal loops, with six vertical loops.
1432D		A rolled up herdsman's shelter of papyrus, with an rectangle at either side and the top of the loop.
1432E		A rolled up herdsman's shelter of papyrus, with an rectangle at either side of the loop, and a cross on top of the loop.
1432F		A cobra in repose (Naja haja) (I10), written over a hobble for cattle without a crossbar (V20).
14331		Two loops of rope, with the ends curling outwards (V58), arranged horizontally, on top of a standard used for carrying religious symbols (R12).
14332		Two loops of rope, with the ends curling outwards (V58), arranged horizontally, on top of a standard used for carrying religious symbols (R12), written on top of a parcel of land with irrigation ditches (N24).
14333		A whip without internal detail and a loop at the back.
14334		A whip with a loop at the front and the back.
14336		A tool used for the fixing of nets, with loops at either ends.
14337		A wick of twisted flax, consisting of a singular loop at the top.
14338		A wick of twisted flax, consisting of four loops.
14339		A wick of twisted flax, consisting of two loops.
1433B		A swap made from a hank of fibre, with three loops, with a horizontal forward line coming from the top loop.
1433C		A swap made from a hank of fibre, with two loops, with a forward tick and a small loop on top of the top loop.
1433D		A mace with a pear-shaped head, written at a 45° angle forwards (T2), with the mace-head leaning against a wick of twisted flax, consisting of three loops (V28).
1433E		A swap made from a hank of fibre, with three loops, with three strokes on the top loop.
1433F		A swab made from a hank of fibre with two loops, with fringes on the wide top-piece.
14340		A swab made from a hank of fibre with three loops.
14341		A swap made from a hank of fibre, with two loops, with ties at the top and bottom.
14342		A wick of twisted flax, orentated on its side, curving upwards.
14343		A wickerwork basket with a handle at either side.
14344		A wickerwork basket, with a vertical string with a bead attached to it on the far side.
14345		Three feathers (H6), arranged horizontally, on top of a wickerwork basket with a handle, orientated with the handle to the back (V31).
14346		Square woven basket (see Jequier, BIFAO 7, 89-94)
14348		A wickerwork frail with ties, and spiked end-pieces.
14349		A wickerwork frail without ties or internal decoration.
1434A		A sheath or receptacle with V shaped indentation at the top.
1434B		A sheath or receptacle with V shaped indentation at the top, on top of a base, with a rectangle written inside it.
1434C		A sheath or receptacle with V shaped indentation at the top, bound at the top with a loop and tie at the backside.
1434D		A sheath or receptacle with V shaped indentation at the top, on top of a base, with a small rectangle inside, bound at the top with a tie and loop at either side.
1434E		A sheath or receptacle with V shaped indentation at the top, with a forward curved line coming from the indentation, bound at the top with a loop and tie at the backside.
1434F		A sheath or receptacle with V shaped indentation at the top, on top of a base, with a forward curved line coming from the indentation, bound at the top with a loop and tie at the backside.
14350		A sheath or receptacle with V shaped indentation at the top, on top of a base, with a forward curved line coming from the indentation.
14352		A basket-shaped bag with a tied of end at the front / A uterus of a cow, tied of at the front.
14353		A basket-shaped bag with a tied of end at the front, with a split end / A uterus of a cow, tied of at the front, with a split end.
14354		A basket-shaped bag with a tied of end at the front, resembling a loop, with a upwards stroke at the back / A uterus of a cow, tied of at the front, with an upwards stroke at the back.
14355		The womb of a cow, with falopian tubes.
14356		A cross-section of a cup or vessel.
14357		A net, with loops at the top ends.
14358		A net, with loops and ties at the top ends.
14359		A sealed ointment jar with a round lid, with ties.
1435A		A perfume jar with triangular handles, with a round lid with a triangular knob.
1435B		A sealed perfume jar with triangular handles, with a round lid, with ties.
1435C		A sealed vessel with triangular handles, with a round lid without knob.
1435D		A tall vessel with a round lid with rim, without knob.
1435E		A tall vessel with a round lid without rim, without knob.
1435F		A sealed perfume jar, without handles, with ties.
14360		An ointment jar witout seal.
14361		A vessel which widens at the base, with a rectangular lid.
14362		A vessel with a rectangular lid.
14363		An alabaster basin with a diamond shaped inner marking, with additional internal decoration / A wickework basket with a diamond shaped inner marking and with the wickerwork structure visible.
14364		An alabaster basin with a diamond shaped inner marking (W3), on top of a butcher's block (T28).
14366		An alabaster basin with a diamond shaped inner marking (W3), with two forearms coming from the top the basin, forearms and hands vertical, handpalms inwards, with a forearm, with the palm of the hand facing upwards (D36) written over the arms.
14367		A stone jug, with the base nearly a point, with a large handle connecting to the rim, without a small handle.
14369		A stone jug with a large handle connecting to the rim, and a small handle.
1436A		A rounded stone jug with a large handle connecting to the rim.
1436B		A tall stone jug, which widens at the base, with a large handle connecting to the rim.
1436C		A tall cup, with a rim wider than its base.
1436D		A ring-stand for jars, without internal decoration.
1436E		A tall water pot with a spout.
1436F		A tall water pot with a spout, with a line of liquid coming from the spout.
14370		A tall water pot with a spout, with a forwards, downwards line of liquid coming from the top of the vessel.
14371		A tall water pot with a spout, with a backwards, downwards line of liquid coming from the top of the vessel.
14372		A tall water pot, written at a 45° forward angle, with a forwards, downwards dotted line coming from the top.
14373		A tall water pot, with a forwards, downwards line of liquid coming from the top of the vessel, with the line ending on top of a canal (N36).
14374		A tall water pot with a spout, with a line of liquid coming from the spout (W15), with the line ending on top of a canal (N36).
14375		A tall water pot with a spout, with a line of liquid coming from the spout (W15), on top of the hill country over the edge of the cultivated areas (N25).
14376		A tall water pot with a spout, with a line of liquid coming from the spout (W15), on top of a  plan of a crossroads in a village (O49).
14377		A tall water pot with a spout, with a line of liquid coming from the spout (W15), in front of its mirror, with the lines of water ending on top of a  plan of a crossroads in a village (O49).
14378		A tall water pot with a spout, with a line of liquid coming from the spout (W15), inside a ring-stand.
14379		A representation of water pots in a rack, with the water pots resembling a rectangle with a slightly wider base and top.
1437A		A schematic representation of three sealed water pots in a rack, with the downwards curving lines connecting to the base.
1437B		A schematic representation of three sealed water pots in a rack, without a base, with a block of four rectangles on top.
1437C		A schematic representation of three sealed water pots in a rack.
1437D		A schematic representation of four sealed water pots in a rack.
1437F		A milk jar in a carrying sling, with upwards lines above the vessel, resembling the horns of a bovid (F13).
14380		A rounded milk jar with two horizontal lines over the jar, with a leaf covering the milk.
14381		A milk jar with a leaf covering the milk.
14384		A tall milk jug in a carrying sling, with upwards lines above the vessel, resembling the horns of a bovid (F13).
14385		A situla which widens at the base, with a triangular stand, with a carrying loop, with two upwards lines coming from the rim.
14386		A vase on its side, with liquid issuing from it.
14387		A vase on its side, with liquid issuing from it (W54), ending in a basin.
14388		A vase with a broad rim, written at a 45° forward angle, with a forwards line of liquid coming from the vase.
14389		Two round vessels with a broad rim, arranged horizontally, connected by two lines.
1438B		A twin set of round vessels with an upstanding rim, bound together.
1438C		Two conical wine jars, arranged horizontally, connected by multiple lines.
1438F		A wine jar with a broad rim, with two horizontal lines over the jar.
14390		A wine jar with a long neck and broad rim, with two horizontal lines over the jar.
14391		A long necked, round bottom vessel, with a broad rim, with a horizontal line over the vessel.
14392		A broad cup (W10), with a triangle of rectangular bricks on top of it.
14393		A flat bottomed jug with a rounded top, with a small upwards rim.
14395		A round vessel with an upstanding rim (W24), on top of long legs in a walking posture, feet orientated towards the reading direction (W25), on top of the sky (N1).
14396		Three round vessel with an upstanding rim (W24), arranged horizontally, on top of a rectangular base or tray.
14397		A granite vessel with small handles, written inside a diamond shaped enclosure, in which the ends of the lines overlap.
14399		A long, pointed vessel, written vertically.
1439A		A long, pointed vessel, written horizontally.
1439B		A long, broken, pointed vessel, written horizontally.
1439C		A rounded beer-jug.
1439D		A vase with the head of a human, seen in profile.
1439E		A well with a straight line of water (N42), with a horizontal tube attached at the back.
1439F		A vessel with a flat bottom, which is the widest point of the vessel, with a broad rim, as wide as the bottom.
143A0		A straight-bottomed vessel with broad rim, with a spout at the front.
143A1		A sealed straight-bottomed vessel, with a spout at the front.
143A2		A vessel with an upstanding rim and spout at the back, inside a wide cup.
143A3		A pointed vessel with a broad rim, with a spout at the front.
143A4		An amphora with a broad rim.
143A5		A vessel with a flat top and a widening base.
143A7		A flat-bottomed vessel which rounds towards the top, with a broad rim.
143A8		A wide bowl with a broad rim.
143A9		A vessel with a small base and a broad rim.
143AB		A broad dish, with a small base.
143AE		A flat bottomed jar without handles, with a broad rim and a small neck.
143AF		A pointed vessel with a spout, with line of fluid coming from the spout.
143B0		A round vessel with a broad rim, with a downwards line at either side of the neck.
143B3		A sealed beer-jug without handles.
143B4		A round vessel with a circular indentation at the top.
143B5		A pile of pottery, arranged in order to be fired.
143B7		The head of a ram without horns, on top of a jar.
143B8		A granite vessel with large handles.
143B9		A plate or low broad vessel, with a half circle on top of it.
143BA		A round vessel with an upstanding rim which curves downwards.
143BB		A rectangualar basin with a trianglular shape on top, with two feet inside the basin.
143BC		A round loaf of bread in a wide cup (W10).
143BD		A round loaf of bread on top of a wide cup (W10).
143BE		A round loaf with four curved imprints (X6B), on top of a wide cup (W10).
143C0		A loaf of bread with a horizontal base.
143C1		A loaf of bread, with a curved inprint at both long sides.
143C2		A conical loaf of bread, with an curved inprint on the sides and the bottom.
143C4		A roll of bread, with a rectangle at the middle of the top line.
143C5		A roll of bread, with a circle in the middle.
143C6		A roll of bread, with oval lines within it.
143C7		A roll of bread, with vertical ovals inside the edges, with a horizontal line in the middle.
143C8		A loaf of bread resembling a quarter oval, with round imprints at all four sides.
143C9		An horizontal oval shape, with two upwards curving strokes added at the front short side (X5), on top of legs in a walking posture, feet orientated towards the reading direction (D54).
143CB		A round loaf with an half circle finger mark at the bottom.
143CC		A round loaf with four curved imprints.
143CE		Four flat breads stacked on top of each other.
143CF		A papyrus scroll, rolled up, tied and sealed, written horizontally, with one tie at the front of the seal.
143D0		Two flutes, connected to each other.
143D1		A scribe's kit, consisting of a palette, ink or paint pouch and a reed pen, with the pallet resembling the counterweight of a necklace (S18A), and the reed pen resembling a forked pole for supporting a roof (O30).
143D2		A scribe's kit, consisting of a palette, ink or paint pouch and a  simplified reed pen.
143D3		A scribe's kit, consisting of a pallet and an ink or paint pouch.
143D4		A scribal pallet.
143D5		Scribe's water pot.
143D7		An writing tablet with a round top, with a vertical line at either side of the tablet.
143D8		A game board of the Mehen game.
143D9		A rounded harp, with a human head in profile on the tip.
143DA		A rounded harp, with a female human head, in profile, with a headdress of bovine horns with a sun disk (F102), on the tip.
143DB		A rounded harp, with a female human head, in profile, with a headdress of bovine horns with a sun disk with two plumes on top of the sun-disk (S65) on the tip.
143DC		A rounded harp, with a female human head, in profile, with a headdress consisting of two plumes, with a sun-disk between them (S63A), on the tip.
143DD		A triangular harp.
143DE	143DF	Tambourine.
143E0		Ball.
143E1		A sistrum, without internal detail on top.
143E2		A sistrum, with horizontal wavy lines over the top piece.
143E3		A sistrum with horizontal wavy lines over the top piece, with the human face on the handle having hair.
143E4		A sistrum, with two curls on top of human head with ears on top of the handle, without central piece.
143E5		A sistrum, with the top piece in the form of a shrine encosed by two curls, with the human face on the handle having hair.
143E6		The headpiece of a sistrum, resembling a façade of a shrine with a flat roof, with oblique sides, with a large doorway (O21B), enclosed by two curls.
143E7		Three diagonal strokes arranged horizontally.
143E9		A horizontal stroke overlaping a vertical stroke, both strokes of the same length.
143EA		A short horizontal stroke overlapped by a longer vertical stroke, the crossing point is near the top of the vertical stroke.
143EE		A diagonal line on top of a horizontal line, open at the front.
143EF		rectangular writing tablet with, with a vertical line at either side of the tablet.
143F0		A diagonal line, with a perpendicular line over the middle.
143F6		Verse-point
143F7		A long horizontal line, crossed by a short vertical line, in the middle.
143F8		A crescent moon shape written on top of a vertical line.
143FA		Two vertical ovals, connected together at the long sides.
END
